@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,16 +7,14 @@
 
 package net.sf.saxon.dom;
 
-import net.sf.saxon.event.Receiver;
-import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.lib.NamespaceConstant;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.AxisInfo;
+import net.sf.saxon.om.NamespaceBinding;
+import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.pattern.AnyNodeTest;
 import net.sf.saxon.pattern.NameTest;
 import net.sf.saxon.pattern.NodeTest;
-import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.AxisIterator;
-import net.sf.saxon.tree.iter.AxisIteratorImpl;
 import net.sf.saxon.tree.iter.LookaheadIterator;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.tree.util.Navigator;
@@ -26,6 +24,7 @@ import net.sf.saxon.tree.wrapper.AbstractNodeWrapper;
 import net.sf.saxon.tree.wrapper.SiblingCountingNode;
 import net.sf.saxon.type.Type;
 import net.sf.saxon.type.UType;
+import org.jetbrains.annotations.NotNull;
 import org.w3c.dom.*;
 
 import java.util.ArrayList;
@@ -33,9 +32,8 @@ import java.util.Arrays;
 
 
 /**
- * A node in the XML parse tree representing an XML element, character content, or attribute.<P>
+ * A node in the XML parse tree representing an XML element, character content, or attribute.
  * This is the implementation of the NodeInfo interface used as a wrapper for DOM nodes.
- * <p/>
  * <p>Because the DOM is not thread-safe even when reading, and because Saxon-EE can spawn multiple
  * threads that access the same input tree, all methods that invoke DOM methods are synchronized
  * on the Document object.</p>
@@ -140,7 +138,7 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
                 throw new IllegalStateException("DOM contains entity reference nodes, which Saxon does not support. The DOM should be built using the expandEntityReferences() option");
 
             default:
-                throw new IllegalArgumentException("Unsupported node type in DOM! " + node.getNodeType() + " instance " + node.toString());
+                throw new IllegalArgumentException("Unsupported node type in DOM! " + node.getNodeType() + " instance " + node);
         }
         wrapper.treeInfo = docWrapper;
         return wrapper;
@@ -154,7 +152,7 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
      * Get the underlying DOM node, to implement the VirtualNode interface
      */
 
-    public Object getUnderlyingNode() {
+    public Node getUnderlyingNode() {
         return node;
     }
 
@@ -169,14 +167,14 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
     }
 
     /**
-     * Determine whether this is the same node as another node. <br />
-     * Note: a.isSameNodeInfo(b) if and only if generateId(a)==generateId(b)
+     * Determine whether this is the same node as another node.
+     * <p>Note: a.equals(b) if and only if generateId(a)==generateId(b)</p>
      *
      * @return true if this Node object and the supplied Node object represent the
      *         same node in the tree.
      */
 
-    public boolean isSameNodeInfo(NodeInfo other) {
+    public boolean equals(Object other) {
         if (!(other instanceof DOMNodeWrapper)) {
             return false;
         }
@@ -189,7 +187,7 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
             return getNodeKind() == ow.getNodeKind() &&
                     equalOrNull(getLocalPart(), ow.getLocalPart()) &&  // redundant, but gives a quick exit
                     getSiblingPosition() == ow.getSiblingPosition() &&
-                    getParent().isSameNodeInfo(ow.getParent());
+                    getParent().equals(ow.getParent());
         }
     }
 
@@ -210,7 +208,7 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
     public int compareOrder(NodeInfo other) {
         // Use the DOM Level-3 compareDocumentPosition() method
         if (other instanceof DOMNodeWrapper && docWrapper.domLevel3) {
-            if (isSameNodeInfo(other)) {
+            if (equals(other)) {
                 return 0;
             }
             try {
@@ -239,51 +237,6 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
         }
     }
 
-    /**
-     * Determine the relative position of this node and another node, in document order,
-     * distinguishing whether the first node is a preceding, following, descendant, ancestor,
-     * or the same node as the second.
-     * <p/>
-     * The other node must always be in the same tree; the effect of calling this method
-     * when the two nodes are in different trees is undefined. If either node is a namespace
-     * or attribute node, the method should throw UnsupportedOperationException.
-     *
-     * @param other The other node, whose position is to be compared with this
-     *              node
-     * @return {@link net.sf.saxon.om.AxisInfo#PRECEDING} if this node is on the preceding axis of the other node;
-     *         {@link net.sf.saxon.om.AxisInfo#FOLLOWING} if it is on the following axis; {@link net.sf.saxon.om.AxisInfo#ANCESTOR} if the first node is an
-     *         ancestor of the second; {@link net.sf.saxon.om.AxisInfo#DESCENDANT} if the first is a descendant of the second;
-     *         {@link net.sf.saxon.om.AxisInfo#SELF} if they are the same node.
-     * @throws UnsupportedOperationException if either node is an attribute or namespace
-     * @since 9.5
-     */
-    public int comparePosition(NodeInfo other) {
-// Use the DOM Level-3 compareDocumentPosition() method
-        if (other instanceof DOMNodeWrapper && docWrapper.domLevel3) {
-            if (isSameNodeInfo(other)) {
-                return AxisInfo.SELF;
-            }
-            try {
-                synchronized (docWrapper.docNode) {
-                    short relationship = node.compareDocumentPosition(((DOMNodeWrapper) other).node);
-                    if ((relationship & Node.DOCUMENT_POSITION_PRECEDING) != 0) {
-                        return AxisInfo.FOLLOWING;
-                    } else if ((relationship & Node.DOCUMENT_POSITION_FOLLOWING) != 0) {
-                        return AxisInfo.PRECEDING;
-                    } else if ((relationship & Node.DOCUMENT_POSITION_CONTAINS) != 0) {
-                        return AxisInfo.ANCESTOR;
-                    } else if ((relationship & Node.DOCUMENT_POSITION_CONTAINED_BY) != 0) {
-                        return AxisInfo.DESCENDANT;
-                    }
-                }
-                // otherwise use fallback implementation (e.g. nodes in different documents)
-            } catch (DOMException e) {
-                //
-            }
-        }
-        return Navigator.comparePosition(this, other);
-    }
-
     /**
      * Get the value of the item as a CharSequence. This is in some cases more efficient than
      * the version of the method that returns a String.
@@ -573,7 +526,6 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
      * thus the sibling position for the text node is that of the first DOM node
      * to which it relates, and the numbering of subsequent XPath nodes is not necessarily
      * consecutive.
-     * <p/>
      * <p>Despite the name, this method also returns a meaningful result for attribute
      * nodes; it returns the position of the attribute among the attributes of its
      * parent element, when they are listed in document order.</p>
@@ -697,9 +649,9 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
     }
 
     /**
-     * Determine whether the node has any children. <br />
-     * Note: the result is equivalent to <br />
-     * getEnumeration(Axis.CHILD, AnyNodeTest.getInstance()).hasNext()
+     * Determine whether the node has any children.
+     * <p>Note: the result is equivalent to
+     * <code>getEnumeration(Axis.CHILD, AnyNodeTest.getInstance()).hasNext()</code></p>
      */
 
     public boolean hasChildNodes() {
@@ -721,14 +673,6 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
         Navigator.appendSequentialKey(this, buffer, true);
     }
 
-    /**
-     * Copy this node to a given outputter (deep copy)
-     */
-
-    public void copy(Receiver out, int copyOptions, Location locationId) throws XPathException {
-        Navigator.copy(this, out, copyOptions, locationId);
-    }
-
     /**
      * Get all namespace undeclarations and undeclarations defined on this element.
      *
@@ -741,7 +685,6 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
      *         If the bottom half is zero, then this is a namespace undeclaration rather than a declaration.
      *         The XML namespace is never included in the list. If the supplied array is larger than required,
      *         then the first unused entry will be set to -1.
-     *         <p/>
      *         <p>For a node other than an element, the method returns null.</p>
      */
 
@@ -815,10 +758,16 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
 
     public DOMNodeWrapper getNextSibling() {
         synchronized (docWrapper.docNode) {
-            Node currNode = node.getNextSibling();
+            Node currNode = node;
+            for (int i=0; i<span; i++) {
+                currNode = currNode.getNextSibling();
+            }
             if (currNode != null) {
-                if (currNode.getNodeType() == Node.DOCUMENT_TYPE_NODE) {
+                short type = currNode.getNodeType();
+                if (type == Node.DOCUMENT_TYPE_NODE) {
                     currNode = currNode.getNextSibling();
+                } else if (type == Node.TEXT_NODE || type == Node.CDATA_SECTION_NODE) {
+                    return spannedWrapper(currNode);
                 }
                 return makeWrapper(currNode, docWrapper);
             }
@@ -826,6 +775,23 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
         }
     }
 
+    @NotNull
+    private DOMNodeWrapper spannedWrapper(Node currNode) {
+        Node currText = currNode;
+        int thisSpan = 1;
+        while (true) {
+            currText = currText.getNextSibling();
+            if (currText != null && (currText.getNodeType() == Node.TEXT_NODE || currText.getNodeType() == Node.CDATA_SECTION_NODE)) {
+                thisSpan++;
+            } else {
+                break;
+            }
+        }
+        DOMNodeWrapper spannedText = makeWrapper(currNode, docWrapper);
+        spannedText.span = thisSpan;
+        return spannedText;
+    }
+
 
     public DOMNodeWrapper getFirstChild() {
         synchronized (docWrapper.docNode) {
@@ -834,6 +800,9 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
                 if (currNode.getNodeType() == Node.DOCUMENT_TYPE_NODE) {
                     currNode = currNode.getNextSibling();
                 }
+                if (currNode.getNodeType() == Node.TEXT_NODE || currNode.getNodeType() == Node.CDATA_SECTION_NODE) {
+                    return spannedWrapper(currNode);
+                }
                 return makeWrapper(currNode, docWrapper);
             }
             return null;
@@ -844,8 +813,23 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
         synchronized (docWrapper.docNode) {
             Node currNode = node.getPreviousSibling();
             if (currNode != null) {
-                if (currNode.getNodeType() == Node.DOCUMENT_TYPE_NODE) {
+                short type = currNode.getNodeType();
+                if (type == Node.DOCUMENT_TYPE_NODE) {
                     return null;
+                } else if (type == Node.TEXT_NODE || type == Node.CDATA_SECTION_NODE) {
+                    int span = 1;
+                    while (true) {
+                        Node prev = currNode.getPreviousSibling();
+                        if (prev != null && (prev.getNodeType() == Node.TEXT_NODE || prev.getNodeType() == Node.CDATA_SECTION_NODE)) {
+                            span++;
+                            currNode = prev;
+                        } else {
+                            break;
+                        }
+                    }
+                    DOMNodeWrapper wrapper = makeWrapper(currNode, docWrapper);
+                    wrapper.span = span;
+                    return wrapper;
                 }
                 return makeWrapper(currNode, docWrapper);
             }
@@ -899,9 +883,9 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
         }
     }
 
-    private final class AttributeEnumeration implements AxisIterator, LookaheadIterator {
+    private final class AttributeEnumeration implements AxisIterator, LookaheadIterator<NodeInfo> {
 
-        private ArrayList<Node> attList = new ArrayList<Node>(10);
+        private ArrayList<Node> attList = new ArrayList<>(10);
         private int ix = 0;
         private DOMNodeWrapper start;
         private DOMNodeWrapper current;
@@ -937,10 +921,6 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
             return current;
         }
 
-        public void close() {
-        }
-
-
         /**
          * Get properties of this iterator, as a bit-significant integer.
          *
@@ -964,7 +944,7 @@ public class DOMNodeWrapper extends AbstractNodeWrapper implements SiblingCounti
      * preceding and preceding-or-ancestor axes (the latter being used by xsl:number)
      */
 
-    private final class ChildEnumeration extends AxisIteratorImpl implements LookaheadIterator {
+    private final class ChildEnumeration implements AxisIterator, LookaheadIterator<NodeInfo> {
 
         private DOMNodeWrapper start;
         private DOMNodeWrapper commonParent;
