@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -39,7 +39,7 @@ public class DOMSender {
     private Receiver receiver;
     protected Node root;
     protected String systemId;
-    private Stack<List<NamespaceBinding>> namespaces = new Stack<List<NamespaceBinding>>();
+    private Stack<List<NamespaceBinding>> namespaces = new Stack<>();
     private Node currentNode;
 
     /**
@@ -164,10 +164,9 @@ public class DOMSender {
      */
 
     private void sendElement(Element startNode) throws XPathException {
-        Element node = startNode;
-        List<Element> ancestors = new java.util.ArrayList<Element>();
-        List<NamespaceBinding> bindings = new java.util.ArrayList<NamespaceBinding>();
-        Node parent = node;
+        List<Element> ancestors = new ArrayList<>();
+        List<NamespaceBinding> bindings = new ArrayList<>();
+        Node parent = startNode;
         while (parent != null && parent.getNodeType() == Type.ELEMENT) {
             ancestors.add((Element) parent);
             parent = parent.getParentNode();
@@ -177,12 +176,12 @@ public class DOMSender {
         }
         // Eliminate duplicate bindings. We need to take the last binding for each prefix. We can't leave this to
         // the NamespaceReducer, because it does it the wrong way.
-        Map<String, String> distinctNamespaces = new HashMap<String, String>();
+        Map<String, String> distinctNamespaces = new HashMap<>();
         for (NamespaceBinding nb : bindings) {
             distinctNamespaces.put(nb.getPrefix(), nb.getURI());
         }
         if (distinctNamespaces.size() != bindings.size()) {
-            bindings = new java.util.ArrayList<NamespaceBinding>();
+            bindings = new ArrayList<>();
             for (Map.Entry<String, String> entry : distinctNamespaces.entrySet()) {
                 bindings.add(new NamespaceBinding(entry.getKey(), entry.getValue()));
             }
@@ -241,10 +240,10 @@ public class DOMSender {
                         break;                  // should not happen
                     case Node.ELEMENT_NODE:
                         Element element = (Element) child;
-                        List<NamespaceBinding> bindings = new java.util.ArrayList<NamespaceBinding>();
+                        List<NamespaceBinding> bindings = new ArrayList<>();
                         namespaces.push(bindings);
                         gatherNamespaces(element, bindings);
-                        outputElement(element, bindings.size() > 0);
+                        outputElement(element, !bindings.isEmpty());
                         namespaces.pop();
                         break;
                     case Node.ATTRIBUTE_NODE:        // have already dealt with attributes
@@ -297,7 +296,7 @@ public class DOMSender {
      * @param element the element to be processed.
      * @param hasNamespaceDeclarations true if the attributes of the element contain namespace declaration
      *                                 attributes (which the method then ignores)
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs
      */
 
     private void outputElement(Element element, boolean hasNamespaceDeclarations) throws XPathException {
