@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -33,7 +33,7 @@ public class DOMTransform extends Transform {
 
     public List<Source> preprocess(List<Source> sources) throws XPathException {
         try {
-            ArrayList<Source> domSources = new ArrayList<Source>(sources.size());
+            ArrayList<Source> domSources = new ArrayList<>(sources.size());
             for (Object source : sources) {
                 StreamSource src = (StreamSource) source;
                 InputSource ins = new InputSource(src.getSystemId());
@@ -52,11 +52,7 @@ public class DOMTransform extends Transform {
                 domSources.add(dom.getRootNode());
             }
             return domSources;
-        } catch (ParserConfigurationException e) {
-            throw new XPathException(e);
-        } catch (SAXException e) {
-            throw new XPathException(e);
-        } catch (IOException e) {
+        } catch (ParserConfigurationException | IOException | SAXException e) {
             throw new XPathException(e);
         }
     }
