@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -41,17 +41,14 @@ import java.util.Stack;
  * A ContentHandlerProxy is a Receiver that converts events into the form expected by an
  * underlying SAX2 ContentHandler. Relevant events (notably comments) can also be
  * fed to a LexicalHandler.
- * <p/>
- * Note that in general the output passed to a Receiver
+ * <p>Note that in general the output passed to a Receiver
  * corresponds to an External General Parsed Entity. A SAX2 ContentHandler only expects
  * to deal with well-formed XML documents, so we only pass it the contents of the first
- * element encountered, unless the saxon:require-well-formed output property is set to "no".
- * </p><p>
- * This ContentHandlerProxy provides no access to type information. For a ContentHandler that
- * makes type information available, see {@link com.saxonica.ee.jaxp.TypedContentHandler}
- * <p></p>
- * The ContentHandlerProxy can also be nominated as a TraceListener, to receive notification
- * of trace events. This will be done automatically if the option setTraceListener(
+ * element encountered, unless the saxon:require-well-formed output property is set to "no".</p>
+ * <p>This ContentHandlerProxy provides no access to type information. For a ContentHandler that
+ * makes type information available, see {@link com.saxonica.ee.jaxp.TypedContentHandler}</p>
+ * <p>The ContentHandlerProxy can also be nominated as a TraceListener, to receive notification
+ * of trace events.</p>
  */
 
 public class ContentHandlerProxy implements Receiver {
@@ -62,8 +59,8 @@ public class ContentHandlerProxy implements Receiver {
     private int depth = 0;
     private boolean requireWellFormed = false;
     private boolean undeclareNamespaces = false;
-    private Stack<String> elementStack = new Stack<String>();
-    private Stack<String> namespaceStack = new Stack<String>();
+    private Stack<String> elementStack = new Stack<>();
+    private Stack<String> namespaceStack = new Stack<>();
     private ContentHandlerProxyTraceListener traceListener;
     protected AttributeCollectionImpl pendingAttributes;
     private NodeName pendingElement = null;
@@ -211,7 +208,7 @@ public class ContentHandlerProxy implements Receiver {
      *                {@link SaxonOutputKeys#REQUIRE_WELL_FORMED} and {@link net.sf.saxon.lib.SaxonOutputKeys#UNDECLARE_PREFIXES}.
      */
 
-    public void setOutputProperties(Properties details) throws XPathException {
+    public void setOutputProperties(Properties details) {
         String prop = details.getProperty(SaxonOutputKeys.REQUIRE_WELL_FORMED);
         if (prop != null) {
             requireWellFormed = prop.equals("yes");
@@ -343,7 +340,7 @@ public class ContentHandlerProxy implements Receiver {
                 return;
             }
             String uri = ns.getURI();
-            if (!undeclareNamespaces && uri.isEmpty() && prefix.length() != 0) {
+            if (!undeclareNamespaces && uri.isEmpty() && !prefix.isEmpty()) {
                 // This is a namespace undeclaration, but the ContentHandler doesn't want to know about undeclarations
                 return;
             }
@@ -408,6 +405,7 @@ public class ContentHandlerProxy implements Receiver {
     public void endElement() throws XPathException {
         if (depth > 0) {
             try {
+                assert !elementStack.isEmpty();
                 String qname = elementStack.pop();
                 String localName = elementStack.pop();
                 String uri = elementStack.pop();
@@ -588,7 +586,7 @@ public class ContentHandlerProxy implements Receiver {
          */
 
         public void open(Controller controller) {
-            contextItemStack = new Stack<Item>();
+            contextItemStack = new Stack<>();
         }
 
         /**
@@ -637,7 +635,7 @@ public class ContentHandlerProxy implements Receiver {
 
         public void startCurrentItem(Item currentItem) {
             if (contextItemStack == null) {
-                contextItemStack = new Stack<Item>();
+                contextItemStack = new Stack<>();
             }
             contextItemStack.push(currentItem);
         }
