@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -24,11 +24,9 @@ import java.util.Comparator;
 import java.util.List;
 
 /**
- * The PIGrabber class is a Receiver that looks for xml-stylesheet processing
+ * The <tt>PIGrabber</tt> class is a {@link ProxyReceiver} that looks for {@code xml-stylesheet} processing
  * instructions and tests whether they match specified criteria; for those that do, it creates
- * an InputSource object referring to the relevant stylesheet
- *
- * @author Michael H. Kay
+ * a {@link Source} object referring to the relevant stylesheet
  */
 
 public class PIGrabber extends ProxyReceiver {
@@ -38,7 +36,7 @@ public class PIGrabber extends ProxyReceiver {
     private String reqTitle = null;
     private String baseURI = null;
     private URIResolver uriResolver = null;
-    private List<String> stylesheets = new ArrayList<String>();
+    private List<String> stylesheets = new ArrayList<>();
     private boolean terminated = false;
 
     public PIGrabber(Receiver next) {
@@ -119,7 +117,9 @@ public class PIGrabber extends ProxyReceiver {
             String piType = ProcInstParser.getPseudoAttribute(value, "type");
             String piAlternate = ProcInstParser.getPseudoAttribute(value, "alternate");
 
-            if (piType == null) return;
+            if (piType == null) {
+                return;
+            }
 
             // System.err.println("Found xml-stylesheet media=" + piMedia + " title=" + piTitle);
 
@@ -159,7 +159,7 @@ public class PIGrabber extends ProxyReceiver {
 
     /*@Nullable*/
     public Source[] getAssociatedStylesheets() throws TransformerException {
-        if (stylesheets.size() == 0) {
+        if (stylesheets.isEmpty()) {
             return null;
         }
         if (uriResolver == null) {
