@@ -12,6 +12,7 @@ import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.linked.LinkedTreeBuilder;
+import net.sf.saxon.tree.tiny.TinyElementImpl;
 import net.sf.saxon.tree.util.Orphan;
 import net.sf.saxon.type.SchemaType;
 import net.sf.saxon.type.SimpleType;
@@ -148,6 +149,7 @@ public abstract class SequenceWriter extends SequenceReceiver {
             NodeInfo doc = builder.getCurrentRoot();
             // add the constructed document to the result sequence
             append(doc, ExplicitLocation.UNKNOWN_LOCATION, ReceiverOptions.ALL_NAMESPACES);
+            systemId = null;
         }
         previousAtomic = false;
     }
@@ -181,6 +183,7 @@ public abstract class SequenceWriter extends SequenceReceiver {
             outputter = null;
             NodeInfo element = builder.getCurrentRoot();
             append(element, ExplicitLocation.UNKNOWN_LOCATION, ReceiverOptions.ALL_NAMESPACES);
+            systemId = null;
         }
         previousAtomic = false;
     }
@@ -300,5 +303,30 @@ public abstract class SequenceWriter extends SequenceReceiver {
     public boolean usesTypeAnnotations() {
         return outputter == null || outputter.usesTypeAnnotations();
     }
+
+
+    public boolean isReadyForBulkCopy() {
+        return level > 0 && outputter instanceof ComplexContentOutputter &&
+                ((ComplexContentOutputter)outputter).isReadyForBulkCopy();
+    }
+
+    public void bulkCopyElementNode(TinyElementImpl node, int copyOptions) throws XPathException {
+        if (inStartTag) {
+            startContent();
+        }
+        ((ComplexContentOutputter) outputter).bulkCopyElementNode(node, copyOptions);
+    }
+
+    public boolean isReadyForGrafting() {
+        return level > 0 && outputter instanceof ComplexContentOutputter &&
+                ((ComplexContentOutputter) outputter).isReadyForGrafting();
+    }
+
+    public void graftElementNode(TinyElementImpl node, int copyOptions) throws XPathException {
+        if (inStartTag) {
+            startContent();
+        }
+        ((ComplexContentOutputter) outputter).graftElementNode(node, copyOptions);
+    }
 }
 
