@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,6 +10,7 @@ package net.sf.saxon.event;
 import net.sf.saxon.Controller;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.XsltController;
 
 import javax.xml.transform.TransformerException;
 
@@ -23,20 +24,21 @@ import javax.xml.transform.TransformerException;
 
 public class TransformerReceiver extends ProxyReceiver {
 
-    private Controller controller;
+    private XsltController controller;
     private Builder builder;
     private Receiver destination;
 
     /**
      * Create a TransformerReceiver and initialise variables.
      *
-     * @param controller the Controller (Saxon's implementation of the JAXP {@link javax.xml.transform.Transformer})
+     * @param controller the Controller
      */
 
-    public TransformerReceiver(Controller controller) {
+    public TransformerReceiver(XsltController controller) {
         super(controller.makeBuilder());
         this.controller = controller;
-        this.builder = (Builder) getUnderlyingReceiver();
+        this.builder = (Builder) getNextReceiver();
+        this.builder.setUseEventLocation(false);
     }
 
     /**
@@ -100,21 +102,26 @@ public class TransformerReceiver extends ProxyReceiver {
      */
 
     public void close() throws XPathException {
-        nextReceiver.close();
-        NodeInfo doc = builder.getCurrentRoot();
-        builder.reset();
-        builder = null;
-        if (doc == null) {
-            throw new XPathException("No source document has been built");
-        }
-        if (destination == null) {
-            throw new XPathException("No output destination has been supplied");
-        }
-        try {
-            controller.setGlobalContextItem(doc);
-            controller.transformDocument(doc, destination);
-        } catch (TransformerException e) {
-            throw XPathException.makeXPathException(e);
+        if (builder == null) {
+            // already closed
+        } else {
+            nextReceiver.close();
+            NodeInfo doc = builder.getCurrentRoot();
+            builder.reset();
+            builder = null;
+            if (doc == null) {
+                throw new XPathException("No source document has been built");
+            }
+            doc.getTreeInfo().setSpaceStrippingRule(controller.getSpaceStrippingRule());
+            if (destination == null) {
+                throw new XPathException("No output destination has been supplied");
+            }
+            try {
+                controller.setGlobalContextItem(doc);
+                controller.applyTemplates(doc, destination);
+            } catch (TransformerException e) {
+                throw XPathException.makeXPathException(e);
+            }
         }
     }
 
