@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -37,7 +37,7 @@ public class AndExpression extends BooleanExpression {
         if (Literal.isConstantBoolean(getLhsExpression(), false) || Literal.isConstantBoolean(getRhsExpression(), false)) {
             // A and false() => false()
             // false() and B => false()
-            return Literal.makeLiteral(BooleanValue.FALSE);
+            return Literal.makeLiteral(BooleanValue.FALSE, this);
         } else if (Literal.hasEffectiveBooleanValue(getLhsExpression(), true)) {
             // true() and B => B
             return forceToBoolean(getRhsExpression());
@@ -52,7 +52,6 @@ public class AndExpression extends BooleanExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -84,7 +83,7 @@ public class AndExpression extends BooleanExpression {
                 th.isSubType(getRhsExpression().getItemType(), BuiltInAtomicType.BOOLEAN) &&
                 !ExpressionTool.isLoopingSubexpression(this, null)) {
             Expression cond = Choose.makeConditional(
-                    getLhsExpression(), getRhsExpression(), Literal.makeLiteral(BooleanValue.FALSE));
+                    getLhsExpression(), getRhsExpression(), Literal.makeLiteral(BooleanValue.FALSE, this));
             ExpressionTool.copyLocationInfo(this, cond);
             return cond;
         }
