@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,20 +7,15 @@
 
 package net.sf.saxon.expr;
 
-import com.saxonica.ee.bytecode.ArithmeticCompiler;
-import com.saxonica.ee.bytecode.ExpressionCompiler;
+import net.sf.saxon.expr.compat.ArithmeticExpression10;
+import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.parser.*;
-import net.sf.saxon.om.SequenceTool;
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
-import net.sf.saxon.value.AtomicValue;
-import net.sf.saxon.value.IntegerValue;
-import net.sf.saxon.value.NumericValue;
-import net.sf.saxon.value.SequenceType;
-
-import java.util.Map;
+import net.sf.saxon.value.*;
 
 /**
  * Arithmetic Expression: an expression using one of the operators
@@ -30,8 +25,7 @@ import java.util.Map;
 
 public class ArithmeticExpression extends BinaryExpression {
 
-    private Calculator calculator;
-    protected boolean simplified = false;
+    protected Calculator calculator;
     private PlainType itemType;
 
     /**
@@ -58,30 +52,16 @@ public class ArithmeticExpression extends BinaryExpression {
         return "arithmetic";
     }
 
-    /*@NotNull*/
-    public Expression simplify() throws XPathException {
-        if (simplified) {
-            // Don't simplify more than once; because in XSLT the static context on subsequent calls
-            // might not know whether backwards compatibility is in force or not
-            return this;
-        }
-        simplified = true;
-        Expression e = super.simplify();
-        if (e == this && getRetainedStaticContext().isBackwardsCompatibility()) {
-            ArithmeticExpression10 ar10 = new ArithmeticExpression10(getLhsExpression(), operator, getRhsExpression());
-            ExpressionTool.copyLocationInfo(this, ar10);
-            return ar10;
-        } else {
-            if (operator == Token.NEGATE && Literal.isAtomic(getRhsExpression())) {
-                // very early evaluation of expressions like "-1", so they are treated as numeric literals
-                AtomicValue val = (AtomicValue) ((Literal) getRhsExpression()).getValue();
-                if (val instanceof NumericValue) {
-                    e = Literal.makeLiteral(((NumericValue) val).negate());
-                    e.setRetainedStaticContext(getRetainedStaticContext());
-                }
-            }
-            return e;
-        }
+    /**
+     * Determine the special properties of this expression
+     *
+     * @return {@link StaticProperty#NO_NODES_NEWLY_CREATED}. This is overridden
+     * for some subclasses.
+     */
+    @Override
+    public int computeSpecialProperties() {
+        int p = super.computeSpecialProperties();
+        return p | StaticProperty.NOT_UNTYPED_ATOMIC;
     }
 
     /**
@@ -112,9 +92,12 @@ public class ArithmeticExpression extends BinaryExpression {
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
 
         resetLocalStaticProperties();
-        typeCheckChildren(visitor, contextInfo);
+        getLhs().typeCheck(visitor, contextInfo);
+        getRhs().typeCheck(visitor, contextInfo);
 
-        final TypeHierarchy th = getConfiguration().getTypeHierarchy();
+        Configuration config = visitor.getConfiguration();
+        final TypeHierarchy th = config.getTypeHierarchy();
+        final TypeChecker tc = config.getTypeChecker(false);
 
         Expression oldOp0 = getLhsExpression();
         Expression oldOp1 = getRhsExpression();
@@ -123,19 +106,19 @@ public class ArithmeticExpression extends BinaryExpression {
 
         RoleDiagnostic role0 = new RoleDiagnostic(RoleDiagnostic.BINARY_EXPR, Token.tokens[operator], 0);
         //role0.setSourceLocator(this);
-        setLhsExpression(TypeChecker.staticTypeCheck(getLhsExpression(), atomicType, false, role0, visitor));
+        setLhsExpression(tc.staticTypeCheck(getLhsExpression(), atomicType, role0, visitor));
         final ItemType itemType0 = getLhsExpression().getItemType();
         if (itemType0 instanceof ErrorType) {
             return Literal.makeEmptySequence();
         }
         AtomicType type0 = (AtomicType) itemType0.getPrimitiveItemType();
         if (type0.getFingerprint() == StandardNames.XS_UNTYPED_ATOMIC) {
-            setLhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(getConfiguration(), getLhsExpression(), BuiltInAtomicType.DOUBLE));
+            setLhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(config, getLhsExpression(), BuiltInAtomicType.DOUBLE));
             type0 = BuiltInAtomicType.DOUBLE;
         } else if (/*!(operand0 instanceof UntypedAtomicConverter)*/
                 (getLhsExpression().getSpecialProperties() & StaticProperty.NOT_UNTYPED_ATOMIC) == 0 &&
                         th.relationship(type0, BuiltInAtomicType.UNTYPED_ATOMIC) != TypeHierarchy.DISJOINT) {
-            setLhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(getConfiguration(), getLhsExpression(), BuiltInAtomicType.DOUBLE));
+            setLhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(config, getLhsExpression(), BuiltInAtomicType.DOUBLE));
             type0 = (AtomicType) getLhsExpression().getItemType().getPrimitiveItemType();
         }
 
@@ -143,19 +126,19 @@ public class ArithmeticExpression extends BinaryExpression {
 
         RoleDiagnostic role1 = new RoleDiagnostic(RoleDiagnostic.BINARY_EXPR, Token.tokens[operator], 1);
         //role1.setSourceLocator(this);
-        setRhsExpression(TypeChecker.staticTypeCheck(getRhsExpression(), atomicType, false, role1, visitor));
+        setRhsExpression(tc.staticTypeCheck(getRhsExpression(), atomicType, role1, visitor));
         final ItemType itemType1 = getRhsExpression().getItemType();
         if (itemType1 instanceof ErrorType) {
             return Literal.makeEmptySequence();
         }
         AtomicType type1 = (AtomicType) itemType1.getPrimitiveItemType();
         if (type1.getFingerprint() == StandardNames.XS_UNTYPED_ATOMIC) {
-            setRhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(getConfiguration(), getRhsExpression(), BuiltInAtomicType.DOUBLE));
+            setRhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(config, getRhsExpression(), BuiltInAtomicType.DOUBLE));
             type1 = BuiltInAtomicType.DOUBLE;
         } else if (/*!(operand1 instanceof UntypedAtomicConverter) &&*/
                 (getRhsExpression().getSpecialProperties() & StaticProperty.NOT_UNTYPED_ATOMIC) == 0 &&
                         th.relationship(type1, BuiltInAtomicType.UNTYPED_ATOMIC) != TypeHierarchy.DISJOINT) {
-            setRhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(getConfiguration(), getRhsExpression(), BuiltInAtomicType.DOUBLE));
+            setRhsExpression(UntypedSequenceConverter.makeUntypedSequenceConverter(config, getRhsExpression(), BuiltInAtomicType.DOUBLE));
             type1 = (AtomicType) getRhsExpression().getItemType().getPrimitiveItemType();
         }
 
@@ -182,7 +165,7 @@ public class ArithmeticExpression extends BinaryExpression {
         if (operator == Token.NEGATE) {
             if (getRhsExpression() instanceof Literal && ((Literal) getRhsExpression()).getValue() instanceof NumericValue) {
                 NumericValue nv = (NumericValue) ((Literal) getRhsExpression()).getValue();
-                return Literal.makeLiteral(nv.negate());
+                return Literal.makeLiteral(nv.negate(), this);
             } else {
                 NegateExpression ne = new NegateExpression(getRhsExpression());
                 ne.setBackwardsCompatible(false);
@@ -209,10 +192,26 @@ public class ArithmeticExpression extends BinaryExpression {
             throw de;
         }
 
+        // If the calculator is going to promote arguments to xs:double, then promote any literal arguments now.
+        // (Could generalize this, but this is the common case)
+        if (calculator.code().matches("d.d")) {
+            if (getLhsExpression() instanceof Literal && !type0.equals(BuiltInAtomicType.DOUBLE)) {
+                GroundedValue value = ((Literal) getLhsExpression()).getValue();
+                if (value instanceof NumericValue) {
+                    setLhsExpression(Literal.makeLiteral(new DoubleValue(((NumericValue) value).getDoubleValue()), this));
+                }
+            }
+            if (getRhsExpression() instanceof Literal && !type1.equals(BuiltInAtomicType.DOUBLE)) {
+                GroundedValue value = ((Literal) getRhsExpression()).getValue();
+                if (value instanceof NumericValue) {
+                    setRhsExpression(Literal.makeLiteral(new DoubleValue(((NumericValue) value).getDoubleValue()), this));
+                }
+            }
+        }
+
         try {
             if ((getLhsExpression() instanceof Literal) && (getRhsExpression() instanceof Literal)) {
-                return Literal.makeLiteral(SequenceTool.toGroundedValue(evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext()))
-                );
+                return Literal.makeLiteral(evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext()).materialize(), this);
             }
         } catch (XPathException err) {
             // if early evaluation fails, suppress the error: the value might
@@ -295,7 +294,6 @@ public class ArithmeticExpression extends BinaryExpression {
         ArithmeticExpression ae = new ArithmeticExpression(getLhsExpression().copy(rebindings), operator, getRhsExpression().copy(rebindings));
         ExpressionTool.copyLocationInfo(this, ae);
         ae.calculator = calculator;
-        ae.simplified = simplified;
         return ae;
     }
 
@@ -386,6 +384,25 @@ public class ArithmeticExpression extends BinaryExpression {
         }
     }
 
+    /**
+     * Get the static type of the expression as a UType, following precisely the type
+     * inference rules defined in the XSLT 3.0 specification.
+     *
+     * @return the static item type of the expression according to the XSLT 3.0 defined rules
+     * @param contextItemType
+     */
+    @Override
+    public UType getStaticUType(UType contextItemType) {
+        // The rationale for this is in the XSLT 3.0 spec
+        if (getParentExpression() instanceof FilterExpression && ((FilterExpression)getParentExpression()).getRhsExpression() == this) {
+            return UType.NUMERIC;
+        } else if (operator == Token.NEGATE) {
+            return UType.NUMERIC;
+        } else {
+            return UType.ANY_ATOMIC;
+        }
+    }
+
     /**
      * Reset the static properties of the expression to -1, so that they have to be recomputed
      * next time they are used.
@@ -416,6 +433,7 @@ public class ArithmeticExpression extends BinaryExpression {
             return calculator.compute(v0, v1, context);
         } catch (XPathException e) {
             e.maybeSetLocation(getLocation());
+            e.maybeSetFailingExpression(this);
             e.maybeSetContext(context);
             throw e;
         }
@@ -433,18 +451,6 @@ public class ArithmeticExpression extends BinaryExpression {
         }
     }
 
-    //#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the Arithmetic expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
-    @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new ArithmeticCompiler();
-    }
-    //#endif
 
 
 }
