@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,6 @@
 
 package net.sf.saxon.expr;
 
-import net.sf.saxon.expr.parser.ExpressionAction;
 import net.sf.saxon.expr.parser.ExpressionTool;
 import net.sf.saxon.expr.parser.PathMap;
 import net.sf.saxon.om.*;
@@ -126,7 +125,7 @@ public abstract class Assignation extends Expression implements LocalBinding {
      * propagated). This method should be called only once, to compute the dependencies;
      * after that, getDependencies should be used.
      *
-     * @return the depencies, as a bit-mask
+     * @return the dependencies, as a bit-mask
      */
     @Override
     public int computeDependencies() {
@@ -144,10 +143,10 @@ public abstract class Assignation extends Expression implements LocalBinding {
      * Get the value of the range variable
      */
 
-    public Sequence evaluateVariable(XPathContext context) throws XPathException {
-        Sequence actual = context.evaluateLocalVariable(slotNumber);
+    public Sequence<?> evaluateVariable(XPathContext context) throws XPathException {
+        Sequence<?> actual = context.evaluateLocalVariable(slotNumber);
         if (!(actual instanceof GroundedValue || actual instanceof NodeInfo)) {
-            actual = SequenceTool.toGroundedValue(actual);
+            actual = actual.materialize();
             context.setLocalVariable(slotNumber, actual);
         }
         return actual;
@@ -307,7 +306,6 @@ public abstract class Assignation extends Expression implements LocalBinding {
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -378,17 +376,14 @@ public abstract class Assignation extends Expression implements LocalBinding {
 
     public void refineTypeInformation(final ItemType type,
                                       final int cardinality,
-                                      final GroundedValue constantValue,
+                                      final GroundedValue<?> constantValue,
                                       final int properties,
                                       final Assignation currentExpression) throws XPathException {
-        ExpressionTool.processExpressionTree(currentExpression.getAction(), null, new ExpressionAction() {
-            @Override
-            public boolean process(Expression exp, Object result) throws XPathException {
-                if (exp instanceof VariableReference && ((VariableReference)exp).getBinding() == currentExpression) {
-                    ((VariableReference) exp).refineVariableType(type, cardinality, constantValue, properties);
-                }
-                return false;
+        ExpressionTool.processExpressionTree(currentExpression.getAction(), null, (exp, result) -> {
+            if (exp instanceof VariableReference && ((VariableReference)exp).getBinding() == currentExpression) {
+                ((VariableReference) exp).refineVariableType(type, cardinality, constantValue, properties);
             }
+            return false;
         });
     }
 
@@ -484,7 +479,7 @@ public abstract class Assignation extends Expression implements LocalBinding {
 
     public void rebuildReferenceList(boolean force) {
         int[] results = new int[]{0, force ? Integer.MAX_VALUE : 500};
-        List<VariableReference> references = new ArrayList<VariableReference>();
+        List<VariableReference> references = new ArrayList<>();
         countReferences(this, getAction(), references, results);
         this.references = results[1] <= 0 ? null : references;
     }
@@ -530,11 +525,9 @@ public abstract class Assignation extends Expression implements LocalBinding {
      * @param seq the expression
      * @return true if the variable was successfully inlined. (Returns false, for example,
      * if a variable reference occurs inside a try/catch, which inhibits inlining).
-     * @throws net.sf.saxon.trans.XPathException if a failure occurs
-     *
      */
 
-    public boolean replaceVariable(Expression seq) throws XPathException {
+    public boolean replaceVariable(Expression seq) {
         boolean done = ExpressionTool.inlineVariableReferences(getAction(), this, seq);
         if (done && isIndexedVariable() && seq instanceof VariableReference) {
             Binding newBinding = ((VariableReference) seq).getBinding();
