@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,14 +10,13 @@ package net.sf.saxon.expr;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.Item;
-import net.sf.saxon.om.SequenceTool;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.jiter.PairIterator;
 import net.sf.saxon.type.AtomicType;
 import net.sf.saxon.value.Cardinality;
 
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
 
 /**
@@ -51,11 +50,7 @@ public abstract class BinaryExpression extends Expression {
 
     @Override
     final public Iterable<Operand> operands() {
-        return new Iterable<Operand>() {
-            public Iterator<Operand> iterator() {
-                return new PairIterator<Operand>(lhs, rhs);
-            }
-        };
+        return () -> new PairIterator<>(lhs, rhs);
     }
 
 
@@ -131,16 +126,14 @@ public abstract class BinaryExpression extends Expression {
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
         resetLocalStaticProperties();
-        //typeCheckChildren(visitor, contextInfo);
         lhs.typeCheck(visitor, contextInfo);
         rhs.typeCheck(visitor, contextInfo);
 
         // if both operands are known, pre-evaluate the expression
         try {
             if ((getLhsExpression() instanceof Literal) && (getRhsExpression() instanceof Literal)) {
-                GroundedValue v = SequenceTool.toGroundedValue(
-                    evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext()));
-                return Literal.makeLiteral(v);
+                GroundedValue<?> v = evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext()).materialize();
+                return Literal.makeLiteral(v, this);
             }
         } catch (XPathException err) {
             // if early evaluation fails, suppress the error: the value might
@@ -151,7 +144,6 @@ public abstract class BinaryExpression extends Expression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -167,15 +159,16 @@ public abstract class BinaryExpression extends Expression {
 
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
-        optimizeChildren(visitor, contextItemType);
+        lhs.optimize(visitor, contextItemType);
+        rhs.optimize(visitor, contextItemType);
 
         // if both operands are known, pre-evaluate the expression
         try {
             if ((getLhsExpression() instanceof Literal) && (getRhsExpression() instanceof Literal)) {
-                Item item = evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext());
+                Item<?> item = evaluateItem(visitor.getStaticContext().makeEarlyEvaluationContext());
                 if (item != null) {
-                    GroundedValue v = SequenceTool.toGroundedValue(item);
-                    return Literal.makeLiteral(v);
+                    GroundedValue<?> v = item.materialize();
+                    return Literal.makeLiteral(v, this);
                 }
             }
         } catch (XPathException err) {
@@ -234,13 +227,13 @@ public abstract class BinaryExpression extends Expression {
     /**
      * Determine the special properties of this expression
      *
-     * @return {@link StaticProperty#NON_CREATIVE}. This is overridden
+     * @return {@link StaticProperty#NO_NODES_NEWLY_CREATED}. This is overridden
      *         for some subclasses.
      */
 
     public int computeSpecialProperties() {
         int p = super.computeSpecialProperties();
-        return p | StaticProperty.NON_CREATIVE;
+        return p | StaticProperty.NO_NODES_NEWLY_CREATED;
     }
 
     /**
@@ -310,29 +303,26 @@ public abstract class BinaryExpression extends Expression {
      */
 
     public boolean equals(Object other) {
-        if (other instanceof BinaryExpression) {
+        if (other instanceof BinaryExpression && hasCompatibleStaticContext((Expression)other)) {
             BinaryExpression b = (BinaryExpression) other;
+            Expression lhs1 = getLhsExpression();
+            Expression rhs1 = getRhsExpression();
+            Expression lhs2 = b.getLhsExpression();
+            Expression rhs2 = b.getRhsExpression();
             if (operator == b.operator) {
-                if (getLhsExpression().equals(b.getLhsExpression()) &&
-                        getRhsExpression().equals(b.getRhsExpression())) {
+                if (lhs1.isEqual(lhs2) && rhs1.isEqual(rhs2)) {
                     return true;
                 }
-                if (isCommutative(operator) &&
-                        getLhsExpression().equals(b.getRhsExpression()) &&
-                        getRhsExpression().equals(b.getLhsExpression())) {
+                if (isCommutative(operator) && lhs1.isEqual(rhs2) && rhs1.isEqual(lhs2)) {
                     return true;
                 }
                 if (isAssociative(operator) &&
-                        pairwiseEqual(flattenExpression(new ArrayList<Expression>(4)),
-                                b.flattenExpression(new ArrayList<Expression>(4)))) {
+                        pairwiseEqual(flattenExpression(new ArrayList<>(4)),
+                                b.flattenExpression(new ArrayList<>(4)))) {
                     return true;
                 }
             }
-            if (isInverse(operator, b.operator) &&
-                    getLhsExpression().equals(b.getRhsExpression()) &&
-                    getRhsExpression().equals(b.getLhsExpression())) {
-                return true;
-            }
+            return isInverse(operator, b.operator) && lhs1.isEqual(rhs2) && rhs1.isEqual(lhs2);
         }
         return false;
     }
@@ -351,10 +341,10 @@ public abstract class BinaryExpression extends Expression {
                 ((BinaryExpression) getLhsExpression()).operator == operator) {
             ((BinaryExpression) getLhsExpression()).flattenExpression(list);
         } else {
-            int h = getLhsExpression().computeHashCode();
+            int h = getLhsExpression().hashCode();
             list.add(getLhsExpression());
             int i = list.size() - 1;
-            while (i > 0 && h > list.get(i - 1).computeHashCode()) {
+            while (i > 0 && h > list.get(i - 1).hashCode()) {
                 list.set(i, list.get(i - 1));
                 list.set(i - 1, getLhsExpression());
                 i--;
@@ -364,10 +354,10 @@ public abstract class BinaryExpression extends Expression {
                 ((BinaryExpression) getRhsExpression()).operator == operator) {
             ((BinaryExpression) getRhsExpression()).flattenExpression(list);
         } else {
-            int h = getRhsExpression().computeHashCode();
+            int h = getRhsExpression().hashCode();
             list.add(getRhsExpression());
             int i = list.size() - 1;
-            while (i > 0 && h > list.get(i - 1).computeHashCode()) {
+            while (i > 0 && h > list.get(i - 1).hashCode()) {
                 list.set(i, list.get(i - 1));
                 list.set(i - 1, getRhsExpression());
                 i--;
@@ -406,8 +396,8 @@ public abstract class BinaryExpression extends Expression {
         // so that (A lt B) has the same hash code as (B gt A)
         int op = Math.min(operator, Token.inverse(operator));
         return ("BinaryExpression " + op).hashCode()
-                ^ getLhsExpression().computeHashCode()
-                ^ getRhsExpression().computeHashCode();
+                ^ getLhsExpression().hashCode()
+                ^ getRhsExpression().hashCode();
     }
 
     /**
@@ -426,7 +416,7 @@ public abstract class BinaryExpression extends Expression {
 
     @Override
     public String toShortString() {
-        return parenthesize(getLhsExpression()) + displayOperator() + parenthesize(getRhsExpression());
+        return parenthesize(getLhsExpression()) + " " + displayOperator() + " " + parenthesize(getRhsExpression());
     }
 
     private String parenthesize(Expression operand) {
