@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -45,7 +45,7 @@ public final class CastableExpression extends CastingExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
 
         // We need to take care here. The usual strategy of wrapping the operand in an expression that
         // does type-checking doesn't work here, because an error in the type checking should be caught,
@@ -60,14 +60,14 @@ public final class CastableExpression extends CastingExpression {
             if (converter == null) {
                 if (!allowsEmpty() || !Cardinality.allowsZero(operand.getCardinality())) {
                     // Conversion from source to target type will never succeed
-                    return Literal.makeLiteral(BooleanValue.FALSE);
+                    return Literal.makeLiteral(BooleanValue.FALSE, this);
                 }
             } else {
                 if (getTargetPrimitiveType().isNamespaceSensitive()) {
                     converter = converter.setNamespaceResolver(getRetainedStaticContext());
                 }
                 if (converter.isAlwaysSuccessful() && !allowsEmpty() && operand.getCardinality() == StaticProperty.ALLOWS_ONE) {
-                    return Literal.makeLiteral(BooleanValue.TRUE);
+                    return Literal.makeLiteral(BooleanValue.TRUE, this);
                 }
             }
         }
@@ -85,14 +85,14 @@ public final class CastableExpression extends CastingExpression {
         GroundedValue literalOperand = ((Literal) getBaseExpression()).getValue();
         if (literalOperand instanceof AtomicValue && converter != null) {
             ConversionResult result = converter.convert((AtomicValue) literalOperand);
-            return Literal.makeLiteral(BooleanValue.get(!(result instanceof ValidationFailure)));
+            return Literal.makeLiteral(BooleanValue.get(!(result instanceof ValidationFailure)), this);
         }
         final int length = literalOperand.getLength();
         if (length == 0) {
-            return Literal.makeLiteral(BooleanValue.get(allowsEmpty()));
+            return Literal.makeLiteral(BooleanValue.get(allowsEmpty()), this);
         }
         if (length > 1) {
-            return Literal.makeLiteral(BooleanValue.FALSE);
+            return Literal.makeLiteral(BooleanValue.FALSE, this);
         }
         return this;
     }
@@ -129,7 +129,7 @@ public final class CastableExpression extends CastingExpression {
 
     public boolean equals(Object other) {
         return other instanceof CastableExpression &&
-                getBaseExpression().equals(((CastableExpression) other).getBaseExpression()) &&
+                getBaseExpression().isEqual(((CastableExpression) other).getBaseExpression()) &&
                 getTargetType() == ((CastableExpression) other).getTargetType() &&
                 allowsEmpty() == ((CastableExpression) other).allowsEmpty();
     }
@@ -140,16 +140,15 @@ public final class CastableExpression extends CastingExpression {
      */
 
     @Override
-    public int hashCode() {
-        return super.hashCode() ^ 0x5555;
+    public int computeHashCode() {
+        return super.computeHashCode() ^ 0x5555;
     }
 
     /**
      * Determine the data type of the result of the Castable expression
-     * <p/>
-     * <p/>
-     * /*@NotNull
      */
+
+    /*@NotNull*/
     public ItemType getItemType() {
         return BuiltInAtomicType.BOOLEAN;
     }
@@ -272,7 +271,7 @@ public final class CastableExpression extends CastingExpression {
 
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("castable", this);
-        out.emitAttribute("as", getTargetType().getTypeName());
+        out.emitAttribute("as", getTargetType().toExportString());
         out.emitAttribute("emptiable", allowsEmpty() ? "1" : "0");
         getBaseExpression().export(out);
         out.endElement();
