@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -43,17 +43,6 @@ public abstract class CastingExpression extends UnaryExpression {
         //derived = (targetType.getFingerprint() != targetPrimitiveType.getFingerprint());
     }
 
-//    /**
-//     * Bind aspects of the static context on which the particular function depends
-//     *
-//     * @param env the static context of the function call
-//     * @throws net.sf.saxon.trans.XPathException if execution with this static context will inevitably fail
-//     */
-//    @Override
-//    public void bindStaticContext(StaticContext env) throws XPathException {
-//        retainStaticContext(env);
-//    }
-
     /**
      * Get the primitive base type of the target type of the cast
      *
@@ -150,12 +139,19 @@ public abstract class CastingExpression extends UnaryExpression {
         return getRetainedStaticContext();
     }
 
+    /**
+     * Get the expression's dependencies. If the target type is namespace-sensitive, then the expression
+     * has a dependency on the namespace bindings in the static context
+     *
+     * @return the expression's dependencies.
+     */
+    @Override
+    public int getIntrinsicDependencies() {
+        return getTargetType().isNamespaceSensitive() ? StaticProperty.DEPENDS_ON_STATIC_CONTEXT : 0;
+    }
 
     /**
      * Simplify the expression
-     *
-     *
-     *
      */
 
     /*@NotNull*/
@@ -178,12 +174,12 @@ public abstract class CastingExpression extends UnaryExpression {
     /**
      * Determine the special properties of this expression
      *
-     * @return {@link net.sf.saxon.expr.StaticProperty#NON_CREATIVE}.
+     * @return {@link net.sf.saxon.expr.StaticProperty#NO_NODES_NEWLY_CREATED}.
      */
 
     public int computeSpecialProperties() {
         int p = super.computeSpecialProperties();
-        return p | StaticProperty.NON_CREATIVE;
+        return p | StaticProperty.NO_NODES_NEWLY_CREATED;
     }
 
 
