@@ -15,7 +15,7 @@ import net.sf.saxon.trans.XPathException;
  * A SequenceIterator that throws an exception as soon as its next() method is called. Used when
  * the method that returns the iterator isn't allowed to throw a checked exception itself.
  */
-public class ErrorIterator implements SequenceIterator<Item> {
+public class ErrorIterator implements SequenceIterator<Item<?>> {
 
     private XPathException exception;
 
