@@ -59,7 +59,7 @@ import java.util.*;
  * no subexpressions.</p>
  */
 
-public abstract class Expression implements IdentityComparable, ExportAgent {
+public abstract class Expression implements IdentityComparable, ExportAgent, Locatable {
 
     public static final int EVALUATE_METHOD = 1;
     public static final int ITERATE_METHOD = 2;
@@ -1039,6 +1039,9 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
 
     public final void explain(Logger out) {
         ExpressionPresenter ep = new ExpressionPresenter(getConfiguration(), out);
+        ExpressionPresenter.ExportOptions options = new ExpressionPresenter.ExportOptions();
+        options.explaining = true;
+        ep.setOptions(options);
         try {
             export(ep);
         } catch (XPathException e) {
@@ -1126,11 +1129,19 @@ public abstract class Expression implements IdentityComparable, ExportAgent {
      */
 
     public final Location getLocation() {
-        if ((location == null || location == ExplicitLocation.UNKNOWN_LOCATION) && getParentExpression() != null) {
-            return getParentExpression().location;  // Only go up one level to avoid infinite recursion: bug 3703#1
-        } else {
-            return location;
+        int limit = 0;
+        Expression exp = this;
+        while (limit < 10) {
+            if ((exp.location == null || exp.location == ExplicitLocation.UNKNOWN_LOCATION)
+                    && exp.getParentExpression() != null) {
+                // avoid infinite recursion: bug 3703#1
+                exp = exp.getParentExpression();
+                limit++;
+            } else {
+                return exp.location;
+            }
         }
+        return exp.location;
     }
 
     /**
