@@ -35,7 +35,6 @@ public class GlobalVariableReference extends VariableReference implements Compon
         super(var);
     }
 
-
     /*@NotNull*/
     public Expression copy(RebindingMap rebindings) {
         if (binding == null) {
@@ -108,7 +107,7 @@ public class GlobalVariableReference extends VariableReference implements Compon
      * @throws net.sf.saxon.trans.XPathException if any error occurs
      */
     @Override
-    public Sequence evaluateVariable(XPathContext c) throws XPathException {
+    public Sequence<?> evaluateVariable(XPathContext c) throws XPathException {
 
         if (bindingSlot >= 0) {
             if (c.getCurrentComponent() == null) {
