@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -45,7 +45,7 @@ public class HomogeneityChecker extends UnaryExpression {
         if (getBaseExpression() instanceof HomogeneityChecker) {
             return getBaseExpression().typeCheck(visitor, contextInfo);
         }
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         TypeHierarchy th = visitor.getConfiguration().getTypeHierarchy();
         ItemType type = getBaseExpression().getItemType();
         if (type.equals(ErrorType.getInstance())) {
@@ -54,15 +54,15 @@ public class HomogeneityChecker extends UnaryExpression {
         int rel = th.relationship(type, AnyNodeTest.getInstance());
         if (rel == TypeHierarchy.DISJOINT) {
             // expression cannot return nodes, so this checker is redundant
-            if (getBaseExpression() instanceof SlashExpression && ((SlashExpression) getBaseExpression()).getLeadingSteps() instanceof SlashExpression &&
-                    (((SlashExpression) getBaseExpression()).getLeadingSteps().getSpecialProperties() & StaticProperty.ORDERED_NODESET) == 0) {
-                DocumentSorter ds = new DocumentSorter(((SlashExpression) getBaseExpression()).getLeadingSteps());
-                SlashExpression se = new SlashExpression(ds, ((SlashExpression) getBaseExpression()).getLastStep());
-                ExpressionTool.copyLocationInfo(this, se);
-                return se;
-            } else {
+//            if (getBaseExpression() instanceof SlashExpression && ((SlashExpression) getBaseExpression()).getLeadingSteps() instanceof SlashExpression &&
+//                    (((SlashExpression) getBaseExpression()).getLeadingSteps().getSpecialProperties() & StaticProperty.ORDERED_NODESET) == 0) {
+//                DocumentSorter ds = new DocumentSorter(((SlashExpression) getBaseExpression()).getLeadingSteps());
+//                SlashExpression se = new SlashExpression(ds, ((SlashExpression) getBaseExpression()).getLastStep());
+//                ExpressionTool.copyLocationInfo(this, se);
+//                return se;
+//            } else {
                 return getBaseExpression();
-            }
+//            }
         } else if (rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMED_BY) {
             // expression always returns nodes, so replace this expression with a DocumentSorter
             Expression savedBase = getBaseExpression();
@@ -120,13 +120,13 @@ public class HomogeneityChecker extends UnaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(final XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(final XPathContext context) throws XPathException {
 
         // This class delivers the result of the path expression in unsorted order,
         // without removal of duplicates. If sorting and deduplication are needed,
         // this is achieved by wrapping the path expression in a DocumentSorter
 
-        SequenceIterator base = getBaseExpression().iterate(context);
+        SequenceIterator<?> base = getBaseExpression().iterate(context);
         return new HomogeneityCheckerIterator(base, getLocation());
     }
 
