@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -76,7 +76,8 @@ public final class IdentityComparison extends BinaryExpression {
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
 
-        typeCheckChildren(visitor, contextInfo);
+        getLhs().typeCheck(visitor, contextInfo);
+        getRhs().typeCheck(visitor, contextInfo);
 
         if (!generateIdEmulation) {
             if (Literal.isEmptySequence(getLhsExpression()) || Literal.isEmptySequence(getRhsExpression())) {
@@ -102,7 +103,6 @@ public final class IdentityComparison extends BinaryExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -205,7 +205,7 @@ public final class IdentityComparison extends BinaryExpression {
 
         switch (operator) {
             case Token.IS:
-                return node0.isSameNodeInfo(node1);
+                return node0.equals(node1);
             case Token.PRECEDES:
                 return GlobalOrderComparer.getInstance().compare(node0, node1) < 0;
             case Token.FOLLOWS:
