@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,6 +15,7 @@ import net.sf.saxon.trans.XPathException;
  * able to return the position of the last item in the sequence.
  */
 
+@FunctionalInterface
 public interface LastPositionFinder {
 
     /**
@@ -29,7 +30,7 @@ public interface LastPositionFinder {
      *                        the number of items
      */
 
-    public int getLength() throws XPathException;
+    int getLength() throws XPathException;
 
 }
 
