@@ -248,11 +248,11 @@ public class LookupAllExpression extends UnaryExpression {
                     } else {
                         if (level1.hasNext()) {
                             Object nextEntry = level1.next();
-                            if (nextEntry instanceof GroundedValue) {
-                                level2 = ((GroundedValue)nextEntry).iterate();
-                            } else if (nextEntry instanceof KeyValuePair) {
+                            if (nextEntry instanceof KeyValuePair) {
                                 GroundedValue value = ((KeyValuePair)nextEntry).value;
                                 level2 = value.iterate();
+                            } else if (nextEntry instanceof GroundedValue) {
+                                level2 = ((GroundedValue) nextEntry).iterate();
                             } else {
                                 throw new IllegalStateException();
                             }
@@ -310,5 +310,9 @@ public class LookupAllExpression extends UnaryExpression {
         return ExpressionTool.parenthesize(getBaseExpression()) + "?*";
     }
 
+    public String toShortString() {
+        return getBaseExpression().toShortString() + "?*";
+    }
+
 }
 
