@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -31,7 +31,7 @@ public class OrExpression extends BooleanExpression {
         if (Literal.hasEffectiveBooleanValue(getLhsExpression(), true) || Literal.hasEffectiveBooleanValue(getRhsExpression(), true)) {
             // A or true() => true()
             // true() or B => true()
-            return Literal.makeLiteral(BooleanValue.TRUE);
+            return Literal.makeLiteral(BooleanValue.TRUE, this);
         } else if (Literal.hasEffectiveBooleanValue(getLhsExpression(), false)) {
             // false() or B => B
             return forceToBoolean(getRhsExpression());
@@ -45,7 +45,6 @@ public class OrExpression extends BooleanExpression {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -69,7 +68,7 @@ public class OrExpression extends BooleanExpression {
 
         // If this is a top-level or-expression then try to replace multiple branches with a general comparison
         if (!(getParentExpression() instanceof OrExpression)) {
-            final Expression e2 = visitor.getConfiguration().obtainOptimizer().tryGeneralComparison(visitor, contextItemType, (OrExpression) this);
+            final Expression e2 = visitor.obtainOptimizer().tryGeneralComparison(visitor, contextItemType, this);
             if (e2 != null && e2 != this) {
                 return e2;
             }
