@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,10 +7,11 @@
 
 package net.sf.saxon.expr;
 
-import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.iter.AtomicIterator;
 import net.sf.saxon.tree.iter.LookaheadIterator;
 import net.sf.saxon.tree.iter.ReversibleIterator;
+import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.Int64Value;
 import net.sf.saxon.value.IntegerValue;
 
@@ -21,8 +22,10 @@ import net.sf.saxon.value.IntegerValue;
  * a RangeIterator that works in descending order.
  */
 
-public class ReverseRangeIterator implements SequenceIterator,
-        ReversibleIterator, LastPositionFinder, LookaheadIterator {
+public class ReverseRangeIterator implements AtomicIterator,
+        ReversibleIterator<AtomicValue>,
+        LastPositionFinder,
+        LookaheadIterator<AtomicValue> {
 
     long start;
     long currentValue;
@@ -32,7 +35,7 @@ public class ReverseRangeIterator implements SequenceIterator,
      * Create an iterator over a range of integers in monotonic descending order
      *
      * @param start the first integer to be delivered (the highest in the range)
-     * @param end   the last integer to be delivered (the lowest in the range). Must be <= start
+     * @param end   the last integer to be delivered (the lowest in the range). Must be &lt;= start
      * @throws XPathException if the values are inconsistent or if the length is greater than 2^31
      */
 
@@ -57,9 +60,6 @@ public class ReverseRangeIterator implements SequenceIterator,
         return Int64Value.makeIntegerValue(currentValue);
     }
 
-    public void close() {
-    }
-
     public int getLength() {
         return (int) ((start - limit) + 1);
     }
@@ -78,7 +78,7 @@ public class ReverseRangeIterator implements SequenceIterator,
         return LOOKAHEAD | LAST_POSITION_FINDER;
     }
 
-    public SequenceIterator getReverseIterator() {
+    public AtomicIterator getReverseIterator() {
         return new RangeIterator(limit, start);
     }
 
