@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.expr;
 
 import net.sf.saxon.expr.parser.ExpressionTool;
 import net.sf.saxon.expr.parser.RebindingMap;
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.value.StringValue;
 
 /**
@@ -23,7 +24,7 @@ public class StringLiteral extends Literal {
      */
 
     public StringLiteral(StringValue value) {
-        super(value);
+        super((GroundedValue)value);
     }
 
     /**
@@ -33,7 +34,7 @@ public class StringLiteral extends Literal {
      */
 
     public StringLiteral(CharSequence value) {
-        super(StringValue.makeStringValue(value));
+        this(StringValue.makeStringValue(value));
     }
 
     /**
