@@ -268,8 +268,8 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         getLhs().optimize(visitor, contextInfo);
         getRhs().optimize(visitor, contextInfo);
 
-        Sequence value0 = null;
-        Sequence value1 = null;
+        Sequence<?> value0 = null;
+        Sequence<?> value1 = null;
 
         if (getLhsExpression() instanceof Literal) {
             value0 = ((Literal) getLhsExpression()).getValue();
@@ -354,6 +354,7 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         if ((p0 == StandardNames.XS_STRING ||
                 p0 == StandardNames.XS_ANY_URI ||
                 p0 == StandardNames.XS_UNTYPED_ATOMIC) &&
+                resultWhenEmpty != BooleanValue.TRUE &&
                 getRhsExpression() instanceof Literal &&
                 ((Literal) getRhsExpression()).getValue() instanceof StringValue &&
                 ((StringValue) ((Literal) getRhsExpression()).getValue()).isZeroLength() &&
@@ -377,6 +378,7 @@ public final class ValueComparison extends BinaryExpression implements Compariso
         if ((p1 == StandardNames.XS_STRING ||
                 p1 == StandardNames.XS_ANY_URI ||
                 p1 == StandardNames.XS_UNTYPED_ATOMIC) &&
+                resultWhenEmpty != BooleanValue.TRUE &&
                 getLhsExpression() instanceof Literal &&
                 ((Literal) getLhsExpression()).getValue() instanceof StringValue &&
                 ((StringValue) ((Literal) getLhsExpression()).getValue()).isZeroLength() &&
@@ -653,7 +655,7 @@ public final class ValueComparison extends BinaryExpression implements Compariso
      * @return true if the operand is the constant zero (of any numeric data type)
      */
 
-    private static boolean isZero(Sequence v) {
+    private static boolean isZero(Sequence<?> v) {
         return v instanceof NumericValue && ((NumericValue) v).compareTo(0) == 0;
     }
 
