@@ -563,13 +563,13 @@ public abstract class VariableReference extends Expression implements BindingRef
             err.printStackTrace();
             String msg = "Internal error: no value for variable $" + getDisplayName() +
                     " at line " + getLocation().getLineNumber() + (getLocation().getSystemId() == null ? "" : " of " + getLocation().getSystemId());
-            StandardErrorListener.printStackTrace(c.getConfiguration().getLogger(), c);
+            StandardErrorListener.printStackTrace(c, c.getConfiguration().getLogger(), 2);
             throw new AssertionError(msg);
         } catch (AssertionError err) {
             err.printStackTrace();
             String msg = err.getMessage() + ". Variable reference $" + getDisplayName() +
                     " at line " + getLocation().getLineNumber() + (getLocation().getSystemId() == null ? "" : " of " + getLocation().getSystemId());
-            StandardErrorListener.printStackTrace(c.getConfiguration().getLogger(), c);
+            StandardErrorListener.printStackTrace(c, c.getConfiguration().getLogger(), 2);
             throw new AssertionError(msg);
         }
     }
