@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,17 +8,18 @@
 package net.sf.saxon.expr;
 
 import net.sf.saxon.Controller;
-import net.sf.saxon.expr.instruct.*;
+import net.sf.saxon.PreparedStylesheet;
+import net.sf.saxon.expr.instruct.Executable;
+import net.sf.saxon.expr.instruct.ParameterSet;
+import net.sf.saxon.expr.instruct.SlotManager;
 import net.sf.saxon.expr.sort.GroupIterator;
 import net.sf.saxon.lib.DelegatingErrorListener;
 import net.sf.saxon.lib.StandardURIResolver;
 import net.sf.saxon.lib.UnfailingErrorListener;
-import net.sf.saxon.om.FocusTrackingIterator;
-import net.sf.saxon.om.Item;
-import net.sf.saxon.om.Sequence;
-import net.sf.saxon.om.StructuredQName;
+import net.sf.saxon.om.*;
 import net.sf.saxon.regex.RegexIterator;
-import net.sf.saxon.trans.*;
+import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.XsltController;
 import net.sf.saxon.trans.rules.Rule;
 import net.sf.saxon.trans.rules.RuleManager;
 import net.sf.saxon.tree.iter.SingletonIterator;
@@ -83,11 +84,13 @@ public class XPathContextMajor extends XPathContextMinor {
      * @param exec the Executable
      */
 
-    public XPathContextMajor(Item item, Executable exec) {
-        controller = new Controller(exec.getConfiguration(), exec);
+    public XPathContextMajor(Item<?> item, Executable exec) {
+        controller = exec instanceof PreparedStylesheet ?
+                new XsltController(exec.getConfiguration(), (PreparedStylesheet)exec) :
+                new Controller(exec.getConfiguration(), exec);
         if (item != null) {
-            UnfailingIterator iter = SingletonIterator.makeIterator(item);
-            currentIterator = new FocusTrackingIterator(iter);
+            UnfailingIterator<?> iter = SingletonIterator.makeIterator(item);
+            currentIterator = new FocusTrackingIterator<>(iter);
             try {
                 currentIterator.next();
             } catch (XPathException e) {
@@ -304,7 +307,7 @@ public class XPathContextMajor extends XPathContextMinor {
                         "Attempting to set more local variables (" + variables.length +
                                 ") than the stackframe can accommodate (" + map.getNumberOfVariables() + ")");
             }
-            stackFrame.slots = new Sequence[map.getNumberOfVariables()];
+            stackFrame.slots = (Sequence<?>[])new Sequence[map.getNumberOfVariables()];
             System.arraycopy(variables, 0, stackFrame.slots, 0, variables.length);
         }
     }
@@ -320,7 +323,8 @@ public class XPathContextMajor extends XPathContextMinor {
     public void resetStackFrameMap(SlotManager map, int numberOfParams) {
         stackFrame.map = map;
         if (stackFrame.slots.length != map.getNumberOfVariables()) {
-            Sequence[] v2 = new Sequence[map.getNumberOfVariables()];
+            Sequence<?>[] v2 =
+                    (Sequence<?>[])new Sequence[map.getNumberOfVariables()];
             System.arraycopy(stackFrame.slots, 0, v2, 0, numberOfParams);
             stackFrame.slots = v2;
         } else {
@@ -370,7 +374,7 @@ public class XPathContextMajor extends XPathContextMinor {
      * @param variables the parameter to be supplied to the user function
      */
 
-    public void requestTailCall(TailCallLoop.TailCallInfo targetFn, Sequence[] variables) {
+    public void requestTailCall(TailCallLoop.TailCallInfo targetFn, Sequence<?>[] variables) {
         if (variables != null) {
             if (variables.length > stackFrame.slots.length) {
                 stackFrame.slots = Arrays.copyOf(variables, variables.length);
@@ -419,7 +423,7 @@ public class XPathContextMajor extends XPathContextMinor {
 
     public void openStackFrame(int numberOfVariables) {
         stackFrame = new StackFrame(new SlotManager(numberOfVariables),
-                new Sequence[numberOfVariables]);
+                                    SequenceTool.makeSequenceArray(numberOfVariables));
     }
 
     /**
@@ -562,7 +566,7 @@ public class XPathContextMajor extends XPathContextMinor {
         if (index < 0) {
             return ParameterSet.NOT_SUPPLIED;
         }
-        Sequence val = params.getValue(index);
+        Sequence<?> val = params.getValue(index);
         stackFrame.slots[slotNumber] = val;
         boolean checked = params.isTypeChecked(index);
         return checked ? ParameterSet.SUPPLIED_AND_CHECKED : ParameterSet.SUPPLIED;
