@@ -24,6 +24,7 @@ import net.sf.saxon.value.DateTimeValue;
 
 import javax.xml.transform.URIResolver;
 import java.util.Iterator;
+import java.util.function.Function;
 
 /**
  * This class represents a minor change in the dynamic context in which an XPath expression is evaluated:
@@ -34,7 +35,7 @@ import java.util.Iterator;
 public class XPathContextMinor implements XPathContext {
 
     Controller controller;
-    FocusIterator<? extends Item> currentIterator;
+    FocusIterator<?> currentIterator;
     /*@Nullable*/ LastValue last = null;
     Receiver currentReceiver;
 
@@ -154,7 +155,7 @@ public class XPathContextMinor implements XPathContext {
      * Set a new sequence iterator.
      */
 
-    public void setCurrentIterator(FocusIterator<? extends Item> iter) {
+    public void setCurrentIterator(FocusIterator<?> iter) {
         currentIterator = iter;
         last = new LastValue(-1);
     }
@@ -166,12 +167,31 @@ public class XPathContextMinor implements XPathContext {
      *             to the current iterator.
      */
 
-    public <T extends Item> FocusIterator<T> trackFocus(SequenceIterator<T> iter) {
-        FocusIterator<T> fit = controller.getFocusTrackerFactory().apply(iter);
+    public <T extends Item<?>> FocusIterator<T> trackFocus(SequenceIterator<T> iter) {
+        Function<SequenceIterator<?>, FocusTrackingIterator<?>> factory =
+                controller.getFocusTrackerFactory(false);
+        //noinspection unchecked
+        FocusIterator<T> fit = (FocusIterator<T>)factory.apply(iter);
         setCurrentIterator(fit);
         return fit;
     }
 
+    /**
+     * Create, set, and return a focus tracking iterator that wraps a supplied sequence iterator,
+     * suitable for use in a multithreaded xsl:for-each iteration
+     *
+     * @param iter the current iterator. The context item, position, and size are determined by reference
+     *             to the current iterator.
+     */
+
+    public <T extends Item<?>> FocusIterator<T> trackFocusMultithreaded(SequenceIterator<T> iter) {
+        Function<SequenceIterator<?>, FocusTrackingIterator<?>> factory = controller.getFocusTrackerFactory(true);
+        FocusIterator<T> fit = (FocusIterator<T>)factory.apply(iter);
+        setCurrentIterator(fit);
+        return fit;
+    }
+
+
     /**
      * Get the current iterator.
      * This encapsulates the context item, context position, and context size.
@@ -180,7 +200,7 @@ public class XPathContextMinor implements XPathContext {
      *         (which means the context item, position, and size are undefined).
      */
 
-    public final FocusIterator<? extends Item> getCurrentIterator() {
+    public final FocusIterator<?> getCurrentIterator() {
         return currentIterator;
     }
 
@@ -297,7 +317,7 @@ public class XPathContextMinor implements XPathContext {
 
     public void makeStackFrameMutable() {
         if (stackFrame == StackFrame.EMPTY) {
-            stackFrame = new StackFrame(null, new Sequence[0]);
+            stackFrame = new StackFrame(null, SequenceTool.makeSequenceArray(0));
         }
     }
 
@@ -305,7 +325,7 @@ public class XPathContextMinor implements XPathContext {
      * Get the value of a local variable, identified by its slot number
      */
 
-    public final Sequence evaluateLocalVariable(int slotnumber) {
+    public final Sequence<?> evaluateLocalVariable(int slotnumber) {
         return stackFrame.slots[slotnumber];
     }
 
@@ -313,7 +333,7 @@ public class XPathContextMinor implements XPathContext {
      * Set the value of a local variable, identified by its slot number
      */
 
-    public final void setLocalVariable(int slotNumber, Sequence value) throws XPathException {
+    public final void setLocalVariable(int slotNumber, Sequence<?> value) throws XPathException {
 
         // The following code is deep defence against attempting to store a non-memo Closure in a variable.
         // This should not happen, and if it does, it means that the evaluation mode has been miscalculated.
@@ -321,17 +341,6 @@ public class XPathContextMinor implements XPathContext {
         // value as it is calculated.
         
         value = value.makeRepeatable();
-        //value = value.materialize();
-
-//        String val = "*lazy*";
-//        if (value instanceof GroundedValue) {
-//            val = Err.depictSequence(value).toString();
-//        }
-//        System.err.println("set var " +
-//                                   getStackFrame().getStackFrameMap().getVariableMap().get(slotNumber) +
-//                                   " := " +
-//                                   val +
-//                                   " -- " + value.getClass().getSimpleName());
         try {
             stackFrame.slots[slotNumber] = value;
         } catch (ArrayIndexOutOfBoundsException e) {
