@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,10 +7,9 @@
 
 package net.sf.saxon.expr.flwor;
 
-import com.saxonica.ee.bytecode.ExpressionCompiler;
-import com.saxonica.ee.bytecode.FLWORExpressionCompiler;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.query.QueryModule;
 import net.sf.saxon.trace.ExpressionPresenter;
@@ -97,6 +96,19 @@ public class FLWORExpression extends Expression {
         return false;
     }
 
+    /**
+     * Ask whether common subexpressions found in the operands of this expression can
+     * be extracted and evaluated outside the expression itself. The result is irrelevant
+     * in the case of operands evaluated with a different focus, which will never be
+     * extracted in this way, even if they have no focus dependency.
+     *
+     * @return false for this kind of expression
+     */
+    @Override
+    public boolean allowExtractingCommonSubexpressions() {
+        return false;
+    }
+
     /**
      * Simplify an expression. This performs any static optimization (by rewriting the expression
      * as a different expression). The default implementation does nothing.
@@ -110,11 +122,8 @@ public class FLWORExpression extends Expression {
     /*@NotNull*/
     @Override
     public Expression simplify() throws XPathException {
-        OperandProcessor simplifier = new OperandProcessor() {
-            public void processOperand(Operand op) throws XPathException {
-                op.setChildExpression(op.getChildExpression().simplify());
-            }
-        };
+        OperandProcessor simplifier =
+                op -> op.setChildExpression(op.getChildExpression().simplify());
         for (Clause c : clauses) {
             c.processOperands(simplifier);
         }
@@ -125,16 +134,13 @@ public class FLWORExpression extends Expression {
     /**
      * Perform type checking of an expression and its subexpressions. This is the second phase of
      * static optimization.
-     * <p/>
      * <p>This checks statically that the operands of the expression have
      * the correct type; if necessary it generates code to do run-time type checking or type
      * conversion. A static type error is reported only if execution cannot possibly succeed, that
      * is, if a run-time type error is inevitable. The call may return a modified form of the expression.</p>
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable. However, the types of such functions and
      * variables may not be accurately known if they have not been explicitly declared.</p>
-     * <p/>
      * <p>If the implementation returns a value other than "this", then it is required to ensure that
      * the location information in the returned expression have been set up correctly.
      * It should not rely on the caller to do this, although for historical reasons many callers do so.</p>
@@ -153,18 +159,14 @@ public class FLWORExpression extends Expression {
     public Expression typeCheck(final ExpressionVisitor visitor, final ContextItemStaticInfo contextInfo)
             throws XPathException {
 
-        OperandProcessor typeChecker = new OperandProcessor() {
-            public void processOperand(Operand op) throws XPathException {
-                op.typeCheck(visitor, contextInfo);
-            }
-        };
+        OperandProcessor typeChecker = op -> op.typeCheck(visitor, contextInfo);
         for (int i = 0; i < clauses.size(); i++) {
             clauses.get(i).processOperands(typeChecker);
             clauses.get(i).typeCheck(visitor, contextInfo);
             LocalVariableBinding[] bindings = clauses.get(i).getRangeVariables();
 
             for (Binding b : bindings) {
-                List<VariableReference> references = new ArrayList<VariableReference>();
+                List<VariableReference> references = new ArrayList<>();
                 for (int j = i; j < clauses.size(); j++) {
                     clauses.get(j).gatherVariableReferences(visitor, b, references);
                 }
@@ -216,17 +218,12 @@ public class FLWORExpression extends Expression {
                                       RoleDiagnostic role, ExpressionVisitor visitor)
             throws XPathException {
         // only called if implementsStaticTypeCheck() returns true
+        TypeChecker tc = visitor.getConfiguration().getTypeChecker(backwardsCompatible);
         returnClauseOp.setChildExpression(
-                TypeChecker.staticTypeCheck(getReturnClause(), req, backwardsCompatible, role, visitor));
+                tc.staticTypeCheck(getReturnClause(), req, role, visitor));
         return this;
     }
 
-    /**
-     * This method is required to refine the variabletype of the returnClause
-     * Repeated code in the Clause sub-classes. Room here for commoning-up code
-     * */
-
-
     /**
      * Determine the data type of the items returned by the expression.
      *
@@ -262,21 +259,11 @@ public class FLWORExpression extends Expression {
      */
     @Override
     public Iterable<Operand> operands() {
-        final List<Operand> list = new ArrayList<Operand>(5);
-        OperandProcessor processor0 = new OperandProcessor() {
-            public void processOperand(Operand op) {
-                list.add(op);
-            }
-        };
-        OperandProcessor processor = new OperandProcessor() {
-            public void processOperand(Operand op) {
-                list.add(op);
-            }
-        };
+        final List<Operand> list = new ArrayList<>(5);
         boolean repeatable = false;
         try {
             for (Clause c : clauses) {
-                c.processOperands(processor);
+                c.processOperands(list::add);
                 if (c instanceof ForClause) {
                     repeatable = true;
                 }
@@ -304,13 +291,11 @@ public class FLWORExpression extends Expression {
      */
 
     public void checkForUpdatingSubexpressions() throws XPathException {
-        OperandProcessor processor = new OperandProcessor() {
-            public void processOperand(Operand op) throws XPathException {
-                op.getChildExpression().checkForUpdatingSubexpressions();
-                if (op.getChildExpression().isUpdatingExpression()) {
-                    throw new XPathException(
-                            "An updating expression cannot be used in a clause of a FLWOR expression", "XUST0001");
-                }
+        OperandProcessor processor = op -> {
+            op.getChildExpression().checkForUpdatingSubexpressions();
+            if (op.getChildExpression().isUpdatingExpression()) {
+                throw new XPathException(
+                        "An updating expression cannot be used in a clause of a FLWOR expression", "XUST0001");
             }
         };
         for (Clause c : clauses) {
@@ -343,6 +328,34 @@ public class FLWORExpression extends Expression {
         return ITERATE_METHOD | PROCESS_METHOD;
     }
 
+    /**
+     * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
+     * by an expression in a source tree.
+     * <p>The default implementation of this method assumes that an expression does no navigation other than
+     * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
+     * same context as the containing expression. The method must be overridden for any expression
+     * where these assumptions do not hold. For example, implementations exist for AxisExpression, ParentExpression,
+     * and RootExpression (because they perform navigation), and for the doc(), document(), and collection()
+     * functions because they create a new navigation root. Implementations also exist for PathExpression and
+     * FilterExpression because they have subexpressions that are evaluated in a different context from the
+     * calling expression.</p>
+     *
+     * @param pathMap        the PathMap to which the expression should be added
+     * @param pathMapNodeSet the PathMapNodeSet to which the paths embodied in this expression should be added
+     * @return the pathMapNodeSet representing the points in the source document that are both reachable by this
+     * expression, and that represent possible results of this expression. For an expression that does
+     * navigation, it represents the end of the arc in the path map that describes the navigation route. For other
+     * expressions, it is the same as the input pathMapNode.
+     */
+
+    public PathMap.PathMapNodeSet addToPathMap(PathMap pathMap, /*@Nullable*/ PathMap.PathMapNodeSet pathMapNodeSet) {
+        for (Clause c : clauses) {
+            c.addToPathMap(pathMap, pathMapNodeSet);
+        }
+        return getReturnClause().addToPathMap(pathMap, pathMapNodeSet);
+    }
+
+
     /**
      * Diagnostic print of expression structure. The abstract expression tree
      * is written to the supplied output destination.
@@ -365,14 +378,15 @@ public class FLWORExpression extends Expression {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variable bindings to be replaced
      */
     /*@NotNull*/
     @Override
     public Expression copy(RebindingMap rebindings) {
-        List<Clause> newClauses = new ArrayList<Clause>();
-        List<LocalVariableBinding> oldBindings = new ArrayList<LocalVariableBinding>();
-        List<LocalVariableBinding> newBindings = new ArrayList<LocalVariableBinding>();
+        //verifyParentPointers();
+        List<Clause> newClauses = new ArrayList<>();
+        List<LocalVariableBinding> oldBindings = new ArrayList<>();
+        List<LocalVariableBinding> newBindings = new ArrayList<>();
         FLWORExpression f2 = new FLWORExpression();
         for (Clause c : clauses) {
             Clause c2 = c.copy(f2, rebindings);
@@ -387,48 +401,10 @@ public class FLWORExpression extends Expression {
         for (int i = 0; i < oldBindings.size(); i++) {
             ExpressionTool.rebindVariableReferences(f2, oldBindings.get(i), newBindings.get(i));
         }
+        //f2.verifyParentPointers();
         return f2;
     }
 
-    /**
-     * Promote this expression if possible
-     */
-
-    public Expression promote(final PromotionOffer offer) throws XPathException {
-        OperandProcessor processor = new OperandProcessor() {
-            /*@Nullable*/
-            public void processOperand(Operand op) throws XPathException {
-                op.setChildExpression(doPromotion(op.getChildExpression(), offer));
-            }
-        };
-        Expression exp = offer.accept(this);
-        if (exp != null) {
-            return exp;
-        } else if (offer.action == PromotionOffer.RANGE_INDEPENDENT ||
-                offer.action == PromotionOffer.FOCUS_INDEPENDENT) {
-            // Pass the offer to the action expression only if the action isn't dependent on a
-            // variable bound within the FLWOR expression
-            Binding[] savedBindingList = offer.bindingList;
-            for (Clause c : clauses) {
-                offer.bindingList = extendBindingList(offer.bindingList, c.getRangeVariables());
-                c.processOperands(processor);
-            }
-            offer.bindingList = savedBindingList;
-            return this;
-        } else {
-
-            try {
-                for (Clause c : clauses) {
-                    c.processOperands(processor);
-                }
-            } catch (XPathException e) {
-                throw new IllegalStateException(e);
-            }
-            getReturnClause().promote(offer);
-            return this;
-        }
-    }
-
     /**
      * Replace this expression by a simpler expression that delivers the results without regard
      * to order.
@@ -475,13 +451,10 @@ public class FLWORExpression extends Expression {
             final ExpressionVisitor visitor,
             final ContextItemStaticInfo contextItemType) throws XPathException {
 
+        //verifyParentPointers();
         // Optimize all the subexpressions
         for (Clause c : clauses) {
-            c.processOperands(new OperandProcessor() {
-                public void processOperand(Operand op) throws XPathException {
-                    op.optimize(visitor, contextItemType);
-                }
-            });
+            c.processOperands(op -> op.optimize(visitor, contextItemType));
             c.optimize(visitor, contextItemType);
         }
 
@@ -620,7 +593,7 @@ public class FLWORExpression extends Expression {
             int whereIndex = 0;
             WhereClause whereClause;
         }
-        List<WhereClauseStruct> whereList = new ArrayList<WhereClauseStruct>();
+        List<WhereClauseStruct> whereList = new ArrayList<>();
 
         for (Clause c : clauses) {
             if (c instanceof WhereClause) {
@@ -644,7 +617,7 @@ public class FLWORExpression extends Expression {
             whereClause = whereList.get(0).whereClause;
             whereIndex = whereList.get(0).whereIndex;
             Expression condition = whereClause.getPredicate();
-            List<Expression> list = new ArrayList<Expression>(5);
+            List<Expression> list = new ArrayList<>(5);
             BooleanExpression.listAndComponents(condition, list);
             for (int i = list.size() - 1; i >= 0; i--) {
                 Expression term = list.get(i);
@@ -744,7 +717,7 @@ public class FLWORExpression extends Expression {
 
                 forExpr.setLocation(forClause.getLocation());
                 forExpr.setRetainedStaticContext(getRetainedStaticContext());
-                forExpr.setParentExpression(getParentExpression());
+                //forExpr.setParentExpression(getParentExpression());
                 forExpr.setAction(action);
 
                 forExpr.setSequence(forClause.getSequence());
@@ -757,17 +730,19 @@ public class FLWORExpression extends Expression {
                     action = injector.inject(action, visitor.getStaticContext(), LocationKind.FOR_EXPRESSION, forExpr.getVariableQName());
                 }
 
-
             } else {
                 LetClause letClause = (LetClause) clauses.get(i);
                 LetExpression letExpr = new LetExpression();
                 letExpr.setLocation(letClause.getLocation());
                 letExpr.setRetainedStaticContext(getRetainedStaticContext());
-                letExpr.setParentExpression(getParentExpression());
+                //letExpr.setParentExpression(getParentExpression());
                 letExpr.setAction(action);
                 letExpr.setSequence(letClause.getSequence());
                 letExpr.setVariableQName(letClause.getRangeVariable().getVariableQName());
                 letExpr.setRequiredType(letClause.getRangeVariable().getRequiredType());
+                if (letClause.getRangeVariable().isIndexedVariable()) {
+                    letExpr.setIndexedVariable();
+                }
                 //letExpr.setRefCount(letClause.getRangeVariable().getNominalReferenceCount());
                 ExpressionTool.rebindVariableReferences(action, letClause.getRangeVariable(), letExpr);
                 action = letExpr;
@@ -799,16 +774,7 @@ public class FLWORExpression extends Expression {
      */
     /*@NotNull*/
     @Override
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
-        //force push mode if there is a window clause that is not the first clause
-        // TODO: this is a temporary solution (pre 9.4 release) because WindowPull fails to call pullTuple
-        // on the previous clause in the FLWOR expression
-        for (int i = 1; i < clauses.size(); i++) {
-            if (clauses.get(i).getClauseKey() == Clause.WINDOW) {
-                return ExpressionTool.getIteratorFromProcessMethod(this, context);
-            }
-        }
-
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         TuplePull stream = new SingularityPull();
         for (Clause c : clauses) {
             stream = c.getPullStream(stream, context);
@@ -859,6 +825,32 @@ public class FLWORExpression extends Expression {
         }
     }
 
+    /**
+     * Get a name identifying the kind of expression, in terms meaningful to a user.
+     *
+     * @return a name identifying the kind of expression, in terms meaningful to a user.
+     * The name will always be in the form of a lexical XML QName, and should match the name used
+     * in export() output displaying the expression.
+     */
+    @Override
+    public String getExpressionName() {
+        return "FLWOR";
+    }
+
+    /**
+     * Produce a short string identifying the expression for use in error messages
+     *
+     * @return a short string, sufficient to identify the expression
+     */
+    @Override
+    public String toShortString() {
+        FastStringBuffer sb = new FastStringBuffer(FastStringBuffer.C64);
+        sb.append(clauses.get(0).toShortString());
+        sb.append(" ... return ");
+        sb.append(getReturnClause().toShortString());
+        return sb.toString();
+    }
+
     /**
      * Display the expression as a string
      */
@@ -909,13 +901,11 @@ public class FLWORExpression extends Expression {
         int lastReferencingClause = clauses.size(); // indicates the return clause
         if (!ExpressionTool.dependsOnVariable(getReturnClause(), new Binding[]{binding})) {
             // artifice to get a response value from the generic processExpression() method
-            final List<Boolean> response = new ArrayList<Boolean>();
-            OperandProcessor checker = new OperandProcessor() {
-                public void processOperand(Operand op) throws XPathException {
-                    if (response.isEmpty() &&
-                            ExpressionTool.dependsOnVariable(op.getChildExpression(), new Binding[]{binding})) {
-                        response.add(true);
-                    }
+            final List<Boolean> response = new ArrayList<>();
+            OperandProcessor checker = op -> {
+                if (response.isEmpty() &&
+                        ExpressionTool.dependsOnVariable(op.getChildExpression(), new Binding[]{binding})) {
+                    response.add(true);
                 }
             };
             for (int i = clauses.size() - 1; i >= 0; i--) {
@@ -945,18 +935,5 @@ public class FLWORExpression extends Expression {
         return false;
     }
 
-//#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the FLWOR expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
-    @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new FLWORExpressionCompiler();
-    }
-//#endif
-
 }
 
