@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -23,12 +23,20 @@ public class LetClause extends Clause {
 
     private LocalVariableBinding rangeVariable;
     private Operand sequenceOp;
+    private Evaluator evaluator;
 
     @Override
     public int getClauseKey() {
         return LET;
     }
 
+    public Evaluator getEvaluator() {
+        if (evaluator == null) {
+            evaluator = ExpressionTool.lazyEvaluator(getSequence(), true);
+        }
+        return evaluator;
+    }
+
     public LetClause copy(FLWORExpression flwor, RebindingMap rebindings) {
         LetClause let2 = new LetClause();
         let2.setLocation(getLocation());
@@ -115,6 +123,7 @@ public class LetClause extends Clause {
         RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.VARIABLE, rangeVariable.getVariableQName().getDisplayName(), 0);
         setSequence(TypeChecker.strictTypeCheck(
                 getSequence(), rangeVariable.getRequiredType(), role, visitor.getStaticContext()));
+        evaluator = ExpressionTool.lazyEvaluator(getSequence(), true);
     }
 
     @Override
@@ -184,7 +193,7 @@ public class LetClause extends Clause {
     @Override
     public void explain(ExpressionPresenter out) throws XPathException {
         out.startElement("let");
-        out.emitAttribute("var", getRangeVariable().getVariableQName().getDisplayName());
+        out.emitAttribute("var", getRangeVariable().getVariableQName());
         out.emitAttribute("slot", getRangeVariable().getLocalSlotNumber() + "");
         getSequence().export(out);
         out.endElement();
