@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,8 +9,8 @@ package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.event.ReceiverOptions;
-import net.sf.saxon.event.SequenceReceiver;
 import net.sf.saxon.expr.StaticProperty;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.lib.ConversionRules;
@@ -175,15 +175,12 @@ public abstract class AttributeCreator extends SimpleNodeConstructor implements
      *
      * @param value   the string value of the new node
      * @param context the dynamic evaluation context
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs
      */
 
     public void processValue(CharSequence value, XPathContext context) throws XPathException {
         NodeName attName = evaluateNodeName(context);
-//        if (nameCode == -1) {
-//            return null;
-//        }
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
         int opt = getOptions();
         SimpleType ann;
 
