@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,10 +7,6 @@
 
 package net.sf.saxon.expr.instruct;
 
-import com.saxonica.ee.bytecode.CallTemplateCompiler;
-import com.saxonica.ee.bytecode.ExpressionCompiler;
-import com.saxonica.ee.stream.adjunct.CallTemplateAdjunct;
-import com.saxonica.ee.stream.adjunct.StreamingAdjunct;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.om.StandardNames;
@@ -27,7 +23,6 @@ import net.sf.saxon.value.SequenceType;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Map;
 
 /**
  * Instruction representing an xsl:call-template element in the stylesheet.
@@ -40,7 +35,7 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
     private WithParam[] actualParams = WithParam.EMPTY_ARRAY;
     private WithParam[] tunnelParams = WithParam.EMPTY_ARRAY;
     private boolean useTailRecursion = false;
-    private int bindingSlot;
+    private int bindingSlot = -1;
     private boolean isWithinDeclaredStreamableConstruct;
 
     /**
@@ -82,6 +77,15 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
         }
     }
 
+    /**
+     * Get the name (QName) of the template being called
+     * @return the name of the target template
+     */
+
+    public StructuredQName getCalledTemplateName() {
+        return calledTemplateName;
+    }
+
     /**
      * Get the symbolic name of the template being called. This is essentially the component kind (template)
      * plus the QName of the target template
@@ -137,7 +141,7 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
 
     public void setTargetTemplate(NamedTemplate target) {
         this.template = target;
-    };
+    }
 
     /**
      * Get the target template
@@ -212,26 +216,26 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
         WithParam.typeCheck(actualParams, visitor, contextInfo);
         WithParam.typeCheck(tunnelParams, visitor, contextInfo);
-        if (template.getBody() != null) {
-            // For non-tunnel parameters, see if the supplied value is type-safe against the declared
-            // type of the value, and if so, avoid the dynamic type check
-            // Can't do this check unless the target template has been compiled.
-            boolean backwards = visitor.getStaticContext().isInBackwardsCompatibleMode();
-            for (int p = 0; p < actualParams.length; p++) {
-                WithParam wp = actualParams[p];
-                //int id = wp.getParameterId();
-                LocalParam lp = template.getLocalParam(wp.getVariableQName());
-                if (lp != null) {
-                    SequenceType req = lp.getRequiredType();
-                    RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.PARAM, wp.getVariableQName().getDisplayName(), p);
-                    Expression select = TypeChecker.staticTypeCheck(
-                            wp.getSelectExpression(), req, backwards, role, visitor);
-                    wp.setSelectExpression(this, select);
-                    wp.setTypeChecked(true);
-                }
+        // For non-tunnel parameters, see if the supplied value is type-safe against the declared
+        // type of the value, and if so, avoid the dynamic type check
+        // Can't do this check unless the target template has been compiled.
+        boolean backwards = visitor.getStaticContext().isInBackwardsCompatibleMode();
+        TypeChecker tc = visitor.getConfiguration().getTypeChecker(backwards);
+        for (int p = 0; p < actualParams.length; p++) {
+            WithParam wp = actualParams[p];
+            //int id = wp.getParameterId();
+            NamedTemplate.LocalParamInfo lp = template.getLocalParamInfo(wp.getVariableQName());
+            if (lp != null) {
+                SequenceType req = lp.requiredType;
+                RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.PARAM, wp.getVariableQName().getDisplayName(), p);
+                role.setErrorCode("XTTE0590");
+                Expression select = tc.staticTypeCheck(
+                        wp.getSelectExpression(), req, role, visitor);
+                wp.setSelectExpression(this, select);
+                wp.setTypeChecked(true);
             }
-
         }
+
         return this;
     }
 
@@ -276,7 +280,7 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings Variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -300,32 +304,19 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
      * This implementation currently returns true unconditionally.
      */
 
-    public final boolean createsNewNodes() {
+    public final boolean mayCreateNewNodes() {
         return true;
     }
 
     @Override
     public Iterable<Operand> operands() {
-        ArrayList<Operand> list = new ArrayList<Operand>(10);
+        ArrayList<Operand> list = new ArrayList<>(10);
         WithParam.gatherOperands(this, actualParams, list);
         WithParam.gatherOperands(this, tunnelParams, list);
         return list;
     }
 
 
-    /**
-     * Handle promotion offers, that is, non-local tree rewrites.
-     *
-     * @param offer The type of rewrite being offered
-     * @throws net.sf.saxon.trans.XPathException
-     *
-     */
-
-    protected void promoteChildren(PromotionOffer offer) throws XPathException {
-        WithParam.promoteParams(actualParams, offer);
-        WithParam.promoteParams(tunnelParams, offer);
-    }
-
     /**
      * Process this instruction, without leaving any tail calls.
      *
@@ -339,13 +330,15 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
         Component target = getFixedTarget();
         if (bindingSlot >= 0) {
             target = context.getTargetComponent(bindingSlot);
-            if (target.getVisibility() == Visibility.ABSENT) {
-                XPathException err = new XPathException("Cannot call a template declared with visibility='ABSENT'", "XTDE3052");
+            if (target.isHiddenAbstractComponent()) {
+                XPathException err = new XPathException("Cannot call an abstract template (" +
+                                                                calledTemplateName.getDisplayName() +
+                                                                ") with no implementation", "XTDE3052");
                 err.setLocation(getLocation());
                 throw err;
             }
         }
-        t = (NamedTemplate) target.getCode();
+        t = (NamedTemplate) target.getActor();
         XPathContextMajor c2 = context.newContext();
         c2.setCurrentComponent(target);
         c2.setOrigin(this);
@@ -371,7 +364,7 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
 
     /**
      * Process this instruction. If the called template contains a tail call (which may be
-     * an xsl:call-template of xsl:apply-templates instruction) then the tail call will not
+     * an xsl:call-template or xsl:apply-templates instruction) then the tail call will not
      * actually be evaluated, but will be returned in a TailCall object for the caller to execute.
      *
      * @param context the dynamic context for this transformation
@@ -390,8 +383,10 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
             if (targetComponent == null) {
                 throw new XPathException("Internal Saxon error: No binding available for call-template instruction", SaxonErrorCode.SXPK0001, this.getLocation());
             }
-            if (targetComponent.getVisibility() == Visibility.ABSENT) {
-                throw new XPathException("Cannot call a template declared with visibility=absent", "XTDE3052", this.getLocation());
+            if (targetComponent.isHiddenAbstractComponent()) {
+                throw new XPathException("Cannot call an abstract template (" +
+                                                 calledTemplateName.getDisplayName() +
+                                                 ") with no implementation", "XTDE3052", this.getLocation());
             }
 
             // handle parameters if any
@@ -429,12 +424,19 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
 
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("callT", this);
+        String flags = "";
         if (template != null && template.getTemplateName() != null) {
             out.emitAttribute("name", template.getTemplateName());
         }
         out.emitAttribute("bSlot", ""+getBindingSlot());
         if (isWithinDeclaredStreamableConstruct) {
-            out.emitAttribute("flags", "d");
+            flags += "d";
+        }
+        if (useTailRecursion) {
+            flags += "t";
+        }
+        if (!flags.isEmpty()) {
+            out.emitAttribute("flags", flags);
         }
         if (actualParams.length > 0) {
             WithParam.exportParameters(actualParams, out, false);
@@ -479,7 +481,7 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
                                    CallTemplate instruction,
                                    XPathContext evaluationContext) {
             this.targetComponent = targetComponent;
-            if (!(targetComponent.getCode() instanceof NamedTemplate)) {
+            if (!(targetComponent.getActor() instanceof NamedTemplate)) {
                 throw new ClassCastException("Target of call-template must be a named template");
             }
             this.params = params;
@@ -503,7 +505,7 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
             // creating a new one. We're doing this for the Java stack, but not for the context stack where
             // local variables are held. It should be possible to avoid creating a new context, and instead
             // to update the existing one in situ.
-            NamedTemplate template = (NamedTemplate)targetComponent.getCode();
+            NamedTemplate template = (NamedTemplate)targetComponent.getActor();
             XPathContextMajor c2 = evaluationContext.newContext();
             c2.setCurrentComponent(targetComponent);
             c2.setOrigin(instruction);
@@ -522,7 +524,9 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
         // fallback implementation
         FastStringBuffer buff = new FastStringBuffer(FastStringBuffer.C64);
         buff.append("CallTemplate#");
-        buff.append(template.getObjectName().getDisplayName());
+        if (template.getObjectName() != null) {
+            buff.append(template.getObjectName().getDisplayName());
+        }
         boolean first = true;
         for (WithParam p : getActualParams()) {
             buff.append(first ? "(" : ", ");
@@ -546,31 +550,10 @@ public class CallTemplate extends Instruction implements ITemplateCall, Componen
         return buff.toString();
     }
 
-    //#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the CallTemplate expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
     @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new CallTemplateCompiler();
+    public String getStreamerName() {
+        return "CallTemplate";
     }
-//#endif
 
-//#ifdefined STREAM
-
-
-    /**
-     * Get a class that supports streamed evaluation of this expression
-     *
-     * @return the relevant StreamingAdjunct, or null if none is available
-     */
-    @Override
-    protected StreamingAdjunct getStreamingAdjunct() {
-        return new CallTemplateAdjunct();
-    }
-//#endif
 }
 
