@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -270,7 +270,9 @@ public final class ComputedAttribute extends AttributeCreator {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings a mutable list of (old binding, new binding) pairs
+     *                   that is used to update the bindings held in any
+     *                   local variable references that are copied.
      */
 
     /*@NotNull*/
@@ -315,7 +317,7 @@ public final class ComputedAttribute extends AttributeCreator {
      *
      * @param context Dynamic evaluation context
      * @return the integer name code for the attribute name
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs (for example, if the attribute name is invalid)
      */
 
     public NodeName evaluateNodeName(XPathContext context) throws XPathException {
@@ -473,7 +475,7 @@ public final class ComputedAttribute extends AttributeCreator {
         }
         SimpleType type = getSchemaType();
         if (type != null) {
-            out.emitAttribute("type", type.getStructuredQName().getEQName());
+            out.emitAttribute("type", type.getStructuredQName());
         }
         String flags = "";
         if (isLocal()) {
