@@ -221,6 +221,9 @@ public class Copy extends ElementCreator {
         if (!getPackageData().isSchemaAware()) {
             return selectItemType;
         }
+        if (selectItemType.getUType().overlaps(UType.ANY_ATOMIC.union(UType.FUNCTION))) {
+            return selectItemType;
+        }
         // The rest of the code handles the complications of schema-awareness
         Configuration config = th.getConfiguration();
         if (getSchemaType() != null) {
@@ -476,6 +479,9 @@ public class Copy extends ElementCreator {
                         out = sr;
                     }
                 }
+                if (out.getSystemId() == null) {
+                    out.setSystemId(source.getBaseURI());
+                }
                 out.startDocument(0);
                 copyUnparsedEntities(source, out);
                 getContentExpression().process(context);
