@@ -8,7 +8,10 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Controller;
-import net.sf.saxon.event.*;
+import net.sf.saxon.event.ComplexContentOutputter;
+import net.sf.saxon.event.PipelineConfiguration;
+import net.sf.saxon.event.Receiver;
+import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.Operand;
 import net.sf.saxon.expr.OperandRole;
@@ -20,9 +23,7 @@ import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.tree.tiny.Statistics;
 import net.sf.saxon.tree.tiny.TinyBuilder;
-import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.type.Type;
 
 /**
@@ -88,7 +89,7 @@ public class Doctype extends Instruction {
         pipe.setXPathContext(context);
         pipe.setHostLanguage(getPackageData().getHostLanguage());
         TinyBuilder builder = new TinyBuilder(pipe);
-        builder.setStatistics(Statistics.RESULT_TREE_STATISTICS);
+        builder.setStatistics(pipe.getConfiguration().getTreeStatistics().RESULT_TREE_STATISTICS);
         builder.open();
         builder.startDocument(0);
         context.setReceiver(ComplexContentOutputter.makeComplexContentReceiver(builder, null));
@@ -105,9 +106,9 @@ public class Doctype extends Instruction {
             e.setXPathContext(context);
             throw e;
         }
-        String name = Navigator.getAttributeValue(docType, "", "name");
-        String system = Navigator.getAttributeValue(docType, "", "system");
-        String publicid = Navigator.getAttributeValue(docType, "", "public");
+        String name = docType.getAttributeValue("", "name");
+        String system = docType.getAttributeValue("", "system");
+        String publicid = docType.getAttributeValue("", "public");
 
         if (name == null) {
             XPathException e = new XPathException("dtd:doctype must have a name attribute");
@@ -137,8 +138,8 @@ public class Doctype extends Instruction {
             String localname = child.getLocalPart();
 
             if ("element".equals(localname)) {
-                String elname = Navigator.getAttributeValue(child, "", "name");
-                String content = Navigator.getAttributeValue(child, "", "content");
+                String elname = child.getAttributeValue("", "name");
+                String content = child.getAttributeValue("", "content");
                 if (elname == null) {
                     XPathException e = new XPathException("dtd:element must have a name attribute");
                     e.setXPathContext(context);
@@ -152,7 +153,7 @@ public class Doctype extends Instruction {
                 write(out, "\n  <!ELEMENT " + elname + ' ' + content + '>');
 
             } else if (localname.equals("attlist")) {
-                String elname = Navigator.getAttributeValue(child, "", "element");
+                String elname = child.getAttributeValue("", "element");
                 if (elname == null) {
                     XPathException e = new XPathException("dtd:attlist must have an attribute named 'element'");
                     e.setXPathContext(context);
@@ -169,9 +170,9 @@ public class Doctype extends Instruction {
 
                     if ("attribute".equals(attDef.getLocalPart())) {
 
-                        String atname = Navigator.getAttributeValue(attDef, "", "name");
-                        String type = Navigator.getAttributeValue(attDef, "", "type");
-                        String value = Navigator.getAttributeValue(attDef, "", "value");
+                        String atname = attDef.getAttributeValue("", "name");
+                        String type = attDef.getAttributeValue("", "type");
+                        String value = attDef.getAttributeValue("", "value");
                         if (atname == null) {
                             XPathException e = new XPathException("dtd:attribute must have a name attribute");
                             e.setXPathContext(context);
@@ -198,11 +199,11 @@ public class Doctype extends Instruction {
 
             } else if (localname.equals("entity")) {
 
-                String entname = Navigator.getAttributeValue(child, "", "name");
-                String parameter = Navigator.getAttributeValue(child, "", "parameter");
-                String esystem = Navigator.getAttributeValue(child, "", "system");
-                String epublicid = Navigator.getAttributeValue(child, "", "public");
-                String notation = Navigator.getAttributeValue(child, "", "notation");
+                String entname = child.getAttributeValue("", "name");
+                String parameter = child.getAttributeValue("", "parameter");
+                String esystem = child.getAttributeValue("", "system");
+                String epublicid = child.getAttributeValue("", "public");
+                String notation = child.getAttributeValue("", "notation");
 
                 if (entname == null) {
                     XPathException e = new XPathException("dtd:entity must have a name attribute");
@@ -239,9 +240,9 @@ public class Doctype extends Instruction {
                 write(out, ">");
 
             } else if (localname.equals("notation")) {
-                String notname = Navigator.getAttributeValue(child, "", "name");
-                String nsystem = Navigator.getAttributeValue(child, "", "system");
-                String npublicid = Navigator.getAttributeValue(child, "", "public");
+                String notname = child.getAttributeValue("", "name");
+                String nsystem = child.getAttributeValue("", "system");
+                String npublicid = child.getAttributeValue("", "public");
                 if (notname == null) {
                     XPathException e = new XPathException("dtd:notation must have a name attribute");
                     e.setXPathContext(context);
