@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -60,7 +60,7 @@ public class FixedElement extends ElementCreator {
         setValidationAction(validation, schemaType);
         preservingTypes = schemaType == null && validation == Validation.PRESERVE;
     }
-
+    
     @Override
     public Iterable<Operand> operands() {
         return contentOp;
@@ -87,7 +87,7 @@ public class FixedElement extends ElementCreator {
      * after creating any child nodes
      *
      * @param env the static context
-     * @throws net.sf.saxon.trans.XPathException
+     * @throws net.sf.saxon.trans.XPathException if the content sequence is invalid
      *
      */
 
@@ -206,7 +206,9 @@ public class FixedElement extends ElementCreator {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings a mutable list of (old binding, new binding) pairs
+     *                   that is used to update the bindings held in any
+     *                   local variable references that are copied.
      */
 
     /*@NotNull*/
@@ -247,19 +249,19 @@ public class FixedElement extends ElementCreator {
                 SchemaDeclaration decl = config.getElementDeclaration(fp);
                 if (decl == null) {
                     XPathException err = new XPathException("There is no global element declaration for " +
-                            elementName.getStructuredQName().getClarkName() +
+                            elementName.getStructuredQName().getEQName() +
                             ", so strict validation will fail");
                     err.setErrorCode(instr.isXSLT() ? "XTTE1512" : "XQDY0084");
-                    err.setIsTypeError(true);
+                    err.setIsTypeError(true); // technically this is a type error in XSLT but not in XQuery
                     err.setLocation(instr.getLocation());
                     throw err;
                 }
                 if (decl.isAbstract()) {
                     XPathException err = new XPathException("The element declaration for " +
-                            elementName.getStructuredQName().getClarkName() +
+                            elementName.getStructuredQName().getEQName() +
                             " is abstract, so strict validation will fail");
                     err.setErrorCode(instr.isXSLT() ? "XTTE1512" : "XQDY0027");
-                    err.setIsTypeError(true);
+                    err.setIsTypeError(true); // technically this is a type error in XSLT but not in XQuery
                     err.setLocation(instr.getLocation());
                     throw err;
                 }
@@ -489,7 +491,8 @@ public class FixedElement extends ElementCreator {
 
         SchemaType type;
         try {
-            type = ((ComplexType) parentType).getElementParticleType(elementName.getStructuredQName(), true);
+            int fp = elementName.obtainFingerprint(getConfiguration().getNamePool());
+            type = ((ComplexType) parentType).getElementParticleType(fp, true);
         } catch (SchemaException e) {
             throw new XPathException(e);
         }
@@ -509,9 +512,7 @@ public class FixedElement extends ElementCreator {
         try {
             getContentExpression().checkPermittedContents(type, true);
         } catch (XPathException e) {
-            if (e.getLocator() == null || e.getLocator() == e) {
-                e.setLocation(getLocation());
-            }
+            e.maybeSetLocation(getLocation());
             throw e;
         }
     }
@@ -549,11 +550,7 @@ public class FixedElement extends ElementCreator {
 
     public void export(ExpressionPresenter out) throws XPathException {
         out.startElement("elem", this);
-        if (elementName.getPrefix().isEmpty()) {
-            out.emitAttribute("name", elementName.getLocalPart());
-        } else {
-            out.emitAttribute("name", elementName.getStructuredQName());
-        }
+        out.emitAttribute("name", elementName.getDisplayName());
         out.emitAttribute("nsuri", elementName.getURI());
         String flags = getInheritanceFlags();
         if (!elementName.getURI().isEmpty() && elementName.getPrefix().isEmpty()) {
