@@ -7,6 +7,7 @@
 
 package net.sf.saxon.expr.instruct;
 
+import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.om.StructuredQName;
@@ -32,6 +33,14 @@ public final class InstructionDetails implements InstructionInfo {
     public InstructionDetails() {
     }
 
+    public InstructionDetails(Expression expr) {
+        setConstructType(expr.getConstructType());
+        setLineNumber(expr.getLocation().getLineNumber());
+        setSystemId(expr.getLocation().getSystemId());
+        setColumnNumber(expr.getLocation().getColumnNumber());
+        setProperty("name", expr.getExpressionName());
+    }
+
     /**
      * Set the type of construct
      *
