@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,8 +8,8 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Configuration;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.event.ReceiverOptions;
-import net.sf.saxon.event.SequenceReceiver;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.lib.NamespaceConstant;
@@ -146,7 +146,7 @@ public class NamespaceConstructor extends SimpleNodeConstructor {
         checkPrefixAndUri(prefix, uri, context);
 
         NamespaceBinding nscode = new NamespaceBinding(prefix, uri);
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
         out.namespace(nscode, ReceiverOptions.REJECT_DUPLICATES);
     }
 
@@ -187,7 +187,7 @@ public class NamespaceConstructor extends SimpleNodeConstructor {
             throw dynamicError(getLocation(), err, context);
         }
 
-        if (context.getConfiguration().getXsdVersion() == Configuration.XSD10 &&
+        if (context.getConfiguration().getXsdVersion() == Configuration.XSD10 &&   //W3C bug 30180
                 !StandardURIChecker.getInstance().isValidURI(uri)) {
             XPathException de = new XPathException(
                 "The string value of the constructed namespace node must be a valid URI", "XTDE0905", getLocation());
