@@ -7,7 +7,6 @@
 
 package net.sf.saxon.expr.instruct;
 
-import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.trans.XPathException;
@@ -80,6 +79,15 @@ public class ParameterSet {
         }
     }
 
+    /**
+     * Get the number of parameters in the parameter set
+     * @return the number of parameters
+     */
+
+    public int size() {
+        return used;
+    }
+
     /**
      * Add a parameter to the ParameterSet
      *
