@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,7 @@
 
 package net.sf.saxon.expr.instruct;
 
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.om.NameChecker;
@@ -19,10 +19,7 @@ import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.ItemType;
-import net.sf.saxon.value.AnyURIValue;
-import net.sf.saxon.value.AtomicValue;
-import net.sf.saxon.value.SequenceType;
-import net.sf.saxon.value.Whitespace;
+import net.sf.saxon.value.*;
 
 
 /**
@@ -103,13 +100,14 @@ public class ProcessingInstruction extends SimpleNodeConstructor {
         // mechanism to get an atomic value, and then check the type "by hand" at run time.
         setNameExp(visitor.getConfiguration().getTypeChecker(false).staticTypeCheck(
                 getNameExp(), SequenceType.SINGLE_ATOMIC, role, visitor));
-        adoptChildExpression(getNameExp());
+        Expression nameExp = getNameExp();
+        adoptChildExpression(nameExp);
 
         // Do early checking of name if known statically
 
-        if (getNameExp() instanceof Literal) {
-            String s = ((Literal) getNameExp()).getValue().getStringValue();
-            checkName(Whitespace.trim(s), env.makeEarlyEvaluationContext());
+        if (nameExp instanceof Literal && ((Literal)nameExp).getValue() instanceof AtomicValue) {
+            AtomicValue val = (AtomicValue) ((Literal) nameExp).getValue();
+            checkName(val, env.makeEarlyEvaluationContext());
         }
 
         // Do early checking of content if known statically
@@ -140,7 +138,7 @@ public class ProcessingInstruction extends SimpleNodeConstructor {
         String expandedName = evaluateName(context);
         if (expandedName != null) {
             String data = checkContent(value.toString(), context);
-            SequenceReceiver out = context.getReceiver();
+            Receiver out = context.getReceiver();
             out.processingInstruction(expandedName, data, getLocation(), 0);
         }
     }
@@ -211,11 +209,9 @@ public class ProcessingInstruction extends SimpleNodeConstructor {
      */
     private String evaluateName(XPathContext context) throws XPathException {
         AtomicValue av = (AtomicValue) getNameExp().evaluateItem(context);
-        if (av instanceof net.sf.saxon.value.StringValue && !(av instanceof AnyURIValue)) {
+        if (av instanceof StringValue && !(av instanceof AnyURIValue)) {
             // Always true under XSLT
-            String expandedName = Whitespace.trim(av.getStringValueCS());
-            checkName(expandedName, context);
-            return expandedName;
+            return checkName(av, context);
         } else {
             XPathException e = new XPathException("Processing instruction name is not a string");
             e.setXPathContext(context);
@@ -224,17 +220,28 @@ public class ProcessingInstruction extends SimpleNodeConstructor {
         }
     }
 
-    private void checkName(String expandedName, XPathContext context) throws XPathException {
-        if (!NameChecker.isValidNCName(expandedName)) {
-            XPathException e = new XPathException("Processing instruction name " + Err.wrap(expandedName) + " is not a valid NCName");
+    private String checkName(AtomicValue name, XPathContext context) throws XPathException {
+        if (name instanceof StringValue && !(name instanceof AnyURIValue)) {
+            String expandedName = Whitespace.trim(name.getStringValue());
+            if (!NameChecker.isValidNCName(expandedName)) {
+                XPathException e = new XPathException("Processing instruction name " + Err.wrap(expandedName) + " is not a valid NCName");
+                e.setXPathContext(context);
+                e.setErrorCode(isXSLT() ? "XTDE0890" : "XQDY0041");
+                throw dynamicError(getLocation(), e, context);
+            }
+            if (expandedName.equalsIgnoreCase("xml")) {
+                XPathException e = new XPathException("Processing instructions cannot be named 'xml' in any combination of upper/lower case");
+                e.setXPathContext(context);
+                e.setErrorCode(isXSLT() ? "XTDE0890" : "XQDY0064");
+                throw dynamicError(getLocation(), e, context);
+            }
+            return expandedName;
+        } else {
+            XPathException e = new XPathException("Processing instruction name " + Err.wrap(name.getStringValue()) +
+                                                          " is not of type xs:string or xs:untypedAtomic");
             e.setXPathContext(context);
-            e.setErrorCode(isXSLT() ? "XTDE0890" : "XQDY0041");
-            throw dynamicError(getLocation(), e, context);
-        }
-        if (expandedName.equalsIgnoreCase("xml")) {
-            XPathException e = new XPathException("Processing instructions cannot be named 'xml' in any combination of upper/lower case");
-            e.setXPathContext(context);
-            e.setErrorCode(isXSLT() ? "XTDE0890" : "XQDY0064");
+            e.setErrorCode("XPTY0004");
+            e.setIsTypeError(true);
             throw dynamicError(getLocation(), e, context);
         }
     }
