@@ -7,6 +7,8 @@
 
 package net.sf.saxon.expr.instruct;
 
+import net.sf.saxon.expr.XPathContext;
+import net.sf.saxon.lib.Logger;
 import net.sf.saxon.om.StructuredQName;
 
 import java.util.ArrayList;
@@ -32,7 +34,7 @@ public class SlotManager {
 
     public static SlotManager EMPTY = new SlotManager(0);
 
-    private ArrayList<StructuredQName> variableMap = new ArrayList<StructuredQName>(10);
+    private ArrayList<StructuredQName> variableMap = new ArrayList<>(10);
     // values are StructuredQName objects representing the variable names
     private int numberOfVariables = 0;
 
@@ -43,7 +45,7 @@ public class SlotManager {
 
     public SlotManager() {
         numberOfVariables = 0;
-        variableMap = new ArrayList<StructuredQName>();
+        variableMap = new ArrayList<>();
     }
 
     /**
@@ -54,7 +56,7 @@ public class SlotManager {
 
     public SlotManager(int n) {
         numberOfVariables = n;
-        variableMap = new ArrayList<StructuredQName>(n);
+        variableMap = new ArrayList<>(n);
     }
 
     /**
@@ -102,5 +104,16 @@ public class SlotManager {
         return variableMap;
     }
 
+    /**
+     * Display the values of the variables and parameters in an XPathContext stack frame
+     * to a Logger
+     * @param context the XPathContext holding the stack frame
+     * @param logger the destination for the output
+     */
+
+    public void showStackFrame(XPathContext context, Logger logger) {
+        // Saxon-HE version does nothing
+    }
+
 }
 
