@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,37 +10,40 @@ package net.sf.saxon.expr.instruct;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.expr.parser.RebindingMap;
+import net.sf.saxon.expr.parser.RoleDiagnostic;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.pattern.Pattern;
 import net.sf.saxon.style.Compilation;
 import net.sf.saxon.style.ComponentDeclaration;
 import net.sf.saxon.trace.ExpressionPresenter;
+import net.sf.saxon.trace.InstructionInfo;
 import net.sf.saxon.trace.LocationKind;
+import net.sf.saxon.trans.UncheckedXPathException;
+import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.trans.rules.Rule;
 import net.sf.saxon.trans.rules.RuleTarget;
-import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.jiter.MonoIterator;
 import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.Type;
+import net.sf.saxon.type.TypeHierarchy;
 import net.sf.saxon.value.SequenceType;
 
 import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.List;
 
 /**
  * The runtime object corresponding to an xsl:template element in the stylesheet.
- * <p/>
- * Note that the Template object no longer has precedence information associated with it; this is now
+ * <p>Note that the Template object no longer has precedence information associated with it; this is now
  * only in the Rule object that references this Template. This allows two rules to share the same template,
  * with different precedences. This occurs when a stylesheet module is imported more than once, from different
- * places, with different import precedences.
- * <p/>
+ * places, with different import precedences.</p>
  * <p>From Saxon 9.7, the old Template class is split into NamedTemplate and TemplateRule.</p>
  * <p>From Saxon 9.8, a subclass is used in Saxon-EE</p>
  */
 
-public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
+public class TemplateRule implements RuleTarget, Location, ExpressionOwner, InstructionInfo {
 
     // The body of the template is represented by an expression,
     // which is responsible for any type checking that's needed.
@@ -59,8 +62,8 @@ public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
     private int lineNumber;
 
 
-    private List<Rule> rules = new ArrayList<Rule>();
-    protected List<TemplateRule> slaveCopies = new ArrayList<TemplateRule>();
+    private List<Rule> rules = new ArrayList<>();
+    protected List<TemplateRule> slaveCopies = new ArrayList<>();
 
     /**
      * Create a template
@@ -264,7 +267,7 @@ public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
 
 
     public List<LocalParam> getLocalParams() {
-        List<LocalParam> result = new ArrayList<LocalParam>();
+        List<LocalParam> result = new ArrayList<>();
         gatherLocalParams(getInterpretedBody(), result);
         return result;
     }
@@ -328,11 +331,13 @@ public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
 
     public TailCall applyLeavingTail(XPathContext context) throws XPathException {
         //initialize();
+        TypeHierarchy th = context.getConfiguration().getTypeHierarchy();
         if (requiredContextItemType != AnyItemType.getInstance() &&
-                !requiredContextItemType.matches(context.getContextItem(), context.getConfiguration().getTypeHierarchy())) {
-            XPathException err = new XPathException("The template requires a context item of type " + requiredContextItemType +
-                                                            ", but the supplied context item has type " +
-                                                            Type.getItemType(context.getContextItem(), context.getConfiguration().getTypeHierarchy()), "XTTE0590");
+                !requiredContextItemType.matches(context.getContextItem(), th)) {
+            RoleDiagnostic role = new RoleDiagnostic(
+                    RoleDiagnostic.MISC, "context item for the template rule", 0);
+            String message = role.composeErrorMessage(requiredContextItemType, context.getContextItem(), th);
+            XPathException err = new XPathException(message, "XTTE0590");
             err.setLocation(this);
             err.setIsTypeError(true);
             throw err;
@@ -349,6 +354,11 @@ public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
                 body.process(context);
                 return null;
             }
+        } catch (UncheckedXPathException e) {
+            XPathException xe = e.getXPathException();
+            xe.maybeSetLocation(this);
+            xe.maybeSetContext(context);
+            throw xe;
         } catch (XPathException e) {
             e.maybeSetLocation(this);
             e.maybeSetContext(context);
@@ -357,6 +367,7 @@ public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
             String message = "Internal error evaluating template rule "
                     + (getLineNumber() > 0 ? " at line " + getLineNumber() : "")
                     + (getSystemId() != null ? " in module " + getSystemId() : "");
+            e2.printStackTrace();
             throw new RuntimeException(message, e2);
         }
     }
@@ -416,8 +427,8 @@ public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
     public void explainProperties(ExpressionPresenter presenter) throws XPathException {
         if (getRequiredContextItemType() != AnyItemType.getInstance()) {
             presenter.emitAttribute("cxt", getRequiredContextItemType().toString());
-            if ("JS".equals(presenter.getOption("target"))) {
-                int targetVersion = presenter.getIntOption("targetVersion", 1);
+            if ("JS".equals(((ExpressionPresenter.ExportOptions) presenter.getOptions()).target)) {
+                int targetVersion = ((ExpressionPresenter.ExportOptions) presenter.getOptions()).targetVersion;
                 presenter.emitAttribute("jsTest", getRequiredContextItemType().generateJavaScriptItemTypeTest(AnyItemType.getInstance(), targetVersion));
             }
         }
@@ -494,6 +505,21 @@ public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
         setBody(expr);
     }
 
+    @Override
+    public StructuredQName getObjectName() {
+        return StandardNames.getStructuredQName(StandardNames.XSL_TEMPLATE);
+    }
+
+    @Override
+    public Object getProperty(String name) {
+        return name.equals("match") ? getMatchPattern().toShortString() : null;
+    }
+
+    @Override
+    public Iterator<String> getProperties() {
+        return new MonoIterator<>("match");
+    }
+
 
 }
 
