@@ -45,6 +45,7 @@ import java.util.Objects;
 public class UserFunction extends Actor implements Function, ContextOriginator {
 
     public enum Determinism {DETERMINISTIC, PROACTIVE, ELIDABLE}
+    private final static int MAX_INLININGS = 100;
 
     private StructuredQName functionName;  // null for an anonymous function
     private boolean tailCalls = false;
@@ -57,6 +58,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
     protected Evaluator evaluator = null;
     private boolean isUpdating = false;
     private int inlineable = -1; // 0:no 1:yes -1:don't know
+    private int inliningCount = 0;
     private boolean overrideExtensionFunction = true;
     private AnnotationList annotations = AnnotationList.EMPTY;
     private FunctionStreamability declaredStreamability = FunctionStreamability.UNCLASSIFIED;
@@ -251,8 +253,11 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
             evaluator = ExpressionTool.eagerEvaluator(getBody());
 //        } else if ((getBody().getSpecialProperties() & StaticProperty.ALL_NODES_NEWLY_CREATED) != 0) {
 //            evaluationMode = ExpressionTool.PROCESS;
+//        } else if (getFunctionName().getLocalPart().equals("check-syntax-primary")) {  // KILROY
+//            evaluator = ExpressionTool.eagerEvaluator(getBody());
         } else {
             evaluator = ExpressionTool.lazyEvaluator(getBody(), true);
+            //evaluator = ExpressionTool.eagerEvaluator(getBody());
         }
     }
 
@@ -265,7 +270,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
     /*@Nullable*/
     public Boolean isInlineable() {
         if (inlineable != -1) {
-            return inlineable == 1;
+            return inlineable > 0 && inliningCount < MAX_INLININGS;
         }
         if (body == null) {
             // bug 2226
@@ -282,7 +287,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
                 if (inlineable < 0) {
                     return null;
                 } else {
-                    return inlineable == 1;
+                    return inlineable > 0;
                 }
             } else {
                 return false;
@@ -303,6 +308,15 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
         this.inlineable = inlineable ? 1 : 0;
     }
 
+    /**
+     * Indicate that the function is being inlined, incrementing a count of how
+     * many times it has been inlined
+     */
+
+    public void markAsInlined() {
+        inliningCount++;
+    }
+
     /**
      * Set the definitions of the declared parameters for this function
      *
@@ -545,16 +559,17 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      * Create a context for evaluating this function
      *
      * @param oldContext the existing context of the caller
+     * @param originator
      * @return a new context which should be supplied to the call() method.
      */
 
-    public XPathContextMajor makeNewContext(XPathContext oldContext) {
+    public XPathContextMajor makeNewContext(XPathContext oldContext, ContextOriginator originator) {
         XPathContextMajor c2 = oldContext.newCleanContext();
-        c2.setOrigin(this);
         c2.setReceiver(oldContext.getReceiver());
         c2.setTemporaryOutputState(StandardNames.XSL_FUNCTION);
         c2.setCurrentOutputUri(null);
         c2.setCurrentComponent(getDeclaringComponent());  // default value for the caller to override if necessary
+        c2.setOrigin(originator);
         return c2;
     }
 
@@ -563,7 +578,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
      * Call this function to return a value.
      *
      * @param context    This provides the run-time context for evaluating the function. This should be created
-     *                   using {@link #makeNewContext(XPathContext)}. It must be an instance of XPathContextMajor.
+     *                   using {@link Function#makeNewContext(XPathContext, ContextOriginator)}. It must be an instance of XPathContextMajor.
      * @param actualArgs the arguments supplied to the function. These must have the correct
      *                   types required by the function signature (it is the caller's responsibility to check this).
      *                   It is acceptable to supply a {@link net.sf.saxon.value.Closure} to represent a value whose
