@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -22,7 +22,7 @@ public class NamedTimeZone {
 
     static Set<String> knownTimeZones = new HashSet<String>(50);
     static HashMap<String, List<String>> idForCountry = new HashMap<String, List<String>>(50);
-    static List<String> worldTimeZones = new ArrayList<String>(20);
+    static List<String> worldTimeZones = new ArrayList<>(20);
 
     static {
         Collections.addAll(knownTimeZones, TimeZone.getAvailableIDs());
@@ -52,8 +52,8 @@ public class NamedTimeZone {
      *
      * @param country the two-character code for the country
      * @param zoneId  the Olsen timezone name for the timezone
-     * @param major   true if this is a major world timezone, meaning that its name is recognized
-     *                by people outside its own country
+     * @param major true if this is a major world timezone, meaning that its name is recognized
+     *              by people outside its own country
      */
 
     static void tz(String country, String zoneId, boolean major) {
@@ -313,7 +313,7 @@ public class NamedTimeZone {
         } else {
             possibleZones = idForCountry.get(place.toLowerCase());
             if (possibleZones == null) {
-                possibleZones = new ArrayList<String>();
+                possibleZones = new ArrayList<>();
             }
             long epochDate = date.getCalendar().getTime().getTime();
             for (String z : possibleZones) {
