@@ -105,6 +105,7 @@ public class ExpressionTool {
             terminator = Token.EOF;
         }
         Expression exp = parser.parse(expression, start, terminator, env);
+        // TODO: parser.parse() already sets the retained static context
         setDeepRetainedStaticContext(exp, env.makeRetainedStaticContext());
         exp = exp.simplify();
         return exp;
@@ -882,22 +883,6 @@ public class ExpressionTool {
                 nextFree = allocateSlots(o.getChildExpression(), nextFree, frame);
             }
         }
-        //#if PE==true || EE==true
-//        if (exp instanceof NextIteration) {
-//            WithParam.setSlotNumbers(((NextIteration)exp).getParameters());
-//        }
-        if (exp instanceof IterateInstr) {
-            nextFree = ((IterateInstr) exp).allocateParameterSlots(nextFree);
-        }
-//#endif
-//#if PE==true || EE==true
-//        if (exp instanceof NextIteration) {
-//            WithParam.setSlotNumbers(((NextIteration)exp).getParameters());
-//        }
-        if (exp instanceof IterateInstr) {
-            nextFree = ((IterateInstr) exp).allocateParameterSlots(nextFree);
-        }
-//#endif
         return nextFree;
 
         // Note, we allocate a distinct slot to each range variable, even if the
@@ -1598,12 +1583,6 @@ public class ExpressionTool {
         return expr;
     }
 
-    public static Expression makeRawPathExpression(
-            /*@NotNull*/ Expression start, /*@NotNull*/ Expression step, boolean sortAndDeduplicate) {
-
-        return new RawSlashExpression(start, step);
-    }
-
 
     /**
      * Find the operand corresponding to a particular child expression
