@@ -14,6 +14,7 @@ import net.sf.saxon.expr.sort.DocumentSorter;
 import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.Logger;
 import net.sf.saxon.om.GroundedValue;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.pattern.Pattern;
@@ -203,7 +204,7 @@ public class Optimizer {
      * @throws UnsupportedOperationException: this method should not be called in Saxon-HE
      */
 
-    public GroundedValue makeIndexedValue(SequenceIterator iter) throws XPathException {
+    public <T extends Item<?>> GroundedValue<T> makeIndexedValue(SequenceIterator<? extends T> iter) throws XPathException {
         throw new UnsupportedOperationException("Indexing requires Saxon-EE");
     }
 
@@ -224,7 +225,7 @@ public class Optimizer {
      * @return the (nominal) result of the evaluation
      */
 
-    public Sequence evaluateStreamingArgument(Expression expr, XPathContext context) throws XPathException {
+    public Sequence<?> evaluateStreamingArgument(Expression expr, XPathContext context) throws XPathException {
         // non-streaming fallback implementation
         return ExpressionTool.eagerEvaluate(expr, context);
     }
@@ -448,8 +449,22 @@ public class Optimizer {
         return null;
     }
 
+    /**
+     * Insert a ByteCodeCandidate into the expression tree. A ByteCodeCandidate monitors how many
+     * times it is executed, and after reaching a certain threshold, generates bytecode for faster
+     * evaluation of its target expression
+     *
+     * @param owner                   the owning construct in the expression tree
+     * @param expr                    the child (target) expression
+     * @param objectName              name to be used in forming the name of the bytecode class
+     * @param requiredEvaluationModes the evaluation modes for which code should be generated
+     * @return a new ByteCodeCandidate if the expression is suitable for bytecode generation,
+     * or the supplied target expression otherwise.
+     */
+
+
     public Expression makeByteCodeCandidate(ExpressionOwner owner, Expression expr, String objectName, int requiredEvaluationModes) {
-        return null;
+        return expr;
     }
 
     public void injectByteCodeCandidates(Expression exp) throws XPathException {
