@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -23,7 +23,6 @@ import java.util.*;
 /**
  * This class contains the part of the static context of expressions that (a) can change from one expression
  * to another within a query or stylesheet, and (b) are potentially needed at run-time.
- * <p/>
  * <p>From 9.6, the retained static context is available to every expression in the expression tree (previously,
  * expressions only retained that part of the static context needed by the particular expression). For economy,
  * a new RetainedStaticContext object is only created when the context changes: which is fairly rare (for
@@ -37,7 +36,7 @@ public class RetainedStaticContext implements NamespaceResolver {
     private String staticBaseUriString;
     private String defaultCollationName;
     private Map<String, String> namespaces;
-    private String defaultFunctionNamespace;
+    private String defaultFunctionNamespace = NamespaceConstant.FN;
     private String defaultElementNamespace;
     private DecimalFormatManager decimalFormatManager;
     private boolean backwardsCompatibility;
@@ -46,7 +45,7 @@ public class RetainedStaticContext implements NamespaceResolver {
     public RetainedStaticContext(Configuration config) {
         this.config = config;
         packageData = new PackageData(config);
-        namespaces = new HashMap<String, String>();
+        namespaces = new HashMap<>();
         defaultCollationName = NamespaceConstant.CODEPOINT_COLLATION_URI;
     }
 
@@ -77,7 +76,7 @@ public class RetainedStaticContext implements NamespaceResolver {
         if (Version.platform.JAXPStaticContextCheck(this, sc)) {
             //updated in method
         } else {
-            this.namespaces = new HashMap<String, String>(4);
+            this.namespaces = new HashMap<>(4);
             NamespaceResolver resolver = sc.getNamespaceResolver();
             for (Iterator<String> it = resolver.iteratePrefixes(); it.hasNext(); ) {
                 String prefix = it.next();
@@ -331,19 +330,10 @@ public class RetainedStaticContext implements NamespaceResolver {
             return false;
         }
         RetainedStaticContext r = (RetainedStaticContext) other;
-        if (!ExpressionTool.equalOrNull(staticBaseUriString, r.staticBaseUriString)) {
-            return false;
-        }
-        if (!defaultCollationName.equals(r.defaultCollationName)) {
-            return false;
-        }
-        if (!defaultFunctionNamespace.equals(r.defaultFunctionNamespace)) {
-            return false;
-        }
-        if (!namespaces.equals(r.namespaces)) {
-            return false;
-        }
-        return true;
+        return ExpressionTool.equalOrNull(staticBaseUriString, r.staticBaseUriString)
+                && defaultCollationName.equals(r.defaultCollationName)
+                && defaultFunctionNamespace.equals(r.defaultFunctionNamespace)
+                && namespaces.equals(r.namespaces);
     }
 
 
