@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -27,6 +27,7 @@ public class AlphanumericCollator implements StringCollator, java.io.Serializabl
 
     private StringCollator baseCollator;
     private static Pattern pattern = Pattern.compile("\\d+");
+    public final static String PREFIX = "http://saxon.sf.net/collation/alphaNumeric?base=";
 
     /**
      * Create an alphanumeric collation
@@ -44,14 +45,13 @@ public class AlphanumericCollator implements StringCollator, java.io.Serializabl
      * @return a collation URI that can be used to reconstruct the collation when an XSLT package is reloaded.
      */
     public String getCollationURI() {
-        // TODO: recognize this form when reconstituting collation URIs
-        return "http://saxon.sf.net/collation/alphaNumeric?base=" + baseCollator.getCollationURI();
+        return PREFIX + baseCollator.getCollationURI();
     }
 
     /**
      * Compare two objects.
      *
-     * @return <0 if a<b, 0 if a=b, >0 if a>b
+     * @return &lt;0 if a&lt;b, 0 if a=b, &gt;0 if a&gt;b
      */
 
     public int compareStrings(CharSequence cs1, CharSequence cs2) {
@@ -138,7 +138,7 @@ public class AlphanumericCollator implements StringCollator, java.io.Serializabl
 
             // handle an alphabetic part (even if zero-length)
 
-            sb.append(s.substring(pos1, m1start));
+            sb.append(baseCollator.getCollationKey(s.substring(pos1, m1start)).toString());
 
             // reached end?
 
