@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,6 +11,7 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.trans.NoDynamicContextException;
+import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.Type;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.QNameValue;
@@ -22,9 +23,8 @@ import net.sf.saxon.value.UntypedAtomicValue;
  * a collator that is used when the values to be compared are strings. It also supports
  * a separate method for testing equality of items, which can be used for data types that
  * are not ordered.
- * <p/>
- * The AtomicSortComparer is identical to the GenericAtomicComparer except for its handling
- * of NaN: it treats NaN values as lower than any other value, and as equal to each other.
+ * <p>The AtomicSortComparer is identical to the GenericAtomicComparer except for its handling
+ * of NaN: it treats NaN values as lower than any other value, and as equal to each other.</p>
  *
  * @author Michael H. Kay
  */
@@ -131,7 +131,7 @@ public class AtomicSortComparer implements AtomicComparer {
      *          interface.
      * @param b the second object to be compared. This must be comparable with the first object: for
      *          example, if one is a string, they must both be strings.
-     * @return <0 if a<b, 0 if a=b, >0 if a>b
+     * @return &lt;0 if a&lt;b, 0 if a=b, &gt;0 if a&gt;b
      * @throws ClassCastException if the objects are not comparable
      */
 
@@ -150,11 +150,11 @@ public class AtomicSortComparer implements AtomicComparer {
         // System.err.println("Comparing " + a.getClass() + "(" + a + ") with " + b.getClass() + "(" + b + ") using " + collator);
 
         // Delete the following five lines to fix bug 3450
-        if (a instanceof UntypedAtomicValue) {
-            return ((UntypedAtomicValue) a).compareTo(b, collator, context);
-        } else if (b instanceof UntypedAtomicValue) {
-            return -((UntypedAtomicValue) b).compareTo(a, collator, context);
-        } else
+        //if (a instanceof UntypedAtomicValue) {
+        //    return ((UntypedAtomicValue) a).compareTo(b, collator, context);
+        //} else if (b instanceof UntypedAtomicValue) {
+        //    return -((UntypedAtomicValue) b).compareTo(a, collator, context);
+        //} else
         // End of fix for 3450
 
         if (a.isNaN()) {
@@ -201,8 +201,10 @@ public class AtomicSortComparer implements AtomicComparer {
      */
 
     protected int compareNonComparables(AtomicValue a, AtomicValue b) {
-        throw new ClassCastException("Values are not comparable (" +
-                Type.displayTypeName(a) + ", " + Type.displayTypeName(b) + ')');
+        XPathException err = new XPathException("Values are not comparable (" +
+                                                        Type.displayTypeName(a) + ", " + Type.displayTypeName(b) + ')',
+                                                "XPTY0004");
+        throw new ComparisonException(err);
     }
 
     /**
