@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -69,7 +69,7 @@ public class CollatingAtomicComparer implements AtomicComparer {
      *          interface.
      * @param b the second object to be compared. This must be comparable with the first object: for
      *          example, if one is a string, they must both be strings.
-     * @return <0 if a<b, 0 if a=b, >0 if a>b
+     * @return &lt;0 if a&lt;b, 0 if a=b, &gt;0 if a&gt;b
      * @throws ClassCastException if the objects are not comparable
      */
 
@@ -96,7 +96,7 @@ public class CollatingAtomicComparer implements AtomicComparer {
      *          collator is used to compare the values, otherwise the value must implement the equals() method.
      * @param b the second object to be compared. This must be comparable with the first object: for
      *          example, if one is a string, they must both be strings.
-     * @return <0 if a<b, 0 if a=b, >0 if a>b
+     * @return true if the values are equal, false if not
      * @throws ClassCastException if the objects are not comparable
      */
 
@@ -113,5 +113,15 @@ public class CollatingAtomicComparer implements AtomicComparer {
     public String save() {
         return "CAC|" + getCollator().getCollationURI();
     }
+
+    @Override
+    public int hashCode() {
+        return collator.hashCode();
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+        return obj instanceof CollatingAtomicComparer && collator.equals(((CollatingAtomicComparer)obj).collator);
+    }
 }
 
