@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,8 +15,6 @@ import net.sf.saxon.value.AtomicValue;
 /**
  * A Comparer that modifies a base comparer by sorting empty key values and NaN values last (greatest),
  * as opposed to the default which sorts them first.
- *
- * @author Michael H. Kay
  */
 
 public class EmptyGreatestComparer implements AtomicComparer, java.io.Serializable {
@@ -76,7 +74,7 @@ public class EmptyGreatestComparer implements AtomicComparer, java.io.Serializab
      *          interface.
      * @param b the second object to be compared. This must be comparable with the first object: for
      *          example, if one is a string, they must both be strings.
-     * @return <0 if a<b, 0 if a=b, >0 if a>b
+     * @return &lt;0 if a&lt;b, 0 if a=b, &gt;0 if a&gt;b
      * @throws ClassCastException if the objects are not comparable
      */
 
@@ -92,7 +90,7 @@ public class EmptyGreatestComparer implements AtomicComparer, java.io.Serializab
         }
 
         if (a.isNaN()) {
-            return (b.isNaN() ? 0 : +1);
+            return b.isNaN() ? 0 : +1;
         } else if (b.isNaN()) {
             return -1;
         }
