@@ -115,6 +115,15 @@ public class SortKeyDefinition extends PseudoExpression {
         return sortKey.getChildExpression();
     }
 
+    /**
+     * Get the sort key operand
+     * @return the operand that computes the sort key
+     */
+
+    public Operand getSortKeyOperand() {
+        return sortKey;
+    }
+
     /**
      * Ask whether the sortkey is to be evaluated with the item-being-sorted
      * as the context item
@@ -427,7 +436,7 @@ public class SortKeyDefinition extends PseudoExpression {
             // Otherwise rely on the containing SortExpression to type-check the sort key
         }
         Expression lang = getLanguage();
-        if (lang instanceof StringLiteral && ((StringLiteral) lang).getStringValue().length() != 0) {
+        if (lang instanceof StringLiteral && !((StringLiteral) lang).getStringValue().isEmpty()) {
             ValidationFailure vf = StringConverter.StringToLanguage.INSTANCE.validate(((StringLiteral) lang).getStringValue());
             if (vf != null) {
                 throw new XPathException("The lang attribute of xsl:sort must be a valid language code", "XTDE0030");
@@ -517,7 +526,7 @@ public class SortKeyDefinition extends PseudoExpression {
             String uri = "http://saxon.sf.net/collation";
             boolean firstParam = true;
             Properties props = new Properties();
-            if (languageX.length() != 0) {
+            if (!languageX.isEmpty()) {
                 ValidationFailure vf = StringConverter.StringToLanguage.INSTANCE.validate(languageX);
                 if (vf != null) {
                     throw new XPathException("The lang attribute of xsl:sort must be a valid language code", "XTDE0030");
