@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -94,25 +94,17 @@ public class AnalyzeStringFn extends RegexFunction {
             boolean schemaAware = context.getController().getExecutable().isSchemaAware();
             Configuration config = context.getConfiguration();
             if (schemaAware && !config.isSchemaAvailable(NamespaceConstant.FN)) {
-                //try {
-                    StandardEntityResolver resolver = new StandardEntityResolver();
-                    resolver.setConfiguration(config);
-                    InputStream inputStream = Configuration.locateResource("xpath-functions.xsd", new ArrayList(), new ArrayList());
-                    if (inputStream == null) {
-                        throw new XPathException("Failed to load xpath-functions.xsd from the classpath");
-                    }
-                    InputSource is = new InputSource(inputStream);
-                    //InputSource is = resolver.resolveEntity(null, "classpath:xpath-functions.xsd");
-
-                    if (config.isTiming()) {
-                        config.getLogger().info("Loading schema from resources for: " + NamespaceConstant.FN);
-                    }
-                    config.addSchemaSource(new SAXSource(is));
-//                } catch (SAXException e) {
-//                    throw new XPathException(e);
-//                } catch (IOException e) {
-//                    throw new XPathException(e);
-//                }
+                StandardEntityResolver resolver = new StandardEntityResolver();
+                resolver.setConfiguration(config);
+                InputStream inputStream = Configuration.locateResource("xpath-functions.scm", new ArrayList<>(), new ArrayList<>());
+                if (inputStream == null) {
+                    throw new XPathException("Failed to load xpath-functions.scm from the classpath");
+                }
+                InputSource is = new InputSource(inputStream);
+                if (config.isTiming()) {
+                    config.getLogger().info("Loading schema from resources for: " + NamespaceConstant.FN);
+                }
+                config.addSchemaSource(new SAXSource(is));
             }
             init(context.getConfiguration(), schemaAware);
         }
@@ -160,4 +152,4 @@ public class AnalyzeStringFn extends RegexFunction {
 
 }
 
-// Copyright (c) 2017 Saxonica Limited.
\ No newline at end of file
+// Copyright (c) 2018 Saxonica Limited.
\ No newline at end of file
