@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -20,9 +20,7 @@ import net.sf.saxon.ma.arrays.ArrayItemType;
 import net.sf.saxon.ma.arrays.SquareArrayConstructor;
 import net.sf.saxon.ma.map.MapFunctionSet;
 import net.sf.saxon.ma.map.MapType;
-import net.sf.saxon.om.Function;
-import net.sf.saxon.om.Sequence;
-import net.sf.saxon.om.SequenceTool;
+import net.sf.saxon.om.*;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
@@ -122,14 +120,14 @@ public class ApplyFn extends SystemFunction  {
      * @throws net.sf.saxon.trans.XPathException
      *          if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
         final Function function = (Function) arguments[0].head();
         ArrayItem args = (ArrayItem)arguments[1].head();
 
         if (function.getArity() != args.arrayLength()) {
             String errorCode = isDynamicFunctionCall() ? "XPTY0004" : "FOAP0001";
             XPathException err = new XPathException(
-                    "Number of arguments required for dynamic call to " + function.getDescription() + " = " + function.getArity() +
+                    "Number of arguments required for dynamic call to " + function.getDescription() + " is " + function.getArity() +
                     "; number supplied = " + args.arrayLength(), errorCode);
             err.setIsTypeError(isDynamicFunctionCall());
             err.setXPathContext(context);
@@ -151,12 +149,12 @@ public class ApplyFn extends SystemFunction  {
                 } else {
                     role = new RoleDiagnostic(RoleDiagnostic.FUNCTION, "fn:apply", i + 1);
                 }
-                Sequence converted = th.applyFunctionConversionRules(
+                Sequence<?> converted = th.applyFunctionConversionRules(
                     args.get(i), expected, role, ExplicitLocation.UNKNOWN_LOCATION);
-                argArray[i] = SequenceTool.makeRepeatable(converted);
+                argArray[i] = converted.materialize();
             }
         }
-        Sequence rawResult = dynamicCall(function, context, argArray);
+        Sequence<?> rawResult = dynamicCall(function, context, argArray);
         if (function.isTrustedResultType()) {
             // trust system functions to return a result of the correct type
             return rawResult;
@@ -178,10 +176,9 @@ public class ApplyFn extends SystemFunction  {
      * the exportAttributes method
      *
      * @param attributes the attributes, as a properties object
-     * @throws net.sf.saxon.trans.XPathException
      */
     @Override
-    public void importAttributes(Properties attributes) throws XPathException {
+    public void importAttributes(Properties attributes) {
         dynamicFunctionCall = attributes.getProperty("dyn");
     }
 }
