@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -59,11 +59,9 @@ public class BooleanFn extends SystemFunction {
             if (val instanceof BooleanValue) {
                 return exp;
             }
-            Literal l2 = Literal.makeLiteral(
+            return Literal.makeLiteral(
                 BooleanValue.get(
-                    ExpressionTool.effectiveBooleanValue(val.iterate())));
-            l2.setRetainedStaticContext(exp.getRetainedStaticContext());
-            return l2;
+                    ExpressionTool.effectiveBooleanValue(val.iterate())), exp);
         }
         if (exp instanceof ValueComparison) {
             ValueComparison vc = (ValueComparison) exp;
@@ -79,11 +77,13 @@ public class BooleanFn extends SystemFunction {
         } else if (exp.isCallOn(Count.class)) {
             // rewrite boolean(count(x)) => exists(x)
             Expression exists = SystemFunction.makeCall("exists", exp.getRetainedStaticContext(), ((SystemFunctionCall) exp).getArg(0));
+            assert exists != null;
             ExpressionTool.copyLocationInfo(exp, exists);
             return exists.optimize(visitor, contextItemType);
         } else if (exp.getItemType() instanceof NodeTest) {
             // rewrite boolean(x) => exists(x)
             Expression exists = SystemFunction.makeCall("exists", exp.getRetainedStaticContext(), exp);
+            assert exists != null;
             ExpressionTool.copyLocationInfo(exp, exists);
             return exists.optimize(visitor, contextItemType);
         } else {
