@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -42,6 +42,7 @@ public abstract class CollatingFunctionFixed extends SystemFunction implements S
 
     /**
      * Ask whether this function needs a collation that can handle substring matching
+     *
      * @return true in the case of functions such as contains() and starts-with() where
      * substring matching is required. Returns false by default;
      */
@@ -77,15 +78,6 @@ public abstract class CollatingFunctionFixed extends SystemFunction implements S
     public void setCollationName(String collationName) throws XPathException {
         this.collationName = collationName;
         allocateCollator();
-        if (isSubstringMatchingFunction()) {
-            if (stringCollator instanceof SimpleCollation) {
-                stringCollator = ((SimpleCollation)stringCollator).getSubstringMatcher();
-            }
-            if (!(stringCollator instanceof SubstringMatcher)) {
-                throw new XPathException("The collation requested for " + getFunctionName().getDisplayName() +
-                                                 " does not support substring matching", "FOCH0004");
-            }
-        }
     }
 
     private void allocateCollator() throws XPathException {
@@ -93,19 +85,27 @@ public abstract class CollatingFunctionFixed extends SystemFunction implements S
         if (stringCollator == null) {
             throw new XPathException("Unknown collation " + collationName, "FOCH0002");
         }
+        if (isSubstringMatchingFunction()) {
+            if (stringCollator instanceof SimpleCollation) {
+                stringCollator = ((SimpleCollation) stringCollator).getSubstringMatcher();
+            }
+            if (!(stringCollator instanceof SubstringMatcher)) {
+                throw new XPathException("The collation requested for " + getFunctionName().getDisplayName() +
+                                                 " does not support substring matching", "FOCH0004");
+            }
+        }
     }
-
+    
     /**
      * During static analysis, if types are known and the collation is known, pre-allocate a comparer
-     * for comparing atomic values. Called by some collating functions during type-checking
-     *
+     * for comparing atomic values. Called by some collating functions during type-checking. The comparer
+     * that is allocated treats NaN as not equal to NaN.
      * @param type0        the type of the first comparand
      * @param type1        the type of the second comparand
      * @param env          the static context
-     * @param NaNequalsNaN true if two NaN values are to be considered equal
      */
 
-    protected void preAllocateComparer(AtomicType type0, AtomicType type1, StaticContext env, boolean NaNequalsNaN) {
+    protected void preAllocateComparer(AtomicType type0, AtomicType type1, StaticContext env) {
         StringCollator collation = getStringCollator();
         if (type0 == ErrorType.getInstance() || type1 == ErrorType.getInstance()) {
             // there will be no instances to compare, so we can use any comparer
@@ -113,14 +113,10 @@ public abstract class CollatingFunctionFixed extends SystemFunction implements S
             return;
         }
 
-        if (NaNequalsNaN) { // TODO: always false?
-            atomicComparer = AtomicSortComparer.makeSortComparer(
-                    collation, type0.getPrimitiveType(), env.makeEarlyEvaluationContext());
-        } else {
-            atomicComparer = GenericAtomicComparer.makeAtomicComparer(
-                    (BuiltInAtomicType) type0.getBuiltInBaseType(), (BuiltInAtomicType) type1.getBuiltInBaseType(),
+        atomicComparer = GenericAtomicComparer.makeAtomicComparer(
+                (BuiltInAtomicType) type0.getBuiltInBaseType(), (BuiltInAtomicType) type1.getBuiltInBaseType(),
                 stringCollator, env.makeEarlyEvaluationContext());
-        }
+
     }
 
 
@@ -136,9 +132,10 @@ public abstract class CollatingFunctionFixed extends SystemFunction implements S
 
     /**
      * During evaluation, get the pre-allocated atomic comparer if available, or allocate a new one otherwise
+     *
      * @param context the dynamic evaluation context
      * @return the pre-allocated comparer if one is available; otherwise, a newly allocated one, using the specified
-     *         StringCollator for comparing strings
+     * StringCollator for comparing strings
      */
 
     public AtomicComparer getAtomicComparer(XPathContext context) {
@@ -176,7 +173,7 @@ public abstract class CollatingFunctionFixed extends SystemFunction implements S
         SystemFunction copy = SystemFunction.makeFunction(getFunctionName().getLocalPart(), getRetainedStaticContext(), getArity());
         if (copy instanceof CollatingFunctionFree) {
             try {
-                copy = ((CollatingFunctionFree)copy).bindCollation(collationName);
+                copy = ((CollatingFunctionFree) copy).bindCollation(collationName);
             } catch (XPathException e) {
                 throw new AssertionError(e);
             }
@@ -185,7 +182,7 @@ public abstract class CollatingFunctionFixed extends SystemFunction implements S
             ((CollatingFunctionFixed) copy).collationName = collationName;
             ((CollatingFunctionFixed) copy).atomicComparer = atomicComparer;
             ((CollatingFunctionFixed) copy).stringCollator = stringCollator;
-            return (CollatingFunctionFixed)copy;
+            return (CollatingFunctionFixed) copy;
         }
         throw new IllegalStateException();
     }
