@@ -119,7 +119,7 @@ public class CurrentGroupingKeyCall extends Expression implements Callable {
 
     /*@NotNull*/
     @Override
-    public SequenceIterator<? extends Item> iterate(XPathContext c) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext c) throws XPathException {
         GroupIterator gi = c.getCurrentGroupIterator();
         AtomicSequence result = gi==null ? null : gi.getCurrentGroupingKey();
         if (result == null) {
@@ -150,7 +150,7 @@ public class CurrentGroupingKeyCall extends Expression implements Callable {
      * of the callee to ensure that the type of result conforms to the expected result type.
      * @throws XPathException if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
         return SequenceTool.toLazySequence(iterate(context));
     }
 }
