@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -37,7 +37,7 @@ public class DateTimeConstructor extends SystemFunction {
         if (arg0 == null || arg1 == null) {
             return ZeroOrOne.empty();
         }
-        return new ZeroOrOne<DateTimeValue>(DateTimeValue.makeDateTimeValue(arg0, arg1));
+        return new ZeroOrOne<>(DateTimeValue.makeDateTimeValue(arg0, arg1));
     }
 
     public String getCompilerName() {
