@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -20,16 +20,12 @@ import net.sf.saxon.value.BooleanValue;
 
 public class DocAvailable extends SystemFunction  {
 
-
-
-
     private boolean isDocAvailable(AtomicValue hrefVal, XPathContext context) throws XPathException {
         if (hrefVal == null) {
             return false;
         }
         String href = hrefVal.getStringValue();
         return docAvailable(href, context);
-
     }
 
     /**
@@ -45,15 +41,7 @@ public class DocAvailable extends SystemFunction  {
         return BooleanValue.get(isDocAvailable((AtomicValue) arguments[0].head(), context));
     }
 
-//    public static class ErrorDiscarder implements ErrorListener {
-//        public static ErrorDiscarder THE_INSTANCE = new ErrorDiscarder();
-//        public void warning(TransformerException exception) {}
-//        public void error(TransformerException exception) {}
-//        public void fatalError(TransformerException exception) {}
-//
-//    }
-
-    public boolean docAvailable(String href, XPathContext context) throws XPathException {
+    public boolean docAvailable(String href, XPathContext context) {
         try {
             PackageData packageData = getRetainedStaticContext().getPackageData();
             DocumentURI documentKey = DocumentFn.computeDocumentKey(href, getStaticBaseUriString(), packageData, context);
