@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -32,7 +32,7 @@ public class DocumentUri_1 extends ScalarSystemFunction  {
     public static AnyURIValue getDocumentURI(NodeInfo node, XPathContext c) {
         if (node.getNodeKind() == Type.DOCUMENT) {
             Object o = node.getTreeInfo().getUserData("saxon:document-uri");
-            if (o != null && o instanceof String) {
+            if (o instanceof String) {
                 return o.toString().isEmpty() ? null : new AnyURIValue(o.toString());
             }
             final Controller controller = c.getController();
