@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -29,9 +29,9 @@ public class Empty extends Aggregate {
         // See if we can deduce the answer from the cardinality
         int c = arguments[0].getCardinality();
         if (c == StaticProperty.ALLOWS_ONE_OR_MORE) {
-            return Literal.makeLiteral(BooleanValue.FALSE);
+            return Literal.makeLiteral(BooleanValue.FALSE, arguments[0]);
         } else if (c == StaticProperty.ALLOWS_ZERO) {
-            return Literal.makeLiteral(BooleanValue.TRUE);
+            return Literal.makeLiteral(BooleanValue.TRUE, arguments[0]);
         }
 
         // Rewrite
@@ -79,7 +79,6 @@ public class Empty extends Aggregate {
 
             /**
              * Perform optimisation of an expression and its subexpressions.
-             * <p/>
              * <p>This method is called after all references to functions and variables have been resolved
              * to the declaration of the function or variable, and after all type checking has been done.</p>
              *
@@ -102,9 +101,9 @@ public class Empty extends Aggregate {
                 // See if we can deduce the answer from the cardinality
                 int c = getArg(0).getCardinality();
                 if (c == StaticProperty.ALLOWS_ONE_OR_MORE) {
-                    return Literal.makeLiteral(BooleanValue.FALSE);
+                    return Literal.makeLiteral(BooleanValue.FALSE, e2);
                 } else if (c == StaticProperty.ALLOWS_ZERO) {
-                    return Literal.makeLiteral(BooleanValue.TRUE);
+                    return Literal.makeLiteral(BooleanValue.TRUE, e2);
                 }
 
                 // Don't sort the argument
