@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,7 @@ package net.sf.saxon.functions;
 
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.lib.EnvironmentVariableResolver;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.ZeroOrOne;
 import net.sf.saxon.trans.XPathException;
@@ -21,7 +21,6 @@ import net.sf.saxon.value.StringValue;
 
 public class EnvironmentVariable extends SystemFunction {
 
-
     /**
      * Evaluate the expression
      *
@@ -33,24 +32,24 @@ public class EnvironmentVariable extends SystemFunction {
      */
     public ZeroOrOne<StringValue> call(XPathContext context, Sequence[] arguments) throws XPathException {
 
-        return new ZeroOrOne<StringValue>(
+        return new ZeroOrOne<>(
                 getVariable((StringValue) arguments[0].head(), context));
     }
 
-    private static StringValue getVariable(StringValue environVar, XPathContext context) throws XPathException {
-        EnvironmentVariableResolver resolver = (EnvironmentVariableResolver) context.getConfiguration().getConfigurationProperty(
-                FeatureKeys.ENVIRONMENT_VARIABLE_RESOLVER);
+    private static StringValue getVariable(StringValue environVar, XPathContext context) {
+        EnvironmentVariableResolver resolver = context.getConfiguration().getConfigurationProperty(
+                Feature.ENVIRONMENT_VARIABLE_RESOLVER);
         String environVarName = environVar.getStringValue();
-        String environValue;
-        try {
-            environValue = resolver.getEnvironmentVariable(environVarName);
-            if (environValue == null) {
-                return null;
+        String environValue = "";
+        if (context.getConfiguration().getBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS)) {
+            try {
+                environValue = resolver.getEnvironmentVariable(environVarName);
+                if (environValue == null) {
+                    return null;
+                }
+            } catch (SecurityException | NullPointerException e) {
+                // no action;
             }
-        } catch (SecurityException e) {
-            environValue = "";
-        } catch (NullPointerException e) {
-            environValue = "";
         }
 
         return new StringValue(environValue);
@@ -58,4 +57,4 @@ public class EnvironmentVariable extends SystemFunction {
 
 }
 
-//Copyright (c) 2010-2012 Saxonica Limited.
\ No newline at end of file
+// Copyright (c) 2010-2017 Saxonica Limited.
\ No newline at end of file
