@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,6 +18,7 @@ import net.sf.saxon.trans.XPathException;
 
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 
 /**
  * An ExecutableFunctionLibrary is a function library that contains definitions of functions for use at
@@ -25,13 +26,12 @@ import java.util.Iterator;
  * the information is needed dynamically, for example (a) to support the XSLT function-available() call
  * (in the pathological case where the argument is not known statically), (b) to allow functions to be
  * called from saxon:evaluate(), (c) to allow functions to be called from a debugging breakpoint.
- * <p/>
  */
 
 public class ExecutableFunctionLibrary implements FunctionLibrary {
 
     private transient Configuration config;
-    private HashMap<SymbolicName, UserFunction> functions = new HashMap<SymbolicName, UserFunction>(20);
+    private HashMap<SymbolicName, UserFunction> functions = new HashMap<>(20);
     // The key of the hash table is a String that combines the QName of the function with the arity.
 
     /**
@@ -64,17 +64,14 @@ public class ExecutableFunctionLibrary implements FunctionLibrary {
      *                     that the static type of the arguments (obtainable via getItemType() and getCardinality() may
      *                     be used as part of the binding algorithm.
      * @param env          the static evaluation context
+     * @param reasons      If no matching function is found by the function library, it may add
+     *                     a diagnostic explanation to this list explaining why none of the available
+     *                     functions could be used.
      * @return An object representing the extension function to be called, if one is found;
      *         null if no extension function was found matching the required name and arity.
-     * @throws net.sf.saxon.trans.XPathException
-     *          if a function is found with the required name and arity, but
-     *          the implementation of the function cannot be loaded or used; or if an error occurs
-     *          while searching for the function; or if this function library "owns" the namespace containing
-     *          the function call, but no function was found.
      */
 
-    public Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env)
-            throws XPathException {
+    public Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env, List<String> reasons) {
         UserFunction fn = functions.get(functionName);
         if (fn == null) {
             return null;
@@ -92,7 +89,6 @@ public class ExecutableFunctionLibrary implements FunctionLibrary {
     /**
      * Test whether a function with a given name and arity is available; if so, return a function
      * item that can be dynamically called.
-     * <p/>
      * <p>This supports the function-lookup() function in XPath 3.0.</p>
      *
      * @param functionName  the qualified name of the function being called
@@ -135,7 +131,7 @@ public class ExecutableFunctionLibrary implements FunctionLibrary {
 
     public FunctionLibrary copy() {
         ExecutableFunctionLibrary efl = new ExecutableFunctionLibrary(config);
-        efl.functions = new HashMap<SymbolicName, UserFunction>(functions);
+        efl.functions = new HashMap<>(functions);
         return efl;
     }
 
