@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,12 +7,15 @@
 
 package net.sf.saxon.functions;
 
+import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.StaticContext;
 import net.sf.saxon.om.Function;
 import net.sf.saxon.trans.SymbolicName;
 import net.sf.saxon.trans.XPathException;
 
+import java.util.List;
+
 /**
  * A FunctionLibrary handles the binding of function calls in XPath (or XQuery) expressions.
  * There are a number of implementations of this
@@ -24,6 +27,8 @@ import net.sf.saxon.trans.XPathException;
 
 public interface FunctionLibrary {
 
+    default void setConfiguration(Configuration config) {}
+
     /**
      * Test whether a function with a given name and arity is available
      * <p>This supports the function-available() function in XSLT.</p>
@@ -49,24 +54,21 @@ public interface FunctionLibrary {
      *                     that the static type of the arguments (obtainable via getItemType() and getCardinality()) may
      *                     be used as part of the binding algorithm. In some cases it may be possible for the function
      *                     to be pre-evaluated at compile time, for example if these expressions are all constant values.
-     *                     <p/>
-     *                     The conventions of the XPath language demand that the results of a function depend only on the
+     *                     <p>The conventions of the XPath language demand that the results of a function depend only on the
      *                     values of the expressions supplied as arguments, and not on the form of those expressions. For
      *                     example, the result of f(4) is expected to be the same as f(2+2). The actual expression is supplied
      *                     here to enable the binding mechanism to select the most efficient possible implementation (including
-     *                     compile-time pre-evaluation where appropriate).
+     *                     compile-time pre-evaluation where appropriate).</p>
      * @param env          The static context of the function call
+     * @param reasons      If no matching function is found by the function library, it may add
+     *                     a diagnostic explanation to this list explaining why none of the available
+     *                     functions could be used.
      * @return An expression equivalent to a call on the specified function, if one is found;
      *         null if no function was found matching the required name and arity.
-     * @throws net.sf.saxon.trans.XPathException
-     *          if a function is found with the required name and arity, but
-     *          the implementation of the function cannot be loaded or used; or if an error occurs
-     *          while searching for the function.
      */
 
     /*@Nullable*/
-    Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env)
-            throws XPathException;
+    Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env, List<String> reasons);
 
     /**
      * This method creates a copy of a FunctionLibrary: if the original FunctionLibrary allows
@@ -83,7 +85,6 @@ public interface FunctionLibrary {
     /**
      * Test whether a function with a given name and arity is available; if so, return a function
      * item that can be dynamically called.
-     * <p/>
      * <p>This supports the function-lookup() function in XPath 3.0.</p>
      *
      * @param functionName  the qualified name of the function being called
