@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,7 @@ package net.sf.saxon.functions;
 
 import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.StaticContext;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.Logger;
 import net.sf.saxon.om.Function;
 import net.sf.saxon.om.StructuredQName;
@@ -27,7 +27,7 @@ import java.util.List;
  */
 public class FunctionLibraryList implements FunctionLibrary, XQueryFunctionBinder {
 
-    public List<FunctionLibrary> libraryList = new ArrayList<FunctionLibrary>(8);
+    public List<FunctionLibrary> libraryList = new ArrayList<>(8);
 
     public FunctionLibraryList() {
     }
@@ -58,7 +58,6 @@ public class FunctionLibraryList implements FunctionLibrary, XQueryFunctionBinde
     /**
      * Test whether a function with a given name and arity is available; if so, return a function
      * item that can be dynamically called.
-     * <p/>
      * <p>This supports the function-lookup() function in XPath 3.0.</p>
      *
      * @param functionName  the qualified name of the function being called
@@ -107,32 +106,30 @@ public class FunctionLibraryList implements FunctionLibrary, XQueryFunctionBinde
      *                     be used as part of the binding algorithm. In some cases it may be possible for the function
      *                     to be pre-evaluated at compile time, for example if these expressions are all constant values.
      * @param env          The static context
+     * @param reasons      If no matching function is found by the function library, it may add
+     *                     a diagnostic explanation to this list explaining why none of the available
+     *                     functions could be used.
      * @return An object representing the extension function to be called, if one is found;
      *         null if no extension function was found matching the required name and arity.
-     * @throws net.sf.saxon.trans.XPathException
-     *          if a function is found with the required name and arity, but
-     *          the implementation of the function cannot be loaded or used; or if an error occurs
-     *          while searching for the function.
      */
 
-    public Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env)
-            throws XPathException {
-        boolean debug = env.getConfiguration().getBooleanProperty(FeatureKeys.TRACE_EXTERNAL_FUNCTIONS);
+    public Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env, List<String> reasons) {
+        boolean debug = env.getConfiguration().getBooleanProperty(Feature.TRACE_EXTERNAL_FUNCTIONS);
         Logger err = env.getConfiguration().getLogger();
         if (debug) {
-            err.info("Looking for function " + functionName.getComponentName().getClarkName() + "#" + functionName.getArity());
+            err.info("Looking for function " + functionName.getComponentName().getEQName() + "#" + functionName.getArity());
         }
         for (FunctionLibrary lib : libraryList) {
             if (debug) {
                 err.info("Trying " + lib.getClass().getName());
             }
-            Expression func = lib.bind(functionName, staticArgs, env);
+            Expression func = lib.bind(functionName, staticArgs, env, reasons);
             if (func != null) {
                 return func;
             }
         }
         if (debug) {
-            err.info("Function " + functionName.getComponentName().getClarkName() + " not found!");
+            err.info("Function " + functionName.getComponentName().getEQName() + " not found!");
         }
         return null;
     }
@@ -179,7 +176,7 @@ public class FunctionLibraryList implements FunctionLibrary, XQueryFunctionBinde
 
     public FunctionLibrary copy() {
         FunctionLibraryList fll = new FunctionLibraryList();
-        fll.libraryList = new ArrayList<FunctionLibrary>(libraryList.size());
+        fll.libraryList = new ArrayList<>(libraryList.size());
         for (int i = 0; i < libraryList.size(); i++) {
             fll.libraryList.add(libraryList.get(i).copy());
         }
