@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -30,12 +30,12 @@ public class GenerateId_1 extends ScalarSystemFunction {
      * Determine the special properties of this expression. The generate-id()
      * function is a special case: it is considered creative if its operand
      * is creative, so that generate-id(f()) is not taken out of a loop
-     * @param arguments
+     * @param arguments the expressions supplied as arguments in the function call
      */
 
     public int getSpecialProperties(Expression[] arguments) {
         int p = super.getSpecialProperties(arguments);
-        return p & ~StaticProperty.NON_CREATIVE;
+        return p & ~StaticProperty.NO_NODES_NEWLY_CREATED;
     }
 
     @Override
