@@ -35,7 +35,7 @@ public class InsertBefore extends SystemFunction {
      * @throws net.sf.saxon.trans.XPathException
      *          if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
         NumericValue n = (NumericValue) arguments[1].head();
         int pos = (int) n.longValue();
         return SequenceTool.toLazySequence(
@@ -57,15 +57,15 @@ public class InsertBefore extends SystemFunction {
      * an iterator over the sequence to be inserted, and the insert position.
      */
 
-    public static class InsertIterator implements SequenceIterator<Item> {
+    public static class InsertIterator implements SequenceIterator<Item<?>> {
 
-        private SequenceIterator base;
-        private SequenceIterator insert;
+        private SequenceIterator<?> base;
+        private SequenceIterator<?> insert;
         private int insertPosition;
         private int position = 0;
         private boolean inserting = false;
 
-        public InsertIterator(SequenceIterator<? extends Item> base, SequenceIterator<? extends Item> insert, int insertPosition) {
+        public InsertIterator(SequenceIterator<?> base, SequenceIterator<?> insert, int insertPosition) {
             this.base = base;
             this.insert = insert;
             this.insertPosition = insertPosition < 1 ? 1 : insertPosition;
@@ -73,8 +73,8 @@ public class InsertBefore extends SystemFunction {
         }
 
 
-        public Item next() throws XPathException {
-            Item nextItem;
+        public Item<?> next() throws XPathException {
+            Item<?> nextItem;
             if (inserting) {
                 nextItem = insert.next();
                 if (nextItem == null) {
