@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,12 +7,9 @@
 
 package net.sf.saxon.functions;
 
-import com.saxonica.ee.bytecode.ExpressionCompiler;
-import com.saxonica.ee.bytecode.ForceCaseCompiler;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.ZeroOrOne;
-import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.StringValue;
 
 
@@ -23,7 +20,7 @@ import net.sf.saxon.value.StringValue;
 public class LowerCase extends ScalarSystemFunction {
 
     @Override
-    public AtomicValue evaluate(Item arg, XPathContext context) {
+    public StringValue evaluate(Item arg, XPathContext context) {
         return StringValue.makeStringValue(arg.getStringValue().toLowerCase());
     }
 
@@ -32,19 +29,9 @@ public class LowerCase extends ScalarSystemFunction {
         return ZERO_LENGTH_STRING;
     }
 
-//#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the LowerCase expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
-    @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new ForceCaseCompiler();
+    public String getCompilerName() {
+        return "ForceCaseCompiler";
     }
-//#endif
-
 
 }
 
