@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,7 +8,6 @@
 package net.sf.saxon.functions;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.Version;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.regex.RegularExpression;
@@ -51,8 +50,8 @@ public class Matches extends RegexFunction {
             if (context.getConfiguration().getXsdVersion() == Configuration.XSD11) {
                 lang += "/XSD11";
             }
-            re = Version.platform.compileRegularExpression(
-                    context.getConfiguration(), regex.getStringValueCS(), flags.toString(), lang, null);
+            re = context.getConfiguration().compileRegularExpression(
+                    regex.getStringValueCS(), flags.toString(), lang, null);
 
         } catch (XPathException err) {
             XPathException de = new XPathException(err);
