@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,7 +12,6 @@ import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.ZeroOrOne;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.StringValue;
 
 /**
@@ -22,12 +21,12 @@ import net.sf.saxon.value.StringValue;
 public class Name_1 extends ScalarSystemFunction {
 
     @Override
-    public AtomicValue evaluate(Item item, XPathContext context) throws XPathException {
+    public StringValue evaluate(Item item, XPathContext context) throws XPathException {
         return StringValue.makeStringValue(((NodeInfo) item).getDisplayName());
     }
 
     @Override
-    public ZeroOrOne<? extends AtomicValue> resultWhenEmpty() {
+    public ZeroOrOne<StringValue> resultWhenEmpty() {
         return ZERO_LENGTH_STRING;
     }
 
