@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,8 +7,6 @@
 
 package net.sf.saxon.functions;
 
-import com.saxonica.ee.bytecode.ExpressionCompiler;
-import com.saxonica.ee.bytecode.NamespaceUriFnCompiler;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
@@ -32,21 +30,12 @@ public class NamespaceUri_1 extends ScalarSystemFunction {
 
     @Override
     public ZeroOrOne<? extends AtomicValue> resultWhenEmpty() {
-        return new One<AnyURIValue>(new AnyURIValue(""));
+        return new One<>(new AnyURIValue(""));
     }
 
-
-//#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the NameFn expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
-    @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new NamespaceUriFnCompiler();
+    public String getCompilerName() {
+        return "NamespaceUriFnCompiler";
     }
-//#endif
+
 }
 
