@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -40,7 +40,7 @@ public class NormalizeUnicode extends SystemFunction {
     }
 
     public static StringValue normalize(StringValue sv, String form, XPathContext c) throws XPathException {
-        byte fb = Normalizer.C;
+        byte fb;
 
         if (form.equalsIgnoreCase("NFC")) {
             fb = Normalizer.C;
