@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,11 +14,11 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.om.Item;
+import net.sf.saxon.om.One;
 import net.sf.saxon.om.ZeroOrOne;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.ConversionResult;
 import net.sf.saxon.type.Converter;
-import net.sf.saxon.type.ValidationException;
 import net.sf.saxon.type.ValidationFailure;
 import net.sf.saxon.value.*;
 
@@ -36,7 +36,7 @@ public class Number_1 extends ScalarSystemFunction {
 
     @Override
     public ZeroOrOne<NumericValue> resultWhenEmpty() {
-        return new ZeroOrOne<NumericValue>(DoubleValue.NaN);
+        return new One<>(DoubleValue.NaN);
     }
 
     /**
@@ -58,25 +58,21 @@ public class Number_1 extends ScalarSystemFunction {
      * @return the result of the conversion
      */
 
-    public static DoubleValue toNumber(AtomicValue arg0) throws XPathException {
-        try {
-            if (arg0 instanceof BooleanValue) {
-                return (DoubleValue) Converter.BOOLEAN_TO_DOUBLE.convert(arg0).asAtomic();
-            } else if (arg0 instanceof NumericValue) {
-                return (DoubleValue) Converter.NUMERIC_TO_DOUBLE.convert(arg0).asAtomic();
-            } else if (arg0 instanceof StringValue && !(arg0 instanceof AnyURIValue)) {
-                // Always use the XSD 1.1 rules, which permit "+INF"
-                ConversionResult cr = StringToDouble11.getInstance().convert(arg0);
-                if (cr instanceof ValidationFailure) {
-                    return DoubleValue.NaN;
-                } else {
-                    return (DoubleValue) cr;
-                }
-            } else {
+    public static DoubleValue toNumber(AtomicValue arg0) {
+        if (arg0 instanceof BooleanValue) {
+            return Converter.BooleanToDouble.INSTANCE.convert((BooleanValue)arg0);
+        } else if (arg0 instanceof NumericValue) {
+            return (DoubleValue) Converter.NumericToDouble.INSTANCE.convert((NumericValue)arg0).asAtomic();
+        } else if (arg0 instanceof StringValue && !(arg0 instanceof AnyURIValue)) {
+            // Always use the XSD 1.1 rules, which permit "+INF"
+            ConversionResult cr = StringToDouble11.getInstance().convert((StringValue)arg0);
+            if (cr instanceof ValidationFailure) {
                 return DoubleValue.NaN;
+            } else {
+                return (DoubleValue) cr;
             }
-        } catch (ValidationException e) {
-            return DoubleValue.NaN;  // should not happen
+        } else {
+            return DoubleValue.NaN;
         }
     }
 
