@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -32,7 +32,7 @@ public class ResolveQName extends SystemFunction {
      */
     public ZeroOrOne<QNameValue> call(XPathContext context, Sequence[] arguments) throws XPathException {
         AtomicValue lex = (AtomicValue) arguments[0].head();
-        return new ZeroOrOne<QNameValue>(
+        return new ZeroOrOne<>(
                 lex == null ? null :
                         resolveQName(lex.getStringValueCS(), (NodeInfo) arguments[1].head()));
     }
