@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -32,7 +32,7 @@ public class Root_1 extends SystemFunction {
     public int getSpecialProperties(Expression[] arguments) {
         int prop = StaticProperty.ORDERED_NODESET |
                 StaticProperty.SINGLE_DOCUMENT_NODESET |
-                StaticProperty.NON_CREATIVE;
+                StaticProperty.NO_NODES_NEWLY_CREATED;
         if ((getArity() == 0) ||
                 (arguments[0].getSpecialProperties() & StaticProperty.CONTEXT_DOCUMENT_NODESET) != 0) {
             prop |= StaticProperty.CONTEXT_DOCUMENT_NODESET;
@@ -54,7 +54,7 @@ public class Root_1 extends SystemFunction {
         if (node == null) {
             return ZeroOrOne.empty();
         } else {
-            return new ZeroOrOne<NodeInfo>(node.getRoot());
+            return new ZeroOrOne<>(node.getRoot());
         }
     }
 
