@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -60,7 +60,7 @@ public class StartsWith extends CollatingFunctionFixed {
     }
 
 
-    public static boolean startsWith(StringValue arg0, StringValue arg1, SubstringMatcher collator) throws XPathException {
+    public static boolean startsWith(StringValue arg0, StringValue arg1, SubstringMatcher collator) {
         if (arg1 == null || arg1.isZeroLength() || collator.comparesEqual(arg1.getPrimitiveStringValue(), "")) {
             return true;
         }
