@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,6 +13,7 @@ import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.om.One;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.ZeroOrOne;
+import net.sf.saxon.regex.EmptyString;
 import net.sf.saxon.regex.UnicodeString;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.BuiltInAtomicType;
@@ -73,7 +74,7 @@ public class Substring extends SystemFunction implements Callable {
             //noinspection RedundantCast
             lstart = ((Int64Value) start).longValue();
             if (lstart > slength) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             } else if (lstart <= 0) {
                 lstart = 1;
             }
@@ -81,20 +82,20 @@ public class Substring extends SystemFunction implements Callable {
             //NumericValue rstart = start.round();
             // We need to be careful to handle cases such as plus/minus infinity
             if (start.isNaN()) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             } else if (start.signum() <= 0) {
                 return s;
             } else if (start.compareTo(slength) > 0) {
                 // this works even where the string contains surrogate pairs,
                 // because the Java length is always >= the XPath length
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             } else {
                 lstart = Math.round(start.getDoubleValue());
             }
         }
 
         if (lstart > s.uLength()) {
-            return UnicodeString.EMPTY_STRING;
+            return EmptyString.THE_INSTANCE;
         }
         return s.uSubstring((int) lstart - 1, s.uLength());
     }
@@ -119,16 +120,16 @@ public class Substring extends SystemFunction implements Callable {
             //noinspection RedundantCast
             lstart = ((Int64Value) start).longValue();
             if (lstart > slength) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             }
         } else {
             // We need to be careful to handle cases such as plus/minus infinity and NaN
             if (start.isNaN()) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             } else if (start.compareTo(slength) > 0) {
                 // this works even where the string contains surrogate pairs,
                 // because the Java length is always >= the XPath length
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             } else {
                 double dstart = start.getDoubleValue();
                 lstart = Double.isInfinite(dstart) ? -Integer.MAX_VALUE : Math.round(dstart);
@@ -139,14 +140,14 @@ public class Substring extends SystemFunction implements Callable {
         if (len instanceof Int64Value) {
             llen = ((Int64Value) len).longValue();
             if (llen <= 0) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             }
         } else {
             if (len.isNaN()) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             }
             if (len.signum() <= 0) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             }
             double dlen = len.getDoubleValue();
             if (Double.isInfinite(dlen)) {
@@ -157,19 +158,19 @@ public class Substring extends SystemFunction implements Callable {
         }
         long lend = lstart + llen;
         if (lend < lstart) {
-            return UnicodeString.EMPTY_STRING;
+            return EmptyString.THE_INSTANCE;
         }
 
         UnicodeString us = sv.getUnicodeString();
         int clength = us.uLength();
         int a1 = (int) lstart - 1;
         if (a1 >= clength) {
-            return UnicodeString.EMPTY_STRING;
+            return EmptyString.THE_INSTANCE;
         }
         int a2 = Math.min(clength, (int) lend - 1);
         if (a1 < 0) {
             if (a2 < 0) {
-                return UnicodeString.EMPTY_STRING;
+                return EmptyString.THE_INSTANCE;
             } else {
                 a1 = 0;
             }
@@ -193,10 +194,10 @@ public class Substring extends SystemFunction implements Callable {
         }
         NumericValue arg1 = (NumericValue) arguments[1].head();
         if (arguments.length == 2) {
-            return new ZeroOrOne<StringValue>(StringValue.makeStringValue(substring(arg0, arg1)));
+            return new ZeroOrOne<>(StringValue.makeStringValue(substring(arg0, arg1)));
         } else {
             NumericValue arg2 = (NumericValue) arguments[2].head();
-            return new ZeroOrOne<StringValue>(StringValue.makeStringValue(substring(arg0, arg1, arg2)));
+            return new ZeroOrOne<>(StringValue.makeStringValue(substring(arg0, arg1, arg2)));
         }
     }
 
