@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -38,7 +38,7 @@ public class SubstringAfter extends CollatingFunctionFixed {
         return substringAfter(arg1, arg2, (SubstringMatcher)getStringCollator());
     }
 
-    private static StringValue substringAfter(StringValue arg1, StringValue arg2, SubstringMatcher collator) throws XPathException {
+    private static StringValue substringAfter(StringValue arg1, StringValue arg2, SubstringMatcher collator) {
         if (arg1 == null) {
             arg1 = StringValue.EMPTY_STRING;
         }
