@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -25,8 +25,7 @@ public class SubstringBefore extends CollatingFunctionFixed {
         return true;
     }
 
-    private static StringValue substringBefore(StringValue arg0, StringValue arg1, SubstringMatcher collator)
-            throws XPathException {
+    private static StringValue substringBefore(StringValue arg0, StringValue arg1, SubstringMatcher collator) {
         String s0 = arg0.getStringValue();
         String s1 = arg1.getStringValue();
         StringValue result = new StringValue(collator.substringBefore(s0, s1));
