@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,7 @@ package net.sf.saxon.functions;
 
 import net.sf.saxon.Controller;
 import net.sf.saxon.expr.XPathContext;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.ZeroOrOne;
 import net.sf.saxon.trans.XPathException;
@@ -40,7 +40,14 @@ public class UnparsedText extends UnparsedTextFunction {
     public ZeroOrOne<StringValue> call(XPathContext context, Sequence[] arguments) throws XPathException {
         StringValue hrefVal = (StringValue) arguments[0].head();
         String encoding = getArity() == 2 ? arguments[1].head().getStringValue() : null;
-        return new ZeroOrOne<StringValue>(evalUnparsedText(hrefVal, getStaticBaseUriString(), encoding, context));
+        try {
+            return new ZeroOrOne<>(evalUnparsedText(hrefVal, getStaticBaseUriString(), encoding, context));
+        } catch (XPathException e) {
+            if (getArity() == 2 && e.getErrorCodeLocalPart().equals("FOUT1200")) {
+                e.setErrorCode("FOUT1190");
+            }
+            throw e;
+        }
     }
 
     private static final String errorValue = "\u0000";
@@ -48,7 +55,7 @@ public class UnparsedText extends UnparsedTextFunction {
     public static StringValue evalUnparsedText(StringValue hrefVal, String base, String encoding, XPathContext context) throws XPathException {
         CharSequence content;
         StringValue result;
-        boolean stable = context.getConfiguration().getBooleanProperty(FeatureKeys.STABLE_UNPARSED_TEXT);
+        boolean stable = context.getConfiguration().getBooleanProperty(Feature.STABLE_UNPARSED_TEXT);
         try {
             if (hrefVal == null) {
                 return null;
@@ -64,7 +71,7 @@ public class UnparsedText extends UnparsedTextFunction {
                         String existing = cache.get(absoluteURI);
                         if (existing != null) {
                             if (existing.startsWith(errorValue)) {
-                                throw new XPathException(existing.substring(1), "FOUT" + "1170");
+                                throw new XPathException(existing.substring(1), "FOUT1170");
                             }
                             return new StringValue(existing);
                         }
@@ -77,7 +84,7 @@ public class UnparsedText extends UnparsedTextFunction {
                         content = errorValue + e.getMessage();
                     }
                     if (cache == null) {
-                        cache = new HashMap<URI, String>();
+                        cache = new HashMap<>();
                         controller.setUserData("unparsed-text-cache", "", cache);
                         cache.put(absoluteURI, content.toString());
                     }
@@ -90,7 +97,7 @@ public class UnparsedText extends UnparsedTextFunction {
             }
             result = new StringValue(content);
         } catch (XPathException err) {
-            err.maybeSetErrorCode("FOUT" + "1170");
+            err.maybeSetErrorCode("FOUT1170");
             throw err;
         }
         return result;
@@ -106,15 +113,15 @@ public class UnparsedText extends UnparsedTextFunction {
         while (true) {
             int b = is.read();
             if (b < 0) {
-                System.out.println(sb1.toString());
-                System.out.println(sb2.toString());
+                System.out.println(sb1);
+                System.out.println(sb2);
                 break;
             }
             sb1.append(Integer.toHexString(b) + " ");
             sb2.append((char) b + " ");
             if (sb1.length() > 80) {
-                System.out.println(sb1.toString());
-                System.out.println(sb2.toString());
+                System.out.println(sb1);
+                System.out.println(sb2);
                 sb1 = new FastStringBuffer(FastStringBuffer.C256);
                 sb2 = new FastStringBuffer(FastStringBuffer.C256);
             }
