@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -223,19 +223,6 @@ public class XPath20FunctionSet extends BuiltInFunctionSet {
                 .arg(1, BuiltInAtomicType.STRING, ONE, null)
                 .arg(2, BuiltInAtomicType.STRING, OPT, null);
 
-        register("format-time", 2, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, CARD0)
-                .arg(0, BuiltInAtomicType.TIME, OPT, null)
-                .arg(1, BuiltInAtomicType.STRING, ONE, null);
-
-        register("format-time", 5, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, CARD0)
-                .arg(0, BuiltInAtomicType.TIME, OPT, null)
-                .arg(1, BuiltInAtomicType.STRING, ONE, null)
-                .arg(2, BuiltInAtomicType.STRING, OPT, null)
-                .arg(3, BuiltInAtomicType.STRING, OPT, null)
-                .arg(4, BuiltInAtomicType.STRING, OPT, null);    // TODO: why separated from format-date/dateTime?
-
         register("hours-from-dateTime", 1, AccessorFn.HoursFromDateTime.class,
                  BuiltInAtomicType.INTEGER, OPT, CORE, CARD0)
                 .arg(0, BuiltInAtomicType.DATE_TIME, OPT, EMPTY);
