@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,6 +14,7 @@ import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.BuiltInAtomicType;
 import net.sf.saxon.type.NumericType;
 import net.sf.saxon.type.Type;
+import net.sf.saxon.value.BooleanValue;
 import net.sf.saxon.value.StringValue;
 
 /**
@@ -68,12 +69,12 @@ public class XPath30FunctionSet extends BuiltInFunctionSet {
                 .arg(0, BuiltInAtomicType.STRING, ONE, null);
 
         register("format-date", 2, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, 0)
+                 OPT, XSLT | XPATH30, CARD0)
                 .arg(0, BuiltInAtomicType.DATE, OPT, null)
                 .arg(1, BuiltInAtomicType.STRING, ONE, null);
 
         register("format-date", 5, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, 0)
+                 OPT, XSLT | XPATH30, CARD0)
                 .arg(0, BuiltInAtomicType.DATE, OPT, null)
                 .arg(1, BuiltInAtomicType.STRING, ONE, null)
                 .arg(2, BuiltInAtomicType.STRING, OPT, null)
@@ -81,12 +82,12 @@ public class XPath30FunctionSet extends BuiltInFunctionSet {
                 .arg(4, BuiltInAtomicType.STRING, OPT, null);
 
         register("format-dateTime", 2, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, 0)
+                 OPT, XSLT | XPATH30, CARD0)
                 .arg(0, BuiltInAtomicType.DATE_TIME, OPT, null)
                 .arg(1, BuiltInAtomicType.STRING, ONE, null);
 
         register("format-dateTime", 5, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, 0)
+                 OPT, XSLT | XPATH30, CARD0)
                 .arg(0, BuiltInAtomicType.DATE_TIME, OPT, null)
                 .arg(1, BuiltInAtomicType.STRING, ONE, null)
                 .arg(2, BuiltInAtomicType.STRING, OPT, null)
@@ -112,12 +113,12 @@ public class XPath30FunctionSet extends BuiltInFunctionSet {
                 .arg(2, BuiltInAtomicType.STRING, OPT, null);
 
         register("format-time", 2, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, 0)
+                 OPT, XSLT | XPATH30, CARD0)
                 .arg(0, BuiltInAtomicType.TIME, OPT, null)
                 .arg(1, BuiltInAtomicType.STRING, ONE, null);
 
         register("format-time", 5, FormatDate.class, BuiltInAtomicType.STRING,
-                 OPT, XSLT | XPATH30, 0)
+                 OPT, XSLT | XPATH30, CARD0)
                 .arg(0, BuiltInAtomicType.TIME, OPT, null)
                 .arg(1, BuiltInAtomicType.STRING, ONE, null)
                 .arg(2, BuiltInAtomicType.STRING, OPT, null)
@@ -151,10 +152,10 @@ public class XPath30FunctionSet extends BuiltInFunctionSet {
         register("outermost", 1, Outermost.class, AnyNodeTest.getInstance(), STAR, XPATH30, AS_ARG0 | FILTER)
                 .arg(0, AnyNodeTest.getInstance(), STAR | TRA, null);
 
-        register("parse-xml", 1, ParseXml.class, NodeKindTest.DOCUMENT, OPT, XPATH30, LATE | NEW)
+        register("parse-xml", 1, ParseXml.class, NodeKindTest.DOCUMENT, OPT, XPATH30, LATE)
                 .arg(0, BuiltInAtomicType.STRING, OPT, EMPTY);
 
-        register("parse-xml-fragment", 1, ParseXmlFragment.class, NodeKindTest.DOCUMENT, OPT, XPATH30, LATE | NEW)
+        register("parse-xml-fragment", 1, ParseXmlFragment.class, NodeKindTest.DOCUMENT, OPT, XPATH30, LATE)
                 .arg(0, BuiltInAtomicType.STRING, OPT, EMPTY);
 
         register("path", 0, ContextItemAccessorFunction.class, BuiltInAtomicType.STRING, OPT, XPATH30, CITEM | LATE);
@@ -197,11 +198,11 @@ public class XPath30FunctionSet extends BuiltInFunctionSet {
 
         register("unparsed-text-available", 1, UnparsedTextAvailable.class,
                  BuiltInAtomicType.BOOLEAN, ONE, XSLT | XPATH30, BASE | LATE)
-                .arg(0, BuiltInAtomicType.STRING, ONE, null);
+                .arg(0, BuiltInAtomicType.STRING, OPT, BooleanValue.FALSE);
 
         register("unparsed-text-available", 2, UnparsedTextAvailable.class,
                  BuiltInAtomicType.BOOLEAN, ONE, XSLT | XPATH30, BASE | LATE)
-                .arg(0, BuiltInAtomicType.STRING, ONE, null)
+                .arg(0, BuiltInAtomicType.STRING, OPT, BooleanValue.FALSE)
                 .arg(1, BuiltInAtomicType.STRING, ONE, null);
 
         register("unparsed-text-lines", 1, UnparsedTextLines.class, BuiltInAtomicType.STRING, STAR, XPATH30, BASE | LATE)
