@@ -197,6 +197,32 @@ public class JavaPlatform implements Platform {
         return parser;
     }
 
+    /**
+     * Get a parser suitable for parsing XML fragments
+     *
+     * <p>For background, see bugs 4127 and 4253. The problem is that to implement parse-xml-fragment(),
+     * we need to set an EntityResolver on the returned parser. But if an Apache catalog resolver is
+     * in use, the JAXP system properties may be set so that JAXP returns a custom XMLReader supplied
+     * by the catalog resolver, and that XMLReader ignores any attempt to set an EntityResolver. So
+     * we bypass JAXP and try to load the built-in parser within the JDK, which we know we can trust;
+     * only if this fails (presumably because this is not the Oracle JDK) do we fall back to using a
+     * JAXP-supplied parser. And if this turns out to ignore {@code setEntityResolver()} calls, we're
+     * hosed.
+     * </p>
+     *
+     * @return the parser (XMLReader)
+     */
+    public XMLReader loadParserForXmlFragments() {
+        try {
+            Class<?> factoryClass = Class.forName("com.sun.org.apache.xerces.internal.jaxp.SAXParserFactoryImpl");
+            SAXParserFactory factory = (SAXParserFactory) factoryClass.newInstance();
+            return factory.newSAXParser().getXMLReader();
+        } catch (Exception e) {
+            // no action; try the JAXP loading mechanism
+        }
+        return loadParser();
+    }
+
     /**
      * Convert a StreamSource to either a SAXSource or a PullSource, depending on the native
      * parser of the selected platform
@@ -394,7 +420,7 @@ public class JavaPlatform implements Platform {
     }
 
 
-//#if EE==true
+//#if BYTECODE==true
     /**
      * Return the class loader required to load the bytecode generated classes
      * @param config           The saxon configuration
