@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.jaxp;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.lib.DelegatingErrorListener;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.FeatureKeys;
 import net.sf.saxon.lib.UnfailingErrorListener;
 import net.sf.saxon.s9api.Processor;
@@ -33,22 +34,21 @@ import java.util.Comparator;
 /**
  * A SaxonTransformerFactory instance can be used to create Transformer and Template
  * objects.
- * <p/>
+ * <p>Although the class is public, it is not intended to be used directly by applications;
+ * applications should normally invoke the subclass {@link net.sf.saxon.TransformerFactoryImpl}.</p>
  * <p>The system property that determines which Factory implementation
  * to create is named "javax.xml.transform.TransformerFactory". This
  * property names a concrete subclass of the TransformerFactory abstract
  * class. If the property is not defined, a platform default is be used.</p>
- * <p/>
  * <p>This implementation class implements the abstract methods on both the
  * javax.xml.transform.TransformerFactory and javax.xml.transform.sax.SAXTransformerFactory
  * classes.</p>
- * <p/>
  * <p>Since Saxon 9.6, the JAXP transformation interface is re-implemented as a layer
  * on top of the s9api interface. This will affect applications that attempt to
  * down-cast from JAXP interfaces to the underlying implementation classes.</p>
  */
 
-public class SaxonTransformerFactory extends SAXTransformerFactory {
+public class SaxonTransformerFactory extends SAXTransformerFactory implements Configuration.ApiProvider {
 
     private Processor processor;
 
@@ -76,7 +76,7 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      */
 
     public void setConfiguration(Configuration config) {
-        processor.setConfigurationProperty(FeatureKeys.CONFIGURATION, config);
+        processor.setConfigurationProperty(Feature.CONFIGURATION, config);
     }
 
     /**
@@ -118,15 +118,9 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      *
      * @return A Transformer object that may be used to perform a transformation
      *         in a single thread, never null.
-     * @throws javax.xml.transform.TransformerConfigurationException
-     *          May throw this during
-     *          the parse when it is constructing the
-     *          Templates object and fails.
      */
 
-    public Transformer newTransformer()
-            throws TransformerConfigurationException {
-
+    public Transformer newTransformer() {
         return new IdentityTransformer(processor.getUnderlyingConfiguration());
     }
 
@@ -147,7 +141,7 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      *          is constructing the Templates object and fails.
      */
 
-    public Templates newTemplates(Source source)
+    public synchronized Templates newTemplates(Source source)
             throws TransformerConfigurationException {
 
         try {
@@ -176,7 +170,7 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      *          is constructing the Templates object and fails.
      */
 
-    public Templates newTemplates(Source source, CompilerInfo info)
+    public synchronized Templates newTemplates(Source source, CompilerInfo info)
             throws TransformerConfigurationException {
 
         try {
@@ -254,7 +248,6 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
 
     /**
      * Look up the value of a feature.
-     * <p/>
      * <p>The feature name is any absolute URI.</p>
      *
      * @param name The feature name, which is an absolute URI.
@@ -262,22 +255,25 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      */
 
     public boolean getFeature(String name) {
-        if (name.equals(SAXSource.FEATURE)) return true;
-        if (name.equals(SAXResult.FEATURE)) return true;
-        if (name.equals(DOMSource.FEATURE)) return true;
-        if (name.equals(DOMResult.FEATURE)) return true;
-        if (name.equals(StreamSource.FEATURE)) return true;
-        if (name.equals(StreamResult.FEATURE)) return true;
-        if (name.equals(SAXTransformerFactory.FEATURE)) return true;
-        if (name.equals(SAXTransformerFactory.FEATURE_XMLFILTER)) return true;
-        if (name.equals(FEATURE_SECURE_PROCESSING)) {
-            return !getConfiguration().getBooleanProperty(FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS);
-        }
-        try {
-            Object val = getConfiguration().getConfigurationProperty(name);
-            return (val instanceof Boolean && (Boolean) val);
-        } catch (IllegalArgumentException err) {
-            return false;
+        switch (name) {
+            case SAXSource.FEATURE:
+            case SAXResult.FEATURE:
+            case DOMSource.FEATURE:
+            case DOMResult.FEATURE:
+            case StreamSource.FEATURE:
+            case StreamResult.FEATURE:
+            case SAXTransformerFactory.FEATURE:
+            case SAXTransformerFactory.FEATURE_XMLFILTER:
+                return true;
+            case FEATURE_SECURE_PROCESSING:
+                return !getConfiguration().getBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS);
+            default:
+                try {
+                    Object val = getConfiguration().getConfigurationProperty(name);
+                    return val instanceof Boolean && (Boolean) val;
+                } catch (IllegalArgumentException err) {
+                    return false;
+                }
         }
     }
 
@@ -286,6 +282,10 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      * implementation.  An attribute in this context is defined to
      * be an option that the implementation provides.
      *
+     * <p>Note: setting configuration properties using the method
+     * {@link Configuration#setConfigurationProperty(Feature, Object)}
+     * is more efficient, and gives better type safety.</p>
+     *
      * @param name  The name of the attribute. This must be one of the constants
      *              defined in class {@link net.sf.saxon.lib.FeatureKeys}.
      * @param value The value of the attribute.
@@ -295,15 +295,21 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      */
 
     public void setAttribute(String name, Object value) throws IllegalArgumentException {
-        if (name.equals(FeatureKeys.CONFIGURATION_FILE)) {
-            ConfigurationReader reader = new ConfigurationReader();
-            try {
-                setConfiguration(reader.makeConfiguration(new StreamSource(new File((String) (value)))));
-            } catch (XPathException err) {
-                throw new IllegalArgumentException(err);
-            }
-        } else {
-            processor.setConfigurationProperty(name, value);
+        switch (name) {
+            case FeatureKeys.CONFIGURATION:
+                setConfiguration((Configuration) value);
+                break;
+            case FeatureKeys.CONFIGURATION_FILE:
+                ConfigurationReader reader = new ConfigurationReader();
+                try {
+                    setConfiguration(reader.makeConfiguration(new StreamSource(new File((String) value))));
+                } catch (XPathException err) {
+                    throw new IllegalArgumentException(err);
+                }
+                break;
+            default:
+                processor.getUnderlyingConfiguration().setConfigurationProperty(name, value);
+                break;
         }
     }
 
@@ -326,7 +332,6 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      * Set the error event listener for the TransformerFactory, which
      * is used for the processing of transformation instructions,
      * and not for the transformation itself.
-     * <p/>
      * <p>This method is defined in JAXP but its use with Saxon is deprecated,
      * because the errorListener will be shared by all stylesheet compilations
      * running under this factory, which may be operating concurrently
@@ -406,13 +411,9 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      *
      * @return A non-null reference to a TransformerHandler, that may
      *         be used as a ContentHandler for SAX parse events.
-     * @throws javax.xml.transform.TransformerConfigurationException
-     *          If for some reason the
-     *          TransformerHandler cannot be created.
      */
 
-    public TransformerHandler newTransformerHandler()
-            throws TransformerConfigurationException {
+    public TransformerHandler newTransformerHandler() {
         IdentityTransformer transformer = new IdentityTransformer(getConfiguration());
         return new IdentityTransformerHandler(transformer);
     }
@@ -423,13 +424,9 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      *
      * @return A non-null reference to a TransformerHandler, that may
      *         be used as a ContentHandler for SAX parse events.
-     * @throws javax.xml.transform.TransformerConfigurationException
-     *          If for some reason the
-     *          TemplatesHandler cannot be created.
      */
 
-    public TemplatesHandler newTemplatesHandler()
-            throws TransformerConfigurationException {
+    public TemplatesHandler newTemplatesHandler() {
         return new TemplatesHandlerImpl(processor);
     }
 
@@ -454,10 +451,9 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
      * Create an XMLFilter, based on the Templates argument..
      *
      * @param templates The compiled transformation instructions.
-     * @return An XMLFilter object, or null if this feature is not supported.
+     * @return An XMLFilter object.
      * @throws javax.xml.transform.TransformerConfigurationException
-     *          If for some reason the
-     *          XMLFilter cannot be created.
+     *          if (for example) the <code>templates</code> object was not created by Saxon.
      */
 
     public XMLFilter newXMLFilter(Templates templates)
@@ -472,15 +468,11 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
     /**
      * <p>Set a feature for this <code>TransformerFactory</code> and <code>Transformer</code>s
      * or <code>Template</code>s created by this factory.</p>
-     * <p/>
-     * <p/>
-     * Feature names are fully qualified {@link java.net.URI}s.
+     * <p>Feature names are fully qualified {@link java.net.URI}s.
      * Implementations may define their own features.
      * An {@link javax.xml.transform.TransformerConfigurationException} is thrown if this <code>TransformerFactory</code> or the
      * <code>Transformer</code>s or <code>Template</code>s it creates cannot support the feature.
-     * It is possible for an <code>TransformerFactory</code> to expose a feature value but be unable to change its state.
-     * </p>
-     * <p/>
+     * It is possible for an <code>TransformerFactory</code> to expose a feature value but be unable to change its state.</p>
      * <p>All implementations are required to support the FEATURE_SECURE_PROCESSING feature.
      * When the feature is:</p>
      * <ul>
@@ -510,7 +502,7 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
 
     public void setFeature(String name, boolean value) throws TransformerConfigurationException {
         if (name.equals(FEATURE_SECURE_PROCESSING)) {
-            getConfiguration().setBooleanProperty(FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS, !value);
+            getConfiguration().setBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS, !value);
         } else {
             try {
                 getConfiguration().setBooleanProperty(name, value);
@@ -523,6 +515,8 @@ public class SaxonTransformerFactory extends SAXTransformerFactory {
     /**
      * Get the underlying s9api Processor. (Trapdoor method providing access to underlying
      * implementation details which may change in subsequent releases)
+     *
+     * @return the processor
      * @since 9.8.0.5
      */
 
