@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,12 +11,10 @@ import net.sf.saxon.s9api.XsltExecutable;
 
 import javax.xml.transform.Templates;
 import javax.xml.transform.Transformer;
-import javax.xml.transform.TransformerConfigurationException;
 import java.util.Properties;
 
 /**
  * Saxon implementation of the JAXP Templates interface.
- * <p/>
  * <p>Since Saxon 9.6, JAXP interfaces are implemented as a layer above the s9api interface</p>
  */
 public class TemplatesImpl implements Templates {
@@ -32,10 +30,8 @@ public class TemplatesImpl implements Templates {
      * Create a new transformation context for this Templates object.
      *
      * @return A valid non-null instance of a Transformer.
-     * @throws javax.xml.transform.TransformerConfigurationException
-     *          if a Transformer can not be created.
      */
-    public Transformer newTransformer() throws TransformerConfigurationException {
+    public Transformer newTransformer() {
         if (forceStreaming) {
             return new StreamingTransformerImpl(executable, executable.load30());
         } else {
@@ -49,7 +45,6 @@ public class TemplatesImpl implements Templates {
      * be a clone of the internal values. Accordingly, it can be mutated
      * without mutating the Templates object, and then handed in to
      * {@link javax.xml.transform.Transformer#setOutputProperties}.
-     * <p/>
      * <p>The properties returned should contain properties set by the stylesheet,
      * and these properties are "defaulted" by default properties specified by
      * <a href="http://www.w3.org/TR/xslt#output">section 16 of the
@@ -62,7 +57,6 @@ public class TemplatesImpl implements Templates {
      * properties, while
      * getOutputProperties().get(String key) will only retrieve properties
      * that were explicitly set in the stylesheet.</p>
-     * <p/>
      * <p>For XSLT,
      * <a href="http://www.w3.org/TR/xslt#attribute-value-templates">Attribute
      * Value Templates</a> attribute values will
@@ -73,7 +67,7 @@ public class TemplatesImpl implements Templates {
      * @return A Properties object, never null.
      */
     public Properties getOutputProperties() {
-        Properties details = executable.getUnderlyingCompiledStylesheet().getDefaultOutputProperties();
+        Properties details = executable.getUnderlyingCompiledStylesheet().getPrimarySerializationProperties().getProperties();
         return new Properties(details);
     }
 
@@ -105,4 +99,13 @@ public class TemplatesImpl implements Templates {
     public void setForceStreaming(boolean forceStreaming) {
         this.forceStreaming = forceStreaming;
     }
+
+    /**
+     * Get the underlying implementation object
+     * @return the underlying S9API {@link XsltExecutable} object
+     */
+
+    public XsltExecutable getImplementation() {
+        return executable;
+    }
 }
