@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,11 +7,11 @@
 
 package net.sf.saxon.jaxp;
 
-import net.sf.saxon.Controller;
 import net.sf.saxon.event.Receiver;
 import net.sf.saxon.lib.SerializerFactory;
 import net.sf.saxon.s9api.*;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.trans.XsltController;
 import org.xml.sax.XMLFilter;
 
 import javax.xml.transform.ErrorListener;
@@ -22,7 +22,6 @@ import javax.xml.transform.sax.TransformerHandler;
 
 /**
  * Saxon implementation of the JAXP Transformer interface.
- * <p/>
  * <p>Since Saxon 9.6, JAXP interfaces are implemented as a layer above the s9api interface</p>
  */
 public class TransformerImpl extends AbstractTransformerImpl {
@@ -40,7 +39,6 @@ public class TransformerImpl extends AbstractTransformerImpl {
      * <code>TransformerFactory</code> in effect when the
      * <code>Transformer</code> was instantiated and any modifications made to
      * the <code>Transformer</code> instance.</p>
-     * <p/>
      * <p>An empty <code>Source</code> is represented as an empty document
      * as constructed by {@link javax.xml.parsers.DocumentBuilder#newDocument()}.
      * The result of transforming an empty <code>Source</code> depends on
@@ -96,7 +94,6 @@ public class TransformerImpl extends AbstractTransformerImpl {
     /**
      * Set an object that will be used to resolve URIs used in
      * document().
-     * <p/>
      * <p>If the resolver argument is null, the URIResolver value will
      * be cleared and the transformer will no longer have a resolver.</p>
      *
@@ -125,19 +122,15 @@ public class TransformerImpl extends AbstractTransformerImpl {
     /**
      * Supply an initial template for a transformation.
      * <p>This is a Saxon extension to the JAXP interface, needed for XSLT 2.0</p>
+     * <p>Changed in 9.9 so it no longer validates the supplied name; an incorrect name will
+     * lead to an error later.</p>
      *
      * @param name the name of the initial template, in Clark notation (either a local name,
      *             or "{uri}local")
-     * @throws IllegalArgumentException if the argument is invalid, for example if the
-     *                                  format of the name is incorrect or if there is no template with this name
      */
 
-    public void setInitialTemplate(String name) throws IllegalArgumentException {
-        try {
-            xsltTransformer.setInitialTemplate(QName.fromClarkName(name));
-        } catch (SaxonApiException e) {
-            throw new IllegalArgumentException(e);
-        }
+    public void setInitialTemplate(String name) {
+        xsltTransformer.setInitialTemplate(QName.fromClarkName(name));
     }
 
     /**
@@ -169,7 +162,7 @@ public class TransformerImpl extends AbstractTransformerImpl {
      * @return the underlying Saxon Controller instance
      */
 
-    public Controller getUnderlyingController() {
+    public XsltController getUnderlyingController() {
         return xsltTransformer.getUnderlyingController();
     }
 
@@ -197,5 +190,6 @@ public class TransformerImpl extends AbstractTransformerImpl {
         super.reset();
         getUnderlyingController().reset();
     }
+
 }
 
