@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,12 +11,12 @@ import net.sf.saxon.expr.sort.LRUCache;
 import net.sf.saxon.om.NotationSet;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.type.*;
+import net.sf.saxon.value.StringToDouble11;
 
 /**
  * This class defines a set of rules for converting between different atomic types. It handles the variations
  * that arise between different versions of the W3C specifications, for example the changes in Name syntax
  * between XML 1.0 and XML 1.1, the introduction of "+INF" as a permitted xs:double value in XSD 1.1, and so on.
- * <p/>
  * <p>It is possible to nominate a customized <code>ConversionRules</code> object at the level of the
  * {@link net.sf.saxon.Configuration}, either by instantiating this class and changing the properties, or
  * by subclassing.</p>
@@ -27,17 +27,21 @@ import net.sf.saxon.type.*;
 
 public class ConversionRules {
 
-    private StringToDouble stringToDouble = StringToDouble.getInstance();
+    private StringToDouble stringToDouble = StringToDouble11.getInstance();
     private NotationSet notationSet; // may be null
     private URIChecker uriChecker;
-    private boolean allowYearZero;
+    private boolean allowYearZero = true;
     private TypeHierarchy typeHierarchy; // may be null
 
     // These two tables need to be synchronised to make the caching thread-safe
     private LRUCache<Integer, Converter> converterCache =
-            new LRUCache<Integer, Converter>(100, true);
-
+            new LRUCache<>(100, true);
 
+    /**
+     * Default conversion rules. Changed in Saxon 9.9 so these are the XSD 1.1 rules (year zero allowed in dates,
+     * {@code -INF} allowed in {@code xs:double}). Modifying the default conversion rules is inadvisable,
+     * but it could potentially be done in order to retain compatibility with earlier Saxon releases.
+     */
     public final static ConversionRules DEFAULT = new ConversionRules();
 
 
@@ -192,13 +196,13 @@ public class ConversionRules {
     public Converter getConverter(AtomicType source, AtomicType target) {
         // Handle some common cases before looking in the cache
         if (source == target) {
-            return Converter.IDENTITY_CONVERTER;
+            return StringConverter.IdentityConverter.INSTANCE;
         } else if (source == BuiltInAtomicType.STRING || source == BuiltInAtomicType.UNTYPED_ATOMIC) {
             return target.getStringConverter(this);
         } else if (target == BuiltInAtomicType.STRING) {
-            return Converter.TO_STRING;
+            return Converter.ToStringConverter.INSTANCE;
         } else if (target == BuiltInAtomicType.UNTYPED_ATOMIC) {
-            return Converter.TO_UNTYPED_ATOMIC;
+            return Converter.ToUntypedAtomicConverter.INSTANCE;
         }
         // For a lookup key, use the primitive type of the source type (always 10 bits) and the
         // fingerprint of the target type (20 bits)
@@ -217,7 +221,6 @@ public class ConversionRules {
 
     /**
      * Create a converter that handles conversion from one primitive type to another.
-     * <p/>
      * <p>This method is intended for internal use only. The approved way to get a converter is using the
      * factory method {@link net.sf.saxon.lib.ConversionRules#getConverter(net.sf.saxon.type.AtomicType, net.sf.saxon.type.AtomicType)}}</p>
      *
@@ -227,9 +230,9 @@ public class ConversionRules {
      */
 
     /*@Nullable*/
-    private Converter makeConverter(/*@NotNull*/ AtomicType sourceType, /*@NotNull*/ AtomicType targetType) {
+    private Converter makeConverter(AtomicType sourceType, AtomicType targetType) {
         if (sourceType == targetType) {
-            return StringConverter.IdentityConverter.THE_INSTANCE;
+            return StringConverter.IdentityConverter.INSTANCE;
         }
 
         int tt = targetType.getFingerprint();
@@ -270,9 +273,9 @@ public class ConversionRules {
 
         switch (tt) {
             case StandardNames.XS_UNTYPED_ATOMIC:
-                return Converter.TO_UNTYPED_ATOMIC;
+                return Converter.ToUntypedAtomicConverter.INSTANCE;
             case StandardNames.XS_STRING:
-                return Converter.TO_STRING;
+                return Converter.ToStringConverter.INSTANCE;
             case StandardNames.XS_FLOAT:
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
@@ -282,9 +285,9 @@ public class ConversionRules {
                     case StandardNames.XS_DECIMAL:
                     case StandardNames.XS_INTEGER:
                     case StandardNames.XS_NUMERIC:
-                        return Converter.NUMERIC_TO_FLOAT;
+                        return Converter.NumericToFloat.INSTANCE;
                     case StandardNames.XS_BOOLEAN:
-                        return Converter.BOOLEAN_TO_FLOAT;
+                        return Converter.BooleanToFloat.INSTANCE;
                     default:
                         return null;
                 }
@@ -298,9 +301,9 @@ public class ConversionRules {
                     case StandardNames.XS_DECIMAL:
                     case StandardNames.XS_INTEGER:
                     case StandardNames.XS_NUMERIC:
-                        return Converter.NUMERIC_TO_DOUBLE;
+                        return Converter.NumericToDouble.INSTANCE;
                     case StandardNames.XS_BOOLEAN:
-                        return Converter.BOOLEAN_TO_DOUBLE;
+                        return Converter.BooleanToDouble.INSTANCE;
                     default:
                         return null;
                 }
@@ -308,17 +311,17 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_DECIMAL;
+                        return StringConverter.StringToDecimal.INSTANCE;
                     case StandardNames.XS_FLOAT:
-                        return Converter.FLOAT_TO_DECIMAL;
+                        return Converter.FloatToDecimal.INSTANCE;
                     case StandardNames.XS_DOUBLE:
-                        return Converter.DOUBLE_TO_DECIMAL;
+                        return Converter.DoubleToDecimal.INSTANCE;
                     case StandardNames.XS_INTEGER:
-                        return Converter.INTEGER_TO_DECIMAL;
+                        return Converter.IntegerToDecimal.INSTANCE;
                     case StandardNames.XS_NUMERIC:
-                        return Converter.NUMERIC_TO_DECIMAL;
+                        return Converter.NumericToDecimal.INSTANCE;
                     case StandardNames.XS_BOOLEAN:
-                        return Converter.BOOLEAN_TO_DECIMAL;
+                        return Converter.BooleanToDecimal.INSTANCE;
                     default:
                         return null;
                 }
@@ -327,17 +330,17 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_INTEGER;
+                        return StringConverter.StringToInteger.INSTANCE;
                     case StandardNames.XS_FLOAT:
-                        return Converter.FLOAT_TO_INTEGER;
+                        return Converter.FloatToInteger.INSTANCE;
                     case StandardNames.XS_DOUBLE:
-                        return Converter.DOUBLE_TO_INTEGER;
+                        return Converter.DoubleToInteger.INSTANCE;
                     case StandardNames.XS_DECIMAL:
-                        return Converter.DECIMAL_TO_INTEGER;
+                        return Converter.DecimalToInteger.INSTANCE;
                     case StandardNames.XS_NUMERIC:
-                        return Converter.NUMERIC_TO_INTEGER;
+                        return Converter.NumericToInteger.INSTANCE;
                     case StandardNames.XS_BOOLEAN:
-                        return Converter.BOOLEAN_TO_INTEGER;
+                        return Converter.BooleanToInteger.INSTANCE;
                     default:
                         return null;
                 }
@@ -345,7 +348,7 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_DURATION;
+                        return StringConverter.StringToDuration.INSTANCE;
                     case StandardNames.XS_DAY_TIME_DURATION:
                     case StandardNames.XS_YEAR_MONTH_DURATION:
                         return new Converter.UpCastingConverter(BuiltInAtomicType.DURATION);
@@ -356,10 +359,10 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_YEAR_MONTH_DURATION;
+                        return StringConverter.StringToYearMonthDuration.INSTANCE;
                     case StandardNames.XS_DURATION:
                     case StandardNames.XS_DAY_TIME_DURATION:
-                        return Converter.DURATION_TO_YEAR_MONTH_DURATION;
+                        return Converter.DurationToYearMonthDuration.INSTANCE;
                     default:
                         return null;
                 }
@@ -367,10 +370,10 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_DAY_TIME_DURATION;
+                        return StringConverter.StringToDayTimeDuration.INSTANCE;
                     case StandardNames.XS_DURATION:
                     case StandardNames.XS_YEAR_MONTH_DURATION:
-                        return Converter.DURATION_TO_DAY_TIME_DURATION;
+                        return Converter.DurationToDayTimeDuration.INSTANCE;
                     default:
                         return null;
                 }
@@ -380,7 +383,7 @@ public class ConversionRules {
                     case StandardNames.XS_STRING:
                         return new StringConverter.StringToDateTime(this);
                     case StandardNames.XS_DATE:
-                        return Converter.DATE_TO_DATE_TIME;
+                        return Converter.DateToDateTime.INSTANCE;
                     default:
                         return null;
                 }
@@ -388,9 +391,9 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_TIME;
+                        return StringConverter.StringToTime.INSTANCE;
                     case StandardNames.XS_DATE_TIME:
-                        return Converter.DATE_TIME_TO_TIME;
+                        return Converter.DateTimeToTime.INSTANCE;
                     default:
                         return null;
                 }
@@ -400,7 +403,7 @@ public class ConversionRules {
                     case StandardNames.XS_STRING:
                         return new StringConverter.StringToDate(this);
                     case StandardNames.XS_DATE_TIME:
-                        return Converter.DATE_TIME_TO_DATE;
+                        return Converter.DateTimeToDate.INSTANCE;
                     default:
                         return null;
                 }
@@ -413,7 +416,7 @@ public class ConversionRules {
                         return Converter.TwoPhaseConverter.makeTwoPhaseConverter(
                                 BuiltInAtomicType.DATE, BuiltInAtomicType.DATE_TIME, BuiltInAtomicType.G_YEAR_MONTH, this);
                     case StandardNames.XS_DATE_TIME:
-                        return Converter.DATE_TIME_TO_G_YEAR_MONTH;
+                        return Converter.DateTimeToGYearMonth.INSTANCE;
                     default:
                         return null;
                 }
@@ -426,7 +429,7 @@ public class ConversionRules {
                         return Converter.TwoPhaseConverter.makeTwoPhaseConverter(BuiltInAtomicType.DATE, BuiltInAtomicType.DATE_TIME,
                                 BuiltInAtomicType.G_YEAR, this);
                     case StandardNames.XS_DATE_TIME:
-                        return Converter.DATE_TIME_TO_G_YEAR;
+                        return Converter.DateTimeToGYear.INSTANCE;
                     default:
                         return null;
                 }
@@ -434,12 +437,12 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_G_MONTH_DAY;
+                        return StringConverter.StringToGMonthDay.INSTANCE;
                     case StandardNames.XS_DATE:
                         return Converter.TwoPhaseConverter.makeTwoPhaseConverter(BuiltInAtomicType.DATE, BuiltInAtomicType.DATE_TIME,
                                 BuiltInAtomicType.G_MONTH_DAY, this);
                     case StandardNames.XS_DATE_TIME:
-                        return Converter.DATE_TIME_TO_G_MONTH_DAY;
+                        return Converter.DateTimeToGMonthDay.INSTANCE;
                     default:
                         return null;
                 }
@@ -447,12 +450,12 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_G_DAY;
+                        return StringConverter.StringToGDay.INSTANCE;
                     case StandardNames.XS_DATE:
                         return Converter.TwoPhaseConverter.makeTwoPhaseConverter(BuiltInAtomicType.DATE, BuiltInAtomicType.DATE_TIME,
                                 BuiltInAtomicType.G_DAY, this);
                     case StandardNames.XS_DATE_TIME:
-                        return Converter.DATE_TIME_TO_G_DAY;
+                        return Converter.DateTimeToGDay.INSTANCE;
                     default:
                         return null;
                 }
@@ -460,12 +463,12 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_G_MONTH;
+                        return StringConverter.StringToGMonth.INSTANCE;
                     case StandardNames.XS_DATE:
                         return Converter.TwoPhaseConverter.makeTwoPhaseConverter(BuiltInAtomicType.DATE, BuiltInAtomicType.DATE_TIME,
                                 BuiltInAtomicType.G_MONTH, this);
                     case StandardNames.XS_DATE_TIME:
-                        return Converter.DATE_TIME_TO_G_MONTH;
+                        return Converter.DateTimeToGMonth.INSTANCE;
                     default:
                         return null;
                 }
@@ -473,13 +476,13 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_BOOLEAN;
+                        return StringConverter.StringToBoolean.INSTANCE;
                     case StandardNames.XS_FLOAT:
                     case StandardNames.XS_DOUBLE:
                     case StandardNames.XS_DECIMAL:
                     case StandardNames.XS_INTEGER:
                     case StandardNames.XS_NUMERIC:
-                        return Converter.NUMERIC_TO_BOOLEAN;
+                        return Converter.NumericToBoolean.INSTANCE;
                     default:
                         return null;
                 }
@@ -487,9 +490,9 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_BASE64_BINARY;
+                        return StringConverter.StringToBase64Binary.INSTANCE;
                     case StandardNames.XS_HEX_BINARY:
-                        return Converter.HEX_BINARY_TO_BASE64_BINARY;
+                        return Converter.HexBinaryToBase64Binary.INSTANCE;
                     default:
                         return null;
                 }
@@ -497,9 +500,9 @@ public class ConversionRules {
                 switch (st) {
                     case StandardNames.XS_UNTYPED_ATOMIC:
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_HEX_BINARY;
+                        return StringConverter.StringToHexBinary.INSTANCE;
                     case StandardNames.XS_BASE64_BINARY:
-                        return Converter.BASE64_BINARY_TO_HEX_BINARY;
+                        return Converter.Base64BinaryToHexBinary.INSTANCE;
                     default:
                         return null;
                 }
@@ -517,7 +520,7 @@ public class ConversionRules {
                     case StandardNames.XS_STRING:
                         return new StringConverter.StringToQName(this);
                     case StandardNames.XS_NOTATION:
-                        return Converter.NOTATION_TO_QNAME;
+                        return Converter.NotationToQName.INSTANCE;
                     default:
                         return null;
                 }
@@ -527,43 +530,19 @@ public class ConversionRules {
                     case StandardNames.XS_STRING:
                         return new StringConverter.StringToNotation(this);
                     case StandardNames.XS_QNAME:
-                        return Converter.QNAME_TO_NOTATION;
+                        return Converter.QNameToNotation.INSTANCE;
                     default:
                         return null;
                 }
 
             case StandardNames.XS_ANY_ATOMIC_TYPE:
-                return Converter.IDENTITY_CONVERTER;
+                return StringConverter.IdentityConverter.INSTANCE;
 
             default:
                 throw new IllegalArgumentException("Unknown primitive type " + tt);
         }
     }
 
-    /**
-     * Get a Converter that converts from strings to a given atomic type. These can be primitive types,
-     * derived types, or user-defined types. The converter implements the casting rules.
-     *
-     * @param target the target type
-     * @return a Converter if conversion between the two types is possible; or null otherwise
-     */
-
-//    public StringConverter getStringConverter(AtomicType target) {
-//        // Beware of infinite recursion. The method AtomicType.getStringConverter() should not call this method,
-//        // except by specifying a target type higher in the type heirarchy.
-//        StringConverter converter = target.getStringConverter(this);
-//        if (converter != null) {
-//            return converter;
-//        }
-//        int key = target.getFingerprint();
-//        converter = stringConverterCache.get(key);
-//        if (converter == null) {
-//            converter = makeStringConverter(target);
-//            stringConverterCache.put(key, converter);
-//        }
-//        return converter;
-//    }
-
     /**
      * Static factory method to get a StringConverter for a specific target type
      *
@@ -580,31 +559,31 @@ public class ConversionRules {
             if (tt == StandardNames.XS_STRING) {
                 switch (targetType.getFingerprint()) {
                     case StandardNames.XS_STRING:
-                        return StringConverter.STRING_TO_STRING;
+                        return StringConverter.StringToString.INSTANCE;
                     case StandardNames.XS_NORMALIZED_STRING:
-                        return StringConverter.STRING_TO_NORMALIZED_STRING;
+                        return StringConverter.StringToNormalizedString.INSTANCE;
                     case StandardNames.XS_TOKEN:
-                        return StringConverter.STRING_TO_TOKEN;
+                        return StringConverter.StringToToken.INSTANCE;
                     case StandardNames.XS_LANGUAGE:
-                        return StringConverter.STRING_TO_LANGUAGE;
+                        return StringConverter.StringToLanguage.INSTANCE;
                     case StandardNames.XS_NAME:
-                        return StringConverter.STRING_TO_NAME;
+                        return StringConverter.StringToName.INSTANCE;
                     case StandardNames.XS_NCNAME:
-                        return StringConverter.STRING_TO_NCNAME;
+                        return StringConverter.StringToNCName.TO_NCNAME;
                     case StandardNames.XS_ID:
-                        return StringConverter.STRING_TO_ID;
+                        return StringConverter.StringToNCName.TO_ID;
                     case StandardNames.XS_IDREF:
-                        return StringConverter.STRING_TO_IDREF;
+                        return StringConverter.StringToNCName.TO_IDREF;
                     case StandardNames.XS_ENTITY:
-                        return StringConverter.STRING_TO_ENTITY;
+                        return StringConverter.StringToNCName.TO_ENTITY;
                     case StandardNames.XS_NMTOKEN:
-                        return StringConverter.STRING_TO_NMTOKEN;
+                        return StringConverter.StringToNMTOKEN.INSTANCE;
                     default:
                         throw new AssertionError("Unknown built-in subtype of xs:string");
 
                 }
             } else if (tt == StandardNames.XS_UNTYPED_ATOMIC) {
-                return StringConverter.STRING_TO_UNTYPED_ATOMIC;
+                return StringConverter.StringToUntypedAtomic.INSTANCE;
             } else if (targetType.isPrimitiveType()) {
                 // converter to built-in types unrelated to xs:string
                 Converter converter = getConverter(BuiltInAtomicType.STRING, targetType);
@@ -615,15 +594,15 @@ public class ConversionRules {
             } else {
                 switch (targetType.getFingerprint()) {
                     case StandardNames.XS_DAY_TIME_DURATION:
-                        return StringConverter.STRING_TO_DAY_TIME_DURATION;
+                        return StringConverter.StringToDayTimeDuration.INSTANCE;
                     case StandardNames.XS_YEAR_MONTH_DURATION:
-                        return StringConverter.STRING_TO_YEAR_MONTH_DURATION;
+                        return StringConverter.StringToYearMonthDuration.INSTANCE;
                     case StandardNames.XS_DATE_TIME_STAMP:
                         StringConverter first = new StringConverter.StringToDateTime(this);
                         Converter.DownCastingConverter second = new Converter.DownCastingConverter(targetType, this);
                         return new StringConverter.StringToNonStringDerivedType(first, second);
                     default:
-                        throw new AssertionError("Unknown built in type " + targetType.toString());
+                        throw new AssertionError("Unknown built in type " + targetType);
                 }
             }
         } else {
@@ -637,7 +616,7 @@ public class ConversionRules {
                 }
             } else {
                 // converter to user-defined types derived from types other than xs:string
-                StringConverter first = ((AtomicType) targetType.getPrimitiveItemType()).getStringConverter(this);
+                StringConverter first = targetType.getPrimitiveItemType().getStringConverter(this);
                 Converter.DownCastingConverter second = new Converter.DownCastingConverter(targetType, this);
                 return new StringConverter.StringToNonStringDerivedType(first, second);
             }
