@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -16,14 +16,12 @@ import javax.xml.transform.URIResolver;
  * identified by this combination. However, to support a stable implementation of the doc() function,
  * Saxon needs to know what the absolute URI is before the resource is fetched, so it can determine whether
  * a document with that absolute URI actually exists.
- * <p/>
- * This extended interface defines a URIResolver that separates the two functions of resolving a relative URI
+ * <p>This extended interface defines a URIResolver that separates the two functions of resolving a relative URI
  * against a base URI, and fetching a resource with that absolute URI. If the URI resolver supplied to Saxon
  * implements this interface, the absolute URI associated with a loaded document will be the URI returned by
- * this resolver.
- * <p/>
- * The particular motivation for providing this interface is to allow a URIResolver to wrap a .NET XmlResolver,
- * which has additional capability not present in the JAXP interface.
+ * this resolver.</p>
+ * <p>The particular motivation for providing this interface is to allow a URIResolver to wrap a .NET XmlResolver,
+ * which has additional capability not present in the JAXP interface.</p>
  */
 
 public interface RelativeURIResolver extends URIResolver {
@@ -43,7 +41,7 @@ public interface RelativeURIResolver extends URIResolver {
      *          if any failure occurs
      */
 
-    public String makeAbsolute(String href, String base)
+    String makeAbsolute(String href, String base)
             throws TransformerException;
 
     /**
@@ -57,16 +55,14 @@ public interface RelativeURIResolver extends URIResolver {
      *          if an error occurs when trying to
      *          resolve the URI.
      */
-    public Source dereference(String uri)
+    Source dereference(String uri)
             throws TransformerException;
 
     /**
      * Called by the processor when it encounters
      * an xsl:include, xsl:import, or document() function.
-     * <p/>
      * <p>Despite the name, the main purpose of this method is to dereference the URI, not merely
      * to resolve it.</p>
-     * <p/>
      * <p>This method is provided because it is required by the interface. When using a RelativeURIResolver,
      * the single-argument dereference() method is preferred. The result of calling this method should be the
      * same as the result of calling <code>dereference(makeAbsolute(href, base))</code></p>
@@ -80,7 +76,7 @@ public interface RelativeURIResolver extends URIResolver {
      *          if an error occurs when trying to
      *          resolve the URI.
      */
-    public Source resolve(String href, String base)
+    Source resolve(String href, String base)
             throws TransformerException;
 
 
