@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,6 @@
 
 package net.sf.saxon.lib;
 
-import net.sf.saxon.Configuration;
 import net.sf.saxon.om.NameChecker;
 import net.sf.saxon.om.NamespaceResolver;
 import net.sf.saxon.om.QNameException;
@@ -25,11 +24,10 @@ import java.util.regex.Pattern;
  * Provides string constants that can be used to set
  * output properties for a Transformer, or to retrieve
  * output properties from a Transformer or Templates object.
- * <p/>
- * These keys are private Saxon keys that supplement the standard keys
+ * <p>These keys are private Saxon keys that supplement the standard keys
  * defined in javax.xml.transform.OutputKeys. As well as Saxon extension
  * attributes, the list includes new attributes defined in XSLT 2.0 which
- * are not yet supported in JAXP
+ * are not yet supported in JAXP</p>
  */
 
 public class SaxonOutputKeys {
@@ -67,7 +65,6 @@ public class SaxonOutputKeys {
 
     /**
      * allow-duplicate-names = yes|no.
-     * <p/>
      * <p>Defines whether duplicate keys are allowed in a JSON map (new in 3.1)</p>
      */
 
@@ -75,7 +72,6 @@ public class SaxonOutputKeys {
 
     /**
      * build-tree = yes|no.
-     * <p/>
      * <p>Defines whether the raw output is used to build an XML document tree</p>
      */
 
@@ -83,7 +79,6 @@ public class SaxonOutputKeys {
 
     /**
      * saxon:indent-spaces = integer.
-     * <p/>
      * <p>Defines the number of spaces used for indentation of output</p>
      */
 
@@ -91,7 +86,6 @@ public class SaxonOutputKeys {
 
     /**
      * saxon:line-length = integer.
-     * <p/>
      * <p>Defines the desired maximum line length used when indenting output</p>
      */
 
@@ -99,7 +93,6 @@ public class SaxonOutputKeys {
 
     /**
      * saxon:single-quotes = boolean.
-     * <p/>
      * <p>Indicates that attributes should be delimited by apostrophes rather than quotation marks</p>
      */
 
@@ -108,7 +101,6 @@ public class SaxonOutputKeys {
 
     /**
      * suppress-indentation = list of element names
-     * <p/>
      * <p>Defines elements within which no indentation will occur</p>
      */
 
@@ -133,7 +125,6 @@ public class SaxonOutputKeys {
 
     /**
      * json-node-output-method = method-name
-     * <p/>
      * <p>Defines the serialization method for nodes encountered while serializing as JSON</p>
      */
 
@@ -142,7 +133,6 @@ public class SaxonOutputKeys {
 
     /**
      * saxon:attribute-order = list of attribute names
-     * <p/>
      * <p>Defines an ordering for attributes in the serialized output. Any attribute present in the list
      * will appear correctly ordered according to the list; other attributes will be ordered first by namespace,
      * then by local name.</p>
@@ -150,9 +140,27 @@ public class SaxonOutputKeys {
 
     /*@NotNull*/ public static final String ATTRIBUTE_ORDER = "{http://saxon.sf.net/}attribute-order";
 
+    /**
+     * saxon:canonical = yes/no
+     * <p>When used in conjunction with the XML output method, delivers the output in C14N canonical form.
+     * Any serialization properties inconsistent with C14N (for example, encoding, indent, or character maps) are
+     * ignored if saxon:canonical=yes is specified. Provisions of the W3C serialization specification
+     * (for example, requiring the use of empty element tags) are also ignored.</p>
+     * @since 9.9
+     */
+
+    /*@NotNull*/ public static final String CANONICAL = "{http://saxon.sf.net/}canonical";
+
+    /**
+     * saxon:property-order = list of strings
+     * <p>Defines an ordering for properties in the serialized JSON output of a map. Any property present in the list
+     * will appear correctly ordered according to the list; other attributes will be ordered by name.</p>
+     */
+
+    /*@NotNull*/ public static final String PROPERTY_ORDER = "{http://saxon.sf.net/}property-order";
+
     /**
      * saxon:double-space = list of element names
-     * <p/>
      * <p>Defines elements that will have an extra blank line added before the start tag, in addition
      * to normal indentation</p>
      */
@@ -161,7 +169,6 @@ public class SaxonOutputKeys {
 
     /**
      * saxon:newline = string
-     * <p/>
      * <p>Defines the sequence of characters used to represent a newline when using the text
      * output method</p>
      */
@@ -177,7 +184,6 @@ public class SaxonOutputKeys {
 
     /**
      * use-character-map = list-of-qnames.
-     * <p/>
      * <p>Defines the character maps used in this output definition. The QNames
      * are represented in Clark notation as {uri}local-name.</p>
      */
@@ -187,7 +193,6 @@ public class SaxonOutputKeys {
 
     /**
      * include-content-type = "yes" | "no". This attribute is defined in XSLT 2.0
-     * <p/>
      * <p>Indicates whether the META tag is to be added to HTML output</p>
      */
 
@@ -195,7 +200,6 @@ public class SaxonOutputKeys {
 
     /**
      * undeclare-prefixes = "yes" | "no". This attribute is defined in XSLT 2.0
-     * <p/>
      * <p>Indicates XML 1.1 namespace undeclarations are to be output when required</p>
      */
 
@@ -203,7 +207,6 @@ public class SaxonOutputKeys {
 
     /**
      * escape-uri-attributes = "yes" | "no". This attribute is defined in XSLT 2.0
-     * <p/>
      * <p>Indicates whether HTML attributes of type URI are to be URI-escaped</p>
      */
 
@@ -211,7 +214,6 @@ public class SaxonOutputKeys {
 
     /**
      * representation = rep1[;rep2].
-     * <p/>
      * <p>Indicates the preferred way of representing non-ASCII characters in HTML
      * and XML output. rep1 is for characters in the range 128-256, rep2 for those
      * above 256.</p>
@@ -220,7 +222,6 @@ public class SaxonOutputKeys {
 
     /**
      * saxon:next-in-chain = URI.
-     * <p/>
      * <p>Indicates that the output is to be piped into another XSLT stylesheet
      * to perform another transformation. The auxiliary property NEXT_IN_CHAIN_BASE_URI
      * records the base URI of the stylesheet element where this attribute was found.</p>
@@ -230,7 +231,6 @@ public class SaxonOutputKeys {
 
     /**
      * parameter-document = URI.
-     * <p/>
      * <p>Indicates that the output is to be piped into another XSLT stylesheet
      * to perform another transformation. The auxiliary property NEXT_IN_CHAIN_BASE_URI
      * records the base URI of the stylesheet element where this attribute was found.</p>
@@ -241,7 +241,6 @@ public class SaxonOutputKeys {
 
     /**
      * byte-order-mark = yes|no.
-     * <p/>
      * <p>Indicates whether UTF-8/UTF-16 output is to start with a byte order mark. Values are "yes" or "no",
      * default is "no"
      */
@@ -250,7 +249,6 @@ public class SaxonOutputKeys {
 
     /**
      * normalization-form = NFC|NFD|NFKC|NFKD|non.
-     * <p/>
      * <p>Indicates that a given Unicode normalization form (or no normalization) is required.
      */
 
@@ -258,7 +256,6 @@ public class SaxonOutputKeys {
 
     /**
      * recognize-binary = yes|no.
-     * <p/>
      * <p>If set to "yes", and the output is being written using output method "text", Saxon will recognize
      * two processing instructions &lt;?hex XXXX?&gt; and &lt;b64 XXXX?&gt; containing binary data encoded
      * as a hexBinary or base64 string respectively. The corresponding strings will be decoded as characters
@@ -270,7 +267,6 @@ public class SaxonOutputKeys {
 
     /**
      * saxon:require-well-formed = yes|no.
-     * <p/>
      * <p>Indicates whether a user-supplied ContentHandler requires the stream of SAX events to be
      * well-formed (that is, to have a single element node and no text nodes as children of the root).
      * The default is "no".</p>
@@ -280,7 +276,6 @@ public class SaxonOutputKeys {
 
     /**
      * supply-source-locator = yes|no.
-     * <p/>
      * <p>If set to "yes", and the output is being sent to a SAXResult (or to a user-supplied content handler),
      * indicates that the SAX Locator made available to the ContentHandler will contain information about the
      * location of the context node in the source document as well as the location in the stylesheet or query.</p>
@@ -290,11 +285,10 @@ public class SaxonOutputKeys {
 
     /**
      * wrap="yes"|"no".
-     * <p/>
-     * This property is only available in the XQuery API. The value "yes" indicates that the result
+     * <p>This property is only available in the XQuery API. The value "yes" indicates that the result
      * sequence produced by the query is to be wrapped, that is, each item in the result is represented
      * as a separate element. This format allows any sequence to be represented as an XML document,
-     * including for example sequences consisting of parentless attribute nodes.
+     * including for example sequences consisting of parentless attribute nodes.</p>
      */
 
     /*@NotNull*/ public static final String WRAP = "{http://saxon.sf.net/}wrap-result-sequence";
@@ -314,23 +308,6 @@ public class SaxonOutputKeys {
 
     public static final String UNFAILING = "{http://saxon.sf.net/}unfailing";
 
-
-    private static void checkExtensions(String key, /*@Nullable*/ Configuration config) throws XPathException {
-        if (config != null) {
-            config.checkLicensedFeature(Configuration.LicenseFeature.PROFESSIONAL_EDITION, "custom serialization " + key, -1);
-        }
-    }
-
-    private static boolean isValidClarkName(/*@NotNull*/ String value) {
-        if (value.charAt(0) != '{') {
-            return false;
-        }
-        int closer = value.indexOf('}');
-        return closer >= 1 &&
-                closer != value.length() - 1 &&
-                NameChecker.isValidNCName(value.substring(closer + 1));
-    }
-
     private static Pattern publicIdPattern = Pattern.compile("^[\\s\\r\\na-zA-Z0-9\\-'()+,./:=?;!*#@$_%]*$");
 
     /**
@@ -340,25 +317,27 @@ public class SaxonOutputKeys {
      * @param value        The value of the property as written
      * @param nsResolver   The namespace resolver to use; may be null if prevalidated is set or if names are supplied
      *                     in Clark format
-     * @param useDefaultNS
+     * @param useDefaultNS True if unprefixed names are to be treated as being in the default namespace
      * @param prevalidated true if the property has already been validated
-     * @param errorCode    The error code to return in the event of problems
-     * @return The list of element names with lexical QNames replaced by Clark names, starting with a single space
+     * @param allowStar  true if the pseudo-name "*" is permitted; it will be retained in the output
+     *@param errorCode    The error code to return in the event of problems  @return The list of element names with lexical QNames replaced by Clark names, starting with a single space
      * @throws XPathException if any error is found in the list of element names, for example, an undeclared namespace prefix
      */
 
     /*@NotNull*/
     public static String parseListOfNodeNames(
-            String value, /*@Nullable*/ NamespaceResolver nsResolver, boolean useDefaultNS, boolean prevalidated, /*@NotNull*/  String errorCode)
+            String value, /*@Nullable*/ NamespaceResolver nsResolver, boolean useDefaultNS, boolean prevalidated, /*@NotNull*/  boolean allowStar, String errorCode)
             throws XPathException {
-        String s = "";
+        StringBuilder s = new StringBuilder();
         StringTokenizer st = new StringTokenizer(value, " \t\n\r", false);
         while (st.hasMoreTokens()) {
             String displayname = st.nextToken();
-            if (prevalidated || (nsResolver == null)) {
-                s += ' ' + displayname;
+            if (allowStar && "*".equals(displayname)) {
+                s.append(' ').append(displayname);
+            } else if (prevalidated || (nsResolver == null)) {
+                s.append(' ').append(displayname);
             } else if (displayname.startsWith("Q{")) {
-                s += ' ' + displayname.substring(1);
+                s.append(' ').append(displayname.substring(1));
             } else {
                 try {
                     String[] parts = NameChecker.getQNameParts(displayname);
@@ -366,13 +345,36 @@ public class SaxonOutputKeys {
                     if (muri == null) {
                         throw new XPathException("Namespace prefix '" + parts[0] + "' has not been declared", errorCode);
                     }
-                    s += " {" + muri + '}' + parts[1];
+                    s.append(" {").append(muri).append('}').append(parts[1]);
                 } catch (QNameException err) {
-                    throw new XPathException("Invalid element name. " + err.getMessage(), errorCode);
+                    throw new XPathException("Invalid QName. " + err.getMessage(), errorCode);
                 }
             }
         }
-        return s;
+        return s.toString();
+    }
+
+    /**
+     * Ask whether a particular serialization property is to be considered as string-valued,
+     * in which case the value is used exactly as specified without any whitespace stripping.
+     *
+     * <p>The logic here is a little pragmatic.</p>
+     * <p>For XSLT (xsl:output and xsl:result-document) the properties doctype-system, doctype-public,
+     *    item-separator, and media-type have string type and whitespace should therefore be retained.</p>
+     * <p>For values in a parameter document, doctype-system is not whitespace stripped (and is restricted
+     *    by a pattern); doctype-public is whitespace-stripped; media-type is whitespace-stripped</p>
+     * <p>For values in an fn:serialize parameter map, doctype-system, doctype-public, encoding,
+     *    item-separator, media-type, and version are non-stripped strings</p>
+     * <p>XQuery output declarations follow the rules for parameter documents</p>
+     * <p>Pragmatically, it makes sense to be consistent. Including whitespace in doctype-system,
+     *    doctype-public, encoding, version, or media-type is never useful. So we apply
+     *    whitespace-stripping to all properties other than item-separator.</p>
+     *
+     * @param key the property name, in Clark notation
+     * @return true if the property is retained as written without whitespace stripping
+     */
+    public static boolean isUnstrippedProperty(String key) {
+        return ITEM_SEPARATOR.equals(key) || NEWLINE.equals(key);
     }
 
     /**
@@ -388,7 +390,7 @@ public class SaxonOutputKeys {
         String htmlVersion = properties.getProperty(SaxonOutputKeys.HTML_VERSION);
         try {
             return htmlVersion != null &&
-                    ((BigDecimalValue) BigDecimalValue.makeDecimalValue(htmlVersion, false).asAtomic())
+                    ((BigDecimalValue)BigDecimalValue.makeDecimalValue(htmlVersion, false).asAtomic())
                             .getDecimalValue().equals(BigDecimal.valueOf(5));
         } catch (ValidationException e) {
             return false;
@@ -411,7 +413,7 @@ public class SaxonOutputKeys {
         }
         if (htmlVersion != null) {
             try {
-                return ((BigDecimalValue) BigDecimalValue.makeDecimalValue(htmlVersion, false).asAtomic())
+                return ((BigDecimalValue)BigDecimalValue.makeDecimalValue(htmlVersion, false).asAtomic())
                                 .getDecimalValue().equals(BigDecimal.valueOf(5));
             } catch (ValidationException e) {
                 return false;
