@@ -561,6 +561,12 @@ public class SerializerFactory {
     protected SequenceReceiver customizeAdaptiveSerializer(
             AdaptiveEmitter emitter, Properties props,
             CharacterMapExpander characterMapExpander, ProxyReceiver normalizer) {
+        if (normalizer instanceof UnicodeNormalizer) {
+            emitter.setNormalizer(((UnicodeNormalizer) normalizer).getNormalizer());
+        }
+        if (characterMapExpander != null) {
+            emitter.setCharacterMap(characterMapExpander.getCharacterMap());
+        }
         return emitter;
     }
 
@@ -1207,7 +1213,7 @@ public class SerializerFactory {
             if (isValidClarkName(value)) {
                 checkExtensions(value);
             } else {
-                throw new XPathException("Invalid value for serialization method: " +
+                throw new XPathException("Invalid value (" + value + ") for serialization method: " +
                                                  "must be xml|html|xhtml|text|json|adaptive, or a QName in '{uri}local' form", "SEPM0016");
             }
         }
