@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,9 +8,11 @@
 package net.sf.saxon.lib;
 
 import net.sf.saxon.Configuration;
+import net.sf.saxon.resource.BinaryResource;
+import net.sf.saxon.resource.DataURIScheme;
+import net.sf.saxon.resource.UnparsedTextResource;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
-import org.jetbrains.annotations.NotNull;
 
 import java.io.*;
 import java.net.MalformedURLException;
@@ -22,7 +24,10 @@ import java.util.zip.GZIPInputStream;
 
 /**
  * Default implementation of the UnparsedTextURIResolver, used if no other implementation
- * is nominated to the Configuration.
+ * is nominated to the Configuration. This implementation
+ *  * handles anything that the java URL class will handle, plus the <code>classpath</code>
+ *  * URI scheme defined in the Spring framework, and the <code>data</code> URI scheme defined in
+ *  * RFC 2397.
  */
 
 public class StandardUnparsedTextResolver implements UnparsedTextURIResolver {
@@ -67,46 +72,85 @@ public class StandardUnparsedTextResolver implements UnparsedTextURIResolver {
             throw new XPathException("Resolved URI supplied to unparsed-text() is not absolute: " + absoluteURI.toString(),
                     "FOUT1170");
         }
-        try {
-            absoluteURL = absoluteURI.toURL();
-        } catch (MalformedURLException mue) {
-            XPathException e = new XPathException("Cannot convert absolute URI to URL", mue);
-            e.setErrorCode("FOUT1170");
-            throw e;
-        }
-        try {
-            InputStream is;
-            URLConnection connection = absoluteURL.openConnection();
-            connection.setRequestProperty("Accept-Encoding", "gzip");
+        InputStream inputStream = null;
+        String contentEncoding;
+        boolean isXmlMediaType = false;
+        if (absoluteURI.getScheme().equals("data")) {
+            Resource resource;
             try {
-                connection.connect();
-            } catch (IOException ioe) {
-                if (debug) {
-                    err.error("unparsed-text(): connection failure. " + ioe.getMessage());
+                resource = DataURIScheme.decode(absoluteURI);
+            } catch (IllegalArgumentException e) {
+                throw new XPathException("Invalid URI in 'data' scheme: " + e.getMessage(), "FOUT1170");
+            }
+            if (resource instanceof BinaryResource) {
+                byte[] octets = ((BinaryResource) resource).getData();
+                inputStream = new ByteArrayInputStream(octets);
+                contentEncoding = "utf-8";
+
+            } else {
+                assert resource instanceof UnparsedTextResource;
+                inputStream = ((UnparsedTextResource) resource).getInputStream();
+                contentEncoding = ((UnparsedTextResource) resource).getEncoding();
+            }
+            if (encoding == null) {
+                encoding = contentEncoding;
+            }
+            String mediaType = resource.getContentType();
+            isXmlMediaType = (mediaType.startsWith("application/") || mediaType.startsWith("text/")) &&
+                    (mediaType.endsWith("/xml") || mediaType.endsWith("+xml"));
+        } else if (absoluteURI.getScheme().equals("classpath")) {
+            InputStream is = config.getDynamicLoader().getResourceAsStream(absoluteURI.toString().substring(10));
+            if (is != null) {
+                try {
+                    if (encoding == null) {
+                        encoding = "UTF-8";
+                    }
+                    return new InputStreamReader(is, encoding);
+                } catch (UnsupportedEncodingException e) {
+                    throw new AssertionError(e);
                 }
-                XPathException xpe = new XPathException("Failed to read input file", ioe);
-                xpe.setErrorCode("FOUT1170");
-                throw xpe;
             }
-
-            is = connection.getInputStream();
-            String contentEncoding = connection.getContentEncoding();
-
-            if ("gzip".equals(contentEncoding)) {
-                is = new GZIPInputStream(is);
-            }
-            if (debug) {
-                err.info("unparsed-text(): established connection " +
-                        ("gzip".equals(contentEncoding) ? " (zipped)" : ""));
+        } else {
+            try {
+                absoluteURL = absoluteURI.toURL();
+            } catch (MalformedURLException mue) {
+                XPathException e = new XPathException("Cannot convert absolute URI to URL", mue);
+                e.setErrorCode("FOUT1170");
+                throw e;
             }
             try {
 
-                if (!is.markSupported()) {
-                    is = new BufferedInputStream(is);
+                URLConnection connection = absoluteURL.openConnection();
+                connection.setRequestProperty("Accept-Encoding", "gzip");
+                try {
+                    connection.connect();
+                } catch (IOException ioe) {
+                    if (debug) {
+                        err.error("unparsed-text(): connection failure. " + ioe.getMessage());
+                    }
+                    XPathException xpe = new XPathException("Failed to read input file", ioe);
+                    xpe.setErrorCode("FOUT1170");
+                    throw xpe;
+                }
+
+                inputStream = connection.getInputStream();
+                contentEncoding = connection.getContentEncoding();
+
+                if ("gzip".equals(contentEncoding)) {
+                    inputStream = new GZIPInputStream(inputStream);
+                }
+                if (debug) {
+                    err.info("unparsed-text(): established connection " +
+                                     ("gzip".equals(contentEncoding) ? " (zipped)" : ""));
+                }
+
+
+                if (!inputStream.markSupported()) {
+                    inputStream = new BufferedInputStream(inputStream);
                 }
 
                 // Get any external (HTTP) encoding label.
-                boolean isXmlMediaType = false;
+                isXmlMediaType = false;
 
                 // The file:// URL scheme gives no useful information...
                 if (!"file".equals(connection.getURL().getProtocol())) {
@@ -149,7 +193,7 @@ public class StandardUnparsedTextResolver implements UnparsedTextURIResolver {
                             // ... and values may be quoted
                             if ((pos = charset.indexOf('"')) > 0) {
                                 charset = charset.substring(pos + 1,
-                                        charset.indexOf('"', pos + 2));
+                                                            charset.indexOf('"', pos + 2));
                             }
                             if (debug) {
                                 err.info("unparsed-text(): charset = " + charset.trim());
@@ -159,35 +203,35 @@ public class StandardUnparsedTextResolver implements UnparsedTextURIResolver {
                     }
                 }
 
-                if (encoding == null || isXmlMediaType) {
-                    encoding = inferStreamEncoding(is, debug ? err : null);
-                    if (debug) {
-                        err.info("unparsed-text(): inferred encoding = " + encoding);
+
+                try {
+                    if (encoding == null || isXmlMediaType) {
+                        encoding = inferStreamEncoding(inputStream, debug ? err : null);
+                        if (debug) {
+                            err.info("unparsed-text(): inferred encoding = " + encoding);
+                        }
                     }
+                } catch (IOException e) {
+                    encoding = "UTF-8";
                 }
 
-            } catch (IOException e) {
-                encoding = "UTF-8";
+            } catch (IOException ioe) {
+                throw new XPathException(ioe.getMessage(), "FOUT1170");
+            } catch (IllegalCharsetNameException icne) {
+                throw new XPathException("Invalid encoding name: " + encoding, "FOUT1190");
+            } catch (UnsupportedCharsetException uce) {
+                throw new XPathException("Invalid encoding name: " + encoding, "FOUT1190");
             }
 
-            //}
-
-            // The following is necessary to ensure that encoding errors are not recovered.
-            Charset charset = Charset.forName(encoding);
-            CharsetDecoder decoder = charset.newDecoder();
-            decoder = decoder.onMalformedInput(CodingErrorAction.REPORT);
-            decoder = decoder.onUnmappableCharacter(CodingErrorAction.REPORT);
-            return new BufferedReader(new InputStreamReader(is, decoder));
-
-
-        } catch (IOException ioe) {
-            throw new XPathException(ioe.getMessage(), "FOUT1170");
-        } catch (IllegalCharsetNameException icne) {
-            throw new XPathException("Invalid encoding name: " + encoding, "FOUT1190");
-        } catch (UnsupportedCharsetException uce) {
-            throw new XPathException("Invalid encoding name: " + encoding, "FOUT1190");
         }
 
+        // The following is necessary to ensure that encoding errors are not recovered.
+        Charset charset = Charset.forName(encoding);
+        CharsetDecoder decoder = charset.newDecoder();
+        decoder = decoder.onMalformedInput(CodingErrorAction.REPORT);
+        decoder = decoder.onUnmappableCharacter(CodingErrorAction.REPORT);
+        return new BufferedReader(new InputStreamReader(inputStream, decoder));
+
     }
 
 
@@ -200,7 +244,6 @@ public class StandardUnparsedTextResolver implements UnparsedTextURIResolver {
      * @throws IOException if it isn't possible to mark the current position on the input stream and read ahead
      */
 
-    @NotNull
     public static String inferStreamEncoding(InputStream is, Logger err) throws IOException {
         is.mark(100);
         byte[] start = new byte[100];
