@@ -28,8 +28,8 @@ import java.util.List;
 public class ArraySort extends SystemFunction {
 
     private static class MemberToBeSorted{
-        public GroundedValue<? extends Item> value;
-        public GroundedValue<? extends Item> sortKey;
+        public GroundedValue<?> value;
+        public GroundedValue<?> sortKey;
         int originalPosition;
     }
 
@@ -40,9 +40,9 @@ public class ArraySort extends SystemFunction {
      * @return an expression representing a call of this extension function
      */
     @Override
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public ArrayItem call(XPathContext context, Sequence[] arguments) throws XPathException {
         ArrayItem array = (ArrayItem) arguments[0].head();
-        final List<MemberToBeSorted> inputList = new ArrayList<MemberToBeSorted>(array.arrayLength());
+        final List<MemberToBeSorted> inputList = new ArrayList<>(array.arrayLength());
         int i = 0;
         StringCollator collation;
         if (arguments.length == 1) {
@@ -59,12 +59,12 @@ public class ArraySort extends SystemFunction {
         if (arguments.length == 3){
             key = (Function) arguments[2].head();
         }
-        for (GroundedValue seq: array.members()){
+        for (GroundedValue<?> seq: array.members()){
             MemberToBeSorted member = new MemberToBeSorted();
             member.value = seq;
             member.originalPosition = i++;
             if (key != null) {
-                member.sortKey = ((Sequence<Item>) dynamicCall(key, context, new Sequence[]{seq})).materialize();
+                member.sortKey = dynamicCall(key, context, new Sequence[]{seq}).materialize();
             } else {
                 member.sortKey = atomize(seq);
             }
@@ -95,7 +95,7 @@ public class ArraySort extends SystemFunction {
             err.setErrorCode("XPTY0004");
             throw err;
         }
-        List<GroundedValue<? extends Item>> outputList = new ArrayList<>(array.arrayLength());
+        List<GroundedValue<?>> outputList = new ArrayList<>(array.arrayLength());
         for (MemberToBeSorted member: inputList){
             outputList.add(member.value);
         }
@@ -134,9 +134,9 @@ public class ArraySort extends SystemFunction {
         }
     }
 
-    private static GroundedValue atomize(Sequence input) throws XPathException {
-        SequenceIterator iterator = input.iterate();
-        SequenceIterator mapper = Atomizer.getAtomizingIterator(iterator, false);
-        return ((SequenceIterator<Item>) mapper).materialize();
+    private static GroundedValue<? extends AtomicValue> atomize(Sequence<?> input) throws XPathException {
+        SequenceIterator<?> iterator = input.iterate();
+        SequenceIterator<? extends AtomicValue> mapper = Atomizer.getAtomizingIterator(iterator, false);
+        return mapper.materialize();
     }
 }
