@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -20,7 +20,10 @@ import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.Type;
-import net.sf.saxon.value.*;
+import net.sf.saxon.value.BooleanValue;
+import net.sf.saxon.value.EmptySequence;
+import net.sf.saxon.value.SequenceType;
+import net.sf.saxon.value.StringValue;
 
 import java.util.Map;
 
@@ -35,7 +38,7 @@ public class XMLToJsonFn extends SystemFunction {
         return xmlToJsonOptions;
     }
 
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
         NodeInfo xml = (NodeInfo) arguments[0].head();
         if (xml == null) {
             return EmptySequence.getInstance();
@@ -44,11 +47,12 @@ public class XMLToJsonFn extends SystemFunction {
         boolean indent = false;
         if (getArity() > 1) {
             MapItem suppliedOptions = (MapItem) arguments[1].head();
-            Map<String, Sequence> options = getDetails().optionDetails.processSuppliedOptions(suppliedOptions, context);
+            Map<String, Sequence<?>> options = getDetails().optionDetails.processSuppliedOptions(suppliedOptions, context);
             indent = ((BooleanValue)options.get("indent").head()).getBooleanValue();
         }
 
         PipelineConfiguration pipe = context.getController().makePipelineConfiguration();
+        pipe.setXPathContext(context);
         JsonReceiver receiver = new JsonReceiver(pipe);
         receiver.setIndenting(indent);
         Receiver r = receiver;
@@ -59,7 +63,7 @@ public class XMLToJsonFn extends SystemFunction {
         pipe.setComponent(StartTagBuffer.class.getName(), stb);
         stb.setPipelineConfiguration(pipe);
         stb.open();
-        xml.copy(stb, NodeInfo.NO_NAMESPACES, ExplicitLocation.UNKNOWN_LOCATION);
+        xml.copy(stb, 0, ExplicitLocation.UNKNOWN_LOCATION);
         stb.close();
         return new StringValue(receiver.getJsonString());
     }
