@@ -7,6 +7,8 @@
 
 package net.sf.saxon.ma.map;
 
+import net.sf.saxon.expr.ContextOriginator;
+import net.sf.saxon.expr.Literal;
 import net.sf.saxon.expr.OperandRole;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.sort.AtomicComparer;
@@ -323,10 +325,11 @@ public interface MapItem extends Function {
      * Prepare an XPathContext object for evaluating the function
      *
      * @param callingContext the XPathContext of the function calling expression
+     * @param originator
      * @return a suitable context for evaluating the function (which may or may
      * not be the same as the caller's context)
      */
-    default XPathContext makeNewContext(XPathContext callingContext) {
+    default XPathContext makeNewContext(XPathContext callingContext, ContextOriginator originator) {
         return callingContext;
     }
 
@@ -468,11 +471,15 @@ public interface MapItem extends Function {
     }
 
     /**
-     * Output information about this function item to the diagnostic explain() output
+     * Export information about this function item to the export() or explain() output
      */
     default void export(ExpressionPresenter out) throws XPathException {
         out.startElement("map");
-        out.emitAttribute("size", size() + "");
+        out.emitAttribute("size", "" + size());
+        for (KeyValuePair kvp : keyValuePairs()) {
+            Literal.exportAtomicValue(kvp.key, out);
+            Literal.exportValue(kvp.value, out);
+        }
         out.endElement();
     }
 
