@@ -12,7 +12,10 @@ import net.sf.saxon.expr.sort.AtomicComparer;
 import net.sf.saxon.expr.sort.AtomicMatchKey;
 import net.sf.saxon.expr.sort.CodepointMatchKey;
 import net.sf.saxon.functions.Count;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.Function;
+import net.sf.saxon.om.GroundedValue;
+import net.sf.saxon.om.NodeInfo;
+import net.sf.saxon.om.Sequence;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.AtomicIterator;
@@ -22,7 +25,7 @@ import net.sf.saxon.type.TypeHierarchy;
 import net.sf.saxon.type.UType;
 import net.sf.saxon.value.*;
 
-import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.List;
 import java.util.TreeMap;
 
@@ -49,7 +52,7 @@ public class RangeKey implements MapItem {
      * @param key     the value of the key
      * @return the value associated with the given key, or null if the key is not present in the map
      */
-    public GroundedValue<? extends Item> get(AtomicValue key)  {
+    public GroundedValue<?> get(AtomicValue key)  {
         CodepointMatchKey k = new CodepointMatchKey(key.getStringValue());
         if ((min == null || min.compareTo(k) <= 0) &&
                 (max == null || max.compareTo(k) >= 0)) {
@@ -100,10 +103,23 @@ public class RangeKey implements MapItem {
      * @return an iterator over the key-value pairs
      */
     public Iterable<KeyValuePair> keyValuePairs() {
-        // TODO: avoid constructing the list in memory
-        List<KeyValuePair> list = new ArrayList<>();
-        keys().forEach(key -> list.add(new KeyValuePair(key, get(key))));
-        return list;
+        return () -> new Iterator<KeyValuePair>() {
+            AtomicIterator keys = keys();
+            AtomicValue next = keys.next();
+            public boolean hasNext() {
+                return next != null;
+            }
+
+            public KeyValuePair next() {
+                if (next == null) {
+                    return null;
+                } else {
+                    KeyValuePair kvp = new KeyValuePair(next, get(next));
+                    next = keys.next();
+                    return kvp;
+                }
+            }
+        };
     }
 
     /**
@@ -137,7 +153,7 @@ public class RangeKey implements MapItem {
      * @return the new map containing the additional entry
      */
     @Override
-    public MapItem addEntry(AtomicValue key, GroundedValue value) {
+    public MapItem addEntry(AtomicValue key, GroundedValue<?> value) {
         return HashTrieMap.copy(this).addEntry(key, value);
     }
 
@@ -154,7 +170,7 @@ public class RangeKey implements MapItem {
         AtomicIterator keyIter = keys();
         AtomicValue key;
         while ((key = keyIter.next()) != null) {
-            Sequence value = get(key);
+            Sequence<?> value = get(key);
             try {
                 if (!valueType.matches(value, th)) {
                     return false;
