@@ -39,12 +39,20 @@ import java.util.*;
  * second phase occurs when any of the methods itemAt(), reduce(), or subsequence()
  * is called.</p>
  */
-public class Chain<T extends Item> implements GroundedValue<T> {
+public class Chain implements GroundedValue<Item<?>> {
 
-    private List<GroundedValue<T>> children = new ArrayList<>();
-    private List<T> extent = null;
+    private List<GroundedValue<?>> children = new ArrayList<>();
+    private List<Item<?>> extent = null;
 
-    public Chain(List<GroundedValue<T>> children) {
+    /**
+     * Create a chain from a list of grounded values
+     * @param children the list of grounded values. The implementation may or may not copy
+     *                 this list, and it may or may not modify the list during execution
+     *                 of the {@link #append(Item)} method. The caller must not attempt to
+     *                 modify the list after return from this constructor.
+     */
+
+    public Chain(List<GroundedValue<?>> children) {
         this.children = children;
 
         int size = 0;
@@ -63,10 +71,10 @@ public class Chain<T extends Item> implements GroundedValue<T> {
         }
         if (copy) {
             this.children = new ArrayList<>(size);
-            for (GroundedValue<T> gv : children) {
+            for (GroundedValue<?> gv : children) {
                 if (gv instanceof Chain) {
                     if (((Chain) gv).children.size() < 30) {
-                        this.children.addAll(((Chain<T>) gv).children);
+                        this.children.addAll(((Chain) gv).children);
                     } else {
                         this.children.add(gv);
                     }
@@ -74,14 +82,15 @@ public class Chain<T extends Item> implements GroundedValue<T> {
                     this.children.add(gv);
                 }
             }
-        } else {
-            this.children = children;
         }
     }
 
-    public T head() {
-        for (GroundedValue<T> seq : children) {
-            T head = seq.head();
+    public Item<?> head() {
+        if (extent != null) {
+            return extent.isEmpty() ? null : extent.get(0);
+        }
+        for (GroundedValue<?> seq : children) {
+            Item<?> head = seq.head();
             if (head != null) {
                 return head;
             }
@@ -89,11 +98,11 @@ public class Chain<T extends Item> implements GroundedValue<T> {
         return null;
     }
 
-    public UnfailingIterator<T> iterate() {
+    public UnfailingIterator<Item<?>> iterate() {
         if (extent != null) {
             return new net.sf.saxon.tree.iter.ListIterator<>(extent);
         } else {
-            return new ChainIterator<>(this);
+            return new ChainIterator(this);
         }
     }
 
@@ -105,16 +114,13 @@ public class Chain<T extends Item> implements GroundedValue<T> {
      */
 
 
-    public void append(T item) {
+    public void append(Item item) {
         if (extent != null) {
             throw new IllegalStateException();
         }
         if (item != null) {
-            if (item instanceof GroundedValue) {
-                children.add(item);
-            } else {
-                children.add(new One<T>(item));
-            }
+            //noinspection unchecked
+            children.add((GroundedValue<?>)item);
         }
     }
 
@@ -135,13 +141,16 @@ public class Chain<T extends Item> implements GroundedValue<T> {
      * @param n the index of the required item, with 0 representing the first item in the sequence
      * @return the n'th item if it exists, or null otherwise
      */
-    public T itemAt(int n) {
-        // TODO: avoid consolidating the chain beyond the required item
-        consolidate();
-        if (n >= 0 && n < extent.size()) {
-            return extent.get(n);
+    public Item<?> itemAt(int n) {
+        if (n == 0) {
+            return head();
         } else {
-            return null;
+            consolidate();
+            if (n >= 0 && n < extent.size()) {
+                return extent.get(n);
+            } else {
+                return null;
+            }
         }
     }
 
@@ -157,7 +166,7 @@ public class Chain<T extends Item> implements GroundedValue<T> {
      *               of the sequence
      * @return the required subsequence.
      */
-    public GroundedValue subsequence(int start, int length) {
+    public GroundedValue<Item<?>> subsequence(int start, int length) {
         consolidate();
         int newStart;
         if (start < 0) {
@@ -177,7 +186,7 @@ public class Chain<T extends Item> implements GroundedValue<T> {
                 newEnd = extent.size();
             }
         }
-        return new SequenceExtent(extent.subList(newStart, newEnd));
+        return new SequenceExtent<>(extent.subList(newStart, newEnd));
     }
 
     /**
@@ -242,30 +251,30 @@ public class Chain<T extends Item> implements GroundedValue<T> {
      *
      * @return the simplified sequence
      */
-    public GroundedValue reduce() {
+    public GroundedValue<Item<?>> reduce() {
         consolidate();
         return SequenceExtent.makeSequenceExtent(extent);
     }
 
-    private static class ChainIterator<T extends Item> implements UnfailingIterator<T>, GroundedIterator<T> {
+    private static class ChainIterator implements UnfailingIterator<Item<?>>, GroundedIterator<Item<?>> {
 
         private class ChainPosition {
-            Chain<T> chain;
+            Chain chain;
             int offset;
 
-            public ChainPosition(Chain<T> chain, int offset) {
+            public ChainPosition(Chain chain, int offset) {
                 this.chain = chain;
                 this.offset = offset;
             }
         }
 
-        private Queue<UnfailingIterator<T>> queue = new LinkedList<>();
+        private Queue<UnfailingIterator<Item<?>>> queue = new LinkedList<>();
 
         private Stack<ChainPosition> stack;
 
-        private Chain<T> thisChain;
+        private Chain thisChain;
 
-        public ChainIterator(Chain<T> thisChain) {
+        public ChainIterator(Chain thisChain) {
             this.thisChain = thisChain;
             stack = new Stack<>();
             stack.push(new ChainPosition(thisChain, 0));
@@ -280,14 +289,14 @@ public class Chain<T extends Item> implements GroundedValue<T> {
          * @return the next item, or null if there are no more items.
          */
 
-        public T next() {
+        public Item<?> next() {
 
             // If there are iterators on the queue waiting to be processed, then take the first
             // item from the first iterator on the queue.
             while (!queue.isEmpty()) {
-                UnfailingIterator<T> ui = queue.peek();
+                UnfailingIterator<Item<?>> ui = queue.peek();
                 while (ui != null) {
-                    T current = ui.next();
+                    Item<?> current = ui.next();
                     if (current != null) {
                         return current;
                     } else {
@@ -308,13 +317,13 @@ public class Chain<T extends Item> implements GroundedValue<T> {
                     stack.pop();
                     continue;
                 }
-                GroundedValue<T> gv = cp.chain.children.get(cp.offset++);
+                GroundedValue<?> gv = cp.chain.children.get(cp.offset++);
                 if (gv instanceof Chain) {
                     stack.push(new ChainPosition((Chain) gv, 0));
                 } else if (gv instanceof Item) {
-                    return (T)gv;
+                    return (Item<?>)gv;
                 } else {
-                    queue.offer(gv.iterate());
+                    queue.offer((UnfailingIterator<Item<?>>)gv.iterate());
                     return next();
                 }
             }
@@ -349,7 +358,7 @@ public class Chain<T extends Item> implements GroundedValue<T> {
          * @return the corresponding Value
          */
 
-        public GroundedValue<T> materialize() {
+        public GroundedValue<Item<?>> materialize() {
             return thisChain;
         }
 
@@ -362,8 +371,8 @@ public class Chain<T extends Item> implements GroundedValue<T> {
          *                        which cause evaluation of expressions while materializing the value.
          */
 
-        public GroundedValue<T> getResidue() throws XPathException {
-            return new SequenceExtent<T>(this);
+        public GroundedValue<Item<?>> getResidue() throws XPathException {
+            return new SequenceExtent<>(this);
         }
     }
 
