@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -34,31 +34,37 @@ import java.util.Iterator;
 public interface TreeInfo extends Source {
 
     /**
-     * Get the NodeInfo object representing the document node at the root of the tree
-     * @return the document node
+     * Get the NodeInfo object representing the node at the root of the tree, which will often
+     * but not invariably be a document node
+     * @return the node at the root of the tree
      */
 
     NodeInfo getRootNode();
 
     /**
-     * Get the Configuration to which this document belongs
+     * Get the Configuration to which the nodes in this tree belong
      * @return the configuration
      */
 
     Configuration getConfiguration();
 
     /**
-     * Get the document number, which identifies this tree uniquely within a Configuration
+     * Get the document number, which identifies this tree uniquely within a Configuration.
+     * Document numbers are used when sorting nodes from different trees into document order.
      * @return the document number
      */
 
     long getDocumentNumber();
 
     /**
-     * Ask whether the document contains any nodes whose type annotation is anything other than
+     * Ask whether the tree contains any nodes whose type annotation is anything other than
      * UNTYPED or UNTYPED_ATOMIC
      *
-     * @return true if the document contains elements whose type is other than UNTYPED or UNTYPED_ATOMIC
+     * @return true if the tree contains elements or attributes whose type is other than
+     * UNTYPED or UNTYPED_ATOMIC. This is typically the case only if the document has been
+     * schema-validated (one can imagine an external tree model in which the type annotations
+     * arise some other way, but they must always be consistent with the typed value of the
+     * node).
      */
 
     boolean isTyped();
@@ -105,7 +111,31 @@ public interface TreeInfo extends Source {
     String[] getUnparsedEntity(String name);
 
     /**
-     * Set user data on the document node. The user data can be retrieved subsequently
+     * Set details of space stripping action that was applied to this document during
+     * construction. This ensures that space stripping is not applied twice to the same
+     * document.
+     * @param rule details of the space stripping rules that have been applied to this
+     *             document during its construction.
+     * @since 9.9
+     */
+
+    void setSpaceStrippingRule(SpaceStrippingRule rule);
+
+    /**
+     * Get details of space stripping action that was applied to this document during
+     * construction. This ensures that space stripping is not applied twice to the same
+     * document.
+     *
+     * @return details of the space stripping rules that have been applied to this
+     *             document during its construction. By default, returns {@link NoElementsSpaceStrippingRule},
+     *             indicating that no space stripping has been applied
+     * @since 9.9
+     */
+
+    SpaceStrippingRule getSpaceStrippingRule();
+
+    /**
+     * Set user data that applies to this tree. The user data can be retrieved subsequently
      * using {@link #getUserData}
      *
      * @param key   A string giving the name of the property to be set. Clients are responsible
@@ -123,7 +153,7 @@ public interface TreeInfo extends Source {
     void setUserData(String key, Object value);
 
     /**
-     * Get user data held in the document node. This retrieves properties previously set using
+     * Get user data that applies to this tree. This retrieves properties previously set using
      * {@link #setUserData}
      *
      * @param key A string giving the name of the property to be retrieved.
