@@ -89,7 +89,7 @@ public class AnchorPattern extends Pattern {
      * @return true if the node matches the Pattern, false otherwise
      */
 
-    public boolean matches(Item item, XPathContext context) throws XPathException {
+    public boolean matches(Item<?> item, XPathContext context) throws XPathException {
         throw new AssertionError();
         //return true;
     }
