@@ -72,7 +72,7 @@ public class IntersectPattern extends VennPattern {
      * @return true if the node matches either of the operand patterns
      */
 
-    public boolean matches(Item item, XPathContext context) throws XPathException {
+    public boolean matches(Item<?> item, XPathContext context) throws XPathException {
         return p1.matches(item, context) && p2.matches(item, context);
     }
 
