@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,17 +7,13 @@
 
 package net.sf.saxon.pattern;
 
-import net.sf.saxon.om.NamePool;
-import net.sf.saxon.om.NodeInfo;
-import net.sf.saxon.om.NodeName;
-import net.sf.saxon.om.StructuredQName;
-import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.om.*;
 import net.sf.saxon.tree.tiny.NodeVectorTree;
-import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.SchemaType;
-import net.sf.saxon.type.Type;
-import net.sf.saxon.type.UType;
-import net.sf.saxon.z.IntPredicate;
+import net.sf.saxon.type.*;
+import net.sf.saxon.z.IntSet;
+
+import java.util.Optional;
+import java.util.function.IntPredicate;
 
 /**
  * NodeTest is an interface that enables a test of whether a node has a particular
@@ -61,6 +57,19 @@ public final class LocalNameTest extends NodeTest implements QNameTest {
         return uType;
     }
 
+    /**
+     * Get the set of node names allowed by this NodeTest. Return no result, because the
+     * set of names cannot be represented.
+     *
+     * @return the set of integer fingerprints of the node names that this node test can match; or absent
+     * if the set of names cannot be represented (for example, with the name tests *:xxx or xxx:*)
+     */
+    @Override
+    public Optional<IntSet> getRequiredNodeNames() {
+        // See bug 3713
+        return Optional.empty();
+    }
+
     /**
      * Test whether this node test is satisfied by a given node. This method is only
      * fully supported for a subset of NodeTests, because it doesn't provide all the information
@@ -84,13 +93,8 @@ public final class LocalNameTest extends NodeTest implements QNameTest {
     public IntPredicate getMatcher(final NodeVectorTree tree) {
         final byte[] nodeKindArray = tree.getNodeKindArray();
         final int[] nameCodeArray = tree.getNameCodeArray();
-        return new IntPredicate() {
-            @Override
-            public boolean matches(int nodeNr) {
-                return nodeKindArray[nodeNr] == nodeKind &&
-                        localName.equals(namePool.getLocalName(nameCodeArray[nodeNr] & 0xfffff));
-            }
-        };
+        return nodeNr -> (nodeKindArray[nodeNr] & 0x0f) == nodeKind &&
+                localName.equals(namePool.getLocalName(nameCodeArray[nodeNr] & 0xfffff));
     }
 
     /**
@@ -145,15 +149,6 @@ public final class LocalNameTest extends NodeTest implements QNameTest {
         return nodeKind;
     }
 
-    /**
-     * Get a mask indicating which kinds of nodes this NodeTest can match. This is a combination
-     * of bits: 1<<Type.ELEMENT for element nodes, 1<<Type.TEXT for text nodes, and so on.
-     */
-
-    public int getNodeKindMask() {
-        return 1 << nodeKind;
-    }
-
     public String toString() {
         switch (nodeKind) {
             case Type.ELEMENT:
@@ -191,7 +186,7 @@ public final class LocalNameTest extends NodeTest implements QNameTest {
      * as the body of a JS function in which the argument name "q" is an
      * XdmQName object holding the name. The XdmQName object has properties
      * uri and local.
-     * @param targetVersion
+     * @param targetVersion the version of Saxon-JS being targeted
      */
     public String generateJavaScriptNameTest(int targetVersion) {
         return "q.local==='" + localName + "'";
@@ -203,13 +198,12 @@ public final class LocalNameTest extends NodeTest implements QNameTest {
      * @return a Javascript instruction or sequence of instructions, which can be used as the body
      * of a Javascript function, and which returns a boolean indication whether the value of the
      * variable "item" is an instance of this item type.
-     * @throws XPathException if JS code cannot be generated for this item type, for example because
-     *                        the test is schema-aware.
-     * @param knownToBe
-     * @param targetVersion
+     * @param knownToBe An item type that the supplied item is known to conform to; the generated code
+     *                  can assume that the item is an instance of this type.
+     * @param targetVersion The version of Saxon-JS for which code is being generated. Currently either 1 or 2.
      */
     @Override
-    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) throws XPathException {
+    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) {
         if (targetVersion == 1) {
             return "var q=SaxonJS.U.nameOfNode(item); return SaxonJS.U.isNode(item) && item.nodeType===" +
                     nodeKind + "&&" + generateJavaScriptNameTest(targetVersion);
@@ -217,5 +211,24 @@ public final class LocalNameTest extends NodeTest implements QNameTest {
             return "return SaxonJS.U.isNode(item) && item.nodeType===" + nodeKind + " && SaxonJS.U.hasLocalName(item, '" + localName + "')";
         }
     }
+
+    /**
+     * Get extra diagnostic information about why a supplied item does not conform to this
+     * item type, if available. If extra information is returned, it should be in the form of a complete
+     * sentence, minus the closing full stop. No information should be returned for obvious cases.
+     *
+     * @param item the item that doesn't match this type
+     * @param th   the type hierarchy cache
+     * @return optionally, a message explaining why the item does not match the type
+     */
+    @Override
+    public Optional<String> explainMismatch(Item item, TypeHierarchy th) {
+        Optional<String> explanation = super.explainMismatch(item, th);
+        if (explanation.isPresent()) {
+            return explanation;
+        }
+        return Optional.of("The node has the wrong local name");
+    }
+
 }
 
