@@ -2,7 +2,7 @@
 // Copyright (c) 2013 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.pattern;
@@ -12,9 +12,10 @@ import net.sf.saxon.om.NodeName;
 import net.sf.saxon.tree.tiny.NodeVectorTree;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.*;
-import net.sf.saxon.z.IntPredicate;
 
 import java.util.LinkedList;
+import java.util.function.Function;
+import java.util.function.IntPredicate;
 
 /**
  * An MultipleNodeKindTest is a nodetest that matches nodes belonging to any subset of possible
@@ -93,15 +94,12 @@ public final class MultipleNodeKindTest extends NodeTest {
 
     public IntPredicate getMatcher(final NodeVectorTree tree) {
         final byte[] nodeKindArray = tree.getNodeKindArray();
-        return new IntPredicate() {
-            @Override
-            public boolean matches(int nodeNr) {
-                int nodeKind = nodeKindArray[nodeNr];
-                if (nodeKind == Type.WHITESPACE_TEXT) {
-                    nodeKind = Type.TEXT;
-                }
-                return (nodeKindMask & (1 << nodeKind)) != 0;
+        return nodeNr -> {
+            int nodeKind = nodeKindArray[nodeNr] & 0x0f;
+            if (nodeKind == Type.WHITESPACE_TEXT) {
+                nodeKind = Type.TEXT;
             }
+            return (nodeKindMask & (1 << nodeKind)) != 0;
         };
     }
 
@@ -128,50 +126,29 @@ public final class MultipleNodeKindTest extends NodeTest {
         return -0.5;
     }
 
-    /**
-     * Get a mask indicating which kinds of nodes this NodeTest can match. This is a combination
-     * of bits: 1<<Type.ELEMENT for element nodes, 1<<Type.TEXT for text nodes, and so on.
-     */
-
-    public int getNodeKindMask() {
-        return nodeKindMask;
-    }
-
     /*@NotNull*/
     public String toString() {
         FastStringBuffer fsb = new FastStringBuffer(FastStringBuffer.C64);
-        LinkedList<PrimitiveUType> types = new LinkedList<PrimitiveUType>(uType.decompose());
-        format(types, fsb);
+        LinkedList<PrimitiveUType> types = new LinkedList<>(uType.decompose());
+        format(types, fsb, ItemType::toString);
         return fsb.toString();
     }
 
     public String toExportString() {
         FastStringBuffer fsb = new FastStringBuffer(FastStringBuffer.C64);
-        LinkedList<PrimitiveUType> types = new LinkedList<PrimitiveUType>(uType.decompose());
-        formatForExport(types, fsb);
+        LinkedList<PrimitiveUType> types = new LinkedList<>(uType.decompose());
+        format(types, fsb, ItemType::toExportString);
         return fsb.toString();
     }
 
-    private void format(LinkedList<PrimitiveUType> list, FastStringBuffer fsb) {
+    private void format(LinkedList<PrimitiveUType> list, FastStringBuffer fsb, Function<ItemType, String> show) {
         if (list.size() == 1) {
             fsb.append(list.get(0).toItemType().toString());
         } else {
             fsb.append('(');
             fsb.append(list.removeFirst().toItemType().toString());
             fsb.append('|');
-            format(list, fsb);
-            fsb.append(')');
-        }
-    }
-
-    private void formatForExport(LinkedList<PrimitiveUType> list, FastStringBuffer fsb) {
-        if (list.size() == 1) {
-            fsb.append(list.get(0).toItemType().toExportString());
-        } else {
-            fsb.append('(');
-            fsb.append(list.removeFirst().toItemType().toExportString());
-            fsb.append('|');
-            format(list, fsb);
+            format(list, fsb, show);
             fsb.append(')');
         }
     }
@@ -184,14 +161,21 @@ public final class MultipleNodeKindTest extends NodeTest {
         return uType.hashCode();
     }
 
+    @Override
+    public boolean equals(Object obj) {
+        return obj instanceof MultipleNodeKindTest && uType.equals(((MultipleNodeKindTest)obj).uType);
+    }
+
     /**
      * Generate Javascript code to test whether an item conforms to this item type
      *
      * @return a Javascript instruction or sequence of instructions, which can be used as the body
      * of a Javascript function, and which returns a boolean indication whether the value of the
      * variable "item" is an instance of this item type.
-     * @param knownToBe
-     * @param targetVersion
+     * @param knownToBe An item type that the supplied item is known to conform to; the generated code
+     *                  can assume that the item is an instance of this type.
+     * @param targetVersion The version of Saxon-JS for which code is being generated. Currently either 1 or 2.
+
      */
     @Override
     public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) {
