@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,13 +21,12 @@ public interface PatternParser {
      *
      * @param pattern the pattern expressed as a String
      * @param env     the static context for the pattern
-     * @param lineNumber the line number of the pattern within some containing resource, for diagnostics
      * @return a Pattern object representing the result of parsing
      * @throws net.sf.saxon.trans.XPathException
      *          if the pattern contains a syntax error
      */
 
-    public Pattern parsePattern(String pattern, StaticContext env, int lineNumber) throws XPathException;
+    Pattern parsePattern(String pattern, StaticContext env) throws XPathException;
 
 
 }
