@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,6 +14,7 @@ import net.sf.saxon.functions.Doc;
 import net.sf.saxon.functions.KeyFn;
 import net.sf.saxon.functions.Root_1;
 import net.sf.saxon.functions.SuperId;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.AxisInfo;
 import net.sf.saxon.om.StructuredQName;
@@ -39,26 +40,27 @@ public class PatternParser30 extends XPathParser implements PatternParser {
      *
      * @param pattern the pattern expressed as a String
      * @param env     the static context for the pattern
-     * @param lineNumber
      * @return a Pattern object representing the result of parsing
      * @throws net.sf.saxon.trans.XPathException
      *          if the pattern contains a syntax error
      */
 
     /*@NotNull*/
-    public Pattern parsePattern(String pattern, StaticContext env, int lineNumber) throws XPathException {
+    public Pattern parsePattern(String pattern, StaticContext env) throws XPathException {
         this.env = env;
         charChecker = env.getConfiguration().getValidCharacterChecker();
         language = XSLT_PATTERN;
         String trimmed = pattern.trim();
         if (trimmed.startsWith("(:")) {
-            Tokenizer t = new Tokenizer();
+            t = new Tokenizer();
             t.languageLevel = 30;
             t.tokenize(trimmed, 0, -1);
+
             int start = t.currentTokenStartOffset;
             trimmed = trimmed.substring(start);
         }
-        if (trimmed.startsWith(".")) {
+        allowSaxonExtensions = env.getConfiguration().getBooleanProperty(Feature.ALLOW_SYNTAX_EXTENSIONS);
+        if (isSelectionPattern(trimmed)) {
             Expression e = parse(pattern, 0, Token.EOF, env);
             if (e instanceof ContextItemExpression) {
                 return new UniversalPattern();
@@ -81,14 +83,14 @@ public class PatternParser30 extends XPathParser implements PatternParser {
                             // let $P := predicate return if ($P instance of xs:numeric) then ($P eq 1) else $P
                             LetExpression let = new LetExpression();
                             StructuredQName varName =
-                                new StructuredQName("vv", NamespaceConstant.SAXON_GENERATED_VARIABLE, "v" + filter.hashCode());
+                                    new StructuredQName("vv", NamespaceConstant.SAXON_GENERATED_VARIABLE, "v" + filter.hashCode());
                             let.setVariableQName(varName);
                             InstanceOfExpression condition =
-                                new InstanceOfExpression(new LocalVariableReference(let), SequenceType.SINGLE_NUMERIC);
+                                    new InstanceOfExpression(new LocalVariableReference(let), SequenceType.SINGLE_NUMERIC);
                             LocalVariableReference ref = new LocalVariableReference(let);
                             ref.setStaticType(SequenceType.SINGLE_NUMERIC, null, 0);
                             ValueComparison comparison =
-                                new ValueComparison(ref, Token.FEQ, Literal.makeLiteral(Int64Value.PLUS_ONE));
+                                    new ValueComparison(ref, Token.FEQ, Literal.makeLiteral(Int64Value.PLUS_ONE));
                             Choose choice = new Choose(new Expression[]{condition, Literal.makeLiteral(BooleanValue.TRUE)},
                                                        new Expression[]{comparison, new LocalVariableReference(let)});
                             let.setSequence(filter);
@@ -129,7 +131,11 @@ public class PatternParser30 extends XPathParser implements PatternParser {
                 pat = PatternMaker.fromExpression(exp.simplify(), env.getConfiguration(), true);
             }
             if (exp instanceof FilterExpression && ((FilterExpression)exp).getBase() instanceof ContextItemExpression) {
-                grumble("A predicatePattern can appear only at the outermost level (parentheses not allowed)");
+                if (allowSaxonExtensions && (pattern.startsWith("tuple") || pattern.startsWith("map") || pattern.startsWith("array"))) {
+                    // no action, this is OK
+                } else {
+                    grumble("A predicatePattern can appear only at the outermost level (parentheses not allowed)");
+                }
             }
             if (exp instanceof FilterExpression && pat instanceof NodeTestPattern) {
                 // the pattern has been simplified but needs to retain a default priority based on its syntactic form (test match-058)
@@ -139,6 +145,22 @@ public class PatternParser30 extends XPathParser implements PatternParser {
         }
     }
 
+    private boolean isSelectionPattern(String pattern) throws XPathException {
+        if (pattern.startsWith(".")) {
+            return true;
+        }
+        if (pattern.startsWith("~")) {
+            checkSyntaxExtensions("type alias");
+            return true;
+        }
+        if (pattern.matches("^(tuple|map|array)\\s*\\(.+")) {
+            checkSyntaxExtensions("Patterns matching tuple, array, and map types");
+            return true;
+        }
+
+        return false;
+    }
+
     private void checkNoPredicatePattern(Expression exp) throws XPathException {
         if (exp instanceof ContextItemExpression) {
             grumble("A predicatePattern can appear only at the outermost level (union operator not allowed)");
@@ -170,8 +192,20 @@ public class PatternParser30 extends XPathParser implements PatternParser {
 
     /*@NotNull*/
     public Expression parseExpression() throws XPathException {
+        Tokenizer t = getTokenizer();
         if (inPredicate > 0) {
             return super.parseExpression();
+        } else if (allowSaxonExtensions && t.currentToken == Token.NODEKIND &&
+                (t.currentTokenValue.equals("tuple") || t.currentTokenValue.equals("map") || t.currentTokenValue.equals("array"))) {
+            ItemType type = parserExtension.parseExtendedItemType(this);
+            Expression expr = new InstanceOfExpression(
+                    new ContextItemExpression(), SequenceType.makeSequenceType(type, StaticProperty.EXACTLY_ONE));
+            expr = new FilterExpression(new ContextItemExpression(), expr);
+            setLocation(expr);
+            while (t.currentToken == Token.LSQB) {
+                expr = parsePredicate(expr);
+            }
+            return expr;
         } else {
             return parseBinaryExpression(parsePathExpression(), 10);
         }
@@ -218,6 +252,8 @@ public class PatternParser30 extends XPathParser implements PatternParser {
                         grumble("In an XSLT pattern, a function call is allowed only as the first step in a path");
                     }
                     return super.parseBasicStep(firstInPattern);
+                case Token.TILDE:
+                    return parserExtension.parseTildePattern(this);
                 default:
                     return super.parseBasicStep(firstInPattern);
 
