@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,7 +21,7 @@ public class UnionQNameTest implements QNameTest {
     List<QNameTest> tests;
 
     public UnionQNameTest(List<QNameTest> tests) {
-        this.tests = new ArrayList<QNameTest>(tests);
+        this.tests = new ArrayList<>(tests);
     }
 
     /**
@@ -66,7 +66,7 @@ public class UnionQNameTest implements QNameTest {
      * as the body of a JS function in which the argument name "q" is an
      * XdmQName object holding the name. The XdmQName object has properties
      * uri and local.
-     * @param targetVersion
+     * @param targetVersion the version of Saxon-JS being targeted
      */
     @Override
     public String generateJavaScriptNameTest(int targetVersion) {
