@@ -15,6 +15,7 @@ import net.sf.saxon.expr.ExpressionOwner;
 import net.sf.saxon.expr.PackageData;
 import net.sf.saxon.expr.XPathContextMajor;
 import net.sf.saxon.expr.instruct.Executable;
+import net.sf.saxon.expr.instruct.GlobalContextRequirement;
 import net.sf.saxon.expr.instruct.GlobalVariable;
 import net.sf.saxon.expr.instruct.SlotManager;
 import net.sf.saxon.expr.parser.*;
@@ -27,6 +28,8 @@ import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.ManualIterator;
+import net.sf.saxon.type.AnyItemType;
+import net.sf.saxon.type.ItemType;
 
 import javax.xml.transform.Result;
 import javax.xml.transform.Source;
@@ -73,7 +76,9 @@ public class XQueryExpression implements Location, ExpressionOwner {
             visitor.setOptimizeForStreaming(streaming);
             exp = exp.simplify();
             exp.checkForUpdatingSubexpressions();
-            ContextItemStaticInfo cit = config.makeContextItemStaticInfo(mainModule.getUserQueryContext().getRequiredContextItemType(), true);
+            GlobalContextRequirement contextReq = exec.getGlobalContextRequirement();
+            ItemType req = contextReq == null ? AnyItemType.getInstance() : contextReq.getRequiredItemType();
+            ContextItemStaticInfo cit = config.makeContextItemStaticInfo(req, true);
             Expression e2 = exp.typeCheck(visitor, cit);
             if (e2 != exp) {
                 e2.setRetainedStaticContext(exp.getRetainedStaticContext());
@@ -237,7 +242,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
      *         of items returned by the expression. Each item in the list will either be an
      *         object representing a node, or an object representing an atomic value.
      *         For the types of Java object that may be returned, see the description of the
-     *         {@link net.sf.saxon.xpath.XPathEvaluator#evaluate evaluate} method
+     *         {@link net.sf.saxon.xpath.XPathEvaluator#evaluate} method
      *         of class XPathProcessor
      * @throws XPathException if a dynamic error occurs during query evaluation
      */
@@ -261,7 +266,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
      * @return The first item in the sequence returned by the expression. If the expression
      *         returns an empty sequence, this method returns null. Otherwise, it returns the first
      *         item in the result sequence, represented as a Java object using the same mapping as for
-     *         the {@link XQueryExpression#evaluate evaluate} method
+     *         the {@link XQueryExpression#evaluate} method
      * @throws XPathException if evaluation fails with a dynamic error
      */
 
@@ -607,7 +612,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     public void explain(/*@NotNull*/ ExpressionPresenter out) throws XPathException {
         out.startElement("query");
-        getExecutable().getKeyManager().exportKeys(out, null);
+        mainModule.getKeyManager().exportKeys(out, null);
         getExecutable().explainGlobalVariables(out);
         mainModule.explainGlobalFunctions(out);
         out.startElement("body");
