@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -27,6 +27,6 @@ public interface XQueryFunctionBinder extends FunctionLibrary {
      */
 
     /*@Nullable*/
-    public XQueryFunction getDeclaration(StructuredQName functionName, int staticArgs);
+    XQueryFunction getDeclaration(StructuredQName functionName, int staticArgs);
 
 }
\ No newline at end of file
