@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,6 +7,8 @@
 
 package net.sf.saxon.regex;
 
+import net.sf.saxon.Configuration;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.StringValue;
@@ -32,10 +34,11 @@ public class ARegularExpression implements RegularExpression {
      * @param flags        the flags (ixsmq)
      * @param hostLanguage one of "XP20", "XP30", "XSD10", "XSD11". Also allow combinations, e.g. "XP20/XSD11".
      * @param warnings     a list to be populated with any warnings arising during compilation of the regex
+     * @param config       the Saxon Configuration: may be null
      * @throws XPathException if the regular expression is invalid
      */
 
-    public ARegularExpression(CharSequence pattern, String flags, String hostLanguage, List<String> warnings) throws XPathException {
+    public ARegularExpression(CharSequence pattern, String flags, String hostLanguage, List<String> warnings, Configuration config) throws XPathException {
         rawFlags = flags;
         REFlags reFlags;
         try {
@@ -53,6 +56,9 @@ public class ARegularExpression implements RegularExpression {
                     warnings.add(s);
                 }
             }
+            if (config != null) {
+                regex.setBacktrackingLimit(config.getConfigurationProperty(Feature.REGEX_BACKTRACKING_LIMIT));
+            }
         } catch (RESyntaxException err) {
             throw new XPathException(err.getMessage(), "FORX0002");
         }
@@ -89,7 +95,7 @@ public class ARegularExpression implements RegularExpression {
      * @param input the string to be tokenized
      * @return a SequenceIterator containing the resulting tokens, as objects of type StringValue
      */
-    public SequenceIterator tokenize(CharSequence input) {
+    public SequenceIterator<StringValue> tokenize(CharSequence input) {
         return new ATokenIterator(UnicodeString.makeUnicodeString(input), new REMatcher(regex));
     }
 
