@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,6 @@ package net.sf.saxon.regex;
 
 import net.sf.saxon.expr.LastPositionFinder;
 import net.sf.saxon.functions.Count;
-import net.sf.saxon.om.Item;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.value.StringValue;
@@ -71,7 +70,7 @@ public class JRegexIterator implements RegexIterator, LastPositionFinder {
      * @return the next item in the sequence
      */
 
-    public Item next() {
+    public StringValue next() {
         if (next == null && prevEnd >= 0) {
             // we've returned a match (or we're at the start), so find the next match
             if (matcher.find()) {
@@ -114,9 +113,6 @@ public class JRegexIterator implements RegexIterator, LastPositionFinder {
         return StringValue.makeStringValue(current);
     }
 
-    public void close() {
-    }
-
     /**
      * Get properties of this iterator, as a bit-significant integer.
      *
