@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,13 +13,15 @@ import net.sf.saxon.regex.charclass.CharacterClass;
 import net.sf.saxon.regex.charclass.EmptyCharacterClass;
 import net.sf.saxon.regex.charclass.IntSetCharacterClass;
 import net.sf.saxon.regex.charclass.SingletonCharacterClass;
+import net.sf.saxon.trans.UncheckedXPathException;
+import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.z.*;
 
 import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Stack;
+import java.util.function.IntPredicate;
 
 /**
  * Represents an operation or instruction in the regular expression program. The class Operation
@@ -27,6 +29,11 @@ import java.util.Stack;
  */
 public abstract class Operation {
 
+    public static final int MATCHES_ZLS_AT_START = 1;
+    public static final int MATCHES_ZLS_AT_END = 2;
+    public static final int MATCHES_ZLS_ANYWHERE = 7;
+    public static final int MATCHES_ZLS_NEVER = 1024;
+
     /**
      * Get an iterator returning all the matches for this operation
      *
@@ -59,11 +66,29 @@ public abstract class Operation {
     }
 
     /**
-     * Ask whether the regular expression is capable of matching a zero-length string
-     * @return true if the regex matches a zero-length string
+     * Ask whether the regular expression is known, after static analysis, to match a
+     * zero-length string
+     * @return a value indicating whether the regex is statically known to match
+     * a zero-length string. Specifically:
+     * <ul>
+     *     <li>returns {@link #MATCHES_ZLS_AT_START}
+     * if the expression is statically known to match a zero-length string at the
+     * start of the supplied input;</li>
+     * <li>returns
+     * {@link #MATCHES_ZLS_AT_END} if it is statically known to return a zero-length
+     * string at the end of the supplied input;</li>
+     * <li>returns {@link #MATCHES_ZLS_ANYWHERE}
+     * if it is statically known to match a zero-length string anywhere in the input.
+     * </li>
+     * <li>returns {@link #MATCHES_ZLS_NEVER} if it is statically known that the
+     * regex will never match a zero length string.</li>
+     * </ul>
+     * Returning 0 means that it is not known statically whether or not the regex
+     * will match a zero-length string; this case typically arises when back-references
+     * are involved.
      */
 
-    public abstract boolean matchesEmptyString();
+    public abstract int matchesEmptyString();
 
     /**
      * Ask whether the expression contains any capturing sub-expressions
@@ -144,13 +169,15 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
+        public int matchesEmptyString() {
+            int m = 0;
             for (Operation branch : branches) {
-                if (branch.matchesEmptyString()) {
-                    return true;
+                int b = branch.matchesEmptyString();
+                if (b != MATCHES_ZLS_NEVER) {
+                    m |= b;
                 }
             }
-            return false;
+            return m;
         }
 
         @Override
@@ -281,13 +308,54 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
+        public int matchesEmptyString() {
+
+            // The operation matches empty anywhere if every suboperation matches empty anywhere
+            boolean matchesEmptyAnywhere = true;
+            boolean matchesEmptyNowhere = false;
             for (Operation o : operations) {
-                if (!o.matchesEmptyString()) {
-                    return false;
+                int m = o.matchesEmptyString();
+                if (m == MATCHES_ZLS_NEVER) {
+                    return MATCHES_ZLS_NEVER;
+                }
+                if (m != MATCHES_ZLS_ANYWHERE) {
+                    matchesEmptyAnywhere = false;
+                    break;
                 }
             }
-            return true;
+
+            if (matchesEmptyAnywhere) {
+                return MATCHES_ZLS_ANYWHERE;
+            }
+
+            // The operation matches BOL if every suboperation matches BOL (which includes
+            // the case of matching empty anywhere)
+            boolean matchesBOL = true;
+            for (Operation o : operations) {
+                if ((o.matchesEmptyString() & MATCHES_ZLS_AT_START) == 0) {
+                    matchesBOL = false;
+                    break;
+                }
+            }
+            if (matchesBOL) {
+                return MATCHES_ZLS_AT_START;
+            }
+
+            // The operation matches EOL if every suboperation matches EOL (which includes
+            // the case of matching empty anywhere)
+            boolean matchesEOL = true;
+
+            for (Operation o : operations) {
+                if ((o.matchesEmptyString() & MATCHES_ZLS_AT_END) == 0) {
+                    matchesEOL = false;
+                    break;
+                }
+            }
+            if (matchesEOL) {
+                return MATCHES_ZLS_AT_END;
+            }
+
+            return 0;
         }
 
         @Override
@@ -305,7 +373,7 @@ public abstract class Operation {
             CharacterClass result = EmptyCharacterClass.getInstance();
             for (Operation o : operations) {
                 result = RECompiler.makeUnion(result, o.getInitialCharacterClass(caseBlind));
-                if (!o.matchesEmptyString()) {
+                if (o.matchesEmptyString() == MATCHES_ZLS_NEVER) {
                     return result;
                 }
             }
@@ -329,7 +397,7 @@ public abstract class Operation {
 
         @Override
         public Operation optimize(REProgram program, REFlags flags) {
-            if (operations.isEmpty()) {
+            if (operations.size() == 0) {
                 return new OpNothing();
             } else if (operations.size() == 1) {
                 return operations.get(0);
@@ -361,9 +429,10 @@ public abstract class Operation {
         public IntIterator iterateMatches(final REMatcher matcher, final int position) {
 
             // A stack of iterators, one for each piece in the sequence
-            final Stack<IntIterator> iterators = new Stack<IntIterator>();
+            final Stack<IntIterator> iterators = new Stack<>();
             final REMatcher.State savedState =
                     containsCapturingExpressions() ? matcher.captureState() : null;
+            final int backtrackingLimit = matcher.getProgram().getBacktrackingLimit();
 
             return new IntIterator() {
 
@@ -381,6 +450,7 @@ public abstract class Operation {
                  */
 
                 private int advance() {
+                    int counter = 0;
                     while (!iterators.isEmpty()) {
                         IntIterator top = iterators.peek();
                         while (top.hasNext()) {
@@ -394,6 +464,12 @@ public abstract class Operation {
                             iterators.push(top);
                         }
                         iterators.pop();
+                        if (backtrackingLimit >=0 && counter++ > backtrackingLimit) {
+                            throw new UncheckedXPathException(new XPathException(
+                                    "Regex backtracking limit exceeded processing " +
+                                            matcher.operation.display() + ". Simplify the regular expression, "
+                                            + "or set Feature.REGEX_BACKTRACKING_LIMIT to -1 to remove this limit." ));
+                        }
                     }
                     if (savedState != null) {
                         matcher.resetState(savedState);
@@ -441,8 +517,8 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return false;
+        public int matchesEmptyString() {
+            return MATCHES_ZLS_NEVER;
         }
 
         @Override
@@ -458,7 +534,7 @@ public abstract class Operation {
         public IntIterator iterateMatches(REMatcher matcher, int position) {
             //System.err.println("CharClass " + predicate + " matching at position " + position);
             UnicodeString in = matcher.search;
-            if (position < in.uLength() && predicate.matches(in.uCharAt(position))) {
+            if (position < in.uLength() && predicate.test(in.uCharAt(position))) {
                 return new IntSingletonIterator(position + 1);
             } else {
                 return EmptyIntIterator.getInstance();
@@ -520,8 +596,8 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return len == 0;
+        public int matchesEmptyString() {
+            return len == 0 ? MATCHES_ZLS_ANYWHERE : MATCHES_ZLS_NEVER;
         }
 
         @Override
@@ -590,8 +666,11 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return op.matchesEmptyString() || min == 0;
+        public int matchesEmptyString() {
+            if (min == 0) {
+                return MATCHES_ZLS_ANYWHERE;
+            }
+            return op.matchesEmptyString();
         }
 
         @Override
@@ -663,8 +742,11 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return op.matchesEmptyString() || min == 0;
+        public int matchesEmptyString() {
+            if (min == 0) {
+                return MATCHES_ZLS_ANYWHERE;
+            }
+            return op.matchesEmptyString();
         }
 
         @Override
@@ -707,7 +789,7 @@ public abstract class Operation {
 
 
     /**
-     * Handle a greedy repetition (with possible min and max) where the
+     * Handle a repetition (with possible min and max) where the
      * size of the repeated unit is variable.
      */
 
@@ -734,8 +816,11 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return min == 0 || op.matchesEmptyString();
+        public int matchesEmptyString() {
+            if (min == 0) {
+                return MATCHES_ZLS_ANYWHERE;
+            }
+            return op.matchesEmptyString();
         }
 
         @Override
@@ -761,7 +846,7 @@ public abstract class Operation {
         @Override
         public Operation optimize(REProgram program, REFlags flags) {
             op = op.optimize(program, flags);
-            if (min == 0 && op.matchesEmptyString()) {
+            if (min == 0 && op.matchesEmptyString() == MATCHES_ZLS_ANYWHERE) {
                 // turns (a?)* into (a?)+
                 min = 1;
             }
@@ -770,8 +855,8 @@ public abstract class Operation {
 
         @Override
         public IntIterator iterateMatches(final REMatcher matcher, int position) {
-            final Stack<IntIterator> iterators = new Stack<IntIterator>();
-            final Stack<Integer> positions = new Stack<Integer>();
+            final Stack<IntIterator> iterators = new Stack<>();
+            final Stack<Integer> positions = new Stack<>();
             final int bound = Math.min(max, matcher.search.uLength() - position + 1);
             int p = position;
             if (greedy) {
@@ -788,83 +873,94 @@ public abstract class Operation {
                         iterators.push(it);
                         positions.push(p);
                     } else if (iterators.isEmpty()) {
-                        //if (min == 0) {
-                        //    return new IntSingletonIterator(position);
-                        //} else {
-                            return EmptyIntIterator.getInstance();
-                        //}
+                        return EmptyIntIterator.getInstance();
                     } else {
                         break;
                     }
                 }
-            } else {
-                LinkedList<Integer> posSet = new LinkedList<Integer>();
-                posSet.add(p);
-                while (!posSet.isEmpty()) {
-                    int q = posSet.getFirst();
-                    IntIterator it = op.iterateMatches(matcher, q);
-                    while (it.hasNext()) {
-                        int q2 = it.next();
-                        if (q2 != q) {
-                            iterators.push(it);
-                            positions.push(q2);
-                            posSet.add(q2);
-                        }
-                    }
-                    posSet.removeFirst();
-                }
-            }
-            // Now return an iterator which returns all the matching positions in order
-            IntIterator base = new IntIterator() {
-                boolean primed = true;
+                // Now return an iterator which returns all the matching positions in order
+                IntIterator base = new IntIterator() {
+                    boolean primed = true;
 
-                /**
-                 * advance() moves to the next (potential) match position,
-                 * ignoring constraints on the minimum number of occurrences
-                 */
+                    /**
+                     * advance() moves to the next (potential) match position,
+                     * ignoring constraints on the minimum number of occurrences
+                     */
 
-                private void advance() {
-                    IntIterator top = iterators.peek();
-                    if (top.hasNext()) {
-                        int p = top.next();
-                        positions.pop();
-                        positions.push(p);
-                        while (iterators.size() < bound) {  // bug 3787
-                            IntIterator it = op.iterateMatches(matcher, p);
-                            if (it.hasNext()) {
-                                p = it.next();
-                                iterators.push(it);
-                                positions.push(p);
-                            } else {
-                                break;
+                    private void advance() {
+                        IntIterator top = iterators.peek();
+                        if (top.hasNext()) {
+                            int p = top.next();
+                            positions.pop();
+                            positions.push(p);
+                            while (iterators.size() < bound) {  // bug 3787
+                                IntIterator it = op.iterateMatches(matcher, p);
+                                if (it.hasNext()) {
+                                    p = it.next();
+                                    iterators.push(it);
+                                    positions.push(p);
+                                } else {
+                                    break;
+                                }
                             }
+                        } else {
+                            iterators.pop();
+                            positions.pop();
                         }
-                    } else {
-                        iterators.pop();
-                        positions.pop();
                     }
-                }
 
-                public boolean hasNext() {
-                    if (primed && iterators.size() >= min) {
-                        return !iterators.isEmpty();
-                    } else if (iterators.isEmpty()) {
-                        return false;
-                    } else {
+                    public boolean hasNext() {
+                        if (primed && iterators.size() >= min) {
+                            return !iterators.isEmpty();
+                        } else if (iterators.isEmpty()) {
+                            return false;
+                        } else {
+                            do {
+                                advance();
+                            } while (iterators.size() < min && !iterators.isEmpty());
+                            return !iterators.isEmpty();
+                        }
+                    }
+
+                    public int next() {
+                        primed = false;
+                        return positions.peek();
+                    }
+                };
+                return new ForceProgressIterator(base);
+            } else {
+                // reluctant (non-greedy) repeat.
+                // rewritten for bug 3902
+                IntIterator iter = new IntIterator() {
+                    private int pos = position;
+                    private int counter = 0;
+
+                    private void advance() {
+                        IntIterator it = op.iterateMatches(matcher, pos);
+                        if (it.hasNext()) {
+                            pos = it.next();
+                            if (++counter > max) {
+                                pos = -1;
+                            }
+                        } else if (min == 0 && counter == 0) {
+                            counter++;
+                        } else {
+                            pos = -1;
+                        }
+                    }
+                    public boolean hasNext() {
                         do {
                             advance();
-                        } while (iterators.size() < min && !iterators.isEmpty());
-                        return !iterators.isEmpty();
+                        } while (counter < min && pos >= 0);
+                        return pos >= 0;
                     }
-                }
-
-                public int next() {
-                    primed = false;
-                    return positions.peek();
-                }
-            };
-            return new ForceProgressIterator(base);
-            //return base;
+                    
+                    public int next() {
+                        return pos;
+                    }
+                };
+                return new ForceProgressIterator(iter);
+            }
         }
 
         /**
@@ -911,8 +1007,11 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return op.matchesEmptyString() || min == 0;
+        public int matchesEmptyString() {
+            if (min == 0) {
+                return MATCHES_ZLS_ANYWHERE;
+            }
+            return op.matchesEmptyString();
         }
 
         @Override
@@ -974,8 +1073,8 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return true;
+        public int matchesEmptyString() {
+            return MATCHES_ZLS_ANYWHERE;
         }
 
         @Override
@@ -1018,8 +1117,8 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return true;
+        public int matchesEmptyString() {
+            return MATCHES_ZLS_AT_START;
         }
 
         @Override
@@ -1061,8 +1160,8 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return true;
+        public int matchesEmptyString() {
+            return MATCHES_ZLS_AT_END;
         }
 
         @Override
@@ -1118,7 +1217,7 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
+        public int matchesEmptyString() {
             return childOp.matchesEmptyString();
         }
 
@@ -1184,9 +1283,19 @@ public abstract class Operation {
             this.groupNr = groupNr;
         }
 
+        /**
+         * Ask whether the regular expression is known, after static analysis, to match a
+         * zero-length string
+         *
+         * @return false. Returning true means that
+         * the expression is known statically to match ""; returning false means that this cannot
+         * be determined statically; it does not mean that the expression does not match "".
+         * We cannot do the analysis statically where back-references are involved, so we return false.
+         */
+
         @Override
-        public boolean matchesEmptyString() {
-            return false;
+        public int matchesEmptyString() {
+            return 0; // no information available
         }
 
         @Override
@@ -1257,8 +1366,8 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
-            return true;
+        public int matchesEmptyString() {
+            return MATCHES_ZLS_ANYWHERE;
         }
 
         @Override
@@ -1297,10 +1406,10 @@ public abstract class Operation {
             final int iterNr = counter++;
             String clName = baseIter.getClass().getName();
             int lastDot = clName.lastIndexOf(".");
-            String iterName = clName.substring(lastDot+1);
+            String iterName = clName.substring(lastDot + 1);
             System.err.println("Iterating over " + base.getClass().getSimpleName() + " " +
-                    base.display() + " at position " + position + " returning " +
-                    iterName + " " + iterNr);
+                                       base.display() + " at position " + position + " returning " +
+                                       iterName + " " + iterNr);
             return new IntIterator() {
                 public boolean hasNext() {
                     boolean b = baseIter.hasNext();
@@ -1322,7 +1431,7 @@ public abstract class Operation {
         }
 
         @Override
-        public boolean matchesEmptyString() {
+        public int matchesEmptyString() {
             return base.matchesEmptyString();
         }
 
