@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -35,6 +35,8 @@ import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.value.Whitespace;
 import net.sf.saxon.z.*;
 
+import java.util.function.IntPredicate;
+
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
@@ -137,7 +139,7 @@ public class RECompiler {
 
     private void warning(String s) {
         if (warnings == null) {
-            warnings = new ArrayList<String>(4);
+            warnings = new ArrayList<>(4);
         }
         warnings.add(s);
     }
@@ -184,7 +186,7 @@ public class RECompiler {
      */
 
     static Operation trace(Operation base) {
-        if (TRACING) {
+        if (TRACING && !(base instanceof Operation.OpTrace)) {
             return new Operation.OpTrace(base);
         } else {
             return base;
@@ -588,7 +590,9 @@ public class RECompiler {
                 // Actually create a range if the range is ok
                 if (rangeStart > rangeEnd) {
                     syntaxError("Bad character range: start > end");
-                    // TODO: not an error in XSD, merely a no-op?
+                    // Technically this is not an error in XSD, merely a no-op; but it is so
+                    // utterly pointless that it is almost certainly a mistake; and we have no
+                    // way of indicating warnings.
                 }
                 range.addRange(rangeStart, rangeEnd);
                 if (reFlags.isCaseIndependent()) {
@@ -692,7 +696,7 @@ public class RECompiler {
         IntSet is1 = p1.getIntSet();
         IntSet is2 = p2.getIntSet();
         if (is1 == null || is2 == null) {
-            return new PredicateCharacterClass(new IntUnionPredicate(p1, p2));
+            return new PredicateCharacterClass(p1.or(p2));
         } else {
             return new IntSetCharacterClass(is1.union(is2));
         }
@@ -883,18 +887,10 @@ public class RECompiler {
                 IntPredicate predicate;
                 if (reFlags.isSingleLine()) {
                     // in XPath with the 's' flag, '.' matches everything
-                    predicate = new IntPredicate() {
-                        public boolean matches(int value) {
-                            return true;
-                        }
-                    };
+                    predicate = IntSetPredicate.ALWAYS_TRUE;
                 } else {
                     // in XSD, "." matches everything except \n and \r. See also bug 15594.
-                    predicate = new IntPredicate() {
-                        public boolean matches(int value) {
-                            return value != '\n' && value != '\r';
-                        }
-                    };
+                    predicate = value -> value != '\n' && value != '\r';
                 }
                 return trace(new Operation.OpCharClass(predicate));
 
@@ -1007,7 +1003,7 @@ public class RECompiler {
                         quantifierType = 0;
                     }
                 }
-                if (ret.matchesEmptyString()) {
+                if (ret.matchesEmptyString() == Operation.MATCHES_ZLS_ANYWHERE) {
                     if (quantifierType == '?') {
                         // can ignore the quantifier
                         quantifierType = 0;
@@ -1117,7 +1113,7 @@ public class RECompiler {
         // Create open paren node unless we were called from the top level (which has no parens)
         int paren = -1;
         int group = 0;
-        List<Operation> branches = new ArrayList<Operation>();
+        List<Operation> branches = new ArrayList<>();
         int closeParens = capturingOpenParenCount;
         boolean capturing = true;
         if ((compilerFlags[0] & NODE_TOPLEVEL) == 0 && pattern.uCharAt(idx) == '(') {
@@ -1190,7 +1186,7 @@ public class RECompiler {
             l2.add(0, o1);
             return o2;
         } else {
-            List<Operation> list = new ArrayList<Operation>(4);
+            List<Operation> list = new ArrayList<>(4);
             list.add(o1);
             list.add(o2);
             return trace(new Operation.OpSequence(list));
@@ -1211,6 +1207,7 @@ public class RECompiler {
      */
     public REProgram compile(UnicodeString pattern) throws RESyntaxException {
         // Initialize variables for compilation
+        //System.err.println("Compiling regex " + pattern);
         this.pattern = pattern;                         // Save pattern in instance variable
         len = pattern.uLength();                         // Precompute pattern length for speed
         idx = 0;                                        // Set parsing index to the first character
