@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,9 +8,10 @@
 
 package net.sf.saxon.regex.charclass;
 
-import net.sf.saxon.z.IntPredicate;
 import net.sf.saxon.z.IntSet;
 
+import java.util.function.IntPredicate;
+
 /**
  * A character class represents a set of characters for regex matching purposes. A predicate
  * character class is one where the determination of membership is established by executing
@@ -25,8 +26,8 @@ public class PredicateCharacterClass implements CharacterClass {
         this.predicate = predicate;
     }
 
-    public boolean matches(int value) {
-        return predicate.matches(value);
+    public boolean test(int value) {
+        return predicate.test(value);
     }
 
     public boolean isDisjoint(CharacterClass other) {
