@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,6 +13,7 @@ import net.sf.saxon.functions.ResolveURI;
 import net.sf.saxon.functions.URIQueryParameters;
 import net.sf.saxon.lib.CollectionFinder;
 import net.sf.saxon.lib.ResourceCollection;
+import net.sf.saxon.lib.StandardCollectionURIResolver;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
 
@@ -25,7 +26,7 @@ import java.util.Map;
 /**
  * Default implementation of the CollectionFinder interface. The standard CollectionFinder recognizes four
  * types of collection:
- *
+ * <p>
  * <ol>
  *     <li>Any URI may be explicitly registered and associated with an instance of {@link ResourceCollection}</li>
  *     <li>If the file: URI scheme is used, and the relevant file identifies a directory, the directory
@@ -40,7 +41,7 @@ import java.util.Map;
 
 public class StandardCollectionFinder implements CollectionFinder {
 
-    private Map<String, ResourceCollection> registeredCollections = new HashMap<String, ResourceCollection>(2);
+    private Map<String, ResourceCollection> registeredCollections = new HashMap<>(2);
 
     /**
      * Register a specific URI and bind it to a specific ResourceCollection
@@ -61,16 +62,11 @@ public class StandardCollectionFinder implements CollectionFinder {
      *                      base URI
      * @return a ResourceCollection object representing the resources in the collection identified
      * by this collection URI
-     * @throws XPathException
+     * @throws XPathException if the collection URI cannot be resolved to a collection
      */
 
     public ResourceCollection findCollection(XPathContext context, String collectionURI) throws XPathException {
-        if (collectionURI == null) {
-            XPathException err = new XPathException("No default collection has been defined");
-            err.setErrorCode("FODC0002");
-            err.setXPathContext(context);
-            throw err;
-        }
+        checkNotNull(collectionURI, context);
 
         ResourceCollection registeredCollection = registeredCollections.get(collectionURI);
         if (registeredCollection != null) {
@@ -106,12 +102,7 @@ public class StandardCollectionFinder implements CollectionFinder {
 
         if ("file".equals(resolvedURI.getScheme())) {
             File file = new File(resolvedURI);
-            if (!file.exists()) {
-                XPathException err = new XPathException("The file or directory " + resolvedURI + " does not exist");
-                err.setErrorCode("FODC0002");
-                err.setXPathContext(context);
-                throw err;
-            }
+            StandardCollectionURIResolver.checkFileExists(file, resolvedURI, context);
             if (file.isDirectory()) {
                 return new DirectoryCollection(context.getConfiguration(), collectionURI, file, params);
             }
@@ -128,6 +119,22 @@ public class StandardCollectionFinder implements CollectionFinder {
         return new CatalogCollection(context.getConfiguration(), collectionURI);
     }
 
+    /**
+     * If the collectionURI is null, report that no default collection exists
+     * @param collectionURI the collection URI to be tested
+     * @param context XPath evaluation context
+     * @throws XPathException if the collectionURI is null
+     */
+
+    public static void checkNotNull(String collectionURI, XPathContext context) throws XPathException {
+        if (collectionURI == null) {
+            XPathException err = new XPathException("No default collection has been defined");
+            err.setErrorCode("FODC0002");
+            err.setXPathContext(context);
+            throw err;
+        }
+    }
+
     /**
      * Ask whether the collection URI should be interpreted as identifying a JAR (or ZIP) file.
      * This method is provided so that it can be overridden in subclasses. The default implementation
