@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -28,6 +28,7 @@ import java.io.InputStream;
  * @since 9.7
  */
 public class UnparsedTextResource implements Resource {
+    private String contentType;
     private String encoding;
     private String href = null;
     private String unparsedText = null;
@@ -35,20 +36,20 @@ public class UnparsedTextResource implements Resource {
 
     /**
      * Create an UnparsedTextResource
-     * @param config the Saxon Configuration (not currently used)
      * @param href the URI of the resource
      * @param details the input stream giving access to the content of the resource, plus encoding info
      */
 
-    public UnparsedTextResource(Configuration config, String href, AbstractResourceCollection.InputDetails details) {
+    public UnparsedTextResource(String href, AbstractResourceCollection.InputDetails details) {
         this.href = href;
         this.inputStream = details.inputStream;
+        this.contentType = details.contentType;
         this.encoding = details.encoding;
     }
 
     public final static ResourceFactory FACTORY = new ResourceFactory() {
         public Resource makeResource(Configuration config, String resourceURI, String contentType, AbstractResourceCollection.InputDetails details) throws XPathException {
-            return new UnparsedTextResource(config, resourceURI, details);
+            return new UnparsedTextResource(resourceURI, details);
         }
     };
 
@@ -56,6 +57,14 @@ public class UnparsedTextResource implements Resource {
         return href;
     }
 
+    public InputStream getInputStream() {
+        return inputStream;
+    }
+
+    public String getEncoding() {
+        return encoding;
+    }
+
     public Item getItem(XPathContext context) throws XPathException {
         if (unparsedText == null) {
             StringBuilder builder = null;
@@ -81,7 +90,7 @@ public class UnparsedTextResource implements Resource {
      */
 
     public String getContentType() {
-        return "text/plain";
+        return contentType == null ? "text/plain" : contentType;
     }
 
 
