@@ -19,6 +19,7 @@ import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.pattern.AnyNodeTest;
+import net.sf.saxon.pattern.NodeKindTest;
 import net.sf.saxon.pattern.NodeTest;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
@@ -76,16 +77,12 @@ public abstract class ItemType {
 
     public static ItemType ANY_FUNCTION = new ItemType() {
 
-        public ConversionRules getConversionRules() {
-            return defaultConversionRules;
-        }
-
         public boolean matches(XdmItem item) {
             return item.getUnderlyingValue() instanceof Function;
         }
 
         public boolean subsumes(ItemType other) {
-            return other.getUnderlyingItemType() instanceof FunctionItemType ;
+            return other.getUnderlyingItemType() instanceof FunctionItemType;
         }
 
         public net.sf.saxon.type.ItemType getUnderlyingItemType() {
@@ -99,10 +96,6 @@ public abstract class ItemType {
 
     public static final ItemType ANY_NODE = new ItemType() {
 
-        public ConversionRules getConversionRules() {
-            return defaultConversionRules;
-        }
-
         public boolean matches(XdmItem item) {
             return item.getUnderlyingValue() instanceof NodeInfo;
         }
@@ -116,16 +109,159 @@ public abstract class ItemType {
         }
     };
 
+
+    /**
+     * ItemType representing the ATTRIBUTE node() type
+     */
+
+    public static final ItemType ATTRIBUTE_NODE = new ItemType() {
+
+        public boolean matches(XdmItem item) {
+            Item<?> it = item.getUnderlyingValue();
+            return it instanceof NodeInfo && ((NodeInfo) it).getNodeKind() == Type.ATTRIBUTE;
+        }
+
+        public boolean subsumes(ItemType other) {
+            return other.getUnderlyingItemType().getUType() == UType.ATTRIBUTE;
+        }
+
+        public net.sf.saxon.type.ItemType getUnderlyingItemType() {
+            return NodeKindTest.ATTRIBUTE;
+        }
+    };
+
+
+    /**
+     * ItemType representing the COMMENT node() type
+     */
+
+    public static final ItemType COMMENT_NODE = new ItemType() {
+
+        public boolean matches(XdmItem item) {
+            Item<?> it = item.getUnderlyingValue();
+            return it instanceof NodeInfo && ((NodeInfo) it).getNodeKind() == Type.COMMENT;
+        }
+
+        public boolean subsumes(ItemType other) {
+            return other.getUnderlyingItemType().getUType() == UType.COMMENT;
+        }
+
+        public net.sf.saxon.type.ItemType getUnderlyingItemType() {
+            return NodeKindTest.COMMENT;
+        }
+    };
+
+
+    /**
+     * ItemType representing the TEXT node() type
+     */
+
+    public static final ItemType TEXT_NODE = new ItemType() {
+
+        public boolean matches(XdmItem item) {
+            Item<?> it = item.getUnderlyingValue();
+            return it instanceof NodeInfo && ((NodeInfo) it).getNodeKind() == Type.TEXT;
+        }
+
+        public boolean subsumes(ItemType other) {
+            return other.getUnderlyingItemType().getUType() == UType.TEXT;
+        }
+
+        public net.sf.saxon.type.ItemType getUnderlyingItemType() {
+            return NodeKindTest.TEXT;
+        }
+    };
+
+
+    /**
+     * ItemType representing the ELEMENT node() type
+     */
+
+    public static final ItemType ELEMENT_NODE = new ItemType() {
+
+        public boolean matches(XdmItem item) {
+            Item<?> it = item.getUnderlyingValue();
+            return it instanceof NodeInfo && ((NodeInfo) it).getNodeKind() == Type.ELEMENT;
+        }
+
+        public boolean subsumes(ItemType other) {
+            return other.getUnderlyingItemType().getUType() == UType.ELEMENT;
+        }
+
+        public net.sf.saxon.type.ItemType getUnderlyingItemType() {
+            return NodeKindTest.ELEMENT;
+        }
+    };
+
+    /**
+     * ItemType representing the DOCUMENT node() type
+     */
+
+    public static final ItemType DOCUMENT_NODE = new ItemType() {
+
+        public boolean matches(XdmItem item) {
+            Item<?> it = item.getUnderlyingValue();
+            return it instanceof NodeInfo && ((NodeInfo) it).getNodeKind() == Type.DOCUMENT;
+        }
+
+        public boolean subsumes(ItemType other) {
+            return other.getUnderlyingItemType().getUType() == UType.DOCUMENT;
+        }
+
+        public net.sf.saxon.type.ItemType getUnderlyingItemType() {
+            return NodeKindTest.DOCUMENT;
+        }
+    };
+
+
+    /**
+     * ItemType representing the NAMESPACE node() type
+     */
+
+    public static final ItemType NAMESPACE_NODE = new ItemType() {
+
+        public boolean matches(XdmItem item) {
+            Item<?> it = item.getUnderlyingValue();
+            return it instanceof NodeInfo && ((NodeInfo) it).getNodeKind() == Type.NAMESPACE;
+        }
+
+        public boolean subsumes(ItemType other) {
+            return other.getUnderlyingItemType().getUType() == UType.NAMESPACE;
+        }
+
+        public net.sf.saxon.type.ItemType getUnderlyingItemType() {
+            return NodeKindTest.NAMESPACE;
+        }
+    };
+
+
+    /**
+     * ItemType representing the PROCESSING_INSTRUCTION node() type
+     */
+
+    public static final ItemType PROCESSING_INSTRUCTION_NODE = new ItemType() {
+
+        public boolean matches(XdmItem item) {
+            Item<?> it = item.getUnderlyingValue();
+            return it instanceof NodeInfo && ((NodeInfo) it).getNodeKind() == Type.PROCESSING_INSTRUCTION;
+        }
+
+        public boolean subsumes(ItemType other) {
+            return other.getUnderlyingItemType().getUType() == UType.PI;
+        }
+
+        public net.sf.saxon.type.ItemType getUnderlyingItemType() {
+            return NodeKindTest.PROCESSING_INSTRUCTION;
+        }
+    };
+
+
     /**
      * ItemType representing the type map(*), that is, any map
      */
 
     public static final ItemType ANY_MAP = new ItemType() {
 
-        public ConversionRules getConversionRules() {
-            return defaultConversionRules;
-        }
-
         public boolean matches(XdmItem item) {
             return item.getUnderlyingValue() instanceof MapItem;
         }
@@ -145,10 +281,6 @@ public abstract class ItemType {
 
     public static final ItemType ANY_ARRAY = new ItemType() {
 
-        public ConversionRules getConversionRules() {
-            return defaultConversionRules;
-        }
-
         public boolean matches(XdmItem item) {
             return item.getUnderlyingValue() instanceof ArrayItem;
         }
@@ -169,10 +301,6 @@ public abstract class ItemType {
 
     public static final ItemType ANY_ATOMIC_VALUE = new ItemType() {
 
-        public ConversionRules getConversionRules() {
-            return defaultConversionRules;
-        }
-
         public boolean matches(XdmItem item) {
             return item.getUnderlyingValue() instanceof AtomicValue;
         }
@@ -519,7 +647,8 @@ public abstract class ItemType {
 
     /**
      * Create an ItemType representing a built-in atomic type.
-     * @param underlyingType the BuiltInAtomicType object representing this type
+     *
+     * @param underlyingType  the BuiltInAtomicType object representing this type
      * @param conversionRules the conversion rules to be used
      * @return the ItemType.
      */
@@ -563,7 +692,9 @@ public abstract class ItemType {
      */
 
     /*@Nullable*/
-    public abstract ConversionRules getConversionRules();
+    public ConversionRules getConversionRules() {
+        return defaultConversionRules;
+    }
 
     /**
      * Determine whether this item type matches a given item.
@@ -581,7 +712,7 @@ public abstract class ItemType {
      *
      * @param other the other ItemType
      * @return true if this ItemType subsumes the other ItemType. This includes the case where A and B
-     *         represent the same ItemType.
+     * represent the same ItemType.
      * @since 9.1
      */
 
@@ -598,6 +729,7 @@ public abstract class ItemType {
 
     /**
      * Get the name of the type, if it has one
+     *
      * @return the name of the type, or null if it is either an anonymous schema-defined type,
      * or an XDM-defined type such as node() or map().
      * @since 9.7
@@ -606,8 +738,8 @@ public abstract class ItemType {
     public QName getTypeName() {
         net.sf.saxon.type.ItemType type = getUnderlyingItemType();
         if (type instanceof SchemaType) {
-            StructuredQName name = ((SchemaType)type).getStructuredQName();
-            return name==null ? null : new QName(name);
+            StructuredQName name = ((SchemaType) type).getStructuredQName();
+            return name == null ? null : new QName(name);
         } else {
             return null;
         }
@@ -641,6 +773,7 @@ public abstract class ItemType {
      * XPath ItemType production, for example a QName (always in "Q{uri}local" format, or a construct
      * such as "node()" or "map(*)". If the type is an anonymous schema type, the name of the nearest
      * named base type will be given, preceded by the character "&lt;".
+     *
      * @return a string representation of the type
      * @since 9.7
      */
@@ -649,7 +782,7 @@ public abstract class ItemType {
         net.sf.saxon.type.ItemType type = getUnderlyingItemType();
         if (type instanceof SchemaType) {
             String marker = "";
-            SchemaType st = (SchemaType)type;
+            SchemaType st = (SchemaType) type;
             StructuredQName name = null;
             while (true) {
                 name = st.getStructuredQName();
@@ -669,6 +802,5 @@ public abstract class ItemType {
     }
 
 
-
 }
 
