@@ -10,14 +10,18 @@ package net.sf.saxon.s9api;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.parser.Token;
 import net.sf.saxon.lib.NamespaceConstant;
+import net.sf.saxon.ma.arrays.ArrayItem;
 import net.sf.saxon.ma.arrays.ArrayItemType;
+import net.sf.saxon.ma.map.MapItem;
 import net.sf.saxon.ma.map.MapType;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.pattern.*;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
+import net.sf.saxon.value.ExternalObject;
 import net.sf.saxon.value.ObjectValue;
 
 import java.util.Map;
@@ -242,19 +246,19 @@ public class ItemTypeFactory {
     public ItemType getNodeKindTest(XdmNodeKind kind) {
         switch (kind) {
             case DOCUMENT:
-                return new ConstructedItemType(NodeKindTest.DOCUMENT, processor);
+                return ItemType.DOCUMENT_NODE;
             case ELEMENT:
-                return new ConstructedItemType(NodeKindTest.ELEMENT, processor);
+                return ItemType.ELEMENT_NODE;
             case ATTRIBUTE:
-                return new ConstructedItemType(NodeKindTest.ATTRIBUTE, processor);
+                return ItemType.ATTRIBUTE_NODE;
             case TEXT:
-                return new ConstructedItemType(NodeKindTest.TEXT, processor);
+                return ItemType.TEXT_NODE;
             case COMMENT:
-                return new ConstructedItemType(NodeKindTest.COMMENT, processor);
+                return ItemType.COMMENT_NODE;
             case PROCESSING_INSTRUCTION:
-                return new ConstructedItemType(NodeKindTest.PROCESSING_INSTRUCTION, processor);
+                return ItemType.PROCESSING_INSTRUCTION_NODE;
             case NAMESPACE:
-                return new ConstructedItemType(NodeKindTest.NAMESPACE, processor);
+                return ItemType.NAMESPACE_NODE;
             default:
                 throw new IllegalArgumentException("XdmNodeKind");
         }
@@ -397,7 +401,7 @@ public class ItemTypeFactory {
     }
 
     /**
-     * Get an ItemType that tests an element name and/or schema type. This is the
+     * Get an ItemType that tests an attribute name and/or schema type. This is the
      * equivalent of the XPath syntax <code>element(element-name, type)</code>
      * <p>It is undefined whether two calls supplying the same argument values will
      * return the same ItemType object.</p>
@@ -533,14 +537,29 @@ public class ItemTypeFactory {
      */
 
     public XdmMap newMap(Map<?, ?> map) throws SaxonApiException {
-        return XdmMap.makeMap(map);
+        try {
+            return XdmMap.makeMap(map);
+        } catch (IllegalArgumentException e) {
+            throw new SaxonApiException(e);
+        }
     }
 
     /**
-     * Get an ItemType representing the type of a supplied XdmItem. If the supplied item is
+     * Get an ItemType representing the type of a supplied XdmItem.
+     *
+     * <p>If the supplied item is
      * an atomic value, the returned ItemType will reflect the most specific atomic type of the
-     * item. If the supplied item is a node, the returned item type will reflect the node kind,
-     * and if the node has a name, then its name. It will not reflect the type annotation.
+     * item.</p>
+     *
+     * <p>If the supplied item is a node, the returned item type will reflect the node kind,
+     * and if the node has a name, then its name. It will not reflect the type annotation.</p>
+     *
+     * <p>For a map, the result is {@link net.sf.saxon.s9api.ItemType#ANY_MAP}. For an array, the result is
+     * {@link net.sf.saxon.s9api.ItemType#ANY_ARRAY}. For any other function, it is {@link net.sf.saxon.s9api.ItemType#ANY_FUNCTION}.</p>
+     *
+     * <p>If the item is an external object, a suitable item type object is constructed.</p>
+     *
+     * <p>Future versions of Saxon may return a more precise type.</p>
      *
      * @param item the supplied item whose type is required
      * @return the type of the supplied item
@@ -549,10 +568,9 @@ public class ItemTypeFactory {
     public ItemType getItemType(XdmItem item) {
         if (item.isAtomicValue()) {
             AtomicValue value = (AtomicValue) item.getUnderlyingValue();
-
             AtomicType type = value.getItemType();
             return new ConstructedItemType(type, processor);
-        } else {
+        } else if (item.getUnderlyingValue() instanceof NodeInfo) {
             NodeInfo node = (NodeInfo) item.getUnderlyingValue();
             int kind = node.getNodeKind();
             if (node.getLocalPart().isEmpty()) {
@@ -560,8 +578,32 @@ public class ItemTypeFactory {
             } else {
                 return new ConstructedItemType(new SameNameTest(node), processor);
             }
+        } else {
+            Item it = item.getUnderlyingValue();
+            if (it instanceof MapItem) {
+                return ItemType.ANY_MAP;
+            } else if (it instanceof ArrayItem) {
+                return ItemType.ANY_ARRAY;
+            } else if (it instanceof ExternalObject) {
+                return new ConstructedItemType(ExternalObjectType.THE_INSTANCE, processor);
+            } else {
+                return ItemType.ANY_FUNCTION;
+            }
         }
     }
 
+    /**
+     * Obtain a s9api {@code ItemType} corresponding to an underlying instance of
+     * {@link net.sf.saxon.type.ItemType}
+     *
+     * @param it the supplied instance of {@link net.sf.saxon.type.ItemType}
+     * @return a corresponding instance of {@link ItemType}
+     * @since 10.0
+     */
+
+    public ItemType exposeItemType(net.sf.saxon.type.ItemType it) {
+        return new ConstructedItemType(it, processor);
+    }
+
 }
 
