@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,10 +7,12 @@
 
 package net.sf.saxon.s9api;
 
-import net.sf.saxon.Configuration;
 import net.sf.saxon.event.EventSource;
+import net.sf.saxon.event.PipelineConfiguration;
 import net.sf.saxon.event.Receiver;
+import net.sf.saxon.jaxp.ReceivingDestination;
 import net.sf.saxon.lib.InvalidityHandler;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.SchemaType;
 
@@ -23,25 +25,21 @@ import javax.xml.transform.Source;
  * cache maintained by the SchemaManager, together with any additional schema components located
  * during the course of validation by means of an xsl:schemaLocation or xsi:noNamespaceSchemaLocation
  * attribute within the instance document.
- * <p/>
  * <p>If validation fails, an exception is thrown. If validation succeeds, the validated document
  * can optionally be written to a specified destination. This will be a copy of the original document,
  * augmented with default values for absent elements and attributes, and carrying type annotations
  * derived from the schema processing. Expansion of defaults can be suppressed by means of the method
  * {@link #setExpandAttributeDefaults(boolean)}.</p>
- * <p/>
- * <p>A <tt>SchemaValidator</tt> is serially resuable but not thread-safe. That is, it should normally
+ * <p>A <tt>SchemaValidator</tt> is serially reusable but not thread-safe. That is, it should normally
  * be used in the thread where it is created, but it can be used more than once, to validate multiple
  * input documents.</p>
- * <p/>
  * <p>A <tt>SchemaValidator</tt> is a <tt>Destination</tt>, which allows it to receive the output of a
  * query or transformation to be validated.</p>
- * <p/>
  * <p>Saxon does not deliver the full PSVI as described in the XML schema specifications,
  * only the subset of the PSVI properties featured in the XDM data model.</p>
  */
 
-public abstract class SchemaValidator implements Destination {
+public abstract class SchemaValidator extends AbstractDestination {
 
     /**
      * The validation mode may be either strict or lax. The default is strict; this method may be called
@@ -155,7 +153,7 @@ public abstract class SchemaValidator implements Destination {
      * The setErrorListener, setInvalidityHandler, and setValidityReporting
      * are mutually exclusive - setting any one of them will cancel the others.
      * @param destination where XML will be sent
-     * @throws SaxonApiException
+     * @throws SaxonApiException if the destination is unsuitable
      */
     public abstract void setValidityReporting(Destination destination) throws SaxonApiException;
 
@@ -339,8 +337,7 @@ public abstract class SchemaValidator implements Destination {
      */
     public abstract void validateMultiple(Iterable<Source> sources) throws SaxonApiException;
 
-    public abstract Receiver getReceiver(Configuration config) throws SaxonApiException;
-
+//#if DOTNET == false
     /**
      * Construct a Source object that applies validation to an underlying input Source.
      * This method does not cause validation to take place immediately: the
@@ -348,7 +345,6 @@ public abstract class SchemaValidator implements Destination {
      * associated with this SchemaValidator is ignored; indeed, it is overwritten. After
      * calling this method, the {@code SchemaValidator} should not be used again for any
      * other purpose.
-     *
      * @param input the input Source to be validated
      * @return a Source object that reads the supplied input source and returns events
      * corresponding to the result of validating the input source. The Source object
@@ -357,27 +353,14 @@ public abstract class SchemaValidator implements Destination {
      * @since 9.9
      */
 
-    public Source asSource(final Source input) {
+    public Source asSource(Source input) {
         return new EventSource() {
             {
                 setSystemId(input.getSystemId());
             }
-
             @Override
-            public void send(final Receiver out) throws XPathException {
-                setDestination(new Destination() {
-                    public Receiver getReceiver(Configuration config) throws SaxonApiException {
-                        return out;
-                    }
-
-                    public void close() throws SaxonApiException {
-                        try {
-                            out.close();
-                        } catch (XPathException e) {
-                            throw new SaxonApiException(e);
-                        }
-                    }
-                });
+            public void send(Receiver out) throws XPathException {
+                setDestination(new ReceivingDestination(out));
                 try {
                     validate(input);
                 } catch (SaxonApiException e) {
@@ -386,7 +369,9 @@ public abstract class SchemaValidator implements Destination {
             }
         };
     }
+//#endif
 
+    public abstract Receiver getReceiver(PipelineConfiguration pipe, SerializationProperties params) throws SaxonApiException;
 
     /**
      * Close the destination, allowing resources to be released. Saxon calls this method when
