@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,6 +18,8 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.SchemaType;
 import net.sf.saxon.value.Whitespace;
 
+import java.util.function.Predicate;
+
 /**
  * WhitespaceStrippingPolicy is class defining the possible policies for handling
  * whitespace text nodes in a source document.
@@ -54,7 +56,8 @@ public class WhitespaceStrippingPolicy {
     public static final WhitespaceStrippingPolicy UNSPECIFIED = new WhitespaceStrippingPolicy(Whitespace.UNSPECIFIED);
 
     /**
-     * Create a custom whitespace stripping policy
+     * Create a custom whitespace stripping policy.
+     * <p>Changed in 9.9 to use the standard Java 8 Predicate class in place of Saxon's version.</p>
      * @param elementTest a predicate applied to element names, which should return true if whitespace-only
      *                    text node children of the element are to be stripped, false if they are to be retained.
      */
@@ -62,8 +65,8 @@ public class WhitespaceStrippingPolicy {
     public static WhitespaceStrippingPolicy makeCustomPolicy(final Predicate<QName> elementTest) {
         SpaceStrippingRule rule = new SpaceStrippingRule() {
             @Override
-            public int isSpacePreserving(NodeName nodeName, SchemaType schemaType) throws XPathException {
-                return elementTest.matches(new QName(nodeName.getStructuredQName()))
+            public int isSpacePreserving(NodeName nodeName, SchemaType schemaType) {
+                return elementTest.test(new QName(nodeName.getStructuredQName()))
                         ? Stripper.ALWAYS_STRIP
                         : Stripper.ALWAYS_PRESERVE;
             }
