@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.s9api;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.StaticContext;
+import net.sf.saxon.expr.parser.OptimizerOptions;
 import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.sxpath.IndependentContext;
 import net.sf.saxon.sxpath.XPathEvaluator;
@@ -21,28 +22,24 @@ import net.sf.saxon.value.SequenceType;
 
 import java.net.URI;
 import java.net.URISyntaxException;
-import java.text.Collator;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.Objects;
 import java.util.concurrent.ConcurrentHashMap;
 
 /**
  * An XPathCompiler object allows XPath queries to be compiled. The compiler holds information that
  * represents the static context for an XPath expression.
- * <p/>
  * <p>To construct an XPathCompiler, use the factory method
  * {@link Processor#newXPathCompiler}.</p>
- * <p/>
  * <p>An XPathCompiler may be used repeatedly to compile multiple
  * queries. Any changes made to the XPathCompiler (that is, to the
  * static context) do not affect queries that have already been compiled.
  * An XPathCompiler may be used concurrently in multiple threads, but
  * it should not then be modified once initialized.</p>
- * <p/>
  * <p>Changes to an XPathCompiler are cumulative. There is no simple way to reset
  * the XPathCompiler to its initial state; instead, simply create a new
  * XPathCompiler.</p>
- * <p/>
  * <p>The <code>XPathCompiler</code> has the ability to maintain a cache of compiled
  * expressions. This is active only if enabled by calling {@link #setCaching(boolean)}.
  * If caching is enabled, then the compiler will recognize an attempt to compile
@@ -61,8 +58,7 @@ public class XPathCompiler {
     private IndependentContext env;
     private ItemType requiredContextItemType;
 
-    /*@Nullable*/ private Map<String, XPathExecutable> cache = null;
-    //private Map<QName, DecimalSymbols> symbolsMap = new HashMap<QName, DecimalSymbols>();
+    private Map<String, XPathExecutable> cache = null;
 
     /**
      * Protected constructor
@@ -88,23 +84,18 @@ public class XPathCompiler {
     }
 
     /**
-     * Set whether XPath 1.0 backwards compatibility mode is to be used. In backwards compatibility
+     * Say whether XPath 1.0 backwards compatibility mode is to be used. In backwards compatibility
      * mode, more implicit type conversions are allowed in XPath expressions, for example it
      * is possible to compare a number with a string. The default is false (backwards compatibility
      * mode is off).
      *
      * @param option true if XPath 1.0 backwards compatibility is to be enabled, false if it is to
      *               be disabled.
-     * @throws SaxonApiException if backwards compatible processing is requested, and
-     *                           is not available in this Saxon Configuration (The feature requires a license
-     *                           for Saxon-PE or higher).
-     * @since 9.1; changed in 9.8 to throw IllegalStateException if the feature is not available
+     * @since 9.1; changed in 9.8 to throw IllegalStateException if the feature is not available; change
+     * reverted by bug 3817.
      */
 
-    public void setBackwardsCompatible(boolean option) throws IllegalStateException {
-        if (option && !processor.getUnderlyingConfiguration().isLicensedFeature(Configuration.LicenseFeature.PROFESSIONAL_EDITION)) {
-            throw new IllegalStateException("XPath 1.0 Compatibility requires a license for Saxon-PE or higher");
-        }
+    public void setBackwardsCompatible(boolean option) {
         if (cache != null) {
             cache.clear();
         }
@@ -123,17 +114,23 @@ public class XPathCompiler {
     }
 
     /**
-     * Say whether XPath expressions compiled using this XPathCompiler are
+     * Say whether XPath expressions compiled using this {@code XPathCompiler} are
      * schema-aware. They will automatically be schema-aware if the method
      * {@link #importSchemaNamespace(String)} is called. An XPath expression
      * must be marked as schema-aware if it is to handle typed (validated)
      * input documents.
      *
      * @param schemaAware true if expressions are to be schema-aware, false otherwise
-     * @since 9.3
+     * @throws UnsupportedOperationException if schema-awareness is requested when this is
+     * not a licensed Saxon-EE configuration.
+     * @since 9.3. Changed in 9.9 to throw an exception if Saxon-EE is not available (previously,
+     * the option was either silently ignored, or caused a failure at some later time.)
      */
 
     public void setSchemaAware(boolean schemaAware) {
+        if (schemaAware && !processor.getUnderlyingConfiguration().isLicensedFeature(Configuration.LicenseFeature.SCHEMA_VALIDATION)) {
+            throw new UnsupportedOperationException("Schema processing requires a licensed Saxon-EE configuration");
+        }
         env.setSchemaAware(schemaAware);
     }
 
@@ -155,11 +152,12 @@ public class XPathCompiler {
     /**
      * Say whether an XPath 2.0, XPath 3.0 or XPath 3.1 processor is required.
      *
-     * @param value Must be numerically equal to 1.0, 2.0, 3.1.
+     * @param value One of the values 1.0, 2.0, 3.0, 3.05, 3.1.
      *              <p>Setting the option to 1.0 requests an XPath 2.0 processor running in 1.0 compatibility mode;
      *              this is equivalent to setting the language version to 2.0 and backwards compatibility mode to true.
-     *              Requesting "3.0" gives an XPath 3.1 processor.</p>
-     * @throws IllegalArgumentException if the version is not numerically equal to 1.0, 2.0, 3.0 or 3.1.
+     *              Requesting "3.05" gives XPath 3.0 plus the extensions defined in the XSLT 3.0 specification
+     *              (map types and map constructors).</p>
+     * @throws IllegalArgumentException if the version is not numerically equal to 1.0, 2.0, 3.0, 3.05, or 3.1.
      * @since 9.3
      */
 
@@ -173,8 +171,8 @@ public class XPathCompiler {
             env.setBackwardsCompatibilityMode(true);
         } else if ("2.0".equals(value)) {
             version = 20;
-        } else if ("3.0".equals(value)) {
-            version = 31;
+        } else if ("3.0".equals(value) || "3.05".equals(value)) {
+            version = 30;
         } else if ("3.1".equals(value)) {
             version = 31;
         } else {
@@ -267,12 +265,10 @@ public class XPathCompiler {
     /**
      * Import a schema namespace: that is, add the element and attribute declarations and type definitions
      * contained in a given namespace to the static context for the XPath expression.
-     * <p/>
      * <p>This method will not cause the schema to be loaded. That must be done separately, using the
      * {@link SchemaManager}. This method will not fail if the schema has not been loaded (but in that case
      * the set of declarations and definitions made available to the XPath expression is empty). The schema
      * document for the specified namespace may be loaded before or after this method is called.</p>
-     * <p/>
      * <p>This method does not bind a prefix to the namespace. That must be done separately, using the
      * {@link #declareNamespace(String, String)} method.</p>
      *
@@ -322,8 +318,9 @@ public class XPathCompiler {
 
     /**
      * Declare a variable as part of the static context for XPath expressions compiled using this
-     * XPathCompiler. It is an error for the XPath expression to refer to a variable unless it has been
-     * declared. This method declares the existence of the variable, but it does not
+     * {@code XPathCompiler}. It is an error for the XPath expression to refer to a variable unless it has been
+     * declared, unless the method {@link #setAllowUndeclaredVariables(boolean)} has been called to permit
+     * undeclared variables. This method declares the existence of the variable, but it does not
      * bind any value to the variable; that is done later, when the XPath expression is evaluated.
      * The variable is allowed to have any type (that is, the required type is <code>item()*</code>).
      *
@@ -335,24 +332,22 @@ public class XPathCompiler {
             cache.clear();
         }
         env.declareVariable(qname.getNamespaceURI(), qname.getLocalName());
-        //declaredVariables.add(var);
     }
 
     /**
      * Declare a variable as part of the static context for XPath expressions compiled using this
-     * XPathCompiler. It is an error for the XPath expression to refer to a variable unless it has been
-     * declared. This method declares the existence of the variable, and defines the required type
+     * {@code XPathCompiler}. It is an error for the XPath expression to refer to a variable unless it has been
+     * declared, unless the method {@link #setAllowUndeclaredVariables(boolean)} has been called to permit
+     * undeclared variables. This method declares the existence of the variable, and defines the required type
      * of the variable, but it does not bind any value to the variable; that is done later,
      * when the XPath expression is evaluated.
      *
-     * @param qname       The name of the variable, expressed as a QName
-     * @param itemType    The required item type of the value of the variable
+     * @param qname       The name of the variable, expressed as a {@link QName}
+     * @param itemType    The required item type of the value of the variable, for example {@code ItemType.BOOLEAN}
      * @param occurrences The allowed number of items in the sequence forming the value of the variable
-     * @throws SaxonApiException if the requiredType is syntactically invalid or if it refers to namespace
-     *                           prefixes or schema components that are not present in the static context
      */
 
-    public void declareVariable(QName qname, ItemType itemType, OccurrenceIndicator occurrences) throws SaxonApiException {
+    public void declareVariable(QName qname, ItemType itemType, OccurrenceIndicator occurrences) {
         if (cache != null) {
             cache.clear();
         }
@@ -367,7 +362,7 @@ public class XPathCompiler {
      * is supplied when the query is invoked, then the context item must conform to this type (no
      * type conversion will take place to force it into this type).
      *
-     * @param type the required type of the context item
+     * @param type the required type of the context item, for example, {@link ItemType#ANY_NODE}.
      * @since 9.3
      */
 
@@ -378,7 +373,7 @@ public class XPathCompiler {
 
     /**
      * Get the required type of the context item. If no type has been explicitly declared for the context
-     * item, an instance of AnyItemType (representing the type item()) is returned.
+     * item, an instance of {@link ItemType#ANY_ITEM} (representing the type {@code item()}) is returned.
      *
      * @return the required type of the context item
      * @since 9.3
@@ -388,29 +383,12 @@ public class XPathCompiler {
         return requiredContextItemType;
     }
 
-    /**
-     * Bind a collation URI to a collation
-     *
-     * @param uri       the absolute collation URI
-     * @param collation a {@link Collator} object that implements the required collation
-     * @throws IllegalArgumentException if an attempt is made to rebind the standard URI
-     *                                  for the Unicode codepoint collation
-     * @since 9.4
-     * @deprecated since 9.6. Collations are now held globally at the level of the Processor
-     * (in fact, at the level of the Processor's underlying Configuration, which can in principle
-     * be shared by multiple Processors). If this method is called, the effect will be to reset
-     * the relevant collation at the global Configuration level.
-     */
-
-    public void declareCollation(String uri, final java.text.Collator collation) {
-        getProcessor().declareCollation(uri, collation);
-    }
-
     /**
      * Declare the default collation
      *
      * @param uri the absolute URI of the default collation. This URI must identify a known collation;
-     *            either one that has been explicitly declared, or one that is recognized implicitly, such as a UCA collation
+     *            either one that has been explicitly declared, or one that is recognized implicitly,
+     *            such as a UCA collation
      * @throws IllegalStateException if the collation URI is not recognized as a known collation
      * @since 9.4
      */
@@ -445,7 +423,7 @@ public class XPathCompiler {
     public void setCaching(boolean caching) {
         if (caching) {
             if (cache == null) {
-                cache = new ConcurrentHashMap<String, XPathExecutable>();
+                cache = new ConcurrentHashMap<>();
             }
         } else {
             cache = null;
@@ -463,10 +441,38 @@ public class XPathCompiler {
         return cache != null;
     }
 
+    /**
+     * Request fast compilation. Fast compilation will generally be achieved at the expense of run-time performance
+     * and quality of diagnostics. Fast compilation is a good trade-off if (a) the expression is known to be correct,
+     * and (b) once compiled, it is only executed once against a document of modest size.
+     * @param fast set to true to request fast compilation; set to false to revert to the optimization options
+     *             defined in the Configuration.
+     *
+     * @since 9.9
+     */
+
+    public void setFastCompilation(boolean fast) {
+        if (fast) {
+            env.setOptimizerOptions(new OptimizerOptions(0));
+        } else {
+            env.setOptimizerOptions(getProcessor().getUnderlyingConfiguration().getOptimizerOptions());
+        }
+    }
+
+    /**
+     * Ask if fast compilation has been enabled.
+     * @return true if fast compilation has been enabled
+     * @since 9.9
+     */
+
+    public boolean isFastCompilation() {
+        return env.getOptimizerOptions().getOptions() == 0;
+    }
+
     /**
      * Compile an XPath expression, supplied as a character string.
      *
-     * @param source A string containing the source text of the XPath expression
+     * @param source A non-null string containing the source text of the XPath expression
      * @return An XPathExecutable which represents the compiled xpath expression object.
      * The XPathExecutable may be run as many times as required, in the same or a different thread.
      * The XPathExecutable is not affected by any changes made to the XPathCompiler once it has been compiled.
@@ -476,6 +482,7 @@ public class XPathCompiler {
      */
 
     public XPathExecutable compile(String source) throws SaxonApiException {
+        Objects.requireNonNull(source);
         if (cache != null) {
             synchronized(this) {
                 XPathExecutable expr = cache.get(source);
@@ -522,11 +529,11 @@ public class XPathCompiler {
      * Compile and evaluate an XPath expression, supplied as a character string, with a given
      * context item.
      *
-     * @param expression  A string containing the source text of the XPath expression
+     * @param expression  A non-null string containing the source text of the XPath expression
      * @param contextItem The context item to be used for evaluating the expression. This
      *                    may be null if the expression is to be evaluated with no context item.
      * @return the result of evaluating the XPath expression with this context item. Note that
-     * the result is an iterable, so that it can be used in a construct such as
+     * the result is an {@link Iterable}, so that it can be used in a "for-each" loop such as
      * <code>for (XdmItem item : xpath.evaluate("//x", doc) {...}</code>
      * @throws SaxonApiException if any static error is detected while analyzing the expression,
      *                           or if any dynamic error is detected while evaluating it.
@@ -534,6 +541,7 @@ public class XPathCompiler {
      */
 
     public XdmValue evaluate(String expression, /*@Nullable*/ XdmItem contextItem) throws SaxonApiException {
+        Objects.requireNonNull(expression);
         XPathSelector selector = compile(expression).load();
         if (contextItem != null) {
             selector.setContextItem(contextItem);
@@ -546,7 +554,7 @@ public class XPathCompiler {
      * a single item, with a given context item. The expression is supplied as
      * a character string.
      *
-     * @param expression  A string containing the source text of the XPath expression
+     * @param expression  A non-null string containing the source text of the XPath expression
      * @param contextItem The context item to be used for evaluating the expression. This
      *                    may be null if the expression is to be evaluated with no context item.
      * @return the result of evaluating the XPath expression with this context item.
@@ -558,7 +566,8 @@ public class XPathCompiler {
      * @since 9.3
      */
 
-    public XdmItem evaluateSingle(String expression, /*@Nullable*/ XdmItem contextItem) throws SaxonApiException {
+    public XdmItem evaluateSingle(String expression, XdmItem contextItem) throws SaxonApiException {
+        Objects.requireNonNull(expression);
         XPathSelector selector = compile(expression).load();
         if (contextItem != null) {
             selector.setContextItem(contextItem);
@@ -572,7 +581,7 @@ public class XPathCompiler {
      * the pattern, and false if it does not. An error is reported if there is no context item or it the context
      * item is not a node.
      *
-     * @param source A string conforming to the syntax of XSLT 2.0 patterns
+     * @param source A non-null string conforming to the syntax of XSLT 2.0 patterns
      * @return An XPathExecutable representing an expression which evaluates to true when the context node matches
      * the pattern, and false when it does not.
      * @throws SaxonApiException if the pattern contains static errors: for example, if its syntax is incorrect,
@@ -581,6 +590,7 @@ public class XPathCompiler {
      */
 
     public XPathExecutable compilePattern(String source) throws SaxonApiException {
+        Objects.requireNonNull(source);
         try {
             env.getDecimalFormatManager().checkConsistency();
         } catch (XPathException e) {
@@ -599,7 +609,7 @@ public class XPathCompiler {
      * Registers the required decimal format properties
      *
      * @param format   The name of the decimal format
-     * @param property The decimal symbols name to update
+     * @param property The decimal symbols name to update, for example {@code "grouping-separator"}
      * @param value    The value to update the decimal symbol property
      * @throws SaxonApiException if there are two conflicting definitions of the named decimal-format
      * @since 9.4
@@ -614,30 +624,42 @@ public class XPathCompiler {
         DecimalSymbols symbols = dfm.obtainNamedDecimalFormat(format.getStructuredQName());
 
         try {
-            if (property.equals("decimal-separator")) {
-                symbols.setDecimalSeparator(value);
-            } else if (property.equals("grouping-separator")) {
-                symbols.setGroupingSeparator(value);
-            } else if (property.equals("exponent-separator")) {
-                symbols.setExponentSeparator(value);
-            } else if (property.equals("infinity")) {
-                symbols.setInfinity(value);
-            } else if (property.equals("NaN")) {
-                symbols.setNaN(value);
-            } else if (property.equals("minus-sign")) {
-                symbols.setMinusSign(value);
-            } else if (property.equals("percent")) {
-                symbols.setPercent(value);
-            } else if (property.equals("per-mille")) {
-                symbols.setPerMille(value);
-            } else if (property.equals("zero-digit")) {
-                symbols.setZeroDigit(value);
-            } else if (property.equals("digit")) {
-                symbols.setDigit(value);
-            } else if (property.equals("pattern-separator")) {
-                symbols.setPatternSeparator(value);
-            } else {
-                throw new IllegalArgumentException("Unknown decimal format attribute " + property);
+            switch (property) {
+                case "decimal-separator":
+                    symbols.setDecimalSeparator(value);
+                    break;
+                case "grouping-separator":
+                    symbols.setGroupingSeparator(value);
+                    break;
+                case "exponent-separator":
+                    symbols.setExponentSeparator(value);
+                    break;
+                case "infinity":
+                    symbols.setInfinity(value);
+                    break;
+                case "NaN":
+                    symbols.setNaN(value);
+                    break;
+                case "minus-sign":
+                    symbols.setMinusSign(value);
+                    break;
+                case "percent":
+                    symbols.setPercent(value);
+                    break;
+                case "per-mille":
+                    symbols.setPerMille(value);
+                    break;
+                case "zero-digit":
+                    symbols.setZeroDigit(value);
+                    break;
+                case "digit":
+                    symbols.setDigit(value);
+                    break;
+                case "pattern-separator":
+                    symbols.setPatternSeparator(value);
+                    break;
+                default:
+                    throw new IllegalArgumentException("Unknown decimal format attribute " + property);
             }
         } catch (XPathException e) {
             throw new SaxonApiException(e);
@@ -651,7 +673,6 @@ public class XPathCompiler {
      *
      * @return the underlying static context object. In the current implementation this will always
      * be an instance of {@link IndependentContext}.
-     * <p/>
      * <p>This method provides an escape hatch to internal Saxon implementation objects that offer a finer and
      * lower-level degree of control than the s9api classes and methods. Some of these classes and methods may change
      * from release to release.</p>
