@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,6 +8,7 @@
 package net.sf.saxon.s9api;
 
 import net.sf.saxon.Configuration;
+import net.sf.saxon.event.PipelineConfiguration;
 import net.sf.saxon.query.XQueryExpression;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
@@ -15,11 +16,9 @@ import net.sf.saxon.trans.XPathException;
 /**
  * An XQueryExecutable represents the compiled form of a query.
  * To execute the query, it must first be loaded to form an {@link net.sf.saxon.s9api.XQueryEvaluator}.
- * <p/>
  * <p>An XQueryExecutable is immutable, and therefore thread-safe.
  * It is simplest to load a new XQueryEvaluator each time the query is to be run.
  * However, the XQueryEvaluator is serially reusable within a single thread. </p>
- * <p/>
  * <p>An XQueryExecutable is created by using one of the <code>compile</code> methods on the
  * {@link net.sf.saxon.s9api.XQueryCompiler} class.</p>
  *
@@ -91,7 +90,7 @@ public class XQueryExecutable {
 
     /**
      * Produce a diagnostic representation of the compiled query, in XML form.
-     * <p><i>The detailed form of this representation is not stable (or even documented).<i></p>
+     * <p><i>The detailed form of this representation is not stable (or even documented).</i></p>
      *
      * @param destination the destination for the XML document containing the diagnostic representation
      *                    of the compiled stylesheet
@@ -101,7 +100,8 @@ public class XQueryExecutable {
     public void explain(Destination destination) throws SaxonApiException {
         Configuration config = processor.getUnderlyingConfiguration();
         try {
-            exp.explain(new ExpressionPresenter(config, destination.getReceiver(config)));
+            PipelineConfiguration pipe = config.makePipelineConfiguration();
+            exp.explain(new ExpressionPresenter(config, destination.getReceiver(pipe, config.obtainDefaultSerializationProperties())));
         } catch (XPathException e) {
             throw new SaxonApiException(e);
         }
