@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,13 +21,14 @@ import java.net.URI;
  * The value may belong to any of the 19 primitive types defined in XML Schema, or to a type
  * derived from these primitive types, or the XPath 2.0 type xs:untypedAtomic. The type may
  * be either a built-in type or a user-defined type.
- * <p/>
  * <p>An <code>XdmAtomicValue</code> is immutable.</p>
  */
 public class XdmAtomicValue extends XdmItem {
 
-    protected XdmAtomicValue(AtomicValue value) {
-        super(value);
+    private XdmAtomicValue(){};
+
+    protected XdmAtomicValue(AtomicValue value, boolean flag) {
+        setValue(value);
     }
 
     /**
@@ -37,7 +38,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(boolean value) {
-        super(BooleanValue.get(value));
+        this(BooleanValue.get(value), true);
     }
 
     /**
@@ -47,7 +48,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(long value) {
-        super(Int64Value.makeDerived(value, BuiltInAtomicType.LONG));
+        this(Int64Value.makeDerived(value, BuiltInAtomicType.LONG), true);
     }
 
     /**
@@ -57,7 +58,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(int value) {
-        super(Int64Value.makeDerived(value, BuiltInAtomicType.INT));
+        this(Int64Value.makeDerived(value, BuiltInAtomicType.INT), true);
     }
 
     /**
@@ -67,7 +68,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(short value) {
-        super(Int64Value.makeDerived(value, BuiltInAtomicType.SHORT));
+        this(Int64Value.makeDerived(value, BuiltInAtomicType.SHORT), true);
     }
 
     /**
@@ -77,7 +78,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(byte value) {
-        super(Int64Value.makeDerived(value, BuiltInAtomicType.BYTE));
+        this(Int64Value.makeDerived(value, BuiltInAtomicType.BYTE), true);
     }
 
     /**
@@ -87,7 +88,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(BigDecimal value) {
-        super(new BigDecimalValue(value));
+        this(new BigDecimalValue(value), true);
     }
 
     /**
@@ -97,7 +98,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(double value) {
-        super(new DoubleValue(value));
+        this(new DoubleValue(value), true);
     }
 
     /**
@@ -107,7 +108,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(float value) {
-        super(new FloatValue(value));
+        this(new FloatValue(value), true);
     }
 
     /**
@@ -117,7 +118,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(String value) {
-        super(new StringValue(value));
+        this(new StringValue(value), true);
     }
 
     /**
@@ -127,7 +128,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(URI value) {
-        super(new AnyURIValue(value.toString()));
+        this(new AnyURIValue(value.toString()), true);
     }
 
     /**
@@ -137,7 +138,7 @@ public class XdmAtomicValue extends XdmItem {
      */
 
     public XdmAtomicValue(QName value) {
-        super(new QNameValue(value.getStructuredQName(), BuiltInAtomicType.QNAME));
+        this(new QNameValue(value.getStructuredQName(), BuiltInAtomicType.QNAME), true);
     }
 
     /**
@@ -174,6 +175,8 @@ public class XdmAtomicValue extends XdmItem {
         }
     }
 
+
+
     /**
      * Attempt conversion from an arbitrary Java object
      * @param value the value to convert from. This must either be an instance of {@link AtomicValue}
@@ -186,7 +189,7 @@ public class XdmAtomicValue extends XdmItem {
 
     public static XdmAtomicValue makeAtomicValue(Object value) {
         if (value instanceof AtomicValue) {
-            return new XdmAtomicValue((AtomicValue)value);
+            return new XdmAtomicValue((AtomicValue)value, true);
         } else if (value instanceof Boolean) {
             return new XdmAtomicValue((Boolean)value);
         } else if (value instanceof Integer) {
@@ -208,7 +211,7 @@ public class XdmAtomicValue extends XdmItem {
         } else if (value instanceof BigDecimal) {
             return new XdmAtomicValue((BigDecimal) value);
         } else if (value instanceof BigInteger) {
-            return new XdmAtomicValue(IntegerValue.makeIntegerValue((BigInteger) value));
+            return new XdmAtomicValue(IntegerValue.makeIntegerValue((BigInteger) value), true);
         } else if (value instanceof URI) {
             return new XdmAtomicValue((URI) value);
         } else if (value instanceof QName) {
@@ -271,9 +274,7 @@ public class XdmAtomicValue extends XdmItem {
 
     /**
      * Get the value as a Java object of the nearest equivalent type.
-     * <p/>
      * <p>The result type is as follows:</p>
-     * <p/>
      * <table>
      * <tr><th>XPath type</th>      <th>Java class</th></tr>
      * <tr><td>xs:string</td>       <td>String</td></tr>
