@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,23 +8,30 @@
 package net.sf.saxon.s9api;
 
 import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.s9api.streams.XdmStream;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.iter.AxisIterator;
+import net.sf.saxon.tree.iter.SingletonIterator;
+import net.sf.saxon.tree.iter.UnfailingIterator;
+import net.sf.saxon.value.AtomicValue;
 
 import java.util.Iterator;
+import java.util.Spliterator;
+import java.util.Spliterators;
+import java.util.stream.Stream;
+import java.util.stream.StreamSupport;
 
 /**
  * An iterator over an XPath sequence.
- * <p/>
  * <p>This class implements the standard Java Iterator interface.</p>
- * <p/>
  * <p>Because the <code>Iterator</code> interface does not define any checked
  * exceptions, the <code>hasNext()</code> method of this iterator throws an unchecked
  * exception if a dynamic error occurs while evaluating the expression. Applications
  * wishing to control error handling should take care to catch this exception.</p>
  */
-public class XdmSequenceIterator implements Iterator<XdmItem> {
+public class XdmSequenceIterator<T extends XdmItem> implements Iterator<T> {
 
-    /*@Nullable*/ private XdmItem next = null;
+    /*@Nullable*/ private T next = null;
     private int state = BEFORE_ITEM;
     private SequenceIterator base;
 
@@ -37,6 +44,23 @@ public class XdmSequenceIterator implements Iterator<XdmItem> {
         this.state = BEFORE_ITEM;
     }
 
+    public XdmSequenceIterator(UnfailingIterator base) {
+        this.base = base;
+        this.state = BEFORE_ITEM;
+    }
+
+    public static XdmSequenceIterator<XdmNode> ofNodes(AxisIterator base) {
+        return new XdmSequenceIterator<>(base);
+    }
+
+    public static XdmSequenceIterator<XdmAtomicValue> ofAtomicValues(UnfailingIterator<? extends AtomicValue> base) {
+        return new XdmSequenceIterator<>(base);
+    }
+
+    protected static XdmSequenceIterator<XdmNode> ofNode(XdmNode node) {
+        return new XdmSequenceIterator<>(SingletonIterator.makeIterator(node.getUnderlyingNode()));
+    }
+
     /**
      * Returns <tt>true</tt> if the iteration has more elements. (In other
      * words, returns <tt>true</tt> if <tt>next</tt> would return an element
@@ -54,7 +78,7 @@ public class XdmSequenceIterator implements Iterator<XdmItem> {
                 return false;
             case BEFORE_ITEM:
                 try {
-                    next = XdmItem.wrapItem(base.next());
+                    next = (T)XdmItem.wrapItem(base.next());
                     if (next == null) {
                         state = FINISHED;
                         return false;
@@ -79,7 +103,7 @@ public class XdmSequenceIterator implements Iterator<XdmItem> {
      * @throws java.util.NoSuchElementException
      *          iteration has no more elements.
      */
-    public XdmItem next() {
+    public T next() {
         switch (state) {
             case ON_ITEM:
                 state = BEFORE_ITEM;
@@ -119,6 +143,27 @@ public class XdmSequenceIterator implements Iterator<XdmItem> {
 
     public void close() {
         base.close();
+        state = FINISHED;
     }
+
+
+    /**
+     * Convert this iterator to a Stream
+     *
+     * @return a Stream delivering the same items as this iterator
+     * @since 9.9
+     */
+    public XdmStream<T> stream() {
+        Stream<T> base = StreamSupport.stream(Spliterators.spliteratorUnknownSize(
+                this, Spliterator.ORDERED), false);
+        base = base.onClose(new Runnable() {
+            @Override
+            public void run() {
+                XdmSequenceIterator.this.close();
+            }
+        });
+        return new XdmStream<T>(base);
+    }
+
 }
 
