@@ -49,7 +49,7 @@ import java.util.stream.StreamSupport;
 
 public class XdmValue implements Iterable<XdmItem> {
 
-    private GroundedValue<? extends Item> value;
+    private GroundedValue<?> value;
 
     protected XdmValue() {
         // must be followed by setValue()
@@ -64,7 +64,7 @@ public class XdmValue implements Iterable<XdmItem> {
      */
 
     public XdmValue(Iterable<? extends XdmItem> items) {
-        List<Item> values = new ArrayList<>();
+        List<Item<?>> values = new ArrayList<>();
         for (XdmItem item : items) {
             values.add(item.getUnderlyingValue());
         }
@@ -81,7 +81,7 @@ public class XdmValue implements Iterable<XdmItem> {
 
     public XdmValue(Iterator<? extends XdmItem> iterator) throws SaxonApiException {
         try {
-            List<Item> values = new ArrayList<>();
+            List<Item<?>> values = new ArrayList<>();
             while (iterator.hasNext()) {
                 values.add(iterator.next().getUnderlyingValue());
             }
@@ -100,13 +100,13 @@ public class XdmValue implements Iterable<XdmItem> {
         this(stream.iterator());
     }
 
-    protected static XdmValue fromGroundedValue(GroundedValue<? extends Item> value) {
+    protected static XdmValue fromGroundedValue(GroundedValue<?> value) {
         XdmValue xv = new XdmValue();
         xv.setValue(value);
         return xv;
     }
 
-    protected void setValue(GroundedValue<? extends Item> value) {
+    protected void setValue(GroundedValue<?> value) {
         this.value = value;
     }
 
@@ -129,11 +129,11 @@ public class XdmValue implements Iterable<XdmItem> {
      * @since 9.5 (previously a protected method)
      */
 
-    public static XdmValue wrap(Sequence<? extends Item> value) {
+    public static XdmValue wrap(Sequence<?> value) {
         if (value == null) {
             return XdmEmptySequence.getInstance();
         }
-        GroundedValue<? extends Item> gv;
+        GroundedValue<?> gv;
         try {
             gv = value.materialize();
         } catch (XPathException e) {
@@ -187,14 +187,14 @@ public class XdmValue implements Iterable<XdmItem> {
      */
 
     public XdmValue append(XdmValue otherValue) {
-        List<Item> values = new ArrayList<>();
+        List<Item<?>> values = new ArrayList<>();
         for (XdmItem item : this) {
             values.add(item.getUnderlyingValue());
         }
         for (XdmItem item : otherValue) {
             values.add(item.getUnderlyingValue());
         }
-        GroundedValue<Item> gv = SequenceExtent.makeSequenceExtent(values);
+        GroundedValue<Item<?>> gv = SequenceExtent.makeSequenceExtent(values);
         return XdmValue.fromGroundedValue(gv);
     }
 
@@ -231,7 +231,7 @@ public class XdmValue implements Iterable<XdmItem> {
             throw new IndexOutOfBoundsException("" + n);
         }
         try {
-            Item item = SequenceTool.itemAt(value, n);
+            Item<?> item = SequenceTool.itemAt(value, n);
             return (XdmItem)XdmItem.wrap(item);
         } catch (XPathException e) {
             throw new SaxonApiUncheckedException(e);
@@ -247,7 +247,7 @@ public class XdmValue implements Iterable<XdmItem> {
      */
     public Iterator<XdmItem> iterator() throws SaxonApiUncheckedException {
         try {
-            Sequence v = getUnderlyingValue();
+            Sequence<?> v = getUnderlyingValue();
             return new XdmSequenceIterator<>(v.iterate());
         } catch (XPathException e) {
             throw new SaxonApiUncheckedException(e);
@@ -264,7 +264,7 @@ public class XdmValue implements Iterable<XdmItem> {
      * @return the underlying implementation object representing the value
      */
 
-    public GroundedValue<? extends Item> getUnderlyingValue() {
+    public GroundedValue<?> getUnderlyingValue() {
         return value;
     }
 
@@ -312,8 +312,8 @@ public class XdmValue implements Iterable<XdmItem> {
      * @throws IllegalArgumentException if conversion is not possible
      */
 
-    public static XdmValue makeSequence(Iterable list) throws IllegalArgumentException {
-        List<Item> result = new ArrayList<>();
+    public static XdmValue makeSequence(Iterable<?> list) throws IllegalArgumentException {
+        List<Item<?>> result = new ArrayList<>();
         for (Object o : list) {
             XdmValue v = XdmValue.makeValue(o);
             if (v instanceof XdmItem) {
@@ -350,7 +350,7 @@ public class XdmValue implements Iterable<XdmItem> {
 
     public static XdmValue makeValue(Object o) throws IllegalArgumentException {
         if (o instanceof Sequence) {
-            return XdmValue.wrap((Sequence) o);
+            return XdmValue.wrap((Sequence<?>) o);
         } else if (o instanceof XdmValue) {
             return (XdmValue) o;
         } else if (o instanceof Map) {
@@ -376,9 +376,9 @@ public class XdmValue implements Iterable<XdmItem> {
 
     public XdmValue documentOrder() throws SaxonApiException {
         try {
-            SequenceIterator iter = value.iterate();
-            SequenceIterator sorted = new DocumentOrderIterator(iter, GlobalOrderComparer.getInstance());
-            return XdmValue.fromGroundedValue(((SequenceIterator<Item>) sorted).materialize());
+            SequenceIterator<?> iter = value.iterate();
+            SequenceIterator<NodeInfo> sorted = new DocumentOrderIterator(iter, GlobalOrderComparer.getInstance());
+            return XdmValue.fromGroundedValue(sorted.materialize());
         } catch (XPathException e) {
             throw new SaxonApiException(e);
         }
