@@ -12,7 +12,6 @@ import net.sf.saxon.PreparedStylesheet;
 import net.sf.saxon.expr.parser.OptimizerOptions;
 import net.sf.saxon.lib.ErrorGatherer;
 import net.sf.saxon.lib.StringCollator;
-import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.style.*;
@@ -49,8 +48,16 @@ import java.util.Objects;
  *  rules might be detected not during execution of the <code>compile()</code> method, but rather
  *  when the relevant code is first executed. In this situation the <code>compile()</code> method will
  *  not throw an exception, but the errors will still (eventually) be notified to the <code>ErrorListener</code>
- *  associated with the compiler. This is problematic if the compiler is used to perform multiple compilations,
- *  because </p>
+ *  associated with the compiler.</p>
+ *  <p>To avoid problems with error reporting, it is recommended that:
+ *  <ul>
+ *      <li>When code is under development and static errors are therefore likely, a new
+ *      {@code XsltCompiler} should be created for each compilation (and JIT should preferably
+ *      be disabled, so that all static errors are detected.)</li>
+ *      <li>When production code is being loaded, and errors are therefore unlikely,
+ *      it is safe to use the same {@code XsltCompiler} for all compilations, and to
+ *      enable JIT for maximum performance.</li>
+ *  </ul>
  *
  * @since 9.0
  */
@@ -510,7 +517,7 @@ public class XsltCompiler {
                 compilation = new Compilation(config, new CompilerInfo(compilerInfo));
             }
             compilation.setLibraryPackage(true);
-            XsltPackage pack = new XsltPackage(processor, compilation.compilePackage(source).getStylesheetPackage());
+            XsltPackage pack = new XsltPackage(this, compilation.compilePackage(source).getStylesheetPackage());
             int errors = compilation.getErrorCount();
             if (errors > 0) {
                 String count = errors == 1 ? "one error" : errors + " errors";
@@ -582,7 +589,7 @@ public class XsltCompiler {
             IPackageLoader loader = processor.getUnderlyingConfiguration().makePackageLoader();
             if(loader != null){
                 StylesheetPackage pack = loader.loadPackage(input);
-                return new XsltPackage(processor, pack);
+                return new XsltPackage(this, pack);
             }
             throw new SaxonApiException("Loading library package requires Saxon PE or higher");
         } catch (XPathException e) {
@@ -691,7 +698,7 @@ public class XsltCompiler {
             PackageDetails details = getPackageLibrary().findPackage(packageName, pvr);
             if (details != null) {
                 if (details.loadedPackage != null) {
-                    return new XsltPackage(processor, details.loadedPackage);
+                    return new XsltPackage(this, details.loadedPackage);
                 } else if (details.sourceLocation != null) {
                     XsltPackage pack = compilePackage(details.sourceLocation);
                     details.loadedPackage = pack.getUnderlyingPreparedPackage();
@@ -719,7 +726,7 @@ public class XsltCompiler {
         }
         try {
             StylesheetPackage pack = getPackageLibrary().obtainLoadedPackage(details, new ArrayList<>());
-            return new XsltPackage(processor, pack);
+            return new XsltPackage(this, pack);
         } catch (XPathException e) {
             throw new SaxonApiException(e);
         }
@@ -803,6 +810,7 @@ public class XsltCompiler {
      * @param errorList a List (typically empty) to which information will be appended about
      *                  static errors found during the compilation. Each such error is represented by a
      *                  {@link StaticError} object.
+     * @since 9.9
      */
 
     public void setErrorList(List<StaticError> errorList) {
