@@ -16,12 +16,12 @@ import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.serialize.charcode.CharacterSet;
 import net.sf.saxon.serialize.charcode.UTF16CharacterSet;
 import net.sf.saxon.serialize.charcode.UTF8CharacterSet;
+import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
 
 import javax.xml.transform.OutputKeys;
 import javax.xml.transform.stream.StreamResult;
 import java.io.*;
-import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.Properties;
 
@@ -129,39 +129,19 @@ public abstract class Emitter extends SequenceReceiver implements ReceiverWithOu
         }
 
         try {
-            URI uri = new URI(uriString);
-            if (!uri.isAbsolute()) {
-                try {
-                    uri = new File(uriString).getAbsoluteFile().toURI();
-                } catch (Exception e) {
-                    // if we fail, we'll get another exception
-                }
-            }
-            File file = new File(uri);
-            try {
-                if ("file".equals(uri.getScheme()) && !file.exists()) {
-                    File directory = file.getParentFile();
-                    if (directory != null && !directory.exists()) {
-                        directory.mkdirs();
-                    }
-                    file.createNewFile();
-                }
-            } catch (IOException err) {
-                throw new XPathException("Failed to create output file " + uri, err);
-            }
+            File file = ExpandedStreamResult.makeWritableOutputFile(uriString);
             setOutputStream(new FileOutputStream(file));
             // Set the outputstream in the StreamResult object so that the
             // call on OutputURIResolver.close() can close it
             streamResult.setOutputStream(outputStream);
             mustClose = true;
-        } catch (FileNotFoundException fnf) {
-            throw new XPathException(fnf);
-        } catch (URISyntaxException use) {
-            throw new XPathException(use);
-        } catch (IllegalArgumentException iae) {
-            // for example, the system ID doesn't use the file: scheme
-            throw new XPathException(iae);
-        }
+        } catch (FileNotFoundException | URISyntaxException | IllegalArgumentException fnf) {
+            XPathException err = new XPathException("Unable to write to output destination", fnf);
+            err.setErrorCode(SaxonErrorCode.SXRD0004);
+            throw err;
+
+        } // for example, the system ID doesn't use the file: scheme
+
         return outputStream;
     }
 
