@@ -12,15 +12,15 @@ import net.sf.saxon.event.Receiver;
 import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.parser.Location;
-import net.sf.saxon.lib.NamespaceConstant;
+import net.sf.saxon.lib.SaxonOutputKeys;
 import net.sf.saxon.om.FingerprintedQName;
 import net.sf.saxon.om.NodeName;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.tiny.CharSlice;
 import net.sf.saxon.type.SchemaType;
 
-import java.util.Arrays;
-import java.util.HashSet;
+import java.util.*;
+
 
 /**
  * HTMLIndenter: This ProxyEmitter indents HTML elements, by adding whitespace
@@ -37,38 +37,19 @@ public class HTMLIndenter extends ProxyReceiver {
     // TODO: some of the logic in this class is probably redundant, e.g. the "sameLine" flag. However,
     // indentation is under-tested in the W3C test suites, so it's safest to avoid unnecessary changes.
 
-    private int level = 0;
+    // The specification has complex rules for deciding whether something is an inline element and whether
+    // indentation is suppressed: the rules for matching names depend on whether it's HTML or XHTML, and which
+    // version. But since the rules are of the form: "if the name matches X, then indentation is not allowed,
+    // otherwise indentation is allowed but not required", there is no harm in having spurious matches. So we
+    // simply do a case-blind match on the local part of the name, which catches all cases where indentation
+    // is not allowed, and is very unlikely to upset anyone by not indenting things that we could have indented.
 
-    protected char[] indentChars = {'\n', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '};
+    // We make one exception (see bug 3877): we don't treat "link" as an inline element under any circumstances,
+    // though HTML5 has some complex rules that treat it as a phrasal element under some conditions, based on the
+    // value of the "rel" attribute.
 
-    private boolean sameLine = false;
-    private boolean inFormattedTag = false;
-    private boolean afterInline = false;
-    private boolean afterFormatted = true;    // to prevent a newline at the start
-    private boolean afterEndElement = false;
-    /*@NotNull*/ private int[] propertyStack = new int[20];
-    private NameClassifier classifier;
-
-    /**
-     * The NameClassifier classifies element names according to whether the element is (a) an inline element,
-     * and/or (b) a formatted element
-     */
-
-    interface NameClassifier {
-
-        int IS_INLINE = 1;
-        int IS_FORMATTED = 2;
-
-        /**
-         * Classify an element name as inline, formatted, or both or neither.
-         * This method is overridden in the XHTML indenter
-         *
-         * @param name the element name
-         * @return a bit-significant integer containing flags IS_INLINE and/or IS_FORMATTED
-         */
-        int classifyTag(NodeName name);
-
-    }
+    final private static String[] formattedTags = {"pre", "script", "style", "textarea", "title", "xmp"};
+    // "xmp" is obsolete but still encountered!
 
     // When elements are classified as inline, indenting whitespace is not added adjacent to the element.
 
@@ -81,189 +62,104 @@ public class HTMLIndenter extends ProxyReceiver {
             "a", "abbr", "acronym", "applet", "area",
             "audio", "b", "basefont", "bdi", "bdo", "big", "br", "button", "canvas", "cite", "code", "data",
             "datalist", "del", "dfn", "em", "embed", "font", "i", "iframe", "img", "input", "ins",
-            "kbd", "label", "link", "map",
+            "kbd", "label", /*"link" -- excluded, see bug 3877,*/ "map",
             "mark", "math", "meter", "noscript", "object", "output", "picture",
             "progress", "q", "ruby", "s", "samp", "script", "select", "small", "span",
             "strike", "strong", "sub", "sup", "svg", "template", "textarea",
             "time", "tt", "u", "var", "video", "wbr"};
 
-    final static String[] formattedTags = {"pre", "script", "style", "textarea", "title", "xmp"};
-    // "xmp" is obsolete but still encountered!
-
-    /**
-     * Class to classify HTML names
-     */
-
-    static class HTMLNameClassifier implements NameClassifier {
-
-        final static HTMLNameClassifier THE_INSTANCE = new HTMLNameClassifier();
-
-        private static HTMLTagHashSet inlineTable = new HTMLTagHashSet(101);
-        private static HTMLTagHashSet formattedTable = new HTMLTagHashSet(23);
-
-        static {
-            for (String inlineTag : inlineTags) {
-                inlineTable.add(inlineTag);
-            }
-
-            for (String formattedTag : formattedTags) {
-                formattedTable.add(formattedTag);
-            }
-        }
-
-        /**
-         * Classify an element name as inline, formatted, or both or neither.
-         * This method is overridden in the XHTML indenter
-         *
-         * @param elemName the element name
-         * @return a bit-significant integer containing flags IS_INLINE and/or IS_FORMATTED
-         */
-        public int classifyTag(NodeName elemName) {
-            int r = 0;
-            String tag = elemName.getDisplayName();
-            if (inlineTable.contains(tag)) {
-                r |= IS_INLINE;
-            }
-            if (formattedTable.contains(tag)) {
-                r |= IS_FORMATTED;
-            }
-            return r;
-        }
-
+    final private static Set<String> inlineTable = new HashSet<>(70);
+    final private static Set<String> formattedTable = new HashSet<>(10);
 
+    static {
+        Collections.addAll(inlineTable, inlineTags);
+        Collections.addAll(formattedTable, formattedTags);
     }
 
-    /**
-     * Class to classify XHTML names for html-version 4
-     */
+    protected char[] indentChars = {'\n', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '};
 
-    static class XHTML4NameClassifier implements NameClassifier {
+    private final static int IS_INLINE = 1;
+    private final static int IS_FORMATTED = 2;
+    private final static int IS_SUPPRESSED = 4;
 
-        final static XHTML4NameClassifier THE_INSTANCE = new XHTML4NameClassifier();
-
-        private final static HashSet<NodeName> inlineTagSet;
-        private final static HashSet<NodeName> formattedTagSet;
-
-        static {
-            inlineTagSet = new HashSet<NodeName>(50);
-            formattedTagSet = new HashSet<NodeName>(10);
-            for (String inlineTag : inlineTags) {
-                String ns = NamespaceConstant.XHTML;
-                if (inlineTag.equals("math")) {
-                    ns = NamespaceConstant.MATH;
-                } else if (inlineTag.equals("svg")) {
-                    ns = NamespaceConstant.SVG;
-                }
-                inlineTagSet.add(new FingerprintedQName("", ns, inlineTag));
-            }
-            for (String formattedTag : formattedTags) {
-                formattedTagSet.add(new FingerprintedQName("", NamespaceConstant.XHTML, formattedTag));
-            }
-        }
-
-        /**
-         * Classify an element name as inline, formatted, or both or neither.
-         * This method is overridden in the XHTML indenter
-         *
-         * @param name the element name
-         * @return a bit-significant integer containing flags IS_INLINE and/or IS_FORMATTED
-         */
-
-        public int classifyTag(NodeName name) {
-            int r = 0;
-            if (inlineTagSet.contains(name)) {
-                r |= IS_INLINE;
-            }
-            if (formattedTagSet.contains(name)) {
-                r |= IS_FORMATTED;
-            }
-            return r;
-        }
-
-    }
-
-    /**
-     * Class to classify XHTML names for html-version 5
-     */
-
-    static class XHTML5NameClassifier implements NameClassifier {
-
-        final static XHTML5NameClassifier THE_INSTANCE = new XHTML5NameClassifier();
-
-        private static HTMLTagHashSet inlineTable = new HTMLTagHashSet(101);
-        private static HTMLTagHashSet formattedTable = new HTMLTagHashSet(23);
-        private static NodeName SVG = new FingerprintedQName("svg", NamespaceConstant.SVG, "svg");
-        private static NodeName MATH = new FingerprintedQName("math", NamespaceConstant.MATHML, "math");
-
-        static {
-            for (String inlineTag : inlineTags) {
-                inlineTable.add(inlineTag);
-            }
-
-            for (String formattedTag : formattedTags) {
-                formattedTable.add(formattedTag);
-            }
-        }
-
-        /**
-         * Classify an element name as inline, formatted, or both or neither.
-         * This method is overridden in the XHTML indenter
-         *
-         * @param name the element name
-         * @return a bit-significant integer containing flags IS_INLINE and/or IS_FORMATTED
-         */
-
-        public int classifyTag(NodeName name) {
-            if (name.hasURI(NamespaceConstant.XHTML) || name.hasURI("") || name.equals(SVG) || name.equals(MATH)) {
-                int r = 0;
-                if (inlineTable.contains(name.getLocalPart())) {
-                    r |= IS_INLINE;
-                }
-                if (formattedTable.contains(name.getLocalPart())) {
-                    r |= IS_FORMATTED;
-                }
-                return r;
-            } else {
-                return 0;
-            }
-        }
-
-    }
+    private String method;
+    private int level = 0;
+    private boolean sameLine = false;
+    private boolean inFormattedTag = false;
+    private boolean afterInline = false;
+    //private boolean afterFormatted = true;    // to prevent a newline at the start
 
+    private boolean afterEndElement = false;
+    private int[] propertyStack = new int[20];
+    private Set<String> suppressed = null;
 
     public HTMLIndenter(Receiver next, String method) {
         super(next);
-        if ("xhtml".equals(method)) {
-            classifier = XHTML4NameClassifier.THE_INSTANCE;
-        } else if ("xhtml5".equals(method)) {
-            classifier = XHTML5NameClassifier.THE_INSTANCE;
-        } else {
-            classifier = HTMLNameClassifier.THE_INSTANCE;
+    }
+
+    /**
+     * Set the properties for this indenter
+     *
+     * @param props the serialization properties
+     */
+
+    public void setOutputProperties(Properties props) {
+        String s = props.getProperty(SaxonOutputKeys.SUPPRESS_INDENTATION);
+        if (s != null) {
+            suppressed = new HashSet<>(8);
+            StringTokenizer st = new StringTokenizer(s, " \t\r\n");
+            while (st.hasMoreTokens()) {
+                String clarkName = st.nextToken();
+                suppressed.add(FingerprintedQName.fromClarkName(clarkName).getLocalPart().toLowerCase());
+            }
         }
     }
 
+    /**
+     * Classify an element name as inline, formatted, or both or neither.
+     * This method is overridden in the XHTML indenter
+     *
+     * @param name the element name
+     * @return a bit-significant integer containing flags IS_INLINE and/or IS_FORMATTED
+     */
+
+    public int classifyTag(NodeName name) {
+        int r = 0;
+        if (inlineTable.contains(name.getLocalPart().toLowerCase())) {
+            r |= IS_INLINE;
+        }
+        if (formattedTable.contains(name.getLocalPart().toLowerCase())) {
+            r |= IS_FORMATTED;
+        }
+        if (suppressed != null && suppressed.contains(name.getLocalPart().toLowerCase())) {
+            r |= IS_SUPPRESSED;
+        }
+        return r;
+    }
+
     /**
      * Output element start tag
      */
 
     public void startElement(NodeName nameCode, SchemaType typeCode, Location location, int properties) throws XPathException {
-        int tagProps = classifier.classifyTag(nameCode);
+        int withinSuppressed = level == 0 ? 0 : (propertyStack[level - 1] & IS_SUPPRESSED);
+        int tagProps = classifyTag(nameCode) | withinSuppressed;
         if (level >= propertyStack.length) {
             propertyStack = Arrays.copyOf(propertyStack, level * 2);
         }
         propertyStack[level] = tagProps;
-        boolean inlineTag = (tagProps & NameClassifier.IS_INLINE) != 0;
-        inFormattedTag = inFormattedTag || ((tagProps & NameClassifier.IS_FORMATTED) != 0);
-        if (!inlineTag && !inFormattedTag &&
-                !afterInline && !afterFormatted) {
+        boolean inlineTag = (tagProps & IS_INLINE) != 0;
+        if (!inlineTag && !inFormattedTag && !afterInline && /*!afterFormatted &&*/ withinSuppressed == 0 && level != 0) {
             indent();
         }
 
         nextReceiver.startElement(nameCode, typeCode, location, properties);
+
+        inFormattedTag = inFormattedTag || ((tagProps & IS_FORMATTED) != 0);
         level++;
         sameLine = true;
         afterInline = false;
-        afterFormatted = false;
+        //afterFormatted = false;
         afterEndElement = false;
     }
 
@@ -273,19 +169,17 @@ public class HTMLIndenter extends ProxyReceiver {
 
     public void endElement() throws XPathException {
         level--;
-        boolean thisInline = (propertyStack[level] & NameClassifier.IS_INLINE) != 0;
-        boolean thisFormatted = (propertyStack[level] & NameClassifier.IS_FORMATTED) != 0;
-        // See bug 3842. The test on afterEndElement is needed for conformance (indenting whitespace
-        // isn't allowed before the end tag) but the change is potentially disruptive so is being
-        // held back to the next major release.
-        if (/*afterEndElement &&*/ !thisInline && !thisFormatted && !afterInline &&
-                !sameLine && !afterFormatted && !inFormattedTag) {
+        boolean thisInline = (propertyStack[level] & IS_INLINE) != 0;
+        boolean thisFormatted = (propertyStack[level] & IS_FORMATTED) != 0;
+        boolean thisSuppressed = (propertyStack[level] & IS_SUPPRESSED) != 0;
+        if (afterEndElement && !thisInline && !thisSuppressed && !afterInline &&
+                !sameLine && !inFormattedTag) {
             indent();
             afterInline = false;
-            afterFormatted = false;
+            //afterFormatted = false;
         } else {
             afterInline = thisInline;
-            afterFormatted = thisFormatted;
+            //afterFormatted = thisFormatted;
         }
         nextReceiver.endElement();
         inFormattedTag = inFormattedTag && !thisFormatted;
@@ -326,16 +220,43 @@ public class HTMLIndenter extends ProxyReceiver {
         afterEndElement = false;
     }
 
+    /**
+     * Output a processing instruction
+     */
+
+    @Override
+    public void processingInstruction(String target, CharSequence data, Location locationId, int properties) throws XPathException {
+        boolean thisInline = (propertyStack[level] & IS_INLINE) != 0;
+        if (afterEndElement && !thisInline) {
+            indent();
+        }
+        nextReceiver.processingInstruction(target, data, locationId, properties);
+        afterEndElement = false;
+    }
+
     /**
      * Output a comment
      */
 
     public void comment(CharSequence chars, Location locationId, int properties) throws XPathException {
-        indent();
+        boolean thisInline = (propertyStack[level] & IS_INLINE) != 0;
+        if (afterEndElement && !thisInline) {
+            indent();
+        }
         nextReceiver.comment(chars, locationId, properties);
         afterEndElement = false;
     }
 
+    /**
+     * Get the maximum length of lines, after which long lines will be word-wrapped
+     *
+     * @return the maximum line length
+     */
+
+    protected int getLineLength() {
+        return 80;
+    }
+
     /**
      * Output white space to reflect the current indentation level
      *
@@ -368,15 +289,5 @@ public class HTMLIndenter extends ProxyReceiver {
         return 3;
     }
 
-    /**
-     * Get the maximum length of lines, after which long lines will be word-wrapped
-     *
-     * @return the maximum line length
-     */
-
-    protected int getLineLength() {
-        return 80;
-    }
-
 }
 
