@@ -264,9 +264,6 @@ public class Compilation {
             throw e;
         }
         CompilerInfo info = getCompilerInfo();
-//        if (info.getXsltVersion() == 0) {
-//            setProcessorVersion(getXPathVersion());
-//        }
         StyleNodeFactory factory = getStyleNodeFactory(true);
         PrincipalStylesheetModule psm = factory.newPrincipalModule(xslpackage);
         StylesheetPackage pack = psm.getStylesheetPackage();
@@ -315,13 +312,13 @@ public class Compilation {
 
 
         // Adjust visibility of components based on xsl:expose
-        if (getErrorCount() == 0) {
-            try {
-                psm.adjustExposedVisibility();
-            } catch (XPathException e) {
-                reportError(e);
-            }
-        }
+//        if (getErrorCount() == 0) {
+//            try {
+//                psm.adjustExposedVisibility();
+//            } catch (XPathException e) {
+//                reportError(e);
+//            }
+//        }
 
         // Compile the stylesheet package
         if (getErrorCount() == 0) {
