@@ -54,7 +54,6 @@ public class StylesheetPackage extends PackageData {
     private boolean createsSecondaryResultDocuments;
 
     private List<Action> completionActions = new ArrayList<>();
-    private boolean isRootPackage = true;
     protected GlobalContextRequirement globalContextRequirement = null;
     private boolean containsGlobalContextItemDeclaration = false;
     protected SpaceStrippingRule stripperRules;
@@ -64,6 +63,7 @@ public class StylesheetPackage extends PackageData {
     private StructuredQName defaultMode;
     private boolean declaredModes;
     protected Map<StructuredQName, Properties> namedOutputProperties = new HashMap<>(4);
+    private boolean implicitPackage;
 
     // table of imported schemas. The members of this set are strings holding the target namespace.
     protected Set<String> schemaIndex = new HashSet<>(10);
@@ -199,14 +199,24 @@ public class StylesheetPackage extends PackageData {
         return packageName;
     }
 
-    public void setRootPackage(boolean isRoot) {
-        isRootPackage = isRoot;
+    /**
+     * Ask if this is an implicit package (one rooted at xsl:stylesheet or xsl:transform rather than xsl:package
+     * @return true if this package was rooted at an xsl:stylesheet or xsl:transform element
+     */
+    public boolean isImplicitPackage() {
+        return implicitPackage;
     }
 
-    public boolean isRootPackage() {
-        return isRootPackage;
+    /**
+     * Say whether this is an implicit package (one rooted at xsl:stylesheet or xsl:transform rather than xsl:package
+     * @param implicitPackage true if this package was rooted at an xsl:stylesheet or xsl:transform element
+     */
+
+    public void setImplicitPackage(boolean implicitPackage) {
+        this.implicitPackage = implicitPackage;
     }
 
+
     /**
      * Ask whether the package was compiled with just-in-time compilation of template rules enabled
      *
@@ -642,9 +652,9 @@ public class StylesheetPackage extends PackageData {
                 Visibility acceptedVisibility = explicitAcceptedVisibility(name, acceptors);
                 if (acceptedVisibility != null) {
                     if (!XSLAccept.isCompatible(oldV, acceptedVisibility)) {
-                        throw new XPathException("Cannot accept a " + oldV +
-                            " component with visibility " + acceptedVisibility +
-                        " (" + name + ")", "XTSE3040");
+                        throw new XPathException("Cannot accept a " + oldV.show() +
+                            " component (" + name + ") from package " + usedPackage.getPackageName()
+                                + " with visibility " + acceptedVisibility.show(), "XTSE3040");
                     }
                     newV = acceptedVisibility;
                 } else {
@@ -665,9 +675,9 @@ public class StylesheetPackage extends PackageData {
                 }
             }
 
-            trace(oldC.getActor().getSymbolicName() + " (" + oldV + ") becomes " + newV);
+            trace(oldC.getActor().getSymbolicName() + " (" + oldV.show() + ") becomes " + newV.show());
 
-            final Component newC = Component.makeComponent(oldC.getActor(), newV, this, oldC.getDeclaringPackage());
+            final Component newC = Component.makeComponent(oldC.getActor(), newV, VisibilityProvenance.DERIVED, this, oldC.getDeclaringPackage());
             correspondence.put(oldC, newC);
             newC.setBaseComponent(oldC);
 
@@ -740,7 +750,7 @@ public class StylesheetPackage extends PackageData {
 
             trace(oldC.getActor().getSymbolicName() + " (HIDDEN, declared in " + oldC.getDeclaringPackage().getPackageName() + ") becomes HIDDEN");
 
-            final Component newC = Component.makeComponent(oldC.getActor(), Visibility.HIDDEN, this, oldC.getDeclaringPackage());
+            final Component newC = Component.makeComponent(oldC.getActor(), Visibility.HIDDEN, VisibilityProvenance.DERIVED, this, oldC.getDeclaringPackage());
             correspondence.put(oldC, newC);
             newC.setBaseComponent(oldC);
 
@@ -829,12 +839,14 @@ public class StylesheetPackage extends PackageData {
 
     public void createFunctionLibrary() {
 
+        String target = getTargetEdition();
         FunctionLibraryList functionLibrary = new FunctionLibraryList();
-        functionLibrary.addFunctionLibrary(config.getXSLT30FunctionSet());
+        boolean includeHOF = !("HE".equals(target) || "JS".equals(target));
+        functionLibrary.addFunctionLibrary(includeHOF ? config.getXSLT30FunctionSet() : new Configuration().getXSLT30FunctionSet());
         functionLibrary.addFunctionLibrary(new StylesheetFunctionLibrary(this, true));
         functionLibrary.addFunctionLibrary(config.getBuiltInExtensionLibraryList());
         functionLibrary.addFunctionLibrary(new ConstructorFunctionLibrary(config));
-        if ("JS".equals(getTargetEdition()) || "JS2".equals(getTargetEdition())) {
+        if ("JS".equals(target) || "JS2".equals(target)) {
             addIxslFunctionLibrary(functionLibrary);
         }
 
@@ -842,7 +854,7 @@ public class StylesheetPackage extends PackageData {
         functionLibrary.addFunctionLibrary(queryFunctions);
         functionLibrary.addFunctionLibrary(config.getIntegratedFunctionLibrary());
         config.addExtensionBinders(functionLibrary);
-
+        functionLibrary.addFunctionLibrary(new StylesheetFunctionLibrary(this, false));
         this.functionLibrary = functionLibrary;
     }
 
@@ -978,8 +990,6 @@ public class StylesheetPackage extends PackageData {
         }
         pss.setFunctionLibrary(libraryList);
 
-        pss.setKeyManager(keyManager);
-
         if (!pss.createsSecondaryResult()) {
             pss.setCreatesSecondaryResult(mayCreateSecondaryResultDocuments());
         }
