@@ -24,6 +24,7 @@ import net.sf.saxon.expr.parser.ExpressionTool;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.expr.parser.Optimizer;
 import net.sf.saxon.expr.parser.RoleDiagnostic;
+import net.sf.saxon.functions.AccumulatorFn;
 import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
@@ -174,7 +175,7 @@ public class XSLAccumulator extends StyleElement implements StylesheetComponent
 
         {
             accumulator.setPackageData(compilation.getPackageData());
-            accumulator.makeDeclaringComponent(Visibility.PRIVATE, getContainingPackage());
+            accumulator.obtainDeclaringComponent(decl.getSourceElement());
             Expression init = accumulator.getInitialValueExpression();
             ExpressionVisitor visitor = ExpressionVisitor.make(getStaticContext());
             init = init.typeCheck(visitor, config.getDefaultContextItemStaticInfo());
@@ -208,7 +209,7 @@ public class XSLAccumulator extends StyleElement implements StylesheetComponent
                 boolean isPreDescent = !rule.isPostDescent();
                 SimpleMode mode = isPreDescent ? accumulator.getPreDescentRules() : accumulator.getPostDescentRules();
                 AccumulatorRule action = new AccumulatorRule(newValueExp, stackFrameMap, rule.isPostDescent());
-                mode.addRule(pattern, action, decl.getModule(), decl.getModule().getPrecedence(), 1, true);
+                mode.addRule(pattern, action, decl.getModule(), decl.getModule().getPrecedence(), 1, 0, 0);
 
                 checkRuleStreamability(rule, pattern, newValueExp);
 
@@ -374,7 +375,8 @@ public class XSLAccumulator extends StyleElement implements StylesheetComponent
             }
             ContextItemStaticInfoEE csi = (ContextItemStaticInfoEE)getConfiguration().makeContextItemStaticInfo(pattern.getItemType(), false);
             csi.setContextPostureStriding();
-            List<String> reasons = new ArrayList<String>(4);
+            csi.setAccumulatorPhase(rule.isPostDescent() ? AccumulatorFn.Phase.AFTER : AccumulatorFn.Phase.BEFORE);
+            List<String> reasons = new ArrayList<>(4);
             PostureAndSweep ps = Streamability.getStreamability(newValueExp, csi, reasons);
             if (!ps.equals(PostureAndSweep.GROUNDED_AND_MOTIONLESS) && !rule.isCapture()) {
                 // saxon:capture="yes" captures a snapshot of the element for access in the phase="end" rule, which
