@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -60,35 +60,41 @@ public class XSLApplyTemplates extends StyleElement {
 
         for (int a = 0; a < atts.getLength(); a++) {
             String f = atts.getQName(a);
-            if (f.equals("mode")) {
-                modeAttribute = Whitespace.trim(atts.getValue(a));
-            } else if (f.equals("select")) {
-                selectAtt = atts.getValue(a);
-                select = makeExpression(selectAtt, a);
-                defaultedSelectExpression = false;
-            } else {
-                checkUnknownAttribute(atts.getNodeName(a));
+            switch (f) {
+                case "mode":
+                    modeAttribute = Whitespace.trim(atts.getValue(a));
+                    break;
+                case "select":
+                    selectAtt = atts.getValue(a);
+                    select = makeExpression(selectAtt, a);
+                    defaultedSelectExpression = false;
+                    break;
+                default:
+                    checkUnknownAttribute(atts.getNodeName(a));
+                    break;
             }
         }
 
         if (modeAttribute != null) {
-            if (modeAttribute.equals("#current")) {
-                useCurrentMode = true;
-            } else if (modeAttribute.equals("#unnamed")) {
-                modeName = Mode.UNNAMED_MODE_NAME;
-            } else if (modeAttribute.equals("#default")) {
-                // do nothing;
-            } else {
-                try {
-                    modeName = makeQName(modeAttribute);
-                } catch (NamespaceException err) {
-                    compileError(err.getMessage(), "XTSE0280");
-                    modeName = null;
-                } catch (XPathException err) {
-                    compileError("Mode name " + Err.wrap(modeAttribute) + " is not a valid QName",
-                            err.getErrorCodeQName());
-                    modeName = null;
-                }
+            switch (modeAttribute) {
+                case "#current":
+                    useCurrentMode = true;
+                    break;
+                case "#unnamed":
+                    modeName = Mode.UNNAMED_MODE_NAME;
+                    break;
+                case "#default":
+                    // do nothing;
+                    break;
+                default:
+                    try {
+                        modeName = makeQName(modeAttribute);
+                    } catch (XPathException err) {
+                        compileError("Mode name " + Err.wrap(modeAttribute) + " is not a valid QName",
+                                     err.getErrorCodeQName());
+                        modeName = null;
+                    }
+                    break;
             }
         }
     }
