@@ -119,7 +119,7 @@ public final class XSLCopyOf extends StyleElement {
     public Expression compile(Compilation exec, ComponentDeclaration decl) {
         CopyOf inst = new CopyOf(select, copyNamespaces, validation, schemaType, false);
         inst.setCopyAccumulators(copyAccumulators);
-        inst.setCopyLineNumbers(exec.getConfiguration().isLineNumbering());
+        //inst.setCopyLineNumbers(exec.getConfiguration().isLineNumbering());
         inst.setSchemaAware(exec.isSchemaAware());
         return inst;
     }
