@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,6 @@ package net.sf.saxon.style;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.om.AttributeCollection;
-import net.sf.saxon.om.NamespaceException;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.trans.DecimalFormatManager;
 import net.sf.saxon.trans.DecimalSymbols;
@@ -63,32 +62,46 @@ public class XSLDecimalFormat extends StyleElement {
 
         for (int a = 0; a < atts.getLength(); a++) {
             String f = atts.getQName(a);
-            if (f.equals("name")) {
-                name = Whitespace.trim(atts.getValue(a));
-            } else if (f.equals("decimal-separator")) {
-                decimalSeparator = atts.getValue(a);
-            } else if (f.equals("grouping-separator")) {
-                groupingSeparator = atts.getValue(a);
-            } else if (f.equals("infinity")) {
-                infinity = atts.getValue(a);
-            } else if (f.equals("minus-sign")) {
-                minusSign = atts.getValue(a);
-            } else if (f.equals("NaN")) {
-                NaN = atts.getValue(a);
-            } else if (f.equals("percent")) {
-                percent = atts.getValue(a);
-            } else if (f.equals("per-mille")) {
-                perMille = atts.getValue(a);
-            } else if (f.equals("zero-digit")) {
-                zeroDigit = atts.getValue(a);
-            } else if (f.equals("digit")) {
-                digit = atts.getValue(a);
-            } else if (f.equals("exponent-separator")) {
-                exponentSeparator = atts.getValue(a);
-            } else if (f.equals("pattern-separator")) {
-                patternSeparator = atts.getValue(a);
-            } else {
-                checkUnknownAttribute(atts.getNodeName(a));
+            switch (f) {
+                case "name":
+                    name = Whitespace.trim(atts.getValue(a));
+                    break;
+                case "decimal-separator":
+                    decimalSeparator = atts.getValue(a);
+                    break;
+                case "grouping-separator":
+                    groupingSeparator = atts.getValue(a);
+                    break;
+                case "infinity":
+                    infinity = atts.getValue(a);
+                    break;
+                case "minus-sign":
+                    minusSign = atts.getValue(a);
+                    break;
+                case "NaN":
+                    NaN = atts.getValue(a);
+                    break;
+                case "percent":
+                    percent = atts.getValue(a);
+                    break;
+                case "per-mille":
+                    perMille = atts.getValue(a);
+                    break;
+                case "zero-digit":
+                    zeroDigit = atts.getValue(a);
+                    break;
+                case "digit":
+                    digit = atts.getValue(a);
+                    break;
+                case "exponent-separator":
+                    exponentSeparator = atts.getValue(a);
+                    break;
+                case "pattern-separator":
+                    patternSeparator = atts.getValue(a);
+                    break;
+                default:
+                    checkUnknownAttribute(atts.getNodeName(a));
+                    break;
             }
         }
     }
@@ -166,8 +179,6 @@ public class XSLDecimalFormat extends StyleElement {
                 symbols.setHostLanguage(Configuration.XSLT, 30);
             } catch (XPathException err) {
                 compileErrorInAttribute("Invalid decimal format name. " + err.getMessage(), "XTSE0020", "name");
-            } catch (NamespaceException err) {
-                compileErrorInAttribute("Invalid decimal format name. " + err.getMessage(), "XTSE0280", "name");
             }
         }
     }
