@@ -27,7 +27,7 @@ public class XSLExpose extends XSLAcceptExpose {
     protected void checkCompatibility(SymbolicName name, Visibility declared, Visibility exposed) throws XPathException {
         if (!isCompatible(declared, exposed)) {
             String code =  "XTSE3010";
-            compileError("The " + name + " is declared as " + declared + " and cannot be exposed as " + exposed, code);
+            compileError("The " + name + " is declared as " + declared.show() + " and cannot be exposed as " + exposed.show(), code);
         }
     }
 
