@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,7 +13,6 @@ import net.sf.saxon.expr.instruct.Choose;
 import net.sf.saxon.om.AttributeCollection;
 import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.value.BooleanValue;
 
 
 /**
@@ -79,12 +78,7 @@ public class XSLIf extends StyleElement {
         if (testAtt == null) {
             return null;
         } else {
-            try {
-                return se.makeExpression(testAtt, atts.getIndex("", "test"));
-            } catch (XPathException err) {
-                // already reported: prevent further errors
-                return Literal.makeLiteral(BooleanValue.FALSE);
-            }
+            return se.makeExpression(testAtt, atts.getIndex("", "test"));
         }
     }
 
