@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,9 +9,12 @@ package net.sf.saxon.style;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.event.PipelineConfiguration;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.NamespaceConstant;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.AttributeCollection;
+import net.sf.saxon.om.AxisInfo;
+import net.sf.saxon.om.Item;
+import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.trans.LicenseException;
 import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
@@ -105,6 +108,13 @@ public class XSLImportSchema extends StyleElement {
                     compileError("The schema-location attribute must be absent if an inline schema is present", "XTSE0215");
                 }
 
+                if (namespace.isEmpty()) {
+                    namespace = inlineSchema.getAttributeValue("", "targetNamespace");
+                    if (namespace == null) {
+                        namespace = "";
+                    }
+                }
+
                 namespace = config.readInlineSchema(inlineSchema, namespace,
                         getCompilation().getCompilerInfo().getErrorListener());
                 getPrincipalStylesheetModule().addImportedSchema(namespace);
@@ -126,7 +136,7 @@ public class XSLImportSchema extends StyleElement {
                     SaxonErrorCode.SXWN9006);
                 return;
             }
-            if (namespaceKnown && !config.getBooleanProperty(FeatureKeys.MULTIPLE_SCHEMA_IMPORTS)) {
+            if (namespaceKnown && !config.getBooleanProperty(Feature.MULTIPLE_SCHEMA_IMPORTS)) {
                 if (schemaLoc != null) {
                     compileWarning("The schema document at " + schemaLoc +
                         " is ignored because a schema for this namespace is already loaded", SaxonErrorCode.SXWN9006);
