@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,7 +12,7 @@ import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.Literal;
 import net.sf.saxon.expr.StringLiteral;
 import net.sf.saxon.expr.instruct.ResultDocument;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.SaxonOutputKeys;
 import net.sf.saxon.lib.Validation;
@@ -30,18 +30,20 @@ import java.util.Map;
 import java.util.Properties;
 
 /**
- * An xsl:result-document element in the stylesheet. <BR>
+ * An xsl:result-document element in the stylesheet.
+ * <p>
  * The xsl:result-document element takes an attribute href="filename". The filename will
  * often contain parameters, e.g. {position()} to ensure that a different file is produced
- * for each element instance. <BR>
- * There is a further attribute "name" which determines the format of the
+ * for each element instance.
+ * <p>
+ * There is a further attribute "format" which determines the format of the
  * output file, it identifies the name of an xsl:output element containing the output
  * format details.
  */
 
 public class XSLResultDocument extends StyleElement {
 
-    public static final HashSet<String> fans = new HashSet<String>(25);    // formatting attribute names
+    public static final HashSet<String> fans = new HashSet<>(40);    // formatting attribute names
 
     static {
         fans.add("allow-duplicate-names");
@@ -68,6 +70,7 @@ public class XSLResultDocument extends StyleElement {
         fans.add("undeclare-prefixes");
 
         fans.add(SaxonOutputKeys.ATTRIBUTE_ORDER);
+        fans.add(SaxonOutputKeys.CANONICAL);
         fans.add(SaxonOutputKeys.CHARACTER_REPRESENTATION);
         fans.add(SaxonOutputKeys.DOUBLE_SPACE);
         fans.add(SaxonOutputKeys.INDENT_SPACES);
@@ -76,9 +79,9 @@ public class XSLResultDocument extends StyleElement {
         fans.add(SaxonOutputKeys.NEXT_IN_CHAIN);
         fans.add(SaxonOutputKeys.RECOGNIZE_BINARY);
         fans.add(SaxonOutputKeys.REQUIRE_WELL_FORMED);
+        fans.add(SaxonOutputKeys.PROPERTY_ORDER);
         fans.add(SaxonOutputKeys.SINGLE_QUOTES);
         fans.add(SaxonOutputKeys.SUPPLY_SOURCE_LOCATOR);
-        fans.add(SaxonOutputKeys.SUPPRESS_INDENTATION);
 
     }
 
@@ -87,7 +90,7 @@ public class XSLResultDocument extends StyleElement {
     private Expression formatExpression;     // used when format is an AVT
     private int validationAction = Validation.STRIP;
     private SchemaType schemaType = null;
-    private Map<StructuredQName, Expression> serializationAttributes = new HashMap<StructuredQName, Expression>(10);
+    private Map<StructuredQName, Expression> serializationAttributes = new HashMap<>(10);
     private boolean async = true;
 
     /**
@@ -135,9 +138,12 @@ public class XSLResultDocument extends StyleElement {
                 typeAtt = Whitespace.trim(atts.getValue(a));
             } else if (f.equals("use-character-maps")) {
                 useCharacterMapsAtt = Whitespace.trim(atts.getValue(a));
-            } else if (fans.contains(f) || (f.startsWith("{") && !StandardNames.SAXON_THREADS.equals(f))) {
+            } else if (fans.contains(f) || (f.startsWith("{") && !StandardNames.SAXON_ASYCHRONOUS.equals(f))) {
                 // this is a serialization attribute
-                String val = Whitespace.trim(atts.getValue(a));
+                String val = atts.getValue(a);
+                if (!f.equals(SaxonOutputKeys.ITEM_SEPARATOR) && !f.equals(SaxonOutputKeys.NEWLINE)) {
+                    val = Whitespace.trim(atts.getValue(a));
+                }
                 Expression exp = makeAttributeValueTemplate(val, a);
                 serializationAttributes.put(name, exp);
             } else if (name.getLocalPart().equals("asynchronous") && name.hasURI(NamespaceConstant.SAXON)) {
@@ -159,8 +165,6 @@ public class XSLResultDocument extends StyleElement {
                 try {
                     formatQName = makeQName(((StringLiteral) formatExpression).getStringValue());
                     formatExpression = null;
-                } catch (NamespaceException err) {
-                    compileError(err.getMessage(), "XTSE0280");
                 } catch (XPathException err) {
                     compileError(err.getMessage(), "XTDE1460");
                 }
@@ -194,7 +198,7 @@ public class XSLResultDocument extends StyleElement {
     }
 
     public void validate(ComponentDeclaration decl) throws XPathException {
-        if (href != null && !getConfiguration().getBooleanProperty(FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS)) {
+        if (href != null && !getConfiguration().getBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS)) {
             compileError("xsl:result-document is disabled when extension functions are disabled");
         }
         href = typeCheck("href", href);
@@ -268,7 +272,7 @@ public class XSLResultDocument extends StyleElement {
 
         Properties localProps = new Properties();
 
-        HashSet<StructuredQName> fixed = new HashSet<StructuredQName>(10);
+        HashSet<StructuredQName> fixed = new HashSet<>(10);
         NamespaceResolver namespaceResolver = getStaticContext().getNamespaceResolver();
         for (StructuredQName property : serializationAttributes.keySet()) {
             Expression exp = serializationAttributes.get(property);
@@ -288,6 +292,7 @@ public class XSLResultDocument extends StyleElement {
                     if (NamespaceConstant.SAXON.equals(e.getErrorCodeNamespace())) {
                         compileWarning(e.getMessage(), e.getErrorCodeQName());
                     } else {
+                        e.setErrorCode("XTSE0020");
                         compileError(e);
                     }
                 }
