@@ -53,7 +53,6 @@ public class XSLUsePackage extends StyleElement {
                 compileError("Package " + getAttributeValue("name") +
                                      " requires a global context item, so it cannot be used as a library package", "XTTE0590");
             }
-            usedPackage.setRootPackage(false);
         }
     }
 
@@ -313,7 +312,7 @@ public class XSLUsePackage extends StyleElement {
 
                             Component.M usedComponent = (Component.M)derivedComponent.getBaseComponent();
 
-                            if (derivedComponent.getVisibility() == Visibility.FINAL || usedComponent.getVisibility() == Visibility.FINAL) {
+                            if (usedComponent.getVisibility() == Visibility.FINAL) {
                                 ((StyleElement) overridingDeclaration).compileError(
                                         "Cannot define overriding template rules in mode " + modeName.getDisplayName() +
                                                 " because it has visibility=final", "XTSE3060");
