@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,6 +13,7 @@ import net.sf.saxon.expr.LocalVariableReference;
 import net.sf.saxon.expr.PackageData;
 import net.sf.saxon.expr.instruct.Executable;
 import net.sf.saxon.expr.instruct.SlotManager;
+import net.sf.saxon.expr.parser.OptimizerOptions;
 import net.sf.saxon.expr.parser.RetainedStaticContext;
 import net.sf.saxon.functions.FunctionLibraryList;
 import net.sf.saxon.lib.NamespaceConstant;
@@ -32,13 +33,11 @@ import java.util.*;
 /**
  * An IndependentContext provides a context for parsing an XPath expression appearing
  * in a context other than a stylesheet.
- * <p/>
  * <p>This class is used in a number of places where freestanding XPath expressions occur.
  * These include the native Saxon XPath API, the .NET XPath API, XPath expressions used
  * in XML Schema identity constraints, and XPath expressions supplied to saxon:evaluate().
  * It is not used by the JAXP XPath API (though it shares code with that API through
  * the common superclass AbstractStaticContext).</p>
- * <p/>
  * <p>This class currently provides no mechanism for binding user-defined functions.</p>
  */
 
@@ -53,6 +52,7 @@ public class IndependentContext extends AbstractStaticContext
     protected boolean autoDeclare = false;
     protected Executable executable;
     protected RetainedStaticContext retainedStaticContext;
+    protected OptimizerOptions optimizerOptions;
 
 
     /**
@@ -72,9 +72,10 @@ public class IndependentContext extends AbstractStaticContext
     public IndependentContext(Configuration config) {
         setConfiguration(config);
         clearNamespaces();
-        setDefaultFunctionLibrary();
+        setDefaultFunctionLibrary(31);
         usingDefaultFunctionLibrary = true;
         setDefaultCollationName(config.getDefaultCollationName());
+        setOptimizerOptions(config.getOptimizerOptions());
         PackageData pd = new PackageData(config);
         pd.setHostLanguage(Configuration.XPATH);
         pd.setSchemaAware(false);
@@ -509,6 +510,27 @@ public class IndependentContext extends AbstractStaticContext
         return requiredContextItemType;
     }
 
+    /**
+     * Set the optimizer options to be used for compiling expressions that use this static context.
+     * By default the optimizer options set in the {@link Configuration} are used.
+     * @param options the optimizer options to be used
+     */
+
+    public void setOptimizerOptions(OptimizerOptions options) {
+        this.optimizerOptions = options;
+    }
+
+    /**
+     * Get the optimizer options being used for compiling expressions that use this static context.
+     * By default the optimizer options set in the {@link Configuration} are used.
+     *
+     * @return the optimizer options being used
+     */
+
+    public OptimizerOptions getOptimizerOptions() {
+        return this.optimizerOptions;
+    }
+
     public void setExecutable(Executable exec) {
         executable = exec;
     }
