@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,6 +10,7 @@ package net.sf.saxon.sxpath;
 import net.sf.saxon.expr.LocalBinding;
 import net.sf.saxon.expr.VariableReference;
 import net.sf.saxon.expr.XPathContext;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.value.IntegerValue;
@@ -22,18 +23,17 @@ import net.sf.saxon.value.SequenceType;
  * Note that once declared, this object is thread-safe: it does not hold the actual variable
  * value, which means it can be used with any number of evaluations of a given XPath expression,
  * in series or in parallel.
- * <p/>
  * <p>A variable can be given a value by calling
  * {@link XPathDynamicContext#setVariable(XPathVariable, net.sf.saxon.om.Sequence)}.
  * Note that the value of the variable is not held in the XPathVariable object, but in the
- * XPathDynamicContext, which means that the XPathVariable itself can be used in multiple threads.
+ * XPathDynamicContext, which means that the XPathVariable itself can be used in multiple threads.</p>
  */
 
 public final class XPathVariable implements LocalBinding {
 
     private StructuredQName name;
     private SequenceType requiredType = SequenceType.ANY_SEQUENCE;
-    private Sequence defaultValue;
+    private Sequence<?> defaultValue;
     private int slotNumber;
 
     /**
@@ -156,7 +156,7 @@ public final class XPathVariable implements LocalBinding {
      * @param defaultValue the default value for the variable
      */
 
-    public void setDefaultValue(Sequence defaultValue) {
+    public void setDefaultValue(Sequence<?> defaultValue) {
         this.defaultValue = defaultValue;
     }
 
@@ -166,7 +166,7 @@ public final class XPathVariable implements LocalBinding {
      * @return the default value if one has been registered, or null otherwise
      */
 
-    public Sequence getDefaultValue() {
+    public Sequence<?> getDefaultValue() {
         return defaultValue;
     }
 
@@ -179,7 +179,7 @@ public final class XPathVariable implements LocalBinding {
      * @return The value of the variable
      */
 
-    public Sequence evaluateVariable(XPathContext context) {
+    public Sequence<?> evaluateVariable(XPathContext context) {
         return context.evaluateLocalVariable(slotNumber);
     }
 
