@@ -12,10 +12,7 @@ import net.sf.saxon.expr.ContextOriginator;
 import net.sf.saxon.expr.UserFunctionCall;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.XPathContextMajor;
-import net.sf.saxon.expr.instruct.ApplyTemplates;
-import net.sf.saxon.expr.instruct.CallTemplate;
-import net.sf.saxon.expr.instruct.GlobalVariable;
-import net.sf.saxon.expr.instruct.UserFunction;
+import net.sf.saxon.expr.instruct.*;
 import net.sf.saxon.trans.rules.BuiltInRuleSet;
 
 import java.util.Iterator;
@@ -26,6 +23,10 @@ import java.util.Iterator;
  */
 public class ContextStackIterator implements Iterator<ContextStackFrame> {
 
+    // TODO: this class is no longer used by StandardErrorListener.printStackTrace(). It's potentially
+    // still useful to have a programmatic way of obtaining the stack trace, but this class is rather
+    // clumsy and probably isn't used.
+
     private XPathContextMajor next;
 
     /**
@@ -84,6 +85,7 @@ public class ContextStackIterator implements Iterator<ContextStackFrame> {
             sf.setLocation(ufc.getLocation());
             sf.setFunctionName(ufc.getFunctionName());
             sf.setContextItem(context.getContextItem());
+            sf.setContext(context);
             next = getMajorCaller(context);
             return sf;
         } else if (origin instanceof UserFunctionCall) {
@@ -93,6 +95,7 @@ public class ContextStackIterator implements Iterator<ContextStackFrame> {
             sf.setLocation(ufc.getLocation());
             sf.setFunctionName(ufc.getFunctionName());
             sf.setContextItem(context.getContextItem());
+            sf.setContext(context);
             next = getMajorCaller(context);
             return sf;
         } else if (origin instanceof ApplyTemplates) {
@@ -100,6 +103,7 @@ public class ContextStackIterator implements Iterator<ContextStackFrame> {
             ApplyTemplates loc = (ApplyTemplates) origin;
             sf.setLocation(loc.getLocation());
             sf.setContextItem(context.getContextItem());
+            sf.setContext(context);
             next = getMajorCaller(context);
             return sf;
         } else if (origin instanceof CallTemplate) {
@@ -108,6 +112,7 @@ public class ContextStackIterator implements Iterator<ContextStackFrame> {
             sf.setLocation(loc.getLocation());
             sf.setTemplateName(loc.getObjectName());
             sf.setContextItem(context.getContextItem());
+            sf.setContext(context);
             next = getMajorCaller(context);
             return sf;
         } else if (origin instanceof GlobalVariable) {
@@ -117,6 +122,7 @@ public class ContextStackIterator implements Iterator<ContextStackFrame> {
             sf.setContextItem(context.getContextItem());
             sf.setVariableName(var.getVariableQName());
             sf.setComponent(var);
+            sf.setContext(context);
             next = getMajorCaller(context);
             return sf;
         } else {
