@@ -277,6 +277,7 @@ public class ExpressionPresenter {
         props.setProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
         props.setProperty(OutputKeys.ENCODING, "utf-8");
         props.setProperty(OutputKeys.VERSION, "1.0");
+        props.setProperty(SaxonOutputKeys.SINGLE_QUOTES, "yes");
         return props;
     }
 
@@ -329,7 +330,7 @@ public class ExpressionPresenter {
 
     public void emitRetainedStaticContext(RetainedStaticContext sc, RetainedStaticContext parentSC) {
         try {
-            if (!relocatable && sc.getStaticBaseUri() != null && (parentSC == null || !sc.getStaticBaseUri().equals(parentSC.getStaticBaseUri()))) {
+            if (!((ExportOptions) options).suppressStaticContext && !relocatable && sc.getStaticBaseUri() != null && (parentSC == null || !sc.getStaticBaseUri().equals(parentSC.getStaticBaseUri()))) {
                 emitAttribute("baseUri", sc.getStaticBaseUriString());
             }
             if (!sc.getDefaultCollationName().equals(NamespaceConstant.CODEPOINT_COLLATION_URI) &&
@@ -343,13 +344,15 @@ public class ExpressionPresenter {
             if (!NamespaceConstant.FN.equals(sc.getDefaultFunctionNamespace())) {
                 emitAttribute("defaultFunctionNS", sc.getDefaultFunctionNamespace());
             }
-            if (parentSC == null || !sc.declaresSameNamespaces(parentSC)) {
+            if (!((ExportOptions)options).suppressStaticContext && (parentSC == null || !sc.declaresSameNamespaces(parentSC))) {
                 FastStringBuffer fsb = new FastStringBuffer(FastStringBuffer.C256);
                 for (Iterator<String> iter = sc.iteratePrefixes(); iter.hasNext(); ) {
                     String p = iter.next();
+
+                    String uri = sc.getURIForPrefix(p, true);
                     fsb.append(p);
                     fsb.append("=");
-                    String uri = sc.getURIForPrefix(p, true);
+
                     if (Whitespace.containsWhitespace(uri)) {
                         throw new XPathException("Cannot export a stylesheet if namespaces contain whitespace: '" + uri + "'");
                     }
@@ -358,6 +361,7 @@ public class ExpressionPresenter {
                     }
                     fsb.append(uri);
                     fsb.append(" ");
+
                 }
                 emitAttribute("ns", Whitespace.trim(fsb));
             }
@@ -669,11 +673,14 @@ public class ExpressionPresenter {
     public interface Options {};
 
     public static class ExportOptions implements Options {
-        public String target;
-        public int targetVersion;
+        public String target = "";
+        public int targetVersion = 0;
         public StylesheetPackage rootPackage;
         public Map<Component, Integer> componentMap;
         public Map<StylesheetPackage, Integer> packageMap;
+        public boolean explaining;
+        public boolean suppressStaticContext;
+        public boolean addStaticType;
     }
 }
 
