@@ -1,5 +1,7 @@
 package net.sf.saxon.trace;
 
+import net.sf.saxon.trans.CommandLineOptions;
+
 import java.util.HashMap;
 import java.util.Map;
 
@@ -22,6 +24,14 @@ public class Instrumentation {
         }
     }
 
+    public static void count(String counter, int increment) {
+        if (counters.containsKey(counter)) {
+            counters.put(counter, counters.get(counter) + increment);
+        } else {
+            counters.put(counter, increment);
+        }
+    }
+
     public static void report() {
         if (ACTIVE && !counters.isEmpty()) {
             System.err.println("COUNTERS");
@@ -31,6 +41,11 @@ public class Instrumentation {
         }
     }
 
+    public static void freeMemory(String label) {
+        System.err.println(label+" "+CommandLineOptions.showMemoryUsed());
+
+    }
+
     public static void reset() {
         counters.clear();
     }
