@@ -9,8 +9,10 @@ package net.sf.saxon.trans;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.event.Builder;
+import net.sf.saxon.expr.number.RegularGroupFormatter;
 import net.sf.saxon.functions.AccessorFn;
 import net.sf.saxon.lib.*;
+import net.sf.saxon.regex.EmptyString;
 import net.sf.saxon.s9api.*;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.SchemaException;
@@ -257,10 +259,27 @@ public class CommandLineOptions {
         for (Enumeration e = configOptions.propertyNames(); e.hasMoreElements(); ) {
             String name = (String) e.nextElement();
             String value = configOptions.getProperty(name);
+            String fullName = "http://saxon.sf.net/feature/" + name;
+            if (!name.startsWith("parserFeature?") && !name.startsWith("parserProperty?")) {
+                Feature<?> f = Feature.byName(fullName);
+                if (f == null) {
+                    throw new XPathException("Unknown configuration feature " + name);
+                }
+                Object typedValue;
+                if (f.type == Boolean.class) {
+                    typedValue = Configuration.requireBoolean(name, value);
+                } else if (f.type == Integer.class) {
+                    typedValue = Integer.valueOf(value);
+                } else if (f.type == String.class) {
+                    typedValue = value;
+                } else {
+                    throw new XPathException("Property --" + name + " cannot be supplied as a string");
+                }
+            }
             try {
-                processor.setConfigurationProperty("http://saxon.sf.net/feature/" + name, value);
+                processor.getUnderlyingConfiguration().setConfigurationProperty(fullName, value);
             } catch (IllegalArgumentException err) {
-                throw new XPathException(err.getMessage());
+                throw new XPathException("Incorrect value for --" + name + ": " + err.getMessage());
             }
         }
 
@@ -335,22 +354,26 @@ public class CommandLineOptions {
 
         value = getOptionValue("ea");
         if (value != null) {
-            config.getDefaultXsltCompilerInfo().setAssertionsEnabled("on".equals(value));
+            boolean on = Configuration.requireBoolean("ea", value);
+            config.getDefaultXsltCompilerInfo().setAssertionsEnabled(on);
         }
 
         value = getOptionValue("expand");
         if (value != null) {
-            config.getParseOptions().setExpandAttributeDefaults("on".equals(value));
+            boolean on = Configuration.requireBoolean("expand", value);
+            config.getParseOptions().setExpandAttributeDefaults(on);
         }
 
         value = getOptionValue("ext");
         if (value != null) {
-            config.setBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS, "on".equals(value));
+            boolean on = Configuration.requireBoolean("ext", value);
+            config.setBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS, on);
         }
 
         value = getOptionValue("l");
         if (value != null) {
-            config.setBooleanProperty(Feature.LINE_NUMBERING, "on".equals(value));
+            boolean on = Configuration.requireBoolean("l", value);
+            config.setBooleanProperty(Feature.LINE_NUMBERING, on);
         }
 
         value = getOptionValue("m");
@@ -397,8 +420,8 @@ public class CommandLineOptions {
 
         value = getOptionValue("TJ");
         if (value != null) {
-            config.setBooleanProperty(Feature.TRACE_EXTERNAL_FUNCTIONS,
-                    "on".equals(value));
+            boolean on = Configuration.requireBoolean("TJ", value);
+            config.setBooleanProperty(Feature.TRACE_EXTERNAL_FUNCTIONS, on);
         }
 
         value = getOptionValue("tree");
@@ -442,7 +465,8 @@ public class CommandLineOptions {
 
         value = getOptionValue("xi");
         if (value != null) {
-            processor.setConfigurationProperty(Feature.XINCLUDE, "on".equals(value));
+            boolean on = Configuration.requireBoolean("xi", value);
+            processor.setConfigurationProperty(Feature.XINCLUDE, on);
         }
 
         value = getOptionValue("xmlversion");
@@ -457,7 +481,8 @@ public class CommandLineOptions {
 
         value = getOptionValue("xsiloc");
         if (value != null) {
-            processor.setConfigurationProperty(Feature.USE_XSI_SCHEMA_LOCATION, "on".equals(value));
+            boolean on = Configuration.requireBoolean("xsiloc", value);
+            processor.setConfigurationProperty(Feature.USE_XSI_SCHEMA_LOCATION, on);
         }
 
         value = getOptionValue("y");
@@ -565,13 +590,25 @@ public class CommandLineOptions {
 
     public void setParams(Processor processor, ParamSetter paramSetter)
             throws SaxonApiException {
-        boolean useURLs = "on".equals(getOptionValue("u"));
-
         for (Enumeration e = paramValues.propertyNames(); e.hasMoreElements(); ) {
             String name = (String) e.nextElement();
             String value = paramValues.getProperty(name);
             paramSetter.setParam(QName.fromClarkName(name), new XdmAtomicValue(value, ItemType.UNTYPED_ATOMIC));
         }
+        applyFileParameters(processor, paramSetter);
+        for (Enumeration e = paramExpressions.propertyNames(); e.hasMoreElements(); ) {
+            String name = (String) e.nextElement();
+            String value = paramExpressions.getProperty(name);
+            // parameters starting with "?" are taken as XPath expressions
+            XPathCompiler xpc = processor.newXPathCompiler();
+            XPathExecutable xpe = xpc.compile(value);
+            XdmValue val = xpe.load().evaluate();
+            paramSetter.setParam(QName.fromClarkName(name), val);
+        }
+    }
+
+    private void applyFileParameters(Processor processor, ParamSetter paramSetter) throws SaxonApiException {
+        boolean useURLs = "on".equals(getOptionValue("u"));
         for (Enumeration e = paramFiles.propertyNames(); e.hasMoreElements(); ) {
             String name = (String) e.nextElement();
             String value = paramFiles.getProperty(name);
@@ -589,15 +626,6 @@ public class CommandLineOptions {
                 paramSetter.setParam(QName.fromClarkName(name), XdmEmptySequence.getInstance());
             }
         }
-        for (Enumeration e = paramExpressions.propertyNames(); e.hasMoreElements(); ) {
-            String name = (String) e.nextElement();
-            String value = paramExpressions.getProperty(name);
-            // parameters starting with "?" are taken as XPath expressions
-            XPathCompiler xpc = processor.newXPathCompiler();
-            XPathExecutable xpe = xpc.compile(value);
-            XdmValue val = xpe.load().evaluate();
-            paramSetter.setParam(QName.fromClarkName(name), val);
-        }
     }
 
     /**
@@ -626,8 +654,10 @@ public class CommandLineOptions {
 
 
     /**
-     * Apply XSLT 3.0 static parameters to a compilerInfo. Actually this sets all parameter values, whether static or dynamic.
-     * This is possible because the stylesheet is compiled for once-only use.
+     * Apply XSLT 3.0 static parameters to a compilerInfo. Actually this sets all parameter values,
+     * whether static or dynamic. This is possible because the stylesheet is compiled for once-only use.
+     * However, file-valued params (+name=value) are always handled as dynamic parameters because
+     * of potential problems with exported SEF files.
      *
      * @param compiler The XsltCompiler object into which the parameters are copied
      * @throws SaxonApiException if invalid options are found
@@ -653,6 +683,24 @@ public class CommandLineOptions {
 
     }
 
+    /**
+     * Apply XSLT 3.0 file-valued parameters to an XSLT transformer. Most parameters are applied
+     * before compilation, so that the compiler can take advantage of knowing their values; but
+     * file-valued parameters (provided as +name=value) are deferred until run-time because of
+     * complications storing their values in a SEF file.
+     *
+     * @param transformer The Xslt30Transformer object into which the parameters are copied
+     * @throws SaxonApiException if invalid options are found
+     */
+
+    public void applyFileParams(Processor processor, Xslt30Transformer transformer) throws SaxonApiException {
+        if (!paramFiles.isEmpty()) {
+            Map<QName, XdmValue> params = new HashMap<>();
+            applyFileParameters(processor, params::put);
+            transformer.setStylesheetParameters(params);
+        }
+    }
+
 
     /**
      * Load a document, or all the documents in a directory, given a filename or URL
@@ -788,5 +836,20 @@ public class CommandLineOptions {
 
         }
     }
+
+    public static String showMemoryUsed() {
+        long value = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();
+        return new RegularGroupFormatter(3, ",", EmptyString.THE_INSTANCE).format(new FastStringBuffer(value + ""));
+    }
+
+
+    public static String getCommandName(Object command) {
+        String s = command.getClass().getName();
+        if (s.startsWith("cli.Saxon.Cmd.DotNet")) {
+            s = s.substring("cli.Saxon.Cmd.DotNet".length());
+        }
+        return s;
+    }
+
 }
 
