@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,15 +10,19 @@ package net.sf.saxon.trans;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.instruct.GlobalParameterSet;
 import net.sf.saxon.expr.parser.CodeInjector;
+import net.sf.saxon.expr.parser.OptimizerOptions;
 import net.sf.saxon.functions.FunctionLibrary;
 import net.sf.saxon.lib.OutputURIResolver;
 import net.sf.saxon.lib.StandardOutputResolver;
-import net.sf.saxon.om.Sequence;
+import net.sf.saxon.om.GroundedValue;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.StructuredQName;
-import net.sf.saxon.pattern.PatternOptimization;
+import net.sf.saxon.query.QueryLibrary;
+import net.sf.saxon.trans.packages.PackageLibrary;
 
 import javax.xml.transform.ErrorListener;
 import javax.xml.transform.URIResolver;
+import java.util.Collection;
 
 /**
  * This class exists to hold information associated with a specific XSLT compilation episode.
@@ -29,34 +33,37 @@ import javax.xml.transform.URIResolver;
 
 public class CompilerInfo {
 
+    private Configuration config;
     private transient URIResolver uriResolver;
     private transient OutputURIResolver outputURIResolver = StandardOutputResolver.getInstance();
     private transient ErrorListener errorListener;
     private CodeInjector codeInjector;
     private int recoveryPolicy = Configuration.RECOVER_WITH_WARNINGS;
     private boolean schemaAware;
-    private boolean versionWarning;
+    //private boolean versionWarning;
     private String messageReceiverClassName = "net.sf.saxon.serialize.MessageEmitter";
     private StructuredQName defaultInitialMode;
     private StructuredQName defaultInitialTemplate;
-    private int xsltVersion = 0;
-        // the externally-requested XSLT Processor version. This is the decimal version number times ten,
-        // for example 10 = 1.0, 20 = 2.0, 30 = 3.0. Zero means uninitialized.
     private FunctionLibrary extensionFunctionLibrary;
-    private GlobalParameterSet variableList = new GlobalParameterSet();
+    private GlobalParameterSet suppliedParameters = new GlobalParameterSet();
     private String defaultCollation;
-    private PackageLibrary packageLibrary = new PackageLibrary();
-    private boolean generateByteCode = false;
+    private PackageLibrary packageLibrary;
     private boolean assertionsEnabled = false;
     private String targetEdition = "HE";
-
-    private PatternOptimization patternOptimization;
+    private boolean relocatable = false;
+    private Collection<QueryLibrary> queryLibraries;
+    private OptimizerOptions optimizerOptions;
 
     /**
-     * Create an empty CompilerInfo object with default settings
+     * Create an empty CompilerInfo object with default settings. (Note, this does not
+     * copy settings from the defaultXsltCompilerInfo held by the configuration. For that,
+     * use <code>new CompilerInfo(config.getDefaultXsltCompilerInfo()</code>).
      */
 
-    public CompilerInfo() {
+    public CompilerInfo(Configuration config) {
+        this.config = config;
+        packageLibrary = new PackageLibrary(this);
+        optimizerOptions = config.getOptimizerOptions();
     }
 
     /**
@@ -77,24 +84,59 @@ public class CompilerInfo {
      */
 
     public void copyFrom(CompilerInfo info) {
+        config = info.config;
         uriResolver = info.uriResolver;
         outputURIResolver = info.outputURIResolver;
         errorListener = info.errorListener;
         codeInjector = info.codeInjector;
         recoveryPolicy = info.recoveryPolicy;
         schemaAware = info.schemaAware;
-        versionWarning = info.versionWarning;
         messageReceiverClassName = info.messageReceiverClassName;
         defaultInitialMode = info.defaultInitialMode;
         defaultInitialTemplate = info.defaultInitialTemplate;
-        xsltVersion = info.xsltVersion;
-        packageLibrary = info.packageLibrary;
-        generateByteCode = info.generateByteCode;
         extensionFunctionLibrary = info.extensionFunctionLibrary;
+        suppliedParameters = new GlobalParameterSet(info.suppliedParameters);
         defaultCollation = info.defaultCollation;
         assertionsEnabled = info.assertionsEnabled;
         targetEdition = info.targetEdition;
-        patternOptimization = info.patternOptimization;      // project: pattern optimization
+        packageLibrary = new PackageLibrary(info.packageLibrary);
+        relocatable = info.relocatable;
+        optimizerOptions = info.optimizerOptions;
+        queryLibraries = info.queryLibraries;
+    }
+
+    public Configuration getConfiguration() {
+        return config;
+    }
+
+    /**
+     * Say whether just-in-time compilation of template rules should be used for this XSLT compilation
+     * <p>The same effect can be achieved by {@link #setOptimizerOptions(OptimizerOptions)} with the appropriate
+     * setting of {@link OptimizerOptions#JIT}.</p>
+     *
+     * @param jit true if just-in-time compilation should be used. With this option, static errors in the
+     *            stylesheet code may be reported only when a template rule is first executed
+     * @since 9.8
+     */
+
+    public void setJustInTimeCompilation(boolean jit) {
+        if (jit) {
+            optimizerOptions = optimizerOptions.union(new OptimizerOptions(OptimizerOptions.JIT));
+        } else {
+            optimizerOptions = optimizerOptions.except(new OptimizerOptions(OptimizerOptions.JIT));
+        }
+    }
+
+    /**
+     * Ask whether just-in-time compilation of template rules is in use for this XSLT compilation
+     *
+     * @return true if just-in-time compilation should be used. With this option, static errors in the
+     * stylesheet code may be reported only when a template rule is first executed
+     * @since 9.8
+     */
+
+    public boolean isJustInTimeCompilation() {
+        return optimizerOptions.isSet(OptimizerOptions.JIT);
     }
 
     /**
@@ -121,35 +163,36 @@ public class CompilerInfo {
      * @param seq  the value of the stylesheet parameter
      */
 
-    public void setParameter(StructuredQName name, Sequence seq) {
-        variableList.put(name, seq);
+    public void setParameter(StructuredQName name, GroundedValue<?> seq) {
+        suppliedParameters.put(name, seq);
     }
 
     /**
      * Get the values of all stylesheet parameters that have been set using the
-     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.Sequence)}
+     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.GroundedValue)}
      * method.
      *
      * @return the set of all parameters that have been set.
      */
 
     public GlobalParameterSet getParameters() {
-        return variableList;
+        return suppliedParameters;
     }
 
     /**
      * Clear the values of all stylesheet parameters that have been set using the
-     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.Sequence)}
+     * {@link #setParameter(net.sf.saxon.om.StructuredQName, net.sf.saxon.om.GroundedValue)}
      * method.
      */
 
     public void clearParameters() {
-        variableList.clear();
+        suppliedParameters.clear();
     }
 
     /**
      * Set the target edition under which the stylesheet will be executed.
-     * @param edition the Saxon edition for the run-time environment. One of "EE", "PE", "HE", or "JS".
+     *
+     * @param edition the Saxon edition for the run-time environment. One of "EE", "PE", "HE", "JS", or "JS2".
      * @since 9.7.0.5. Experimental and subject to change.
      */
 
@@ -159,7 +202,8 @@ public class CompilerInfo {
 
     /**
      * Get the target edition under which the stylesheet will be executed.
-     * @return the Saxon edition for the run-time environment. One of "EE", "PE", "HE", or "JS".
+     *
+     * @return the Saxon edition for the run-time environment. One of "EE", "PE", "HE", "JS", or "JS2".
      * @since 9.7.0.5. Experimental and subject to change.
      */
 
@@ -167,6 +211,28 @@ public class CompilerInfo {
         return targetEdition;
     }
 
+    /**
+     * Ask whether any package produced by this compiler can be deployed to a different location, with a different base URI
+     *
+     * @return if true then static-base-uri() represents the deployed location of the package,
+     * rather than its compile time location
+     */
+
+    public boolean isRelocatable() {
+        return relocatable;
+    }
+
+    /**
+     * Say whether any package produced by this compiler can be deployed to a different location, with a different base URI
+     *
+     * @param relocatable if true then static-base-uri() represents the deployed location of the package,
+     *                    rather than its compile time location
+     */
+
+    public void setRelocatable(boolean relocatable) {
+        this.relocatable = relocatable;
+    }
+
     /**
      * Set the package library to be used during the compilation episode. Any packages referenced
      * using xsl:use-package declarations during the stylesheet compilation must be found in this
@@ -196,6 +262,7 @@ public class CompilerInfo {
      * they are disabled. If assertions are enabled at compile time, then by
      * default they will also be enabled at run time; but they can be
      * disabled at run time by specific request
+     *
      * @return true if assertions are enabled at compile time
      */
 
@@ -217,20 +284,49 @@ public class CompilerInfo {
         this.assertionsEnabled = enabled;
     }
 
+    /**
+     * Set the optimizer options to be used for compiling queries that use this static context.
+     * By default the optimizer options set in the {@link Configuration} are used.
+     *
+     * @param options the optimizer options to be used
+     */
+
+    public void setOptimizerOptions(OptimizerOptions options) {
+        this.optimizerOptions = options;
+    }
+
+    /**
+     * Get the optimizer options being used for compiling queries that use this static context.
+     * By default the optimizer options set in the {@link Configuration} are used.
+     *
+     * @return the optimizer options being used
+     */
+
+    public OptimizerOptions getOptimizerOptions() {
+        return this.optimizerOptions;
+    }
+
 
     /**
      * Set whether bytecode should be generated for the compiled stylesheet. This option
      * is available only with Saxon-EE. The default depends on the setting in the configuration
      * at the time the XsltCompiler is instantiated, and by default is true for Saxon-EE.
+     * <p>The same effect can be achieved by {@link #setOptimizerOptions(OptimizerOptions)} with the appropriate
+     * setting of {@link OptimizerOptions#BYTE_CODE}.</p>
      *
      * @param option true if bytecode is to be generated, false otherwise
      * @since 9.6
      */
 
     public void setGenerateByteCode(boolean option) {
-        generateByteCode = option;
+        if (option) {
+            optimizerOptions = optimizerOptions.union(new OptimizerOptions(OptimizerOptions.BYTE_CODE));
+        } else {
+            optimizerOptions = optimizerOptions.except(new OptimizerOptions(OptimizerOptions.BYTE_CODE));
+        }
     }
 
+
     /**
      * Ask whether bytecode is to be generated in the compiled code.
      *
@@ -239,7 +335,7 @@ public class CompilerInfo {
      */
 
     public boolean isGenerateByteCode() {
-        return generateByteCode;
+        return optimizerOptions.isSet(OptimizerOptions.BYTE_CODE);
     }
 
     /**
@@ -414,36 +510,17 @@ public class CompilerInfo {
         return recoveryPolicy;
     }
 
-    /**
-     * Get the pattern optimization options to be used during this compilation
-     * @return the pattern optimization options
-     * @since 9.7
-     */
-
-    public PatternOptimization getPatternOptimization() {
-        return patternOptimization;
-    }
-
-    /**
-     * Set the pattern optimization options to be used during this compilation
-     * @param p the pattern optimization options
-     * @since 9.7
-     */
-
-    public void setPatternOptimization(PatternOptimization p) {
-        patternOptimization = p;
-    }
-
     /**
      * Ask whether a warning is to be output when the stylesheet version does not match the processor version.
      * In the case of stylesheet version="1.0", the XSLT specification requires such a warning unless the user disables it.
      *
      * @return true if these messages are to be output.
      * @since 9.2
+     * @deprecated since 9.8.0.2. Always now returns false. See bug 3278.
      */
 
     public boolean isVersionWarning() {
-        return versionWarning;
+        return false;
     }
 
     /**
@@ -452,10 +529,11 @@ public class CompilerInfo {
      *
      * @param warn true if these messages are to be output.
      * @since 9.2
+     * @deprecated since 9.8.0.2. The method no longer has any effect. See bug 3278.
      */
 
     public void setVersionWarning(boolean warn) {
-        versionWarning = warn;
+
     }
 
     /**
@@ -542,48 +620,29 @@ public class CompilerInfo {
     }
 
     /**
-     * Set the version of XSLT to be supported by this processor. This determines the version
-     * of the XSLT specification to which the processor conforms. This does not have to match
-     * the version attribute in the stylesheet; for example if the processor version is 2.0
-     * and the stylesheet is version 3.0, then the stylesheet will be processed using the rules
-     * for a 2.0 processor in forwards compatibility mode, rather than the rules for a 3.01
-     * processor.
-     * <p>The value 0.0 (which is the default) indicates that the processor version is to be
-     * taken from the version attribute of the xsl:stylesheet element.</p>
-     * <p>If the value 1.0 is supplied, this is treated as 2.0. Typically this happens when
-     * a stylesheet specifies version="1.0" and selection of the XSLT version is based on what
-     * the stylesheet says. This will use an XSLT 2.0 processor running in backwards compatibilty
-     * mode.</p>
-     * <p><i>XSLT 3.0 features are supported only in Saxon-EE. Setting the version to 3.0
-     * here will not fail if the wrong edition is in use, but use of XSLT 3.0 features will
-     * fail subsequently.</i></p>
+     * Set the version of XSLT to be supported by this processor. From Saxon 9.8 this has
+     * no effect; the processor will always be an XSLT 3.0 processor.
      *
-     * @param version The decimal version number times ten, for example 20 indicates
-     *                XSLT 2.0. This must be numerically equal to 0, 20, or 30.
-     * @throws IllegalArgumentException if the version is invalid
-     * @since 9.3. Changed in 9.7 to take an integer rather than a decimal.
+     * @param version The decimal version number times ten, for example 30 indicates
+     *                XSLT 3.0.
+     * @since 9.3. Changed in 9.7 to take an integer rather than a decimal. Ignored from 9.8.
+     * @deprecated since Saxon 9.8 (has no effect).
      */
 
     public void setXsltVersion(int version) {
-        if (version == 10) {
-            xsltVersion = 20;
-        } else if (version != 20 && version != 30 && version != 0) {
-            throw new IllegalArgumentException("XSLT version must be 0.0, 2.0 or 3.0");
-        } else {
-            xsltVersion = version;
-        }
     }
 
     /**
      * Get the version of XSLT requested for this compilation.
      *
-     * @return 20 (for XSLT 2.0) or 30 (for XSLT 3.0),
-     * or zero indicating that the processor version is taken from the version attribute of the xsl:stylesheet element.
-     * @since 9.3  Changed in 9.7 to take an integer rather than a decimal.
+     * @return 30 (for XSLT 3.0)
+     * @since 9.3  Changed in 9.7 to take an integer rather than a decimal. Changed in 9.8 to return the
+     * value 30 unconditionally.
+     * @deprecated since 9.8 (always returns 30)
      */
 
     public int getXsltVersion() {
-        return xsltVersion;
+        return 30;
     }
 
     /**
@@ -617,5 +676,13 @@ public class CompilerInfo {
         return extensionFunctionLibrary;
     }
 
+    public void setXQueryLibraries(Collection<QueryLibrary> libraries) {
+        this.queryLibraries = libraries;
+    }
+
+    public Collection<QueryLibrary> getQueryLibraries() {
+        return queryLibraries;
+    }
+
 }
 
