@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -64,8 +64,6 @@ public class DecimalSymbols {
     private int[] intValues = new int[propertyNames.length - 2];
     private int[] precedences = new int[propertyNames.length];
     private boolean[] inconsistent = new boolean[propertyNames.length];
-    private int languageLevel = 20;
-
 
     /**
      * Create a DecimalSymbols object with default values for all properties
@@ -93,7 +91,6 @@ public class DecimalSymbols {
         } else {
             errorCodes = XSLT_CODES;
         }
-        this.languageLevel = languageLevel;
     }
 
     /**
@@ -203,15 +200,6 @@ public class DecimalSymbols {
         return NaNValue;
     }
 
-    /**
-     * Get the XPath language level
-     * @return the language level times ten, for example 30 indicates XPath 3.0.
-     */
-
-    public int getLanguageLevel() {
-        return languageLevel;
-    }
-
     /**
      * Set the character to be used as the decimal separator
      *
@@ -407,15 +395,23 @@ public class DecimalSymbols {
     }
 
     public void export(StructuredQName name, ExpressionPresenter out) {
+        DecimalSymbols defaultSymbols = new DecimalSymbols(Configuration.XSLT, 31);
         out.startElement("decimalFormat");
         if (name != null) {
-            out.emitAttribute("name", name.getEQName());
+            out.emitAttribute("name", name);
         }
         for (int i=0; i<intValues.length; i++) {
-            out.emitAttribute(propertyNames[i], intValues[i]+"");
+            int propValue = intValues[i];
+            if (propValue != defaultSymbols.intValues[i]) {
+                out.emitAttribute(propertyNames[i], propValue + "");
+            }
+        }
+        if (!"Infinity".equals(getInfinity())) {
+            out.emitAttribute("infinity", getInfinity());
+        }
+        if (!"NaN".equals(getNaN())) {
+            out.emitAttribute("NaN", getNaN());
         }
-        out.emitAttribute("infinity", getInfinity());
-        out.emitAttribute("NaN", getNaN());
         out.endElement();
     }
 
