@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -22,6 +22,14 @@ public enum FunctionStreamability {
 
     public String streamabilityStr;
 
+    public boolean isConsuming() {
+        return this == ABSORBING || this == SHALLOW_DESCENT || this == DEEP_DESCENT;
+    }
+
+    public boolean isStreaming() {
+        return this != UNCLASSIFIED;
+    }
+
     FunctionStreamability(String v) {
         streamabilityStr = v;
     }
