@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -37,7 +37,7 @@ public class KeyDefinitionSet {
     public KeyDefinitionSet(StructuredQName keyName, int keySetNumber) {
         this.keyName = keyName;
         this.keySetNumber = keySetNumber;
-        keyDefinitions = new ArrayList<KeyDefinition>(3);
+        keyDefinitions = new ArrayList<>(3);
     }
 
     /**
@@ -64,8 +64,8 @@ public class KeyDefinitionSet {
             // a stylesheet module more than once
             List<KeyDefinition> v = getKeyDefinitions();
             for (KeyDefinition other : v) {
-                if (keyDef.getMatch().equals(other.getMatch()) &&
-                        keyDef.getBody().equals(other.getBody())) {
+                if (keyDef.getMatch().isEqual(other.getMatch()) &&
+                        keyDef.getBody().isEqual(other.getBody())) {
                     return;
                 }
             }
