@@ -197,6 +197,12 @@ public class SaxonErrorCode {
 
     public static final String SXPK0003 = "SXPK0003";
 
+    /**
+     * SXPK0004: documentation namespace can be used only for documentation
+     */
+
+    public static final String SXPK0004 = "SXPK0004";
+
     /**
      * SXRD0001: URI supplied to xsl:result-document does not identify a writable destination
      */
@@ -215,6 +221,12 @@ public class SaxonErrorCode {
 
     public static final String SXRD0003 = "SXRD0003";
 
+    /**
+     * SXRD0004: Failure to create an output stream for an output destination
+     */
+
+    public static final String SXRD0004 = "SXRD0004";
+
     /**
      * SXST0060: Template in a streaming mode is not streamable
      */
@@ -269,6 +281,12 @@ public class SaxonErrorCode {
      */
     public static final String SXTA0001 = "SXTA0001";
 
+    /**
+     * SXTM0001: tabulate-maps: selecting an item with no pedigree
+     */
+    public static final String SXTM0001 = "SXTM0001";
+
+
     /**
      * SXTT0001: field name not defined in tuple type
      */
