@@ -25,4 +25,8 @@ public enum Visibility {
         visibilityStr = v;
     }
 
+    public String show() {
+        return visibilityStr;
+    }
+
 }
