@@ -7,6 +7,7 @@
 
 package net.sf.saxon.trans.rules;
 
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.instruct.ParameterSet;
 import net.sf.saxon.expr.parser.Location;
@@ -36,6 +37,7 @@ public class DeepCopyRuleSet implements BuiltInRuleSet {
     private DeepCopyRuleSet() {
     }
 
+    private static int COPY_OPTIONS = CopyOptions.TYPE_ANNOTATIONS | CopyOptions.ALL_NAMESPACES;
     /**
      * Perform the built-in template action for a given node.
      *  @param item
@@ -51,24 +53,25 @@ public class DeepCopyRuleSet implements BuiltInRuleSet {
                         Location locationId) throws XPathException {
         if (item instanceof NodeInfo) {
             NodeInfo node = (NodeInfo) item;
+            Receiver out = context.getReceiver();
             switch (node.getNodeKind()) {
-                case Type.DOCUMENT: {
-                    node.copy(context.getReceiver(), CopyOptions.ALL_NAMESPACES, locationId);
-                    return;
-                }
+                case Type.DOCUMENT: 
                 case Type.ELEMENT: {
-                    node.copy(context.getReceiver(), CopyOptions.ALL_NAMESPACES, locationId);
+                    if (out.getSystemId() == null) {
+                        out.setSystemId(node.getBaseURI());
+                    }
+                    node.copy(out, COPY_OPTIONS, locationId);
                     return;
                 }
                 case Type.TEXT:
-                    context.getReceiver().characters(item.getStringValueCS(), locationId, 0);
+                    out.characters(item.getStringValueCS(), locationId, 0);
                     return;
 
                 case Type.ATTRIBUTE:
                 case Type.COMMENT:
                 case Type.PROCESSING_INSTRUCTION:
                 case Type.NAMESPACE:
-                    node.copy(context.getReceiver(), 0, locationId);
+                    node.copy(out, COPY_OPTIONS, locationId);
                     return;
 
                 default:
@@ -88,4 +91,14 @@ public class DeepCopyRuleSet implements BuiltInRuleSet {
     public int[] getActionForParentNodes(int nodeKind) {
         return new int[]{DEEP_COPY};
     }
+
+    /**
+     * Identify this built-in rule set
+     * @return "deep-copy"
+     */
+
+    @Override
+    public String getName() {
+        return "deep-copy";
+    }
 }
