@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,7 +12,6 @@ import net.sf.saxon.event.Receiver;
 import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.om.*;
-import net.sf.saxon.pattern.AnyNodeTest;
 import net.sf.saxon.pattern.NodeTest;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.*;
@@ -81,7 +80,7 @@ public class NamespaceNode implements NodeInfo {
      * @return a singleton iterator that returns this item
      */
 
-    public UnfailingIterator iterate() {
+    public UnfailingIterator<NodeInfo> iterate() {
         return SingletonIterator.makeIterator(this);
     }
 
@@ -99,7 +98,7 @@ public class NamespaceNode implements NodeInfo {
 
     /**
      * Determine whether this is the same node as another node.
-     * Note: a.isSameNodeInfo(b) if and only if generateId(a)==generateId(b).
+     * Note: a.equals(b) if and only if generateId(a)==generateId(b).
      * This method has the same semantics as isSameNode() in DOM Level 3, but
      * works on Saxon NodeInfo objects rather than DOM Node objects.
      *
@@ -108,30 +107,13 @@ public class NamespaceNode implements NodeInfo {
      *         the same node in the tree.
      */
 
-    public boolean isSameNodeInfo(/*@NotNull*/ NodeInfo other) {
+    public boolean equals(Object other) {
         return other instanceof NamespaceNode &&
-                element.isSameNodeInfo(((NamespaceNode) other).element) &&
+                element.equals(((NamespaceNode) other).element) &&
                 nsBinding.equals(((NamespaceNode) other).nsBinding);
 
     }
 
-    /**
-     * The equals() method compares nodes for identity. It is defined to give the same result
-     * as isSameNodeInfo().
-     *
-     * @param other the node to be compared with this node
-     * @return true if this NodeInfo object and the supplied NodeInfo object represent
-     *         the same node in the tree.
-     * @since 8.7 Previously, the effect of the equals() method was not defined. Callers
-     *        should therefore be aware that third party implementations of the NodeInfo interface may
-     *        not implement the correct semantics. It is safer to use isSameNodeInfo() for this reason.
-     *        The equals() method has been defined because it is useful in contexts such as a Java Set or HashMap.
-     */
-
-    public boolean equals(Object other) {
-        return other instanceof NamespaceNode && isSameNodeInfo((NodeInfo) other);
-    }
-
     /**
      * The hashCode() method obeys the contract for hashCode(): that is, if two objects are equal
      * (represent the same node) then they must have the same hashCode()
@@ -226,45 +208,16 @@ public class NamespaceNode implements NodeInfo {
      */
 
     public int compareOrder(/*@NotNull*/ NodeInfo other) {
-        if (other instanceof NamespaceNode && element.isSameNodeInfo(((NamespaceNode) other).element)) {
-            // alternative: return Integer.signum(position - ((NamespaceNodeI)other).position);
+        if (other instanceof NamespaceNode && element.equals(((NamespaceNode) other).element)) {
             int c = position - ((NamespaceNode) other).position;
-            if (c == 0) {
-                return 0;
-            }
-            if (c < 0) {
-                return -1;
-            }
-            return +1;
-        } else if (element.isSameNodeInfo(other)) {
+            return Integer.compare(c, 0);
+        } else if (element.equals(other)) {
             return +1;
         } else {
             return element.compareOrder(other);
         }
     }
 
-    /**
-     * Determine the relative position of this node and another node, in document order,
-     * distinguishing whether the first node is a preceding, following, descendant, ancestor,
-     * or the same node as the second.
-     * <p/>
-     * The other node must always be in the same tree; the effect of calling this method
-     * when the two nodes are in different trees is undefined. If either node is a namespace
-     * or attribute node, the method should throw UnsupportedOperationException.
-     *
-     * @param other The other node, whose position is to be compared with this
-     *              node
-     * @return {@link net.sf.saxon.om.AxisInfo#PRECEDING} if this node is on the preceding axis of the other node;
-     *         {@link net.sf.saxon.om.AxisInfo#FOLLOWING} if it is on the following axis; {@link net.sf.saxon.om.AxisInfo#ANCESTOR} if the first node is an
-     *         ancestor of the second; {@link net.sf.saxon.om.AxisInfo#DESCENDANT} if the first is a descendant of the second;
-     *         {@link net.sf.saxon.om.AxisInfo#SELF} if they are the same node.
-     * @throws UnsupportedOperationException if either node is an attribute or namespace
-     * @since 9.5
-     */
-    public int comparePosition(NodeInfo other) {
-        throw new UnsupportedOperationException("comparePosition(namespace)");
-    }
-
     /**
      * Return the string value of the node. The interpretation of this depends on the type
      * of node. For a namespace node, it is the namespace URI.
@@ -391,7 +344,6 @@ public class NamespaceNode implements NodeInfo {
     /**
      * Get the type annotation of this node, if any. The type annotation is represented as
      * SchemaType object.
-     * <p/>
      * <p>Types derived from a DTD are not reflected in the result of this method.</p>
      *
      * @return For element and attribute nodes: the type annotation derived from schema
@@ -415,22 +367,6 @@ public class NamespaceNode implements NodeInfo {
         return element;
     }
 
-    /**
-     * Return an iteration over all the nodes reached by the given axis from this node
-     *
-     * @param axisNumber an integer identifying the axis; one of the constants
-     *                   defined in class net.sf.saxon.om.Axis
-     * @return an AxisIterator that scans the nodes reached by the axis in
-     *         turn.
-     * @throws UnsupportedOperationException if the namespace axis is
-     *                                       requested and this axis is not supported for this implementation.
-     * @see net.sf.saxon.om.AxisInfo
-     */
-
-    public AxisIterator iterateAxis(byte axisNumber) {
-        return iterateAxis(axisNumber, AnyNodeTest.getInstance());
-    }
-
     /**
      * Return an iteration over all the nodes reached by the given axis from this node
      * that match a given NodeTest
@@ -518,9 +454,9 @@ public class NamespaceNode implements NodeInfo {
     }
 
     /**
-     * Determine whether the node has any children. <br />
-     * Note: the result is equivalent to <br />
-     * getEnumeration(Axis.CHILD, AnyNodeTest.getInstance()).hasNext()
+     * Determine whether the node has any children.
+     * <p>Note: the result is equivalent to
+     * <code>getEnumeration(Axis.CHILD, AnyNodeTest.getInstance()).hasNext()</code></p>
      *
      * @return True if the node has one or more children
      */
@@ -567,7 +503,6 @@ public class NamespaceNode implements NodeInfo {
      *         If the bottom half is zero, then this is a namespace undeclaration rather than a declaration.
      *         The XML namespace is never included in the list. If the supplied array is larger than required,
      *         then the first unused entry will be set to -1.
-     *         <p/>
      *         <p>For a node other than an element, the method returns null.</p>
      */
 
@@ -578,7 +513,6 @@ public class NamespaceNode implements NodeInfo {
 
     /**
      * Set the system identifier for this Source.
-     * <p/>
      * <p>The system identifier is optional if the source does not
      * get its data from a URL, but it may still be useful to provide one.
      * The application can use a system identifier, for example, to resolve
@@ -602,38 +536,11 @@ public class NamespaceNode implements NodeInfo {
         return new StringValue(getStringValueCS());
     }
 
-
-    /**
-     * Determine whether this node has the is-id property
-     *
-     * @return true if the node is an ID
-     */
-
-    public boolean isId() {
-        return false;
+    @Override
+    public boolean isStreamed() {
+        return element.isStreamed();
     }
 
-    /**
-     * Determine whether this node has the is-idref property
-     *
-     * @return true if the node is an IDREF or IDREFS element or attribute
-     */
-
-    public boolean isIdref() {
-        return false;
-    }
-
-    /**
-     * Determine whether the node has the is-nilled property
-     *
-     * @return true if the node has the is-nilled property
-     */
-
-    public boolean isNilled() {
-        return false;
-    }
-
-
     /**
      * Factory method to create an iterator over the in-scope namespace nodes of an element
      *
@@ -644,7 +551,7 @@ public class NamespaceNode implements NodeInfo {
 
     /*@NotNull*/
     public static AxisIterator makeIterator(final NodeInfo element, /*@NotNull*/ NodeTest test) {
-        List<NamespaceNode> nodes = new ArrayList<NamespaceNode>();
+        List<NodeInfo> nodes = new ArrayList<>();
         Iterator<NamespaceBinding> bindings = NamespaceIterator.iterateNamespaces(element);
         int position = 0;
         while (bindings.hasNext()) {
@@ -659,15 +566,5 @@ public class NamespaceNode implements NodeInfo {
         }
         return new ListIterator.OfNodes(nodes);
     }
-
-    /**
-     * Ask whether this is a node in a streamed document
-     *
-     * @return true if the node is in a document being processed using streaming
-     */
-
-    public boolean isStreamed() {
-        return false;
-    }
 }
 
