@@ -9,7 +9,6 @@ package net.sf.saxon.tree.iter;
 
 import net.sf.saxon.expr.LastPositionFinder;
 import net.sf.saxon.om.GroundedValue;
-import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.EmptySequence;
 
@@ -82,12 +81,12 @@ public class SingleAtomicIterator implements AtomicIterator,
      */
 
     /*@NotNull*/
-    public GroundedValue materialize() {
+    public GroundedValue<AtomicValue> materialize() {
         return item;
     }
 
     @Override
-    public GroundedValue getResidue() throws XPathException {
+    public GroundedValue<AtomicValue> getResidue() {
         return gone ? EmptySequence.getInstance() : item;
     }
 
