@@ -7,8 +7,10 @@
 
 package net.sf.saxon.tree.iter;
 
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.value.SequenceExtent;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -22,7 +24,7 @@ import java.util.function.Predicate;
  * is a SequenceIterator that throws no checked exceptions.
  */
 
-public interface UnfailingIterator<T extends Item> extends SequenceIterator<T> {
+public interface UnfailingIterator<T extends Item<?>> extends SequenceIterator<T> {
 
     /**
      * Get the next item in the sequence.
@@ -56,5 +58,9 @@ public interface UnfailingIterator<T extends Item> extends SequenceIterator<T> {
         forEach(list::add);
         return list;
     }
+
+    default GroundedValue<T> toGroundedValue() {
+        return SequenceExtent.makeSequenceExtent(toList());
+    }
 }
 
