@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -22,6 +22,7 @@ import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.tree.util.NamespaceIterator;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.type.*;
+import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.Whitespace;
 
 import java.util.Arrays;
@@ -161,7 +162,7 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
      */
 
     public String getBaseURI() {
-        return Navigator.getBaseURI(this);
+        return Navigator.getBaseURI(this, n -> getPhysicalRoot().isTopWithinEntity((ElementImpl)n));
     }
 
     /**
@@ -274,9 +275,8 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
 
     /**
      * Copy this node to a given Receiver.
-     * <p/>
-     * This method is primarily for internal use. It should not be considered a stable
-     * part of the Saxon API.
+     * <p>This method is primarily for internal use. It should not be considered a stable
+     * part of the Saxon API.</p>
      *
      * @param out         the Receiver to which the node should be copied. It is the caller's
      *                    responsibility to ensure that this Receiver is open before the method is called
@@ -384,7 +384,7 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
         }
         if (!uc.equals(uri)) {
             if (uc.isEmpty()) {
-                addNamespace(ns, true);
+                addNamespace(ns, false);
             } else {
                 throw new IllegalArgumentException(
                     "Namespace binding of new name conflicts with existing namespace binding");
@@ -401,31 +401,16 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
      *                added. If the target element already has a namespace binding with this (prefix, uri) pair, the call has
      *                no effect. If the target element currently has a namespace binding with this prefix and a different URI, an
      *                exception is raised.
-     * @param inherit If true, the new namespace binding will be inherited by any children of the target element
-     *                that do not already have a namespace binding for the specified prefix, recursively.
-     *                If false, the new namespace binding will not be inherited.
+     * @param unused
      * @throws IllegalArgumentException if the target element already has a namespace binding for this prefix,
      *                                  or if the namespace code represents a namespace undeclaration
      */
 
-    public void addNamespace(/*@NotNull*/ NamespaceBinding nscode, boolean inherit) {
+    public void addNamespace(/*@NotNull*/ NamespaceBinding nscode, boolean unused) {
         if (nscode.getURI().isEmpty()) {
             throw new IllegalArgumentException("Cannot add a namespace undeclaration");
         }
         addNamespaceInternal(nscode, true);
-
-        // The data model is such that namespaces are inherited by default. If inheritance is NOT requested,
-        // we must process the children to add namespace undeclarations
-        if (hasChildNodes() && !inherit) {
-            AxisIterator kids = iterateChildren(NodeKindTest.ELEMENT);
-            while (true) {
-                ElementImpl child = (ElementImpl) kids.next();
-                if (child == null) {
-                    break;
-                }
-                child.addNamespaceInternal(nscode, false);
-            }
-        }
     }
 
     private void addNamespaceInternal(/*@NotNull*/ NamespaceBinding nscode, boolean externalCall) {
@@ -476,11 +461,9 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
 
     /**
      * Add an attribute to this element node.
-     * <p/>
      * <p>If this node is not an element, or if the supplied node is not an attribute, the method
      * takes no action. If the element already has an attribute with this name, the method
      * throws an exception.</p>
-     * <p/>
      * <p>This method does not perform any namespace fixup. It is the caller's responsibility
      * to ensure that any namespace prefix used in the name of the attribute (or in its value
      * if it has a namespace-sensitive type) is declared on this element.</p>
@@ -511,7 +494,7 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
             String uc = getURIForPrefix(prefix, false);
             if (uc == null) {
                 // The namespace is not already declared on the element
-                addNamespace(binding, true);
+                addNamespace(binding, false);
             } else if (!uc.equals(binding.getURI())) {
                 throw new IllegalStateException(
                         "Namespace binding of new name conflicts with existing namespace binding");
@@ -527,10 +510,8 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
 
     /**
      * Remove an attribute from this element node
-     * <p/>
      * <p>If this node is not an element, or if the specified node is not an attribute
      * of this element, this method takes no action.</p>
-     * <p/>
      * <p>The attribute object itself becomes unusable; any attempt to use this attribute object,
      * or any other object representing the same node, is likely to result in an exception.</p>
      *
@@ -602,7 +583,7 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
             for (NamespaceBinding aNamespaceList : namespaceList) {
                 if (aNamespaceList.getPrefix().equals(prefix)) {
                     String uri = aNamespaceList.getURI();
-                    return uri.isEmpty() && prefix.length() != 0 ? null : uri;
+                    return uri.isEmpty() && !prefix.isEmpty() ? null : uri;
                 }
             }
         }
@@ -673,7 +654,6 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
      *         this element. For a node other than an element, return null.
      *         The XML namespace is never included in the list. If the supplied array is larger than required,
      *         then the first unused entry will be set to null.
-     *         <p/>
      *         <p>For a node other than an element, the method returns null.</p>
      */
 
@@ -695,7 +675,7 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
             return;
         }
 
-        Set<NamespaceBinding> childNamespaces = new HashSet<NamespaceBinding>();
+        Set<NamespaceBinding> childNamespaces = new HashSet<>();
         if (namespaceList != null) {
             childNamespaces.addAll(Arrays.asList(namespaceList));
         }
@@ -704,7 +684,7 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
 
         // If the child is in the null namespace but the parent has a default namespace, xmlns="" should be added.
 
-        if (getURI().isEmpty() && inscope.getURIForPrefix("", true).length() != 0) {
+        if (getURI().isEmpty() && !inscope.getURIForPrefix("", true).isEmpty()) {
             childNamespaces.add(NamespaceBinding.DEFAULT_UNDECLARATION);
         }
 
@@ -805,5 +785,34 @@ public class ElementImpl extends ParentNodeImpl implements NamespaceResolver {
             return false;
         }
     }
+
+    public boolean isIdref() {
+        return isIdRefNode(this);
+    }
+
+    static boolean isIdRefNode(NodeImpl node) {
+        SchemaType type = node.getSchemaType();
+        try {
+            if (type.isIdRefType()) {
+                if (type == BuiltInAtomicType.IDREF || type == BuiltInListType.IDREFS) {
+                    return true;
+                }
+                try {
+                    for (AtomicValue av : node.atomize()) {
+                        if (av.getItemType().isIdRefType()) {
+                            return true;
+                        }
+                    }
+                } catch (XPathException err) {
+                    // no action
+                }
+            }
+        } catch (MissingComponentException e) {
+            return false;
+        }
+        return false;
+    }
+
+
 }
 
