@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -48,7 +48,7 @@ abstract class ParentNodeImpl extends NodeImpl {
      */
 
     protected final long getSequenceNumber() {
-        return (getRawSequenceNumber() == -1 ? -1L : ((long) getRawSequenceNumber()) << 32);
+        return getRawSequenceNumber() == -1 ? -1L : (long) getRawSequenceNumber() << 32;
     }
 
     protected final int getRawSequenceNumber() {
@@ -75,7 +75,7 @@ abstract class ParentNodeImpl extends NodeImpl {
      */
 
     public final boolean hasChildNodes() {
-        return (children != null);
+        return children != null;
     }
 
     /**
@@ -147,8 +147,12 @@ abstract class ParentNodeImpl extends NodeImpl {
 
     /*@Nullable*/
     public final NodeImpl getLastChild() {
-        if (children == null) return null;
-        if (children instanceof NodeImpl) return (NodeImpl) children;
+        if (children == null) {
+            return null;
+        }
+        if (children instanceof NodeImpl) {
+            return (NodeImpl) children;
+        }
         NodeImpl[] n = (NodeImpl[]) children;
         return n[n.length - 1];
     }
@@ -162,12 +166,16 @@ abstract class ParentNodeImpl extends NodeImpl {
 
     /*@Nullable*/
     protected final NodeImpl getNthChild(int n) {
-        if (children == null) return null;
+        if (children == null) {
+            return null;
+        }
         if (children instanceof NodeImpl) {
-            return (n == 0 ? (NodeImpl) children : null);
+            return n == 0 ? (NodeImpl) children : null;
         }
         NodeImpl[] nodes = (NodeImpl[]) children;
-        if (n < 0 || n >= nodes.length) return null;
+        if (n < 0 || n >= nodes.length) {
+            return null;
+        }
         return nodes[n];
     }
 
@@ -218,13 +226,12 @@ abstract class ParentNodeImpl extends NodeImpl {
         boolean prevText = false;
         int j = 0;
         NodeImpl[] c2 = new NodeImpl[children.length];
-        for (int i = 0; i < children.length; i++) {
-            NodeImpl node = children[i];
+        for (NodeImpl node : children) {
             if (node instanceof TextImpl) {
                 if (prevText) {
-                    TextImpl prev = ((TextImpl) c2[j - 1]);
+                    TextImpl prev = (TextImpl) c2[j - 1];
                     prev.replaceStringValue(prev.getStringValue() + node.getStringValue());
-                } else if (node.getStringValue().length() > 0) {
+                } else if (!node.getStringValue().isEmpty()) {
                     prevText = true;
                     node.setSiblingPosition(j);
                     c2[j++] = node;
@@ -258,7 +265,7 @@ abstract class ParentNodeImpl extends NodeImpl {
     public CharSequence getStringValueCS() {
         FastStringBuffer sb = null;
 
-        NodeImpl next = (NodeImpl) getFirstChild();
+        NodeImpl next = getFirstChild();
         while (next != null) {
             if (next instanceof TextImpl) {
                 if (sb == null) {
@@ -268,7 +275,9 @@ abstract class ParentNodeImpl extends NodeImpl {
             }
             next = next.getNextInDocument(this);
         }
-        if (sb == null) return "";
+        if (sb == null) {
+            return "";
+        }
         return sb.condense();
     }
 
@@ -304,11 +313,9 @@ abstract class ParentNodeImpl extends NodeImpl {
 
     /**
      * Insert a sequence of nodes as children of this node.
-     * <p/>
      * <p>This method takes no action unless the target node is a document node or element node. It also
      * takes no action in respect of any supplied nodes that are not elements, text nodes, comments, or
      * processing instructions.</p>
-     * <p/>
      * <p>The supplied nodes will form the new children. Adjacent text nodes will be merged, and
      * zero-length text nodes removed. The supplied nodes may be modified in situ, for example to change their
      * parent property and to add namespace bindings, or they may be copied, at the discretion of
@@ -349,24 +356,14 @@ abstract class ParentNodeImpl extends NodeImpl {
      */
 
     protected synchronized void insertChildrenAt(/*@NotNull*/ NodeInfo[] source, int index, boolean inherit) {
-        NodeImpl[] source2 = new NodeImpl[source.length];
         if (source.length == 0) {
             return;
         }
-        for (int i = 0; i < source.length; i++) {
-            source2[i] = convertForeignNode(source[i]);
-            NodeImpl child = (NodeImpl) source2[i];
-            child.setRawParent(this);
-            if (child instanceof ElementImpl) {
-                // If the child has no xmlns="xxx" declaration, then add an xmlns="" to prevent false inheritance
-                // from the new parent
-                ((ElementImpl) child).fixupInsertedNamespaces(inherit);
-            }
-        }
+        NodeImpl[] source2 = adjustSuppliedNodeArray(source, inherit);
         if (children == null) {
             if (source2.length == 1) {
                 children = source2[0];
-                ((NodeImpl)children).setSiblingPosition(0);
+                ((NodeImpl) children).setSiblingPosition(0);
             } else {
                 children = cleanUpChildren(source2);
             }
@@ -402,6 +399,7 @@ abstract class ParentNodeImpl extends NodeImpl {
         }
     }
 
+
     /*@NotNull*/
     private NodeImpl convertForeignNode(/*@NotNull*/ NodeInfo source) {
         if (!(source instanceof NodeImpl)) {
@@ -441,9 +439,32 @@ abstract class ParentNodeImpl extends NodeImpl {
         if (children == null) {
             return;
         }
+        NodeImpl[] source2 = adjustSuppliedNodeArray(source, inherit);
+        if (children instanceof NodeImpl) {
+            if (source2.length == 0) {
+                children = null;
+            } else if (source2.length == 1) {
+                children = source2[0];
+            } else {
+                NodeImpl[] n2 = new NodeImpl[source2.length];
+                System.arraycopy(source2, 0, n2, 0, source.length);
+                children = cleanUpChildren(n2);
+            }
+        } else {
+            NodeImpl[] n0 = (NodeImpl[]) children;
+            NodeImpl[] n2 = new NodeImpl[n0.length + source2.length - 1];
+            System.arraycopy(n0, 0, n2, 0, index);
+            System.arraycopy(source2, 0, n2, index, source2.length);
+            System.arraycopy(n0, index + 1, n2, index + source2.length, n0.length - index - 1);
+            children = cleanUpChildren(n2);
+        }
+    }
+
+    private NodeImpl[] adjustSuppliedNodeArray(NodeInfo[] source, boolean inherit) {
+        NodeImpl[] source2 = new NodeImpl[source.length];
         for (int i = 0; i < source.length; i++) {
-            source[i] = convertForeignNode(source[i]);
-            NodeImpl child = (NodeImpl) source[i];
+            source2[i] = convertForeignNode(source[i]);
+            NodeImpl child = source2[i];
             child.setRawParent(this);
             if (child instanceof ElementImpl) {
                 // If the child has no xmlns="xxx" declaration, then add an xmlns="" to prevent false inheritance
@@ -451,25 +472,7 @@ abstract class ParentNodeImpl extends NodeImpl {
                 ((ElementImpl) child).fixupInsertedNamespaces(inherit);
             }
         }
-        if (children instanceof NodeImpl) {
-            if (source.length == 0) {
-                children = null;
-            } else if (source.length == 1) {
-                children = source[0];
-                ((NodeImpl)children).setSiblingPosition(0);
-            } else {
-                NodeImpl[] n2 = new NodeImpl[source.length];
-                System.arraycopy(source, 0, n2, 0, source.length);
-                children = cleanUpChildren(n2);
-            }
-        } else {
-            NodeImpl[] n0 = (NodeImpl[]) children;
-            NodeImpl[] n2 = new NodeImpl[n0.length + source.length - 1];
-            System.arraycopy(n0, 0, n2, 0, index);
-            System.arraycopy(source, 0, n2, index, source.length);
-            System.arraycopy(n0, index + 1, n2, index + source.length, n0.length - index - 1);
-            children = cleanUpChildren(n2);
-        }
+        return source2;
     }
 
 
