@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,24 +9,26 @@ package net.sf.saxon.tree.tiny;
 
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.pattern.NodeTest;
-import net.sf.saxon.tree.iter.AxisIteratorImpl;
-import net.sf.saxon.z.IntPredicate;
+import net.sf.saxon.tree.iter.AxisIterator;
+import net.sf.saxon.type.Type;
+
+import java.util.function.IntPredicate;
 
 /**
  * This class supports both the descendant:: and descendant-or-self:: axes, which are
  * identical except for the route to the first candidate node.
- * It enumerates descendants of the specified node.
+ * It enumerates descendants of the specified node. This version includes all text nodes
+ * in the result (assuming they are matched), but does not handle grafted external nodes
  * The calling code must ensure that the start node is not an attribute or namespace node.
  */
 
-final class DescendantIterator extends AxisIteratorImpl {
+final class DescendantIterator implements AxisIterator {
 
     private final TinyTree tree;
-    private final TinyNodeImpl startNode;
     private int nextNodeNr;
     private final int startDepth;
-    private final NodeTest test;
     private final IntPredicate matcher;
+    private NodeInfo pending = null;
 
     /**
      * Create an iterator over the descendant axis
@@ -38,8 +40,6 @@ final class DescendantIterator extends AxisIteratorImpl {
 
     DescendantIterator(/*@NotNull*/ TinyTree doc, /*@NotNull*/ TinyNodeImpl node, NodeTest nodeTest) {
         tree = doc;
-        startNode = node;
-        test = nodeTest;
         nextNodeNr = node.nodeNr;
         startDepth = doc.depth[nextNodeNr];
         matcher = nodeTest.getMatcher(doc);
@@ -49,6 +49,11 @@ final class DescendantIterator extends AxisIteratorImpl {
     /*@Nullable*/
     public NodeInfo next() {
         do {
+            if (pending != null) {
+                NodeInfo p = pending;
+                pending = null;
+                return p;
+            }
             nextNodeNr++;
             try {
                 if (tree.depth[nextNodeNr] <= startDepth) {
@@ -63,8 +68,10 @@ final class DescendantIterator extends AxisIteratorImpl {
                 nextNodeNr = -1;
                 return null;
             }
-        } while (!matcher.matches(nextNodeNr));
-
+            if (tree.nodeKind[nextNodeNr] == Type.TEXTUAL_ELEMENT) {
+                pending = ((TinyTextualElement)tree.getNode(nextNodeNr)).getTextNode();
+            }
+        } while (!matcher.test(nextNodeNr));
         return tree.getNode(nextNodeNr);
     }
 
