@@ -18,6 +18,8 @@ import net.sf.saxon.tree.iter.AxisIterator;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.tree.wrapper.VirtualCopy;
 
+import java.util.Optional;
+
 /**
  * This class represents an element node in a TinyTree that is actually a reference to an element
  * node held elsewhere. Some properties of the node come from the containing TinyTree (for example,
@@ -30,6 +32,16 @@ public class GraftedElement extends TinyElementImpl {
 
     private NodeInfo externalNode;
     private VirtualCopy externalNodeCopy;
+    private Optional<Boolean> hasDefaultNamespace = Optional.empty();
+
+    /**
+     * Created a grafted element node
+     * @param tree the host tree that is to contain a reference to an external node
+     * @param nodeNr the node number at which the external reference is to appear
+     * @param externalNode the external node, in some other tree
+     * @param copyNamespaces indicates whether namespaces in the external tree are
+     *                       retained in the virtual copy.
+     */
 
     public GraftedElement(TinyTree tree, int nodeNr, NodeInfo externalNode, boolean copyNamespaces) {
         super(tree, nodeNr);
@@ -139,6 +151,22 @@ public class GraftedElement extends TinyElementImpl {
         }
     }
 
+    @Override
+    public String getAttributeValue(String uri, String local) {
+        return externalNode.getAttributeValue(uri, local);
+    }
 
+    public boolean hasDefaultNamespace() {
+        if (!hasDefaultNamespace.isPresent()) {
+            hasDefaultNamespace = Optional.of(
+                    !new InscopeNamespaceResolver(getParent()).getURIForPrefix("", true).isEmpty());
+        }
+        return hasDefaultNamespace.get();
+    }
+
+    @Override
+    public NamespaceBinding[] getDeclaredNamespaces(NamespaceBinding[] buffer) {
+        return externalNodeCopy.getDeclaredNamespaces(buffer);
+    }
 }
 
