@@ -38,11 +38,17 @@ final public class TinyAttributeImpl extends TinyNodeImpl {
         // no action: an attribute has the same base URI as its parent
     }
 
+    @Override
+    public String getSystemId() {
+        NodeInfo parent = getParent();
+        return parent == null ? null : getParent().getSystemId();
+    }
+
     /**
      * Get the parent node
      */
 
-    public NodeInfo getParent() {
+    public TinyNodeImpl getParent() {
         return tree.getNode(tree.attParent[nodeNr]);
     }
 
@@ -178,7 +184,11 @@ final public class TinyAttributeImpl extends TinyNodeImpl {
      */
 
     public final String getURI() {
-        return tree.getNamePool().getURI(tree.attCode[nodeNr]);
+        int code = tree.attCode[nodeNr];
+        if (!NamePool.isPrefixed(code)) {
+            return "";
+        }
+        return tree.getNamePool().getURI(code);
     }
 
     /**
