@@ -39,7 +39,7 @@ public class TinyBuilder extends Builder {
     private int currentDepth = 0;
     private int nodeNr = 0;             // this is the local sequence within this document
     private boolean ended = false;
-    private Statistics statistics = Statistics.TEMPORARY_TREE_STATISTICS;
+    private Statistics statistics;
     private boolean markDefaultedAttributes = false;
     private int textualElementEligibilityState = 0;
 
@@ -52,6 +52,7 @@ public class TinyBuilder extends Builder {
     public TinyBuilder(/*@NotNull*/ PipelineConfiguration pipe) {
         super(pipe);
         Configuration config = pipe.getConfiguration();
+        statistics = config.getTreeStatistics().TEMPORARY_TREE_STATISTICS;
         //markDefaultedAttributes = true;
         markDefaultedAttributes = config.isExpandAttributeDefaults() && config.getBooleanProperty(Feature.MARK_DEFAULTED_ATTRIBUTES);
         //System.err.println("TinyBuilder " + this);
@@ -190,7 +191,7 @@ public class TinyBuilder extends Builder {
         currentDepth = 0;
         nodeNr = 0;
         ended = false;
-        statistics = Statistics.TEMPORARY_TREE_STATISTICS;
+        statistics = config.getTreeStatistics().TEMPORARY_TREE_STATISTICS;
     }
 
     public void close() throws XPathException {
@@ -523,18 +524,20 @@ public class TinyBuilder extends Builder {
     }
 
     public boolean isPositionedAtElement() {
-        return (tree.nodeKind[tree.numberOfNodes-1]&0x0f) == Type.ELEMENT;
+        return tree.numberOfNodes > 0 && (tree.nodeKind[tree.numberOfNodes-1]&0x0f) == Type.ELEMENT;
     }
 
     /**
      * Copy an element node and its subtree from another TinyTree instance
-     * @param source the TinyTree from which a subtree is to be copied
-     * @param nodeNr the node number of the element node to be copied
+     *
+     * @param sourceNode the element at the root of the subtree to be copied
      */
 
-    public void bulkCopy(TinyTree source, int nodeNr) {
+    public void bulkCopy(TinyElementImpl sourceNode, boolean copyNamespaces) {
+        TinyTree sourceTree = sourceNode.tree;
+        int oldNodeNr = sourceNode.nodeNr;
         int newNodeNr = tree.numberOfNodes;
-        tree.bulkCopy(source, nodeNr, currentDepth);
+        tree.bulkCopy(sourceTree, oldNodeNr, prevAtDepth, currentDepth);
         int prev = prevAtDepth[currentDepth];
         if (prev > 0) {
             tree.next[prev] = newNodeNr;
@@ -542,7 +545,6 @@ public class TinyBuilder extends Builder {
         tree.next[newNodeNr] = prevAtDepth[currentDepth - 1];
         prevAtDepth[currentDepth] = newNodeNr;
         siblingsAtDepth[currentDepth]++;
-        //tree.diagnosticDump();
     }
 
     /**
@@ -553,7 +555,7 @@ public class TinyBuilder extends Builder {
 
     public void graft(NodeInfo node, boolean copyNamespaces) {
         int newNodeNr = tree.numberOfNodes;
-        tree.graft(node, nodeNr, currentDepth, copyNamespaces);
+        tree.graft(node, newNodeNr, currentDepth, copyNamespaces);
         int prev = prevAtDepth[currentDepth];
         if (prev > 0) {
             tree.next[prev] = newNodeNr;
