@@ -62,6 +62,32 @@ public final class Navigator {
         return element.getAttributeValue(uri, localName);
     }
 
+    /**
+     * Get the string value of an inherited attribute of a given element, given the URI and
+     * local part of the attribute name. If the element does not have such an attribute, go up
+     * the ancestor axis to find such an attribute
+     *
+     * @param element   the element where the search should start
+     * @param uri       The namespace URI of the attribute name.
+     *                  The "no namespace" case is represented as an empty string.
+     * @param localName The local part of the attribute name.
+     * @return the attribute value, or null if the attribute is not present on any ancestor
+     * @since 9.9
+     */
+
+    public static String getInheritedAttributeValue(NodeInfo element, String uri, String localName) {
+        NodeInfo node = element;
+        while (node != null) {
+            String value = node.getAttributeValue(uri, localName);
+            if (value == null) {
+                node = node.getParent();
+            } else {
+                return value;
+            }
+        }
+        return null;
+    }
+
     /**
      * Helper method to get the name of a node as a structuredQName. Used from bytecode
      * @param node the node
@@ -151,7 +177,9 @@ public final class Navigator {
 
     /*@Nullable*/
     public static String getBaseURI(/*@NotNull*/ NodeInfo node, Predicate<NodeInfo> isTopElementWithinEntity) {
-        String xmlBase = node.getAttributeValue(NamespaceConstant.XML, "base");
+        String xmlBase = node instanceof TinyElementImpl ?
+                ((TinyElementImpl)node).getAttributeValue(StandardNames.XML_BASE) :
+                node.getAttributeValue(NamespaceConstant.XML, "base");
         if (xmlBase != null) {
             URI baseURI;
             try {
@@ -250,7 +278,7 @@ public final class Navigator {
                     if (pre.equals("/")) {
                         return '/' + node.getDisplayName();
                     } else {
-                        return pre + '/' + node.getDisplayName() + (streamed ? "" : '[' + getNumberSimple(node, context) + ']');
+                        return pre + '/' + node.getDisplayName() + (streamed ? "" : "[" + getNumberSimple(node, context) + "]");
                     }
                 }
             case Type.ATTRIBUTE:
@@ -258,17 +286,17 @@ public final class Navigator {
             case Type.TEXT:
                 pre = getPath(parent, context);
                 return (pre.equals("/") ? "" : pre) +
-                            "/text()" + (streamed ? "" : '[' + getNumberSimple(node, context) + ']');
+                            "/text()" + (streamed ? "" : "[" + getNumberSimple(node, context) + "]");
 
             case Type.COMMENT:
                 pre = getPath(parent, context);
                 return (pre.equals("/") ? "" : pre) +
-                            "/comment()" + (streamed ? "" : '[' + getNumberSimple(node, context) + ']');
+                            "/comment()" + (streamed ? "" : "[" + getNumberSimple(node, context) + "]");
 
             case Type.PROCESSING_INSTRUCTION:
                 pre = getPath(parent, context);
                 return (pre.equals("/") ? "" : pre) +
-                            "/processing-instruction()" + (streamed ? "" : '[' + getNumberSimple(node, context) + ']');
+                            "/processing-instruction()" + (streamed ? "" : "[" + getNumberSimple(node, context) + "]");
                 
             case Type.NAMESPACE:
                 String test = node.getLocalPart();
