@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,6 @@ package net.sf.saxon.tree.util;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.event.Builder;
-import net.sf.saxon.event.Receiver;
 import net.sf.saxon.event.ReceiverOptions;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.om.*;
@@ -18,7 +17,6 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.AxisIterator;
 import net.sf.saxon.tree.iter.EmptyIterator;
 import net.sf.saxon.tree.iter.SingleNodeIterator;
-import net.sf.saxon.tree.iter.SingletonIterator;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.StringValue;
 import net.sf.saxon.value.UntypedAtomicValue;
@@ -27,7 +25,6 @@ import net.sf.saxon.value.UntypedAtomicValue;
  * A node (implementing the NodeInfo interface) representing an attribute, text node,
  * comment, processing instruction, or namespace that has no parent (and of course no children).
  * Exceptionally it is also used (during whitespace stripping) to represent a standalone element.
- * <p/>
  * <p>In general this class does not impose constraints defined in the data model: that is the responsibility
  * of the client. For example, the class does not prevent you from creating a comment or text node that has
  * a name or a non-trivial type annotation.</p>
@@ -35,13 +32,14 @@ import net.sf.saxon.value.UntypedAtomicValue;
  * @author Michael H. Kay
  */
 
-public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
+public final class Orphan implements MutableNodeInfo {
 
     private short kind;
     /*@Nullable*/ private NodeName nodeName = null;
     private CharSequence stringValue;
     private SchemaType typeAnnotation = null;
     private int options = 0; // Bit-settings defined in ReceiverOptions
+    private GenericTreeInfo treeInfo;
 
     /**
      * Create an Orphan node
@@ -50,8 +48,8 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
      */
 
     public Orphan(Configuration config) {
-        super(config);
-        setRootNode(this);
+        treeInfo = new GenericTreeInfo(config);
+        treeInfo.setRootNode(this);
     }
 
     /**
@@ -61,30 +59,63 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
      * @since 9.7
      */
     public TreeInfo getTreeInfo() {
-        return this;
+        return treeInfo;
     }
 
     /**
-     * To implement {@link Sequence}, this method returns the item itself
+     * Get the System ID for the node. Note this is not the
+     * same as the base URI: the base URI can be modified by xml:base, but
+     * the system ID cannot. The base URI is used primarily for resolving
+     * relative URIs within the content of the document. The system ID is
+     * used primarily in conjunction with a line number, for identifying the
+     * location of elements within the source XML, in particular when errors
+     * are found. For a document node, the System ID represents the value of
+     * the document-uri property as defined in the XDM data model.
      *
-     * @return this item
+     * @return the System Identifier of the entity in the source document
+     * containing the node, or null if not known or not applicable.
+     * @since 8.4
      */
-
-    public NodeInfo head() {
-        return this;
+    @Override
+    public String getSystemId() {
+        return treeInfo.getSystemId();
     }
 
     /**
-     * To implement {@link Sequence}, this method returns a singleton iterator
-     * that delivers this item in the form of a sequence
+     * Get the Public ID of the entity containing the node.
      *
-     * @return a singleton iterator that returns this item
+     * @return the Public Identifier of the entity in the source document
+     * containing the node, or null if not known or not applicable
+     * @since 9.7
      */
-
-    public SequenceIterator iterate() {
-        return SingletonIterator.makeIterator(this);
+    @Override
+    public String getPublicId() {
+        return treeInfo.getPublicId();
     }
 
+    /**
+     * Set the system identifier for this Source.
+     * <p>
+     * <p>The system identifier is optional if the source does not
+     * get its data from a URL, but it may still be useful to provide one.
+     * The application can use a system identifier, for example, to resolve
+     * relative URIs and to include in error messages and warnings.</p>
+     *
+     * @param systemId The system identifier as a URL string.
+     */
+    @Override
+    public void setSystemId(String systemId) {
+        treeInfo.setSystemId(systemId);
+    }
+
+    /**
+     * Get the effective boolean value of this sequence
+     * @return the effective boolean value (always true for an Orphan node)
+     */
+    @Override
+    public boolean effectiveBooleanValue() {
+        return true;
+    }
 
     /**
      * Set the node kind
@@ -240,7 +271,6 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
     /**
      * Get the type annotation of this node, if any. The type annotation is represented as
      * SchemaType object.
-     * <p/>
      * <p>Types derived from a DTD are not reflected in the result of this method.</p>
      *
      * @return For element and attribute nodes: the type annotation derived from schema
@@ -262,34 +292,17 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
     }
 
     /**
-     * Determine whether this is the same node as another node. <br />
-     * Note: a.isSameNode(b) if and only if generateId(a)==generateId(b)
+     * Determine whether this is the same node as another node.
+     * <p>Note: a.equals(b) if and only if generateId(a)==generateId(b)</p>
      *
      * @return true if this Node object and the supplied Node object represent the
      *         same node in the tree.
      */
 
-    public boolean isSameNodeInfo(NodeInfo other) {
+    public boolean equals(NodeInfo other) {
         return this == other;
     }
 
-    /**
-     * The equals() method compares nodes for identity. It is defined to give the same result
-     * as isSameNodeInfo().
-     *
-     * @param other the node to be compared with this node
-     * @return true if this NodeInfo object and the supplied NodeInfo object represent
-     *         the same node in the tree.
-     * @since 8.7 Previously, the effect of the equals() method was not defined. Callers
-     *        should therefore be aware that third party implementations of the NodeInfo interface may
-     *        not implement the correct semantics. It is safer to use isSameNodeInfo() for this reason.
-     *        The equals() method has been defined because it is useful in contexts such as a Java Set or HashMap.
-     */
-
-    public boolean equals(Object other) {
-        return other instanceof NodeInfo && isSameNodeInfo((NodeInfo) other);
-    }
-
     /**
      * The hashCode() method obeys the contract for hashCode(): that is, if two objects are equal
      * (represent the same node) then they must have the same hashCode()
@@ -317,26 +330,6 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
         }
     }
 
-    /**
-     * Get line number
-     *
-     * @return the line number of the node in its original source document; or -1 if not available
-     */
-
-    public int getLineNumber() {
-        return -1;
-    }
-
-    /**
-     * Get column number
-     *
-     * @return the column number of the node in its original source document; or -1 if not available
-     */
-
-    public int getColumnNumber() {
-        return -1;
-    }
-
     /**
      * Get an immutable copy of this Location object. By default Location objects may be mutable, so they
      * should not be saved for later use. The result of this operation holds the same location information,
@@ -359,38 +352,10 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
     public int compareOrder(/*@NotNull*/ NodeInfo other) {
 
         // are they the same node?
-        if (this.isSameNodeInfo(other)) {
+        if (this.equals(other)) {
             return 0;
         }
-        return (this.hashCode() < other.hashCode() ? -1 : +1);
-    }
-
-    /**
-     * Determine the relative position of this node and another node, in document order,
-     * distinguishing whether the first node is a preceding, following, descendant, ancestor,
-     * or the same node as the second.
-     * <p/>
-     * The other node must always be in the same tree; the effect of calling this method
-     * when the two nodes are in different trees is undefined. If either node is a namespace
-     * or attribute node, the method should throw UnsupportedOperationException.
-     *
-     * @param other The other node, whose position is to be compared with this
-     *              node
-     * @return {@link net.sf.saxon.om.AxisInfo#PRECEDING} if this node is on the preceding axis of the other node;
-     *         {@link net.sf.saxon.om.AxisInfo#FOLLOWING} if it is on the following axis; {@link net.sf.saxon.om.AxisInfo#ANCESTOR} if the first node is an
-     *         ancestor of the second; {@link net.sf.saxon.om.AxisInfo#DESCENDANT} if the first is a descendant of the second;
-     *         {@link net.sf.saxon.om.AxisInfo#SELF} if they are the same node.
-     * @throws UnsupportedOperationException if either node is an attribute or namespace
-     * @since 9.5
-     */
-    public int comparePosition(NodeInfo other) {
-        if (kind == Type.ATTRIBUTE || kind == Type.NAMESPACE) {
-            throw new UnsupportedOperationException();
-        }
-        if (this.isSameNodeInfo(other)) {
-            return AxisInfo.SELF;
-        }
-        throw new UnsupportedOperationException();
+        return this.hashCode() < other.hashCode() ? -1 : +1;
     }
 
     /**
@@ -597,14 +562,6 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
         buffer.append(Integer.toString(hashCode()));
     }
 
-    /**
-     * Copy this node to a given outputter (deep copy)
-     */
-
-    public void copy(/*@NotNull*/ Receiver out, int copyOptions, Location locationId) throws XPathException {
-        Navigator.copy(this, out, copyOptions, locationId);
-    }
-
     /**
      * Get all namespace undeclarations and undeclarations defined on this element.
      *
@@ -617,7 +574,6 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
      *         If the bottom half is zero, then this is a namespace undeclaration rather than a declaration.
      *         The XML namespace is never included in the list. If the supplied array is larger than required,
      *         then the first unused entry will be set to -1.
-     *         <p/>
      *         <p>For a node other than an element, the method returns null.</p>
      */
 
@@ -646,16 +602,6 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
         return isOption(ReceiverOptions.IS_IDREF);
     }
 
-    /**
-     * Ask whether the node has the is-nilled property
-     *
-     * @return true if the node has the is-nilled property
-     */
-
-    public boolean isNilled() {
-        return false;
-    }
-
     /**
      * Ask whether the node has the disable-output-escaping property
      * @return true if the node has the disable-output-escaping property
@@ -667,11 +613,9 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
 
     /**
      * Insert copies of a sequence of nodes as children of this node.
-     * <p/>
      * <p>This method takes no action unless the target node is a document node or element node. It also
      * takes no action in respect of any supplied nodes that are not elements, text nodes, comments, or
      * processing instructions.</p>
-     * <p/>
      * <p>The supplied nodes will be copied to form the new children. Adjacent text nodes will be merged, and
      * zero-length text nodes removed.</p>
      *
@@ -687,12 +631,10 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
 
     /**
      * Insert copies of a sequence of nodes as siblings of this node.
-     * <p/>
      * <p>This method takes no action unless the target node is an element, text node, comment, or
      * processing instruction, and one that has a parent node. It also
      * takes no action in respect of any supplied nodes that are not elements, text nodes, comments, or
      * processing instructions.</p>
-     * <p/>
      * <p>The supplied nodes must use the same data model implementation as the tree into which they
      * will be inserted.</p>
      *
@@ -708,10 +650,8 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
 
     /**
      * Remove an attribute from this element node
-     * <p/>
      * <p>If this node is not an element, or if it has no attribute with the specified name,
      * this method takes no action.</p>
-     * <p/>
      * <p>The attribute node itself is not modified in any way.</p>
      *
      * @param attribute the attribute node to be removed
@@ -723,7 +663,6 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
 
     /**
      * Add an attribute to this element node.
-     * <p/>
      * <p>If this node is not an element, or if the supplied node is not an attribute, the method
      * takes no action. If the element already has an attribute with this name, the existing attribute
      * is replaced.</p>
@@ -819,14 +758,12 @@ public final class Orphan extends GenericTreeInfo implements MutableNodeInfo {
      *                added. If the target element already has a namespace binding with this (prefix, uri) pair, the call has
      *                no effect. If the target element currently has a namespace binding with this prefix and a different URI, an
      *                exception is raised.
-     * @param inherit If true, the new namespace binding will be inherited by any children of the target element
-     *                that do not already have a namespace binding for the specified prefix, recursively.
-     *                If false, the new namespace binding will not be inherited.
+     * @param unused
      * @throws IllegalArgumentException if the namespace code is not present in the namepool, or if the target
      *                                  element already has a namespace binding for this prefix
      */
 
-    public void addNamespace(NamespaceBinding nscode, boolean inherit) {
+    public void addNamespace(NamespaceBinding nscode, boolean unused) {
         // no action: node is not an element
     }
 
