@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,19 +15,19 @@ import net.sf.saxon.om.*;
 import net.sf.saxon.value.UntypedAtomicValue;
 import net.sf.saxon.value.Whitespace;
 
+import static net.sf.saxon.type.SchemaComponent.ValidationStatus.VALIDATED;
+
 /**
  * This class has a singleton instance which represents the XML Schema built-in type xs:anySimpleType
  */
 
-public final class AnySimpleType implements SimpleType {
+public enum AnySimpleType implements SimpleType {
 
-    /*@NotNull*/ private static AnySimpleType theInstance = new AnySimpleType();
+    // Josh Bloch recommends using a single-element enum type to implement singletons
+    // (see Effective Java 2nd Edition, Item 3: Enforce the singleton property with a
+    // private constructor or an enum type).
 
-    /**
-     * Private constructor
-     */
-    private AnySimpleType() {
-    }
+    INSTANCE;
 
     /**
      * Get the local name of this type
@@ -134,13 +134,13 @@ public final class AnySimpleType implements SimpleType {
 
     /*@NotNull*/
     public static AnySimpleType getInstance() {
-        return theInstance;
+        return INSTANCE;
     }
 
     /**
      * Get the validation status - always valid
      */
-    public int getValidationStatus() {
+    public ValidationStatus getValidationStatus() {
         return VALIDATED;
     }
 
@@ -225,7 +225,7 @@ public final class AnySimpleType implements SimpleType {
      */
 
     public boolean isSameType(SchemaType other) {
-        return other instanceof AnySimpleType;
+        return other == INSTANCE;
     }
 
     /**
