@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,26 +13,23 @@ import net.sf.saxon.expr.StaticProperty;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
 import net.sf.saxon.value.UntypedAtomicValue;
+import net.sf.saxon.z.IntHashSet;
 
-import java.util.HashSet;
+import static net.sf.saxon.type.SchemaComponent.ValidationStatus.VALIDATED;
 
 /**
  * This class has a singleton instance which represents the XML Schema built-in type xs:anyType,
  * also known as the urtype.
- * <p/>
- * See XML Schema 1.1 Part 1 section 3.4.7
+ * <p>See XML Schema 1.1 Part 1 section 3.4.7</p>
  */
 
-public final class AnyType implements ComplexType {
+public enum AnyType implements ComplexType {
 
-    /*@NotNull*/ private static AnyType theInstance = new AnyType();
+    // Josh Bloch recommends using a single-element enum type to implement singletons
+    // (see Effective Java 2nd Edition, Item 3: Enforce the singleton property with a
+    // private constructor or an enum type).
 
-    /**
-     * Private constructor
-     */
-    private AnyType() {
-        super();
-    }
+    INSTANCE;
 
     /**
      * Get the singular instance of this class
@@ -42,7 +39,7 @@ public final class AnyType implements ComplexType {
 
     /*@NotNull*/
     public static AnyType getInstance() {
-        return theInstance;
+        return INSTANCE;
     }
 
     /**
@@ -90,7 +87,7 @@ public final class AnyType implements ComplexType {
     /**
      * Get the validation status - always valid
      */
-    public int getValidationStatus() {
+    public ValidationStatus getValidationStatus() {
         return VALIDATED;
     }
 
@@ -440,19 +437,6 @@ public final class AnyType implements ComplexType {
         return new UntypedAtomicValue(node.getStringValue());
     }
 
-    /**
-     * Test whether this complex type subsumes another complex type. The algorithm
-     * used is as published by Thompson and Tobin, XML Europe 2003.
-     * @param sub the other type (the type that is derived by restriction, validly or otherwise)
-     * @param compiler
-     * @return null indicating that this type does indeed subsume the other; or a string indicating
-     * why it doesn't.
-     */
-
-//    public String subsumes(ComplexType sub, ISchemaCompiler compiler) {
-//        return null;
-//    }
-
     /**
      * Check that this type is validly derived from a given type
      *
@@ -473,12 +457,12 @@ public final class AnyType implements ComplexType {
      * If there is no such particle, return null. If the fingerprint matches an element wildcard,
      * return the type of the global element declaration with the given name if one exists, or AnyType
      * if none exists and lax validation is permitted by the wildcard.
-     *  @param elementName        Identifies the name of the child element within this content model
+     * @param elementName        Identifies the name of the child element within this content model
      * @param considerExtensions true if the analysis should take into account types derived by extension
      */
 
     /*@NotNull*/
-    public SchemaType getElementParticleType(StructuredQName elementName, boolean considerExtensions) {
+    public SchemaType getElementParticleType(int elementName, boolean considerExtensions) {
         return this;
     }
 
@@ -489,10 +473,15 @@ public final class AnyType implements ComplexType {
      * {@link net.sf.saxon.expr.StaticProperty#EXACTLY_ONE}, {@link net.sf.saxon.expr.StaticProperty#ALLOWS_ZERO_OR_ONE},
      * {@link net.sf.saxon.expr.StaticProperty#ALLOWS_ZERO_OR_MORE}, {@link net.sf.saxon.expr.StaticProperty#ALLOWS_ONE_OR_MORE},
      * If there is no such particle, return zero.
-     *  @param elementName        Identifies the name of the child element within this content model
+     * @param elementName        Identifies the name of the child element within this content model
      * @param considerExtensions  true if the analysis should take into account types derived by extension
      */
 
+    public int getElementParticleCardinality(int elementName, boolean considerExtensions) {
+        return StaticProperty.ALLOWS_ZERO_OR_MORE;
+    }
+
+    @Override
     public int getElementParticleCardinality(StructuredQName elementName, boolean considerExtensions) {
         return StaticProperty.ALLOWS_ZERO_OR_MORE;
     }
@@ -519,15 +508,14 @@ public final class AnyType implements ComplexType {
      * If there is no such attribute use, return null. If the fingerprint matches an attribute wildcard,
      * return the type of the global attribute declaration with the given name if one exists, or AnySimpleType
      * if none exists and lax validation is permitted by the wildcard.
-     * <p/>
-     * If there are types derived from this type by extension, search those too.
+     * <p>If there are types derived from this type by extension, search those too.</p>
      *
      * @param attributeName Identifies the name of the child element within this content model
      * @return the schema type associated with the attribute use identified by the fingerprint.
      *         If there is no such attribute use, return null.
      */
 
-    public int getAttributeUseCardinality(StructuredQName attributeName) throws SchemaException {
+    public int getAttributeUseCardinality(StructuredQName attributeName) {
         return StaticProperty.ALLOWS_ZERO_OR_ONE;
     }
 
@@ -546,15 +534,14 @@ public final class AnyType implements ComplexType {
      * Get a list of all the names of elements that can appear as children of an element having this
      * complex type, as integer fingerprints. If the list is unbounded (because of wildcards or the use
      * of xs:anyType), return null.
-     *  @param children        an integer set, initially empty, which on return will hold the fingerprints of all permitted
+     * @param children        an integer set, initially empty, which on return will hold the fingerprints of all permitted
      *                        child elements; if the result contains the value -1, this indicates that it is not possible to enumerate
      *                        all the children, typically because of wildcards. In this case the other contents of the set should
      * @param ignoreWildcards  true if wildcards are to be ignored, rather than a wildcard causing the result to be considered
-     *                         an infinite set.
      */
 
-    public void gatherAllPermittedChildren(/*@NotNull*/ HashSet<StructuredQName> children, boolean ignoreWildcards) throws SchemaException {
-        children.add(StandardNames.getStructuredQName(StandardNames.XS_INVALID_NAME));
+    public void gatherAllPermittedChildren(/*@NotNull*/ IntHashSet children, boolean ignoreWildcards) {
+        children.add(-1);
     }
 
     /**
@@ -568,8 +555,8 @@ public final class AnyType implements ComplexType {
      *                    be ignored.
      */
 
-    public void gatherAllPermittedDescendants(/*@NotNull*/ HashSet<StructuredQName> descendants) throws SchemaException {
-        descendants.add(StandardNames.getStructuredQName(StandardNames.XS_INVALID_NAME));
+    public void gatherAllPermittedDescendants(/*@NotNull*/ IntHashSet descendants) {
+        descendants.add(-1);
     }
 
     /**
@@ -583,7 +570,7 @@ public final class AnyType implements ComplexType {
      */
 
     /*@NotNull*/
-    public SchemaType getDescendantElementType(StructuredQName fingerprint) throws SchemaException {
+    public SchemaType getDescendantElementType(int fingerprint) {
         return this;
     }
 
@@ -591,11 +578,11 @@ public final class AnyType implements ComplexType {
      * Assuming an element is a permitted descendant in the content model of this type, determine
      * the cardinality of the element when it appears as a descendant.
      *
-     * @param elementName the name of the required descendant element
+     * @param elementFingerprint the name of the required descendant element
      * @return the cardinality of the descendant element within this complex type
      */
 
-    public int getDescendantElementCardinality(StructuredQName elementName) throws SchemaException {
+    public int getDescendantElementCardinality(int elementFingerprint) {
         return StaticProperty.ALLOWS_ZERO_OR_MORE;
     }
 
