@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,9 +15,13 @@ import net.sf.saxon.lib.ConversionRules;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
+import net.sf.saxon.tree.iter.AtomicIterator;
+import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.StringValue;
 import net.sf.saxon.value.Whitespace;
 
+import static net.sf.saxon.type.SchemaComponent.ValidationStatus.VALIDATED;
+
 /**
  * <p>This class is used to implement the built-in
  * list types NMTOKENS, ENTITIES, IDREFS. It is also used to represent the anonymous type of the
@@ -28,10 +32,10 @@ public class BuiltInListType implements ListType {
 
     private int fingerprint;
 
-    /*@NotNull*/ public static BuiltInListType ENTITIES = makeListType(NamespaceConstant.SCHEMA, "ENTITIES");
-    /*@NotNull*/ public static BuiltInListType IDREFS = makeListType(NamespaceConstant.SCHEMA, "IDREFS");
-    /*@NotNull*/ public static BuiltInListType NMTOKENS = makeListType(NamespaceConstant.SCHEMA, "NMTOKENS");
-    /*@NotNull*/ public static BuiltInListType ANY_URIS = makeListType(NamespaceConstant.SCHEMA_INSTANCE, "anonymous_schemaLocationType");
+    public static BuiltInListType ENTITIES = makeListType(NamespaceConstant.SCHEMA, "ENTITIES");
+    public static BuiltInListType IDREFS = makeListType(NamespaceConstant.SCHEMA, "IDREFS");
+    public static BuiltInListType NMTOKENS = makeListType(NamespaceConstant.SCHEMA, "NMTOKENS");
+    public static BuiltInListType ANY_URIS = makeListType(NamespaceConstant.SCHEMA_INSTANCE, "anonymous_schemaLocationType");
 
     /**
      * Return true if this is an external object type, that is, a Saxon-defined type for external
@@ -121,7 +125,7 @@ public class BuiltInListType implements ListType {
     /**
      * Get the validation status - always valid
      */
-    public int getValidationStatus() {
+    public ValidationStatus getValidationStatus() {
         return VALIDATED;
     }
 
@@ -385,10 +389,9 @@ public class BuiltInListType implements ListType {
      *
      * @param type  the type from which this type is derived
      * @param block the derivations that are blocked by the relevant element declaration
-     * @throws SchemaException if the derivation is not allowed
      */
 
-    public void checkTypeDerivationIsOK(SchemaType type, int block) throws SchemaException {
+    public void checkTypeDerivationIsOK(SchemaType type, int block) {
         //
     }
 
@@ -491,7 +494,7 @@ public class BuiltInListType implements ListType {
      *
      * @param value    the string whose typed value is required
      * @param resolver namespace resolver for namespace-sensitive content
-     * @param rules
+     * @param rules    the type conversion rules to be used
      */
 
     /*@NotNull*/
@@ -502,7 +505,7 @@ public class BuiltInListType implements ListType {
         map.atomicType = (AtomicType) getItemType();
         map.rules = rules;
         try {
-            return new AtomicArray(new MappingIterator(iter, map));
+            return new AtomicArray(new MappingIterator<>(iter, map));
         } catch (XPathException err) {
             throw new ValidationException(err); // should not happen
         }
@@ -515,7 +518,7 @@ public class BuiltInListType implements ListType {
         return t;
     }
 
-    private static class ListTypeMappingFunction implements MappingFunction {
+    private static class ListTypeMappingFunction implements MappingFunction<StringValue, AtomicValue> {
 
         public NamespaceResolver resolver;
         /*@Nullable*/ public AtomicType atomicType;
@@ -528,7 +531,7 @@ public class BuiltInListType implements ListType {
          * For details see {@link net.sf.saxon.expr.MappingFunction}
          */
 
-        public SequenceIterator map(Item item) throws XPathException {
+        public AtomicIterator map(StringValue item) throws XPathException {
             try {
                 return atomicType.getTypedValue(item.getStringValueCS(), resolver, rules).iterate();
             } catch (ValidationException err) {
@@ -559,7 +562,7 @@ public class BuiltInListType implements ListType {
      * @return the value after postprocessing
      */
 
-    public CharSequence postprocess(CharSequence input) throws ValidationException {
+    public CharSequence postprocess(CharSequence input) {
         return input;
     }
 }
