@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -20,6 +20,9 @@ import net.sf.saxon.value.UntypedAtomicValue;
 import net.sf.saxon.value.Whitespace;
 
 import java.util.Collections;
+import java.util.Optional;
+
+import static net.sf.saxon.type.SchemaComponent.ValidationStatus.VALIDATED;
 
 
 /**
@@ -145,7 +148,7 @@ public final class ErrorType extends NodeTest implements AtomicType, UnionType,
     /**
      * Get the validation status - always valid
      */
-    public int getValidationStatus() {
+    public ValidationStatus getValidationStatus() {
         return VALIDATED;
     }
 
@@ -494,7 +497,7 @@ public final class ErrorType extends NodeTest implements AtomicType, UnionType,
      * @return the value after postprocessing
      */
 
-    public CharSequence postprocess(CharSequence input) throws ValidationException {
+    public CharSequence postprocess(CharSequence input) {
         return input;
     }
 
@@ -527,12 +530,8 @@ public final class ErrorType extends NodeTest implements AtomicType, UnionType,
         return BuiltInAtomicType.UNTYPED_ATOMIC;
     }
 
-    public boolean isAtomizable() {
-        return false;  //To change body of implemented methods use File | Settings | File Templates.
-    }
-
-    public void visitNamedSchemaComponents(SchemaComponentVisitor visitor) throws XPathException {
-        //To change body of implemented methods use File | Settings | File Templates.
+    public boolean isAtomizable(TypeHierarchy th) {
+        return false;
     }
 
     public SequenceType getResultTypeOfCast() {
@@ -542,24 +541,24 @@ public final class ErrorType extends NodeTest implements AtomicType, UnionType,
         return SequenceType.OPTIONAL_ITEM;
     }
 
-    public String toString() {
-        return "xs:error";
-    }
-
     /**
-     * Return a string representation of this SequenceType suitable for use in export (SEF) files.
-     * The difference from the toString() method is that the representation will not contain
-     * references to anonymous types.
+     * Return a string representation of this ItemType suitable for use in stylesheet
+     * export files. This differs from the result of toString() in that it will not contain
+     * any references to anonymous types. Note that it may also use the Saxon extended syntax
+     * for union types and tuple types. The default implementation returns the result of
+     * calling {@code toString()}.
      *
-     * @return the string representation as an instance of the XPath
-     * SequenceType construct, using only names that will be in-scope if the target environment
-     * imports the same schema as the source environment
+     * @return the string representation as an instance of the XPath SequenceType construct
      */
     @Override
     public String toExportString() {
         return toString();
     }
 
+    public String toString() {
+        return "xs:error";
+    }
+
     /**
      * Validate that a primitive atomic value is a valid instance of a type derived from the
      * same primitive type.
@@ -639,16 +638,27 @@ public final class ErrorType extends NodeTest implements AtomicType, UnionType,
      * @return a Javascript instruction or sequence of instructions, which can be used as the body
      * of a Javascript function, and which returns a boolean indication whether the value of the
      * variable "item" is an instance of this item type.
-     * @throws XPathException if JS code cannot be generated for this item type, for example because
-     *                        the test is schema-aware.
      * @param knownToBe
      * @param targetVersion
      */
     @Override
-    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) throws XPathException {
+    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) {
         return "return false;";
     }
 
+    /**
+     * Get extra diagnostic information about why a supplied item does not conform to this
+     * item type, if available. If extra information is returned, it should be in the form of a complete
+     * sentence, minus the closing full stop. No information should be returned for obvious cases.
+     *
+     * @param item the item that doesn't match this type
+     * @param th   the type hierarchy cache
+     * @return optionally, a message explaining why the item does not match the type
+     */
+    @Override
+    public Optional<String> explainMismatch(Item item, TypeHierarchy th) {
+        return Optional.of("Evaluation of the supplied expression will always fail");
+    }
 
     //#ifdefined SCHEMA
 
