@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -19,45 +19,16 @@ import net.sf.saxon.value.SequenceType;
  */
 public interface SchemaComponent {
 
+    enum ValidationStatus {UNVALIDATED, FIXED_UP, VALIDATING, VALIDATED, INVALID, INCOMPLETE}
+
     /**
      * Get the validation status of this component.
      *
-     * @return one of the values {@link #UNVALIDATED}, {@link #VALIDATING},
-     *         {@link #VALIDATED}, {@link #INVALID}, {@link #INCOMPLETE}
+     * @return one of the values {@link ValidationStatus#UNVALIDATED}, {@link ValidationStatus#VALIDATING},
+     *         {@link ValidationStatus#VALIDATED}, {@link ValidationStatus#INVALID}, {@link ValidationStatus#INCOMPLETE}
      */
 
-    int getValidationStatus();
-
-    /**
-     * Validation status: not yet validated
-     */
-    int UNVALIDATED = 0;
-
-    /**
-     * Validation status: fixed up (all references to other components have been resolved)
-     */
-    int FIXED_UP = 1;
-
-    /**
-     * Validation status: currently being validated
-     */
-    int VALIDATING = 2;
-
-    /**
-     * Validation status: successfully validated
-     */
-    int VALIDATED = 3;
-
-    /**
-     * Validation status: validation attempted and failed with fatal errors
-     */
-    int INVALID = 4;
-
-    /**
-     * Validation status: validation attempted, component contains references to
-     * other components that are not (yet) available
-     */
-    int INCOMPLETE = 5;
+    ValidationStatus getValidationStatus();
 
     /**
      * Get the redefinition level. This is zero for a component that has not been redefined;
