@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -16,11 +16,9 @@ import net.sf.saxon.trans.XPathException;
 /**
  * SchemaType is an interface implemented by all schema types: simple and complex types, built-in and
  * user-defined types.
- * <p/>
  * <p>There is a hierarchy of interfaces that extend SchemaType, representing the top levels of the schema
  * type system: SimpleType and ComplexType, with SimpleType further subdivided into List, Union, and Atomic
  * types.</p>
- * <p/>
  * <p>The implementations of these interfaces are organized into a different hierarchy: on the one side,
  * built-in types such as AnyType, AnySimpleType, and the built-in atomic types and list types; on the other
  * side, user-defined types defined in a schema.</p>
@@ -194,6 +192,22 @@ public interface SchemaType extends SchemaComponent {
 
     SchemaType getBaseType();
 
+    /**
+     * Get the nearest named type in the type hierarchy, that is, the nearest type that
+     * is not anonymous. (In practice, since types cannot be derived from anonymous types,
+     * this will either the type itself, or its immediate base type).
+     * @return the nearest type, found by following the {@code getBaseType()} relation
+     * recursively, that is not an anonymous type
+     */
+
+    default SchemaType getNearestNamedType() {
+        SchemaType type = this;
+        while (type.isAnonymousType()) {
+            type = type.getBaseType();
+        }
+        return type;
+    }
+
     /**
      * Gets the integer code of the derivation method used to derive this type from its
      * parent. Returns zero for primitive types.
