@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,6 +7,7 @@
 
 package net.sf.saxon.type;
 
+import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.parser.Token;
 import net.sf.saxon.ma.arrays.ArrayItem;
 import net.sf.saxon.ma.arrays.ArrayItemType;
@@ -15,16 +16,12 @@ import net.sf.saxon.ma.map.MapType;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.*;
 import net.sf.saxon.tree.iter.AxisIterator;
-import net.sf.saxon.value.AtomicValue;
-import net.sf.saxon.value.Cardinality;
-import net.sf.saxon.value.ObjectValue;
-import net.sf.saxon.value.SequenceType;
+import net.sf.saxon.value.*;
 
 
 /**
  * This class contains static information about types and methods for constructing type codes.
  * The class is never instantiated.
- * <p/>
  * <p><i>The constant integers used for type names in earlier versions of this class have been replaced
  * by constants in {@link StandardNames}. The constants representing {@link AtomicType} objects are now
  * available through the {@link BuiltInAtomicType} class.</i></p>
@@ -78,7 +75,14 @@ public abstract class Type {
      * Dummy node kind used in the tiny tree to contain a parent pointer
      */
     public static final short PARENT_POINTER = 12;
-
+    /**
+     * Node kind used in the tiny tree to represent an element node having simple text content
+     */
+    public static final short TEXTUAL_ELEMENT = 17; // Chosen so kind & 0x0f = Type.ELEMENT
+    /**
+     * Item type representing an external element node grafted into the tiny tree
+     */
+    public static final short EXTERNAL_NODE_REFERENCE = 33; // Chosen so kind & 0x0f = Type.ELEMENT
     /**
      * An item type that matches any node
      */
@@ -101,11 +105,6 @@ public abstract class Type {
 
     public static final short FUNCTION = 99;
 
-    public static final short MAX_NODE_TYPE = 13;
-    /**
-     * Item type that matches no items (corresponds to SequenceType empty())
-     */
-    public static final short EMPTY = 15;    // a test for this type will never be satisfied
 
     private Type() {
     }
@@ -208,8 +207,8 @@ public abstract class Type {
                 default:
                     throw new IllegalArgumentException("Unknown node kind " + node.getNodeKind());
             }
-        } else if (item instanceof ObjectValue) {
-            return ((ObjectValue) item).getItemType(th);
+        } else if (item instanceof ExternalObject) {
+            return ((ExternalObject) item).getItemType(th);
         } else if (item instanceof MapItem) {
             return ((MapItem)item).getItemType(th);
         } else if (item instanceof ArrayItem) {
@@ -255,8 +254,8 @@ public abstract class Type {
                 default:
                     return "";
             }
-        } else if (item instanceof ObjectValue) {
-            return ((ObjectValue) item).displayTypeName();
+        } else if (item instanceof ExternalObject) {
+            return ObjectValue.displayTypeName(((ExternalObject) item).getObject());
         } else if (item instanceof AtomicValue) {
             return ((AtomicValue) item).getItemType().toString();
         } else if (item instanceof Function) {
@@ -348,9 +347,10 @@ public abstract class Type {
             return t1;
         }
         if (t1 instanceof JavaExternalObjectType && t2 instanceof JavaExternalObjectType) {
+            Configuration config = ((JavaExternalObjectType) t1).getConfiguration();
             Class c1 = ((JavaExternalObjectType) t1).getJavaClass();
             Class c2 = ((JavaExternalObjectType) t2).getJavaClass();
-            return new JavaExternalObjectType(leastCommonSuperClass(c1, c2));
+            return config.getJavaExternalObjectType(leastCommonSuperClass(c1, c2));
         }
         if (t1 instanceof MapType && t2 instanceof MapType) {
             if (t1 == MapType.EMPTY_MAP_TYPE) {
@@ -426,9 +426,10 @@ public abstract class Type {
             return AnyNodeTest.getInstance();
         }
         if (t1 instanceof JavaExternalObjectType && t2 instanceof JavaExternalObjectType) {
+            Configuration config = ((JavaExternalObjectType) t1).getConfiguration();
             Class c1 = ((JavaExternalObjectType) t1).getJavaClass();
             Class c2 = ((JavaExternalObjectType) t2).getJavaClass();
-            return new JavaExternalObjectType(leastCommonSuperClass(c1, c2));
+            return config.getJavaExternalObjectType(leastCommonSuperClass(c1, c2));
         }
         return AnyItemType.getInstance();
 
@@ -436,7 +437,16 @@ public abstract class Type {
 
     }
 
-    private static Class leastCommonSuperClass(Class class1, Class class2) {
+    public static SequenceType getCommonSuperType(SequenceType t1, SequenceType t2) {
+        if (t1.equals(t2)) {
+            return t1;
+        } else {
+            return SequenceType.makeSequenceType(getCommonSuperType(t1.getPrimaryType(), t2.getPrimaryType()),
+                                                 Cardinality.union(t1.getCardinality(), t2.getCardinality()));
+        }
+    }
+
+    private static Class<?> leastCommonSuperClass(Class<?> class1, Class<?> class2) {
         if (class1 == class2) {
             return class1;
         }
