@@ -2,7 +2,7 @@
 // Copyright (c) 2013 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.type;
@@ -327,6 +327,16 @@ public class UType {
         }
     }
 
+    /**
+     * Ask whether a given Item is an instance of this UType
+     * @param item the item to be tested
+     * @return true if this UType matches the supplied item
+     */
+
+    public boolean matches(Item item) {
+        return subsumes(getUType(item));
+    }
+
     /**
      * Get the UType of an Item
      * @param item the item whose UType is required
