@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,28 +13,38 @@ import net.sf.saxon.expr.StaticProperty;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
 import net.sf.saxon.value.UntypedAtomicValue;
+import net.sf.saxon.z.IntHashSet;
 
-import java.util.HashSet;
+import static net.sf.saxon.type.SchemaComponent.ValidationStatus.VALIDATED;
 
 /**
  * This class has a singleton instance which represents the complex type xdt:untyped,
  * used for elements that have not been validated.
  */
 
-public final class Untyped implements ComplexType {
+public enum Untyped implements ComplexType {
 
-    /*@NotNull*/ private static Untyped theInstance = new Untyped();
+    // Josh Bloch recommends using a single-element enum type to implement singletons
+    // (see Effective Java 2nd Edition, Item 3: Enforce the singleton property with a
+    // private constructor or an enum type).
+
+    INSTANCE;
 
     /**
-     * Private constructor
+     * Get the singular instance of this class
+     *
+     * @return the singular object representing xs:anyType
      */
-    private Untyped() {
+
+    /*@NotNull*/
+    public static Untyped getInstance() {
+        return INSTANCE;
     }
 
     /**
      * Get the validation status - always valid
      */
-    public int getValidationStatus() {
+    public ValidationStatus getValidationStatus() {
         return VALIDATED;
     }
 
@@ -235,7 +245,7 @@ public final class Untyped implements ComplexType {
      */
 
     public boolean isSameType(SchemaType other) {
-        return (other instanceof Untyped);
+        return other == INSTANCE;
     }
 
     /**
@@ -250,18 +260,6 @@ public final class Untyped implements ComplexType {
         return AnyType.getInstance();
     }
 
-
-    /**
-     * Get the singular instance of this class
-     *
-     * @return the singular object representing xs:anyType
-     */
-
-    /*@NotNull*/
-    public static Untyped getInstance() {
-        return theInstance;
-    }
-
     /**
      * Test whether this ComplexType has been marked as abstract.
      *
@@ -432,32 +430,18 @@ public final class Untyped implements ComplexType {
         return new UntypedAtomicValue(node.getStringValueCS());
     }
 
-    /**
-     * Test whether this complex type subsumes another complex type. The algorithm
-     * used is as published by Thompson and Tobin, XML Europe 2003.
-     *
-     * @param sub      the other type (the type that is derived by restriction, validly or otherwise)
-     * @param compiler used for error reporting
-     * @return null indicating that this type does indeed subsume the other; or a string indicating
-     *         why it doesn't.
-     */
-
-//    public String subsumes(ComplexType sub, ISchemaCompiler compiler) {
-//        return null;
-//    }
-
     /**
      * Find an element particle within this complex type definition having a given element name
      * (identified by fingerprint), and return the schema type associated with that element particle.
      * If there is no such particle, return null. If the fingerprint matches an element wildcard,
      * return the type of the global element declaration with the given name if one exists, or AnyType
      * if none exists and lax validation is permitted by the wildcard.
-     *  @param elementName        Identifies the name of the child element within this content model
+     * @param elementName        Identifies the name of the child element within this content model
      * @param considerExtensions  True if types derived from this type by extension are to be included in the search
      */
 
     /*@NotNull*/
-    public SchemaType getElementParticleType(StructuredQName elementName, boolean considerExtensions) {
+    public SchemaType getElementParticleType(int elementName, boolean considerExtensions) {
         return this;
     }
 
@@ -468,10 +452,15 @@ public final class Untyped implements ComplexType {
      * {@link net.sf.saxon.expr.StaticProperty#EXACTLY_ONE}, {@link net.sf.saxon.expr.StaticProperty#ALLOWS_ZERO_OR_ONE},
      * {@link net.sf.saxon.expr.StaticProperty#ALLOWS_ZERO_OR_MORE}, {@link net.sf.saxon.expr.StaticProperty#ALLOWS_ONE_OR_MORE},
      * If there is no such particle, return zero.
-     *  @param elementName        Identifies the name of the child element within this content model
+     * @param elementName        Identifies the name of the child element within this content model
      * @param considerExtensions True if types derived from this type by extension are to be included in the search
      */
 
+    public int getElementParticleCardinality(int elementName, boolean considerExtensions) {
+        return StaticProperty.ALLOWS_ZERO_OR_MORE;
+    }
+
+    @Override
     public int getElementParticleCardinality(StructuredQName elementName, boolean considerExtensions) {
         return StaticProperty.ALLOWS_ZERO_OR_MORE;
     }
@@ -498,15 +487,14 @@ public final class Untyped implements ComplexType {
      * If there is no such attribute use, return null. If the fingerprint matches an attribute wildcard,
      * return the type of the global attribute declaration with the given name if one exists, or AnySimpleType
      * if none exists and lax validation is permitted by the wildcard.
-     * <p/>
-     * If there are types derived from this type by extension, search those too.
+     * <p>If there are types derived from this type by extension, search those too.</p>
      *
      * @param attributeName Identifies the name of the child element within this content model
      * @return the schema type associated with the attribute use identified by the fingerprint.
      *         If there is no such attribute use, return null.
      */
 
-    public int getAttributeUseCardinality(StructuredQName attributeName) throws SchemaException {
+    public int getAttributeUseCardinality(StructuredQName attributeName) {
         return StaticProperty.ALLOWS_ZERO_OR_ONE;
     }
 
@@ -526,14 +514,14 @@ public final class Untyped implements ComplexType {
      * Get a list of all the names of elements that can appear as children of an element having this
      * complex type, as integer fingerprints. If the list is unbounded (because of wildcards or the use
      * of xs:anyType), return null.
-     *  @param children        a set, initially empty, which on return will hold the names of all permitted
+     * @param children        a set, initially empty, which on return will hold the names of all permitted
      *                        child elements; if the result contains the value XS_INVALID_NAME, this indicates that it is not possible to enumerate
      *                        all the children, typically because of wildcards. In this case the other contents of the set should
      * @param ignoreWildcards True if wildcard particles in the content model should be ignored
      */
 
-    public void gatherAllPermittedChildren(HashSet<StructuredQName> children, boolean ignoreWildcards) throws SchemaException {
-        children.add(StandardNames.getStructuredQName(StandardNames.XS_INVALID_NAME));
+    public void gatherAllPermittedChildren(IntHashSet children, boolean ignoreWildcards) {
+        children.add(-1);
     }
 
     /**
@@ -547,8 +535,8 @@ public final class Untyped implements ComplexType {
      *                    be ignored.
      */
 
-    public void gatherAllPermittedDescendants(/*@NotNull*/ HashSet<StructuredQName> descendants) throws SchemaException {
-        descendants.add(StandardNames.getStructuredQName(StandardNames.XS_INVALID_NAME));
+    public void gatherAllPermittedDescendants(/*@NotNull*/ IntHashSet descendants) {
+        descendants.add(-1);
     }
 
     /**
@@ -562,7 +550,7 @@ public final class Untyped implements ComplexType {
      */
 
     /*@NotNull*/
-    public SchemaType getDescendantElementType(StructuredQName fingerprint) throws SchemaException {
+    public SchemaType getDescendantElementType(int fingerprint) {
         return this;
     }
 
@@ -570,11 +558,11 @@ public final class Untyped implements ComplexType {
      * Assuming an element is a permitted descendant in the content model of this type, determine
      * the cardinality of the element when it appears as a descendant.
      *
-     * @param elementName the name of the required descendant element
+     * @param elementFingerprint the name of the required descendant element
      * @return the cardinality of the descendant element within this complex type
      */
 
-    public int getDescendantElementCardinality(StructuredQName elementName) throws SchemaException {
+    public int getDescendantElementCardinality(int elementFingerprint) {
         return StaticProperty.ALLOWS_ZERO_OR_MORE;
     }
 
