@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,8 @@
 
 package net.sf.saxon.value;
 
-import net.sf.saxon.expr.MonoIterator;
+import net.sf.saxon.event.ReceiverOptions;
+import net.sf.saxon.tree.jiter.MonoIterator;
 import net.sf.saxon.expr.StaticContext;
 import net.sf.saxon.expr.StaticProperty;
 import net.sf.saxon.expr.XPathContext;
@@ -15,13 +16,11 @@ import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.sort.AtomicMatchKey;
 import net.sf.saxon.expr.sort.CodepointCollator;
 import net.sf.saxon.functions.AccessorFn;
-import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.NoDynamicContextException;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.tree.iter.AtomicIterator;
 import net.sf.saxon.tree.iter.SingleAtomicIterator;
 import net.sf.saxon.type.*;
 
@@ -33,18 +32,16 @@ import java.util.Iterator;
  * XPath 2.0 data model. Atomic values belong to one of the 19 primitive types
  * defined in XML Schema; or they are of type xs:untypedAtomic; or they are
  * "external objects", representing a Saxon extension to the XPath 2.0 type system.
- * <p/>
- * The AtomicValue class contains some methods that are suitable for applications
+ * <p>The AtomicValue class contains some methods that are suitable for applications
  * to use, and many others that are designed for internal use by Saxon itself.
  * These have not been fully classified. At present, therefore, none of the methods on this
- * class should be considered to be part of the public Saxon API.
- * <p/>
+ * class should be considered to be part of the public Saxon API.</p>
  *
  * @author Michael H. Kay
  */
 
-public abstract class AtomicValue extends AbstractItem
-        implements AtomicSequence, ConversionResult, IdentityComparable {
+public abstract class AtomicValue 
+        implements Item<AtomicValue>, AtomicSequence, ConversionResult, IdentityComparable {
 
     protected AtomicType typeLabel;
 
@@ -70,6 +67,16 @@ public abstract class AtomicValue extends AbstractItem
         return this;
     }
 
+    /**
+     * Get the length of the sequence
+     *
+     * @return the number of items in the sequence (always one)
+     */
+
+    public int getLength() {
+        return 1;
+    }
+
     /**
      * To implement {@link net.sf.saxon.om.Sequence}, this method returns a singleton iterator
      * that delivers this item in the form of a sequence
@@ -77,7 +84,7 @@ public abstract class AtomicValue extends AbstractItem
      * @return a singleton iterator that returns this item
      */
 
-    public final AtomicIterator iterate() {
+    public final SingleAtomicIterator iterate() {
         return new SingleAtomicIterator(this);
     }
 
@@ -93,11 +100,9 @@ public abstract class AtomicValue extends AbstractItem
         typeLabel = type;
     }
 
-
     /**
      * Get a Comparable value that implements the XML Schema ordering comparison semantics for this value.
      * An implementation must be provided for all atomic types.
-     * <p/>
      * <p>In the case of data types that are partially ordered, the returned Comparable extends the standard
      * semantics of the compareTo() method by returning the value {@link SequenceTool#INDETERMINATE_ORDERING} when there
      * is no defined order relationship between two given values. This value is also returned when two values
@@ -156,7 +161,6 @@ public abstract class AtomicValue extends AbstractItem
      * to two atomic values. When the other operand is not an atomic value, the result is undefined
      * (may be false, may be an exception). When the other operand is an atomic value that cannot be
      * compared with this one, the method must throw a ClassCastException.
-     * <p/>
      * <p>The hashCode() method is consistent with equals().</p>
      *
      * @param o the other value
@@ -243,7 +247,7 @@ public abstract class AtomicValue extends AbstractItem
      */
 
     public void process(/*@NotNull*/ XPathContext context) throws XPathException {
-        context.getReceiver().append(this, ExplicitLocation.UNKNOWN_LOCATION, NodeInfo.ALL_NAMESPACES);
+        context.getReceiver().append(this, ExplicitLocation.UNKNOWN_LOCATION, ReceiverOptions.ALL_NAMESPACES);
     }
 
     /**
@@ -253,7 +257,7 @@ public abstract class AtomicValue extends AbstractItem
      *
      * @param n position of the required item, counting from zero.
      * @return the n'th item in the sequence, where the first item in the sequence is
-     *         numbered zero. If n is negative or >= the length of the sequence, returns null.
+     *         numbered zero. If n is negative or &gt;= the length of the sequence, returns null.
      */
 
     /*@Nullable*/
@@ -434,7 +438,6 @@ public abstract class AtomicValue extends AbstractItem
      * Calling this method on a ConversionResult returns the AtomicValue that results
      * from the conversion if the conversion was successful, and throws a ValidationException
      * explaining the conversion error otherwise.
-     * <p/>
      * <p>Use this method if you are calling a conversion method that returns a ConversionResult,
      * and if you want to throw an exception if the conversion fails.</p>
      *
@@ -453,7 +456,7 @@ public abstract class AtomicValue extends AbstractItem
      */
 
     public String toString() {
-        return typeLabel.toString() + "(\"" + getStringValueCS() + "\")";
+        return typeLabel + "(\"" + getStringValueCS() + "\")";
     }
 
     /**
@@ -463,14 +466,18 @@ public abstract class AtomicValue extends AbstractItem
      */
 
     public Iterator<AtomicValue> iterator() {
-        return new MonoIterator<AtomicValue>(this);
+        return new MonoIterator<>(this);
     }
 
     /**
-     * A match key for use in situations where NaN = NaN
+     * Get the genre of this item
+     *
+     * @return the genre: specifically, {@link Genre#ATOMIC};
      */
-
-    public final static AtomicMatchKey NaN_MATCH_KEY = new QNameValue("", NamespaceConstant.SAXON, "+NaN+");
+    @Override
+    public final Genre getGenre() {
+        return Genre.ATOMIC;
+    }
 
 }
 
