@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,12 +15,17 @@ import net.sf.saxon.type.*;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.math.RoundingMode;
 
 /**
- * A value of type xs:dayTimeDuration
+ * A value of type xs:dayTimeDuration.
+ * <p>Internally this is held as an integer number of seconds held in a positive long, a positive integer
+ * number of microseconds in the range 0 to 999,999,999, and a boolean sign. Some of the constructor
+ * and accessor methods cannot handle the full range of values.</p>
  */
 
-public final class DayTimeDurationValue extends DurationValue implements Comparable {
+public final class DayTimeDurationValue extends DurationValue
+        implements Comparable<DayTimeDurationValue> {
 
     /**
      * Private constructor for internal use
@@ -32,10 +37,10 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
 
     /**
      * Factory method: create a duration value from a supplied string, in
-     * ISO 8601 format [-]PnDTnHnMnS
+     * ISO 8601 format {@code [-]PnDTnHnMnS}
      *
      * @param s the lexical representation of the xs:dayTimeDuration value
-     * @return a DayTimeDurationValue if the format is correct, or a ValidationErrorValue if not
+     * @return a {@code DayTimeDurationValue} if the format is correct, or a {@link ValidationFailure} if not
      */
 
     public static ConversionResult makeDayTimeDurationValue(CharSequence s) {
@@ -44,14 +49,15 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
             return d;
         }
         DurationValue dv = (DurationValue) d;
-        return Converter.DURATION_TO_DAY_TIME_DURATION.convert(dv);
-        //return dv.convertPrimitive(BuiltInAtomicType.DAY_TIME_DURATION, false, null);
+        return Converter.DurationToDayTimeDuration.INSTANCE.convert(dv);
     }
 
     /**
      * Create a dayTimeDuration given the number of days, hours, minutes, and seconds. This
-     * constructor performs no validation. The components (apart from sign) must all be non-negative
+     * constructor performs limited validation. The components (apart from sign) must all be non-negative
      * integers; they need not be normalized (for example, 36 hours is acceptable)
+     * <p>Note: for historic reasons this constructor only supports microsecond precision. For nanosecond
+     * precision, use the constructor {@link DayTimeDurationValue#DayTimeDurationValue(int, int, int, long, int)}</p>
      *
      * @param sign         positive number for positive durations, negative for negative duratoins
      * @param days         number of days
@@ -82,13 +88,62 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
             microseconds %= 1000000;
         }
         this.seconds = s;
-        this.microseconds = microseconds;
+        this.nanoseconds = microseconds*1000;
         if (s == 0 && microseconds == 0) {
             negative = false;
         }
         typeLabel = BuiltInAtomicType.DAY_TIME_DURATION;
     }
 
+    /**
+     * Create a dayTimeDuration given the number of days, hours, minutes, seconds, and nanoseconds. This
+     * constructor performs limited validation. The components need not be normalized (for example,
+     * 36 hours is acceptable)
+     * <p>To construct a positive duration, all the component values should be positive integers (or zero).
+     * To construct a negative duration, all the component values should be negative integers (or zero).</p>
+     *
+     * @param days         number of days
+     * @param hours        number of hours
+     * @param minutes      number of minutes
+     * @param seconds      number of seconds
+     * @param nanoseconds  number of nanoseconds
+     * @throws IllegalArgumentException if the value is out of range; specifically, if the total
+     *                                  number of seconds exceeds 2^63; or if some values are positive and
+     *                                  others are negative
+     */
+
+    public DayTimeDurationValue(int days, int hours, int minutes, long seconds, int nanoseconds)
+            throws IllegalArgumentException {
+        boolean somePositive = days > 0 || hours > 0 || minutes > 0 || seconds > 0 || nanoseconds > 0;
+        boolean someNegative = days < 0 || hours < 0 || minutes < 0 || seconds < 0 || nanoseconds < 0;
+        if (somePositive && someNegative) {
+            throw new IllegalArgumentException("Some component values are positive and others are negative");
+        }
+        if (someNegative) {
+            negative = true;
+            days = -days;
+            hours = -hours;
+            minutes = -minutes;
+            seconds = -seconds;
+            nanoseconds = -nanoseconds;
+        }
+        if ((double) days * (24 * 60 * 60) + (double) hours * (60 * 60) +
+                (double) minutes * 60 + (double) seconds > Long.MAX_VALUE) {
+            throw new IllegalArgumentException("Duration seconds limit exceeded");
+        }
+        months = 0;
+        long h = (long) days * 24L + (long) hours;
+        long m = h * 60L + (long) minutes;
+        long s = m * 60L + seconds;
+        if (nanoseconds > 1_000_000_000) {
+            s += nanoseconds / 1_000_000_000;
+            nanoseconds %= 1_000_000_000;
+        }
+        this.seconds = s;
+        this.nanoseconds = nanoseconds;
+        typeLabel = BuiltInAtomicType.DAY_TIME_DURATION;
+    }
+
     /**
      * Create a copy of this atomic value, with a different type label
      *
@@ -98,7 +153,7 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
 
     /*@NotNull*/
     public AtomicValue copyAsSubType(AtomicType typeLabel) {
-        DayTimeDurationValue v = DayTimeDurationValue.fromMicroseconds(getLengthInMicroseconds());
+        DayTimeDurationValue v = DayTimeDurationValue.fromSeconds(getTotalSeconds());
         v.typeLabel = typeLabel;
         return v;
     }
@@ -136,7 +191,7 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
         if (days != 0) {
             sb.append(days + "D");
         }
-        if (days == 0 || hours != 0 || minutes != 0 || seconds != 0 || microseconds != 0) {
+        if (days == 0 || hours != 0 || minutes != 0 || seconds != 0 || nanoseconds != 0) {
             sb.append('T');
         }
         if (hours != 0) {
@@ -145,23 +200,23 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
         if (minutes != 0) {
             sb.append(minutes + "M");
         }
-        if (seconds != 0 || microseconds != 0 || (days == 0 && minutes == 0 && hours == 0)) {
-            if (microseconds == 0) {
+        if (seconds != 0 || nanoseconds != 0 || (days == 0 && minutes == 0 && hours == 0)) {
+            if (nanoseconds == 0) {
                 sb.append(seconds + "S");
             } else {
-                long ms = (seconds * 1000000) + microseconds;
+                long ms = (seconds * 1_000_000_000L) + nanoseconds;
                 String mss = ms + "";
                 if (seconds == 0) {
-                    mss = "0000000" + mss;
-                    mss = mss.substring(mss.length() - 7);
+                    mss = "0000000000" + mss;
+                    mss = mss.substring(mss.length() - 10);
                 }
-                sb.append(mss.substring(0, mss.length() - 6));
+                sb.append(mss.substring(0, mss.length() - 9));
                 sb.append('.');
                 int lastSigDigit = mss.length() - 1;
                 while (mss.charAt(lastSigDigit) == '0') {
                     lastSigDigit--;
                 }
-                sb.append(mss.substring(mss.length() - 6, lastSigDigit + 1));
+                sb.append(mss.substring(mss.length() - 9, lastSigDigit + 1));
                 sb.append('S');
             }
         }
@@ -169,23 +224,44 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
     }
 
     /**
-     * Get length of duration in seconds
+     * Get the length of duration in seconds. Note this may involve loss of precision. For an exact
+     * result, use {@link #getTotalSeconds()}
+     * @return an approximation to the length of the duration in seconds, expressed as a double.
      */
 
     public double getLengthInSeconds() {
-        double a = seconds + ((double) microseconds / 1000000);
+        double a = seconds + ((double) nanoseconds / 1_000_000_000);
         // System.err.println("Duration length " + days + "/" + hours + "/" + minutes + "/" + seconds + " is " + a);
         return negative ? -a : a;
     }
 
     /**
-     * Get length of duration in microseconds, as a long
+     * Get the length of duration in microseconds, as a long
      *
-     * @return the length in microseconds
+     * @return the length in nanoseconds, divided by one thousand, rounded towards zero
+     * @throws ArithmeticException if the number of microseconds is too high to be returned as a long.
      */
 
     public long getLengthInMicroseconds() {
-        long a = seconds * 1000000 + microseconds;
+        if (seconds > Long.MAX_VALUE/1_000_000L) {
+            throw new ArithmeticException("Value is too large to be expressed in microseconds");
+        }
+        long a = seconds * 1_000_000L + (nanoseconds / 1000);
+        return negative ? -a : a;
+    }
+
+    /**
+     * Get the length of duration in microseconds, as a long
+     *
+     * @return the length in nanoseconds, divided by one thousand, rounded towards zero
+     * @throws ArithmeticException if the number of nanoseconds is too high to be returned as a long.
+     */
+
+    public long getLengthInNanoseconds() {
+        if (seconds > Long.MAX_VALUE / 1_000_000_000L) {
+            throw new ArithmeticException("Value is too large to be expressed in nanoseconds");
+        }
+        long a = seconds * 1_000_000_000L + nanoseconds;
         return negative ? -a : a;
     }
 
@@ -195,6 +271,7 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
      *
      * @param seconds the number of seconds in the duration. May be negative
      * @return the xs:dayTimeDuration value with the specified length
+     * @throws ArithmeticException if the number of (whole) seconds exceeds 2^63
      */
 
     public static DayTimeDurationValue fromSeconds(BigDecimal seconds) {
@@ -203,14 +280,20 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
         if (sdv.negative) {
             seconds = seconds.negate();
         }
-        BigDecimal microseconds = seconds.multiply(BigDecimalValue.BIG_DECIMAL_ONE_MILLION);
-        BigInteger intMicros = microseconds.toBigInteger();
-        BigInteger[] parts = intMicros.divideAndRemainder(BigInteger.valueOf(1000000));
-        sdv.seconds = parts[0].longValue();
-        sdv.microseconds = parts[1].intValue();
+        BigInteger wholeSeconds = seconds.toBigInteger();
+        sdv.seconds = wholeSeconds.longValueExact(); // ArithmeticException if out of range
+        BigDecimal fractionalPart = seconds.remainder(BigDecimal.ONE);
+        BigDecimal nanoseconds = fractionalPart.multiply(BigDecimalValue.BIG_DECIMAL_ONE_BILLION);
+        sdv.nanoseconds = nanoseconds.intValue();
+        if (sdv.seconds == 0 && sdv.nanoseconds == 0) {
+            // can happen with underflow (division by a very large number)
+            sdv.negative = false;
+        }
         return sdv;
     }
 
+
+
     /**
      * Construct a duration value as a number of milliseconds.
      *
@@ -237,10 +320,8 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
     /**
      * Construct a duration value as a number of microseconds.
      *
-     * @param microseconds the number of microseconds in the duration. The maximum and minimum
-     *                     limits are such that the number of days in the duration must fit in a 32-bit signed integer.
+     * @param microseconds the number of microseconds in the duration.
      * @return the xs:dayTimeDuration represented by the given number of microseconds
-     * @throws IllegalArgumentException if the value is out of range.
      */
 
     public static DayTimeDurationValue fromMicroseconds(long microseconds) throws IllegalArgumentException {
@@ -249,36 +330,140 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
             microseconds = -microseconds;
         }
         return new DayTimeDurationValue(
-                sign, 0, 0, 0, microseconds / 1000000, (int) (microseconds % 1000000));
+                sign, 0, 0, 0, microseconds / 1_000_000, (int) (microseconds % 1_000_000));
 
     }
 
+    /**
+     * Construct a duration value as a number of nanoseconds.
+     *
+     * @param nanoseconds the number of nanoseconds in the duration.
+     * @return the xs:dayTimeDuration represented by the given number of nanoseconds
+     */
+
+    public static DayTimeDurationValue fromNanoseconds(long nanoseconds) throws IllegalArgumentException {
+        int sign = Long.signum(nanoseconds);
+        return new DayTimeDurationValue(
+                    0, 0, 0, nanoseconds / 1_000_000_000L, (int) (nanoseconds % 1_000_000_000L));
+    }
+
+    /**
+     * Factory method taking a Java 8 {@link java.time.Duration} object
+     * @param duration a duration as a Java 8 {@code java.time.Duration}
+     * @since 9.9
+     */
+
+    public static DayTimeDurationValue fromJavaDuration(java.time.Duration duration) {
+        long seconds = duration.getSeconds();
+        int nanoseconds = duration.getNano();
+        boolean negative = false;
+        if (seconds < 0) {
+            return new DayTimeDurationValue(0, 0, 0, seconds, -1_000_000_000 + nanoseconds);
+        } else {
+            return new DayTimeDurationValue(0, 0, 0, seconds, nanoseconds);
+        }
+    }
+
+    /**
+     * Convert this value to a Java 8 {@link java.time.Duration} object
+     * @return the duration expressed as a Java 8 {@code java.time.Duration}
+     * @since 9.9
+     */
+
+    public java.time.Duration toJavaDuration() {
+        if (negative) {
+            return java.time.Duration.ofSeconds(-seconds, -nanoseconds);
+        } else {
+            return java.time.Duration.ofSeconds(seconds, nanoseconds);
+        }
+    }
+
 
     /**
-     * Multiply duration by a number. This is also used when dividing a duration by a number.
+     * Multiply a duration by an integer
+     *
+     * @param factor the number to multiply by
+     * @return the result of the multiplication
      */
 
-    public DurationValue multiply(double n) throws XPathException {
+    public DurationValue multiply(long factor) throws XPathException {
+        // Fast path for simple cases
+        if (Math.abs(factor) < 0x7fff_ffff && Math.abs(seconds) < 0x7fff_ffff) {
+            return new DayTimeDurationValue(0, 0, 0,
+                                            seconds * factor * (negative ? -1 : 1),
+                                            (int)(nanoseconds * factor * (negative ? -1 : 1)));
+        } else {
+            return multiply(BigDecimal.valueOf(factor));
+        }
+    }
+
+
+    /**
+     * Multiply duration by a number. Follows the semantics of op:multiply-dayTimeDuration.
+     * @param n the number to multiply by.
+     * @throws XPathException if the operand is Infinite or NaN, or if the resulting duration
+     * exceeds Saxon limits (2^63 seconds)
+     */
+
+    public DayTimeDurationValue multiply(double n) throws XPathException {
         if (Double.isNaN(n)) {
-            XPathException err = new XPathException("Cannot multiply/divide a duration by NaN");
+            XPathException err = new XPathException("Cannot multiply a duration by NaN");
             err.setErrorCode("FOCA0005");
             throw err;
         }
-        double m = (double) getLengthInMicroseconds();
-        double product = n * m;
-        if (Double.isInfinite(product) || Double.isNaN(product) ||
-                product > Long.MAX_VALUE || product < Long.MIN_VALUE) {
-            XPathException err = new XPathException("Overflow when multiplying/dividing a duration by a number");
+        if (Double.isInfinite(n)) {
+            XPathException err = new XPathException("Cannot multiply a duration by infinity");
             err.setErrorCode("FODT0002");
             throw err;
         }
+        BigDecimal factor = BigDecimal.valueOf(n);
+        return multiply(factor);
+    }
+
+    private DayTimeDurationValue multiply(BigDecimal factor) throws XPathException {
+        BigDecimal secs = getTotalSeconds();
+        BigDecimal product = secs.multiply(factor);
         try {
-            return fromMicroseconds((long) product);
-        } catch (IllegalArgumentException err) {
+            return fromSeconds(product);
+        } catch (IllegalArgumentException | ArithmeticException err) {
             if (err.getCause() instanceof XPathException) {
                 throw (XPathException) err.getCause();
             } else {
-                XPathException err2 = new XPathException("Overflow when multiplying/dividing a duration by a number", err);
+                XPathException err2 = new XPathException("Overflow when multiplying a duration by a number", err);
+                err2.setErrorCode("FODT0002");
+                throw err2;
+            }
+        }
+    }
+
+    /**
+     * Divide duration by a number. Follows the semantics of op:divide-dayTimeDuration.
+     *
+     * @param n the number to divide by.
+     * @throws XPathException if the operand is zero or NaN, or if the resulting duration
+     *                        exceeds Saxon limits (2^63 seconds)
+     */
+
+    public DurationValue divide(double n) throws XPathException {
+        if (Double.isNaN(n)) {
+            XPathException err = new XPathException("Cannot divide a duration by NaN");
+            err.setErrorCode("FOCA0005");
+            throw err;
+        }
+        if (n == 0) {
+            XPathException err = new XPathException("Cannot divide a duration by zero");
+            err.setErrorCode("FODT0002");
+            throw err;
+        }
+        BigDecimal secs = getTotalSeconds();
+        BigDecimal product = secs.divide(BigDecimal.valueOf(n));
+        try {
+            return fromSeconds(product);
+        } catch (IllegalArgumentException | ArithmeticException err) {
+            if (err.getCause() instanceof XPathException) {
+                throw (XPathException) err.getCause();
+            } else {
+                XPathException err2 = new XPathException("Overflow when dividing a duration by a number", err);
                 err2.setErrorCode("FODT0002");
                 throw err2;
             }
@@ -290,18 +475,18 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
      *
      * @param other the dividend
      * @return the ratio, as a decimal
-     * @throws XPathException
+     * @throws XPathException when dividing by zero, or when dividing two durations of different type
      */
     public BigDecimalValue divide(DurationValue other) throws XPathException {
         if (other instanceof DayTimeDurationValue) {
-            BigDecimal v1 = BigDecimal.valueOf(getLengthInMicroseconds());
-            BigDecimal v2 = BigDecimal.valueOf(((DayTimeDurationValue) other).getLengthInMicroseconds());
+            BigDecimal v1 = getTotalSeconds();
+            BigDecimal v2 = other.getTotalSeconds();
             if (v2.signum() == 0) {
                 XPathException err = new XPathException("Divide by zero (durations)");
                 err.setErrorCode("FOAR0001");
                 throw err;
             }
-            return new BigDecimalValue(v1.divide(v2, 20, BigDecimal.ROUND_HALF_EVEN));
+            return new BigDecimalValue(v1.divide(v2, 20, RoundingMode.HALF_EVEN));
         } else {
             XPathException err = new XPathException("Cannot divide two durations of different type");
             err.setErrorCode("XPTY0004");
@@ -315,13 +500,21 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
 
     public DurationValue add(DurationValue other) throws XPathException {
         if (other instanceof DayTimeDurationValue) {
-            try {
-                return fromMicroseconds(getLengthInMicroseconds() +
-                        ((DayTimeDurationValue) other).getLengthInMicroseconds());
-            } catch (IllegalArgumentException e) {
-                XPathException err = new XPathException("Overflow when adding two durations");
-                err.setErrorCode("FODT0002");
-                throw err;
+            DayTimeDurationValue d2 = (DayTimeDurationValue)other;
+            if (((seconds | d2.seconds) & 0xffff_ffff_0000_0000L) != 0) {
+                // risk of complications, use BigDecimal arithmetic
+                try {
+                    BigDecimal v1 = getTotalSeconds();
+                    BigDecimal v2 = other.getTotalSeconds();
+                    return fromSeconds(v1.add(v2));
+                } catch (IllegalArgumentException e) {
+                    XPathException err = new XPathException("Overflow when adding two durations");
+                    err.setErrorCode("FODT0002");
+                    throw err;
+                }
+            } else {
+                // fast path for common case: no risk of overflow
+                return DayTimeDurationValue.fromNanoseconds(getLengthInNanoseconds() + d2.getLengthInNanoseconds());
             }
         } else {
             XPathException err = new XPathException("Cannot add two durations of different type");
@@ -336,13 +529,21 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
 
     public DurationValue subtract(DurationValue other) throws XPathException {
         if (other instanceof DayTimeDurationValue) {
-            try {
-                return fromMicroseconds(getLengthInMicroseconds() -
-                        ((DayTimeDurationValue) other).getLengthInMicroseconds());
-            } catch (IllegalArgumentException e) {
-                XPathException err = new XPathException("Overflow when subtracting two durations");
-                err.setErrorCode("FODT0002");
-                throw err;
+            DayTimeDurationValue d2 = (DayTimeDurationValue) other;
+            if (((seconds | d2.seconds) & 0xffff_ffff_0000_0000L) != 0) {
+                // risk of complications, use BigDecimal arithmetic
+                try {
+                    BigDecimal v1 = getTotalSeconds();
+                    BigDecimal v2 = other.getTotalSeconds();
+                    return fromSeconds(v1.subtract(v2));
+                } catch (IllegalArgumentException e) {
+                    XPathException err = new XPathException("Overflow when subtracting two durations");
+                    err.setErrorCode("FODT0002");
+                    throw err;
+                }
+            } else {
+                // fast path for common case: no risk of overflow
+                return DayTimeDurationValue.fromNanoseconds(getLengthInNanoseconds() - d2.getLengthInNanoseconds());
             }
         } else {
             XPathException err = new XPathException("Cannot subtract two durations of different type");
@@ -360,30 +561,31 @@ public final class DayTimeDurationValue extends DurationValue implements Compara
      */
 
     public DurationValue negate() throws IllegalArgumentException {
-        return fromMicroseconds(-getLengthInMicroseconds());
+        DayTimeDurationValue d2 = new DayTimeDurationValue();
+        d2.setTypeLabel(typeLabel);
+        d2.seconds = seconds;
+        d2.nanoseconds = nanoseconds;
+        d2.negative = !negative;
+        return d2;
     }
 
     /**
      * Compare the value to another duration value
      *
      * @param other The other dateTime value
-     * @return negative value if this one is the earler, 0 if they are chronologically equal,
-     *         positive value if this one is the later. For this purpose, dateTime values with an unknown
-     *         timezone are considered to be UTC values (the Comparable interface requires
-     *         a total ordering).
-     * @throws ClassCastException if the other value is not a DateTimeValue (the parameter
-     *                            is declared as Object to satisfy the Comparable interface)
+     * @return negative value if this one is the smaller, 0 if they are equal,
+     *         positive value if this one is the greater.
+     * @throws ClassCastException if the other value is not a DayTimeDurationValue
      */
 
-    public int compareTo(Object other) {
+    public int compareTo(DayTimeDurationValue other) {
         if (other instanceof DayTimeDurationValue) {
-            long diff = getLengthInMicroseconds() - ((DayTimeDurationValue) other).getLengthInMicroseconds();
-            if (diff < 0) {
-                return -1;
-            } else if (diff > 0) {
-                return +1;
+            if (this.negative != other.negative) {
+                return this.negative ? -1 : +1;
+            } else if (this.seconds != other.seconds) {
+                return Long.compare(this.seconds, other.seconds) * (this.negative ? -1 : +1);
             } else {
-                return 0;
+                return Integer.compare(this.nanoseconds, other.nanoseconds) * (this.negative ? -1 : +1);
             }
         } else {
             throw new ClassCastException("Cannot compare a dayTimeDuration to an object of class "
