@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,6 +17,7 @@ import net.sf.saxon.type.BuiltInAtomicType;
 import net.sf.saxon.type.ValidationException;
 
 import java.math.BigDecimal;
+import java.math.RoundingMode;
 
 /**
  * A numeric (double precision floating point) value
@@ -153,7 +154,7 @@ public final class DoubleValue extends NumericValue {
         if (value > Integer.MIN_VALUE && value < Integer.MAX_VALUE) {
             return (int) value;
         } else {
-            return new Double(value).hashCode();
+            return Double.valueOf(value).hashCode();
         }
     }
 
@@ -171,7 +172,7 @@ public final class DoubleValue extends NumericValue {
      * @return the effective boolean value (true unless the value is zero or NaN)
      */
     public boolean effectiveBooleanValue() {
-        return (value != 0.0 && !Double.isNaN(value));
+        return value != 0.0 && !Double.isNaN(value);
     }
 
     /**
@@ -268,7 +269,7 @@ public final class DoubleValue extends NumericValue {
         if (Double.isInfinite(d)) {
             // double arithmetic has overflowed - do it in decimal
             BigDecimal dec = new BigDecimal(value);
-            dec = dec.setScale(scale, BigDecimal.ROUND_HALF_UP);
+            dec = dec.setScale(scale, RoundingMode.HALF_UP);
             return new DoubleValue(dec.doubleValue());
         }
 
@@ -276,7 +277,7 @@ public final class DoubleValue extends NumericValue {
 
         double rem = d % 10;
         if (rem >= 5) {
-            d += (10 - rem);
+            d += 10 - rem;
         } else if (rem < 5) {
             d -= rem;
         }
@@ -307,7 +308,7 @@ public final class DoubleValue extends NumericValue {
         if (Double.isInfinite(d)) {
             // double arithmetic has overflowed - do it in decimal
             BigDecimal dec = new BigDecimal(value);
-            dec = dec.setScale(scale, BigDecimal.ROUND_HALF_EVEN);
+            dec = dec.setScale(scale, RoundingMode.HALF_EVEN);
             return new DoubleValue(dec.doubleValue());
         }
 
@@ -315,7 +316,7 @@ public final class DoubleValue extends NumericValue {
 
         double rem = d % 10;
         if (rem > 5) {
-            d += (10 - rem);
+            d += 10 - rem;
         } else if (rem < 5) {
             d -= rem;
         } else {
@@ -347,9 +348,7 @@ public final class DoubleValue extends NumericValue {
         if (Double.isNaN(value)) {
             return 0;
         }
-        if (value > 0) return 1;
-        if (value == 0) return 0;
-        return -1;
+        return value > 0 ? 1 : value == 0 ? 0 : -1;
     }
 
     /**
@@ -421,8 +420,9 @@ public final class DoubleValue extends NumericValue {
      */
 
     public Comparable getSchemaComparable() {
-        // convert negative to positive zero because Double.compareTo() does the wrong thing
-        // TODO: what shall we do with NaN?
+        // Convert negative to positive zero because Double.compareTo() does the wrong thing
+        // Note that for NaN, we return NaN, and rely on the user of the Comparable to use it in a way
+        // that ensures NaN != NaN.
         return value == 0.0 ? 0.0 : value;
     }
 
@@ -466,7 +466,7 @@ public final class DoubleValue extends NumericValue {
         return v instanceof DoubleValue && DoubleSortComparer.getInstance().comparesEqual(this, v);
     }
 
-    /**
+    /*
      * Diagnostic method: print the sign, exponent, and significand
      * @param d the double to be diagnosed
      */
