@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -25,7 +25,7 @@ public abstract class GDateValue extends CalendarValue {
     protected int year;         // unlike the lexical representation, includes a year zero
     protected byte month;
     protected byte day;
-    protected boolean xsd10rules;
+    protected boolean hasNoYearZero;
     /**
      * Test whether a candidate date is actually a valid date in the proleptic Gregorian calendar
      */
@@ -66,19 +66,23 @@ public abstract class GDateValue extends CalendarValue {
     /*@NotNull*/
     public GregorianCalendar getCalendar() {
 
-        int tz = (hasTimezone() ? getTimezoneInMinutes() : 0);
-        TimeZone zone = new SimpleTimeZone(tz * 60000, "LLL");
+        int tz = hasTimezone() ? getTimezoneInMinutes() * 60000 : 0;
+        TimeZone zone = new SimpleTimeZone(tz, "LLL");
         GregorianCalendar calendar = new GregorianCalendar(zone);
         calendar.setGregorianChange(new Date(Long.MIN_VALUE));
+        if (tz < calendar.getMinimum(Calendar.ZONE_OFFSET) || tz > calendar.getMaximum(Calendar.ZONE_OFFSET)) {
+            return adjustTimezone(0).getCalendar();
+        }
         calendar.clear();
         calendar.setLenient(false);
         int yr = year;
         if (year <= 0) {
-            yr = (xsd10rules ? 1 - year : 0 - year);
+            yr = hasNoYearZero ? 1 - year : 0 - year;
             calendar.set(Calendar.ERA, GregorianCalendar.BC);
         }
+        //noinspection MagicConstant
         calendar.set(yr, month - 1, day);
-        calendar.set(Calendar.ZONE_OFFSET, tz * 60000);
+        calendar.set(Calendar.ZONE_OFFSET, tz);
         calendar.set(Calendar.DST_OFFSET, 0);
         calendar.getTime();
         return calendar;
@@ -95,8 +99,8 @@ public abstract class GDateValue extends CalendarValue {
      */
 
     /*@NotNull*/
-    protected static ConversionResult setLexicalValue(/*@NotNull*/ GDateValue d, CharSequence s, boolean allowYearZero) {
-        d.xsd10rules = !allowYearZero;
+    protected static ConversionResult setLexicalValue(GDateValue d, CharSequence s, boolean allowYearZero) {
+        d.hasNoYearZero = !allowYearZero;
         StringTokenizer tok = new StringTokenizer(Whitespace.trimWhitespace(s).toString(), "-:+TZ", true);
         try {
             if (!tok.hasMoreElements()) {
@@ -236,7 +240,7 @@ public abstract class GDateValue extends CalendarValue {
                         return badDate("Continues after timezone", s);
                     }
 
-                    tzOffset = (tzhour * 60 + tzminute);
+                    tzOffset = tzhour * 60 + tzminute;
                     if ("-".equals(delim)) {
                         tzOffset = -tzOffset;
                     }
@@ -315,9 +319,7 @@ public abstract class GDateValue extends CalendarValue {
      * to two atomic values. When the other operand is not an atomic value, the result is undefined
      * (may be false, may be an exception). When the other operand is an atomic value that cannot be
      * compared with this one, the method returns false.
-     * <p/>
      * <p>The hashCode() method is consistent with equals().</p>
-     * <p/>
      * <p>This implementation performs a context-free comparison: it fails with ClassCastException
      * if one value has a timezone and the other does not.</p>
      *
@@ -379,7 +381,7 @@ public abstract class GDateValue extends CalendarValue {
 
     /*@NotNull*/
     public DateTimeValue toDateTime() {
-        return new DateTimeValue(year, month, day, (byte) 0, (byte) 0, (byte) 0, 0, getTimezoneInMinutes(), xsd10rules);
+        return new DateTimeValue(year, month, day, (byte) 0, (byte) 0, (byte) 0, 0, getTimezoneInMinutes(), hasNoYearZero);
     }
 
 
@@ -400,7 +402,7 @@ public abstract class GDateValue extends CalendarValue {
             case YEAR_ALLOWING_ZERO:
                 return Int64Value.makeIntegerValue(year);
             case YEAR:
-                return Int64Value.makeIntegerValue(year > 0 || !xsd10rules ? year : year - 1);
+                return Int64Value.makeIntegerValue(year > 0 || !hasNoYearZero ? year : year - 1);
             case MONTH:
                 return Int64Value.makeIntegerValue(month);
             case DAY:
