@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -26,12 +26,32 @@ import java.math.BigInteger;
 
 public final class Int64Value extends IntegerValue {
 
+    /**
+     * IntegerValue representing the value -1
+     */
+    public static final Int64Value MINUS_ONE = new Int64Value(-1);
+    /**
+     * IntegerValue representing the value zero
+     */
+    public static final Int64Value ZERO = new Int64Value(0);
+    /**
+     * IntegerValue representing the value +1
+     */
+    public static final Int64Value PLUS_ONE = new Int64Value(+1);
+    /**
+     * IntegerValue representing the maximum value for a long
+     */
+    public static final Int64Value MAX_LONG = new Int64Value(Long.MAX_VALUE);
+    /**
+     * IntegerValue representing the minimum value for a long
+     */
+    public static final Int64Value MIN_LONG = new Int64Value(Long.MIN_VALUE);
     private long value;
 
     /**
      * Array of small integer values
      */
-    /*@NotNull*/ private static final Int64Value[] SMALL_INTEGERS = {
+    private static final Int64Value[] SMALL_INTEGERS = {
             new Int64Value(0),
             new Int64Value(1),
             new Int64Value(2),
@@ -132,9 +152,9 @@ public final class Int64Value extends IntegerValue {
 
     public static Int64Value signum(long val) {
         if (val == 0) {
-            return IntegerValue.ZERO;
+            return ZERO;
         } else {
-            return val < 0 ? IntegerValue.MINUS_ONE : IntegerValue.PLUS_ONE;
+            return val < 0 ? MINUS_ONE : PLUS_ONE;
         }
     }
 
@@ -312,7 +332,7 @@ public final class Int64Value extends IntegerValue {
         if (value > Integer.MIN_VALUE && value < Integer.MAX_VALUE) {
             return (int) value;
         } else {
-            return new Double(getDoubleValue()).hashCode();
+            return Double.valueOf(getDoubleValue()).hashCode();
         }
     }
 
@@ -343,16 +363,9 @@ public final class Int64Value extends IntegerValue {
      *         +1 if this value is greater
      */
 
-    public int compareTo(/*@NotNull*/ Object other) {
+    public int compareTo(NumericValue other) {
         if (other instanceof Int64Value) {
-            long val2 = ((Int64Value) other).value;
-            if (value == val2) {
-                return 0;
-            }
-            if (value < val2) {
-                return -1;
-            }
-            return 1;
+            return Long.compare(value, ((Int64Value) other).value);
         } else if (other instanceof BigIntegerValue) {
             return BigInteger.valueOf(value).compareTo(((BigIntegerValue) other).asBigInteger());
         } else if (other instanceof BigDecimalValue) {
@@ -495,7 +508,7 @@ public final class Int64Value extends IntegerValue {
      *
      * @param scale number of digits required after the decimal point; the
      *              value -2 (for example) means round to a multiple of 100
-     * @return if the scale is >=0, return this value unchanged. Otherwise
+     * @return if the scale is &gt;=0, return this value unchanged. Otherwise
      *         round it to a multiple of 10**-scale
      */
 
