@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,9 +9,7 @@ package net.sf.saxon.value;
 
 import net.sf.saxon.expr.sort.AtomicMatchKey;
 import net.sf.saxon.lib.StringCollator;
-import net.sf.saxon.regex.BMPString;
-import net.sf.saxon.regex.LatinString;
-import net.sf.saxon.regex.UnicodeString;
+import net.sf.saxon.regex.*;
 import net.sf.saxon.tree.iter.UnfailingIterator;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.AtomicType;
@@ -25,7 +23,7 @@ import net.sf.saxon.type.BuiltInAtomicType;
 
 public class StringValue extends AtomicValue {
 
-    public static final StringValue EMPTY_STRING = new StringValue(LatinString.EMPTY);
+    public static final StringValue EMPTY_STRING = new StringValue(EmptyString.THE_INSTANCE);
     public static final StringValue SINGLE_SPACE = new StringValue(LatinString.SINGLE_SPACE);
     public static final StringValue TRUE = new StringValue(new LatinString("true"));
     public static final StringValue FALSE = new StringValue(new LatinString("false"));
@@ -78,7 +76,7 @@ public class StringValue extends AtomicValue {
      */
 
     public synchronized void setContainsNoSurrogates() {
-        if (!(value instanceof BMPString || value instanceof LatinString)) {
+        if (!(value instanceof BMPString || value instanceof LatinString || value instanceof EmptyString)) {
             value = new BMPString(value);
         }
     }
@@ -186,7 +184,7 @@ public class StringValue extends AtomicValue {
 
     /**
      * Get an upper bound on the length of the string in Unicode codepoints.
-     * @return a value N such that getStringLength <= N. In practice, if the string is held
+     * @return a value N such that getStringLength &lt;= N. In practice, if the string is held
      * as UTF16 codepoints this will be the length in UTF16 codepoints; if it is held in Unicode
      * codepoints, it will be the length in Unicode codepoints
      */
@@ -273,7 +271,7 @@ public class StringValue extends AtomicValue {
      */
 
     public boolean isKnownToContainNoSurrogates() {
-        return value instanceof BMPString || value instanceof LatinString;
+        return value instanceof BMPString || value instanceof LatinString || value instanceof EmptyString;
     }
 
     /**
@@ -283,7 +281,7 @@ public class StringValue extends AtomicValue {
      */
 
     /*@NotNull*/
-    public synchronized UnfailingIterator iterateCharacters() {
+    public synchronized UnfailingIterator<Int64Value> iterateCharacters() {
         if (value instanceof UnicodeString) {
             return new UnicodeCharacterIterator((UnicodeString)value);
         } else {
@@ -368,7 +366,7 @@ public class StringValue extends AtomicValue {
      * @throws ClassCastException always
      */
 
-    @SuppressWarnings({"EqualsWhichDoesntCheckParameterClass"})
+    @SuppressWarnings({"EqualsWhichDoesntCheckParameterClass", "Contract"})
     public boolean equals(Object other) {
         throw new ClassCastException("equals on StringValue is not allowed");
     }
@@ -416,6 +414,14 @@ public class StringValue extends AtomicValue {
         return "\"" + value + '\"';
     }
 
+    public String toShortString() {
+        String s = value.toString();
+        if (s.length() > 40) {
+            s = s.substring(0,35) + "...";
+        }
+        return "\"" + s + '\"';
+    }
+
     /**
      * Get a Comparable value that implements the XML Schema comparison semantics for this value.
      * Returns null if the value is not comparable according to XML Schema rules. This implementation
@@ -477,7 +483,7 @@ public class StringValue extends AtomicValue {
      */
 
 
-    public final static class CharacterIterator implements UnfailingIterator {
+    public final static class CharacterIterator implements UnfailingIterator<Int64Value> {
 
         int inpos = 0;        // 0-based index of the current Java char
         private CharSequence value;
@@ -514,23 +520,9 @@ public class StringValue extends AtomicValue {
             }
         }
 
-        public void close() {
-        }
-
-        /**
-         * Get properties of this iterator, as a bit-significant integer.
-         *
-         * @return the properties of this iterator. This will be some combination of
-         *         properties such as {@link #GROUNDED} and {@link #LAST_POSITION_FINDER}. It is always
-         *         acceptable to return the value zero, indicating that there are no known special properties.
-         */
-
-        public int getProperties() {
-            return 0;
-        }
     }
 
-    public final static class UnicodeCharacterIterator implements UnfailingIterator {
+    public final static class UnicodeCharacterIterator implements UnfailingIterator<Int64Value> {
 
         UnicodeString uValue;
         int inpos = 0;        // 0-based index of the current Java char
@@ -568,6 +560,5 @@ public class StringValue extends AtomicValue {
         }
     }
 
-
 }
 
