@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -28,11 +28,12 @@ import java.util.Iterator;
  * This class represents a temporary tree whose root document node owns a single text node. <BR>
  */
 
-public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo, SourceLocator {
+public final class TextFragmentValue implements NodeInfo, SourceLocator {
 
     private CharSequence text;
     private String baseURI;
     private String documentURI;
+    private GenericTreeInfo treeInfo;
     private TextFragmentTextNode textNode = null;   // created on demand
 
 
@@ -44,10 +45,10 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
      */
 
     public TextFragmentValue(Configuration config, CharSequence value, String baseURI) {
-        super(config);
-        setRootNode(this);
         this.text = value;
         this.baseURI = baseURI;
+        this.treeInfo = new GenericTreeInfo(config);
+        this.treeInfo.setRootNode(this);
     }
 
     /**
@@ -57,7 +58,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
      * @since 9.7
      */
     public TreeInfo getTreeInfo() {
-        return this;
+        return treeInfo;
     }
 
     /**
@@ -70,27 +71,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
         return this;
     }
 
-    /**
-     * To implement {@link Sequence}, this method returns the item itself
-     *
-     * @return this item
-     */
-
-    public NodeInfo head() {
-        return this;
-    }
-
-    /**
-     * To implement {@link Sequence}, this method returns a singleton iterator
-     * that delivers this item in the form of a sequence
-     *
-     * @return a singleton iterator that returns this item
-     */
-
-    public SequenceIterator iterate() {
-        return SingletonIterator.makeIterator(this);
-    }
-
     /**
      * Ask whether the document contains any nodes whose type annotation is anything other than
      * UNTYPED
@@ -135,7 +115,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
      *         same node in the tree.
      */
 
-    public boolean isSameNodeInfo(NodeInfo other) {
+    public boolean equals(Object other) {
         return this == other;
     }
 
@@ -163,7 +143,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
 
     public void generateId(/*@NotNull*/ FastStringBuffer buffer) {
         buffer.append("tt");
-        buffer.append(Long.toString(getDocumentNumber()));
+        buffer.append(Long.toString(treeInfo.getDocumentNumber()));
     }
 
     /**
@@ -209,34 +189,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
         return -1;
     }
 
-    /**
-     * Determine the relative position of this node and another node, in document order,
-     * distinguishing whether the first node is a preceding, following, descendant, ancestor,
-     * or the same node as the second.
-     * <p/>
-     * The other node must always be in the same tree; the effect of calling this method
-     * when the two nodes are in different trees is undefined. If either node is a namespace
-     * or attribute node, the method should throw UnsupportedOperationException.
-     *
-     * @param other The other node, whose position is to be compared with this
-     *              node
-     * @return {@link net.sf.saxon.om.AxisInfo#PRECEDING} if this node is on the preceding axis of the other node;
-     *         {@link net.sf.saxon.om.AxisInfo#FOLLOWING} if it is on the following axis; {@link net.sf.saxon.om.AxisInfo#ANCESTOR} if the first node is an
-     *         ancestor of the second; {@link net.sf.saxon.om.AxisInfo#DESCENDANT} if the first is a descendant of the second;
-     *         {@link net.sf.saxon.om.AxisInfo#SELF} if they are the same node.
-     * @throws UnsupportedOperationException if either node is an attribute or namespace
-     * @since 9.5
-     */
-    public int comparePosition(NodeInfo other) {
-        if (this == other) {
-            return AxisInfo.SELF;
-        }
-        if (textNode == other) {
-            return AxisInfo.ANCESTOR;
-        }
-        throw new UnsupportedOperationException();
-    }
-
     /**
      * Get the fingerprint of the node, used for matching names
      */
@@ -302,18 +254,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
      */
 
     public boolean hasChildNodes() {
-        return !"".equals(text);
-    }
-
-    /**
-     * Get line number
-     *
-     * @return the line number of the node in its original source document; or
-     *         -1 if not available
-     */
-
-    public int getLineNumber() {
-        return -1;
+        return text.length() != 0;
     }
 
     /**
@@ -328,7 +269,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
     /**
      * Get the type annotation of this node, if any. The type annotation is represented as
      * SchemaType object.
-     * <p/>
      * <p>Types derived from a DTD are not reflected in the result of this method.</p>
      *
      * @return For element and attribute nodes: the type annotation derived from schema
@@ -354,7 +294,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
      *         If the bottom half is zero, then this is a namespace undeclaration rather than a declaration.
      *         The XML namespace is never included in the list. If the supplied array is larger than required,
      *         then the first unused entry will be set to -1.
-     *         <p/>
      *         <p>For a node other than an element, the method returns null.</p>
      */
 
@@ -377,41 +316,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
         return new UntypedAtomicValue(text);
     }
 
-    /**
-     * Return the public identifier for the current document event.
-     * <p/>
-     * <p>The return value is the public identifier of the document
-     * entity or of the external parsed entity in which the markup that
-     * triggered the event appears.</p>
-     *
-     * @return A string containing the public identifier, or
-     *         null if none is available.
-     * @see #getSystemId
-     */
-    /*@Nullable*/
-    public String getPublicId() {
-        return null;
-    }
-
-    /**
-     * Return the character position where the current document event ends.
-     * <p/>
-     * <p><strong>Warning:</strong> The return value from the method
-     * is intended only as an approximation for the sake of error
-     * reporting; it is not intended to provide sufficient information
-     * to edit the character content of the original XML document.</p>
-     * <p/>
-     * <p>The return value is an approximation of the column number
-     * in the document entity or external parsed entity where the
-     * markup that triggered the event appears.</p>
-     *
-     * @return The column number, or -1 if none is available.
-     * @see #getLineNumber
-     */
-    public int getColumnNumber() {
-        return -1;
-    }
-
     /**
      * Get the string value of a given attribute of this node
      *
@@ -573,7 +477,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
      */
 
     public Iterator<String> getUnparsedEntityNames() {
-        return Collections.EMPTY_LIST.iterator();
+        return Collections.emptyIterator();
     }
 
     /**
@@ -588,37 +492,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
         return null;
     }
 
-
-    /**
-     * Determine whether this node has the is-id property
-     *
-     * @return true if the node is an ID
-     */
-
-    public boolean isId() {
-        return false;
-    }
-
-    /**
-     * Determine whether this node has the is-idref property
-     *
-     * @return true if the node is an IDREF or IDREFS element or attribute
-     */
-
-    public boolean isIdref() {
-        return false;
-    }
-
-    /**
-     * Determine whether the node has the is-nilled property
-     *
-     * @return true if the node has the is-nilled property
-     */
-
-    public boolean isNilled() {
-        return false;
-    }
-
     /**
      * Make an instance of the text node
      * @return the new or existing instance
@@ -661,28 +534,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
          * @since 9.7
          */
         public TreeInfo getTreeInfo() {
-            return TextFragmentValue.this;
-        }
-
-        /**
-         * To implement {@link Sequence}, this method returns the item itself
-         *
-         * @return this item
-         */
-
-        public NodeInfo head() {
-            return this;
-        }
-
-        /**
-         * To implement {@link Sequence}, this method returns a singleton iterator
-         * that delivers this item in the form of a sequence
-         *
-         * @return a singleton iterator that returns this item
-         */
-
-        public SequenceIterator iterate() {
-            return SingletonIterator.makeIterator(this);
+            return treeInfo;
         }
 
         /**
@@ -692,14 +544,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
         public void setSystemId(String systemId) {
         }
 
-        /**
-         * Get the configuration
-         */
-
-        public Configuration getConfiguration() {
-            return getTreeInfo().getConfiguration();
-        }
-
         /**
          * Return the type of node.
          *
@@ -734,7 +578,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
          *         same node in the tree.
          */
 
-        public boolean isSameNodeInfo(NodeInfo other) {
+        public boolean equals(NodeInfo other) {
             return this == other;
         }
 
@@ -744,7 +588,7 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
 
         public void generateId(/*@NotNull*/ FastStringBuffer buffer) {
             buffer.append("tt");
-            buffer.append(Long.toString(getDocumentNumber()));
+            buffer.append(Long.toString(treeInfo.getDocumentNumber()));
             buffer.append("t1");
         }
 
@@ -783,34 +627,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
             return +1;
         }
 
-        /**
-         * Determine the relative position of this node and another node, in document order,
-         * distinguishing whether the first node is a preceding, following, descendant, ancestor,
-         * or the same node as the second.
-         * <p/>
-         * The other node must always be in the same tree; the effect of calling this method
-         * when the two nodes are in different trees is undefined. If either node is a namespace
-         * or attribute node, the method should throw UnsupportedOperationException.
-         *
-         * @param other The other node, whose position is to be compared with this
-         *              node
-         * @return {@link net.sf.saxon.om.AxisInfo#PRECEDING} if this node is on the preceding axis of the other node;
-         *         {@link net.sf.saxon.om.AxisInfo#FOLLOWING} if it is on the following axis; {@link net.sf.saxon.om.AxisInfo#ANCESTOR} if the first node is an
-         *         ancestor of the second; {@link net.sf.saxon.om.AxisInfo#DESCENDANT} if the first is a descendant of the second;
-         *         {@link net.sf.saxon.om.AxisInfo#SELF} if they are the same node.
-         * @throws UnsupportedOperationException if either node is an attribute or namespace
-         * @since 9.5
-         */
-        public int comparePosition(NodeInfo other) {
-            if (this == other) {
-                return AxisInfo.SELF;
-            }
-            if (TextFragmentValue.this == other) {
-                return AxisInfo.DESCENDANT;
-            }
-            throw new UnsupportedOperationException();
-        }
-
         /**
          * Get the fingerprint of the node, used for matching names
          */
@@ -894,17 +710,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
             return null;
         }
 
-        /**
-         * Get line number
-         *
-         * @return the line number of the node in its original source document; or
-         *         -1 if not available
-         */
-
-        public int getLineNumber() {
-            return -1;
-        }
-
         /**
          * Get an immutable copy of this Location object. By default Location objects may be mutable, so they
          * should not be saved for later use. The result of this operation holds the same location information,
@@ -917,7 +722,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
         /**
          * Get the type annotation of this node, if any. The type annotation is represented as
          * SchemaType object.
-         * <p/>
          * <p>Types derived from a DTD are not reflected in the result of this method.</p>
          *
          * @return For element and attribute nodes: the type annotation derived from schema
@@ -943,7 +747,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
          *         If the bottom half is zero, then this is a namespace undeclaration rather than a declaration.
          *         The XML namespace is never included in the list. If the supplied array is larger than required,
          *         then the first unused entry will be set to -1.
-         *         <p/>
          *         <p>For a node other than an element, the method returns null.</p>
          */
 
@@ -966,41 +769,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
             return new UntypedAtomicValue(text);
         }
 
-        /**
-         * Return the public identifier for the current document event.
-         * <p/>
-         * <p>The return value is the public identifier of the document
-         * entity or of the external parsed entity in which the markup that
-         * triggered the event appears.</p>
-         *
-         * @return A string containing the public identifier, or
-         *         null if none is available.
-         * @see #getSystemId
-         */
-        /*@Nullable*/
-        public String getPublicId() {
-            return null;
-        }
-
-        /**
-         * Return the character position where the current document event ends.
-         * <p/>
-         * <p><strong>Warning:</strong> The return value from the method
-         * is intended only as an approximation for the sake of error
-         * reporting; it is not intended to provide sufficient information
-         * to edit the character content of the original XML document.</p>
-         * <p/>
-         * <p>The return value is an approximation of the column number
-         * in the document entity or external parsed entity where the
-         * markup that triggered the event appears.</p>
-         *
-         * @return The column number, or -1 if none is available.
-         * @see #getLineNumber
-         */
-        public int getColumnNumber() {
-            return -1;
-        }
-
         /**
          * Return an enumeration over the nodes reached by the given axis from this node
          *
@@ -1062,9 +830,9 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
                     if (matchesDoc && matchesText) {
                         NodeInfo[] nodes = {this, TextFragmentValue.this};
                         return new ArrayIterator.OfNodes(nodes);
-                    } else if (matchesDoc && !matchesText) {
+                    } else if (matchesDoc /* && !matchesText */) {
                         return SingleNodeIterator.makeIterator(TextFragmentValue.this);
-                    } else if (matchesText && !matchesDoc) {
+                    } else if (matchesText /* && !matchesDoc */) {
                         return SingleNodeIterator.makeIterator(this);
                     } else {
                         return EmptyIterator.OfNodes.THE_INSTANCE;
@@ -1119,47 +887,6 @@ public final class TextFragmentValue extends GenericTreeInfo implements NodeInfo
                 throws XPathException {
             out.characters(text, locationId, 0);
         }
-
-
-        /**
-         * Determine whether this node has the is-id property
-         *
-         * @return true if the node is an ID
-         */
-
-        public boolean isId() {
-            return false;
-        }
-
-        /**
-         * Determine whether this node has the is-idref property
-         *
-         * @return true if the node is an IDREF or IDREFS element or attribute
-         */
-
-        public boolean isIdref() {
-            return false;
-        }
-
-        /**
-         * Determine whether the node has the is-nilled property
-         *
-         * @return true if the node has the is-nilled property
-         */
-
-        public boolean isNilled() {
-            return false;
-        }
-
-        /**
-         * Ask whether this is a node in a streamed document
-         *
-         * @return true if the node is in a document being processed using streaming
-         */
-
-        public boolean isStreamed() {
-            return false;
-        }
     }
 
 }
