@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,8 +10,10 @@ package net.sf.saxon.value;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.sort.CodepointCollator;
+import net.sf.saxon.expr.sort.ComparisonException;
 import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.om.StandardNames;
+import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
 
 /**
@@ -100,13 +102,14 @@ public class UntypedAtomicValue extends StringValue {
                     doubleValue = new DoubleValue(context.getConfiguration().getConversionRules()
                                                           .getStringToDoubleConverter().stringToNumber(value));
                 } catch (NumberFormatException e) {
-                    throw new ClassCastException("Cannot convert untyped atomic value '" + getStringValue()
-                                                         + "' to a double ");
+                    XPathException xe = new XPathException("Cannot convert untyped atomic value '" + getStringValue()
+                                                                   + "' to a double ", "FORG0001");
+                    throw new ComparisonException(xe);
                 }
                 cachedConversionType = StandardNames.XS_DOUBLE;
                 cachedConversionResult = doubleValue;
             }
-            return doubleValue.compareTo(other);
+            return doubleValue.compareTo((NumericValue)other);
         } else if (other instanceof StringValue) {
             if (collator instanceof CodepointCollator) {
                 // This optimization avoids creating String objects for the purpose of the comparison
@@ -119,8 +122,9 @@ public class UntypedAtomicValue extends StringValue {
             Converter converter = config.getConversionRules().getConverter(BuiltInAtomicType.UNTYPED_ATOMIC, other.getItemType());
             ConversionResult result = converter.convert(this);
             if (result instanceof ValidationFailure) {
-                throw new ClassCastException("Cannot convert untyped atomic value '" + getStringValue()
-                        + "' to type " + other.getItemType());
+                XPathException xe = new XPathException("Cannot convert untyped atomic value '" + getStringValue()
+                                                               + "' to type " + other.getItemType(), "FORG0001");
+                throw new ComparisonException(xe);
             }
             return ((Comparable) result).compareTo(other);
 
@@ -138,9 +142,16 @@ public class UntypedAtomicValue extends StringValue {
 
     /**
      * Compute and cache a conversion result converting this value to a particular type
+     * <p>This method should not be used if the converter is namespace-sensitive, since the cached
+     * value may then be incorrect.</p>
+     *
+     * @param atomicType the fingerprint of the type to which the value is being convertered
+     * @param converter the converter used to convert to the target type
      */
 
     public synchronized ConversionResult obtainConversionResult(int atomicType, StringConverter converter) {
+        // The logic here doesn't take into account the possibility that the cached result was obtained
+        // using a different converter. The risk is small, but
         ConversionResult knownResult = getConversionResultIfKnown(atomicType);
         if (knownResult == null) {
             cachedConversionType = atomicType;
@@ -159,6 +170,16 @@ public class UntypedAtomicValue extends StringValue {
         cachedConversionResult = value;
     }
 
+    /**
+     * For displaying the value in error messages, prefix with "u" to highlight that it's
+     * untyped atomic
+     * @return a short depiction of the string for use in error messages
+     */
+
+    public String toShortString() {
+        return "u" + super.toShortString();
+    }
+
 
 }
 
