@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.xpath;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Version;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.FeatureKeys;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.Validation;
@@ -19,7 +20,7 @@ import javax.xml.xpath.*;
 /**
  * Saxon implementation of the JAXP 1.3 XPathFactory
  */
-public class XPathFactoryImpl extends XPathFactory {
+public class XPathFactoryImpl extends XPathFactory implements Configuration.ApiProvider {
 
     private Configuration config;
     private XPathVariableResolver variableResolver;
@@ -119,7 +120,7 @@ public class XPathFactoryImpl extends XPathFactory {
 
     public void setFeature(String feature, boolean b) throws XPathFactoryConfigurationException {
         if (feature.equals(FEATURE_SECURE_PROCESSING)) {
-            config.setBooleanProperty(FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS, !b);
+            config.setBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS, !b);
         } else if (feature.equals(FeatureKeys.SCHEMA_VALIDATION)) {
             config.setSchemaValidationMode(b ? Validation.STRICT : Validation.STRIP);
         } else {
@@ -150,14 +151,14 @@ public class XPathFactoryImpl extends XPathFactory {
 
     public boolean getFeature(String feature) throws XPathFactoryConfigurationException {
         if (feature.equals(FEATURE_SECURE_PROCESSING)) {
-            return !config.getBooleanProperty(FeatureKeys.ALLOW_EXTERNAL_FUNCTIONS);
+            return !config.getBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS);
         } else if (feature.equals(FeatureKeys.SCHEMA_VALIDATION)) {
             return config.getSchemaValidationMode() == Validation.STRICT;
         } else {
             try {
                 Object o = config.getConfigurationProperty(feature);
                 if (o instanceof Boolean) {
-                    return ((Boolean) o);
+                    return (Boolean) o;
                 } else {
                     throw new XPathFactoryConfigurationException(
                             "Configuration property " + feature + " is not a boolean (it is an instance of " + o.getClass() + ")");
